import posix1e
import getopt
import os
import sys
import stat
import pwd
import grp
import re

import Exceptions
import DataTransferConstants

# for creating the temporary directory change ownership/group of data files
import tempfile
import shutil

# for a given database directory, returns a hash table whose keys are the project names, for each key
# there is a hash table whose keys are: 'groupReadOnly', 'userFull', 'groupReadOnly', 'groupFull'
# the values are arrays of usernames whom have these permissions
import errno

import UNIXUtilities

ACCESS_METAREAD = 1
ACCESS_METAWRITE = 2
ACCESS_ENCREAD = 4
ACCESS_DATAREAD = 8
ACCESS_DATAWRITE = 16
#ACCESS_PROJECTCREATE = 32

def isACLSupported(databaseDirectory = None):
	ACLSupport = True
	if not os.path.isdir(databaseDirectory):
		return None
	else:
		try:
			projectACLDefault = posix1e.ACL(filedef = databaseDirectory)
		except IOError, e:
			if e.errno == errno.EOPNOTSUPP:
				#print "ACL not supported"
				ACLSupport = False
			if e.errno != errno.EOPNOTSUPP:
				raise
	return ACLSupport

# we don't need this file anymore because we are using the data directory, which we can use to set an ACL that has the appropriate access rights
#def setDataReadWritePermissions(databaseDirectory = None, projectName = None, accessPerms = None):
#	if databaseDirectory == None:
#		return None
#	else:
#		if not DataTransferConstants.isProjectExist(databaseDirectory, projectName):
#			raise Exceptions.ProjectNotExist()
#		else:
#			projectDir = os.path.join(databaseDirectory, projectName)
#			if not isinstance(accessPerms, dict):
#				raise Exceptions.InvalidType()
#			else:
#				if not ('userRead' in accessPerms and 'userWrite' in accessPerms and 'groupRead' in accessPerms and 'groupWrite' in accessPerms):
#					raise Exceptions.InvalidValue()
#				else:
#					fp = open(os.path.join(projectDir, 'datapermissions.txt'), 'w')
#					fp.write('userRead: ' + ",".join(accessPerms['userRead']) + "\n")
#					fp.write('userWrite: ' + ",".join(accessPerms['userWrite']) + "\n")
#					fp.write('groupRead: ' + ",".join(accessPerms['groupRead']) + "\n")
#					fp.write('groupWrite: ' + ",".join(accessPerms['groupWrite']) + "\n")
#					fp.close()
#
#def getDataReadWritePermissions(databaseDirectory = None, projectName = None):
#	accessPerms = {}
#	accessPerms['userRead'] = []
#	accessPerms['userWrite'] = []
#	accessPerms['groupRead'] = []
#	accessPerms['groupWrite'] = []
#	if databaseDirectory == None:
#		return None
#	else:
#		if not DataTransferConstants.isProjectExist(databaseDirectory, projectName):
#			raise Exceptions.ProjectNotExist()
#		else:
#			projectDir = os.path.join(databaseDirectory, projectName)
#			if not os.path.isfile(os.path.join(projectDir, 'datapermissions.txt')):
#				setDataReadWritePermissions(databaseDirectory = databaseDirectory, projectName = projectName, accessPerms = accessPerms)
#				# the data access file does not exist, write an empty one
#				#fp = open(os.path.join(projectDir, 'datapermissions.txt'), 'w')
#				#fp.write("userRead: \nuserWrite: \ngroupRead: \ngroupWrite: \n")
#				#fp.close()
#			else:
#				fp = open(os.path.join(projectDir, 'datapermissions.txt'), 'r')
#				pat = re.compile('(userRead|userWrite|groupRead|groupWrite): (\S*)')
#				for curLine in fp:
#					mat = pat.match(curLine)
#					if mat != None:
#						if len(mat.group(2)) > 0:
#							accessPerms[mat.group(1)] = mat.group(2).split(',')
#				fp.close()
#	return accessPerms

# the strategy that we will use for unknown userids is to keep them numeric
# we can tell if the user doesnt exist if we get KeyError exceptions from the pwd and grp commands


def getProjectACLs(databaseDirectory = None, projectName = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	ACLs = {}

	projectDir = os.path.join(databaseDirectory, projectName)
	(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(projectDir)
	
	ACLs['owner'] = UNIXUtilities.getUserNameOrID(st_uid)
	ACLs['maingroup'] = UNIXUtilities.getGroupNameOrID(st_gid)
	# get the ACL entries for the project directory
	
	ACLs['users'] = {}
	ACLs['groups'] = {}

	# first, form a list of users/groups who actually have access to the project as a whole
	# these are users who have rx access to the project's main directory
	
	projectDirACLDefault = posix1e.ACL(filedef = projectDir)
	projectDirACL = posix1e.ACL(file = projectDir)
	# dataDirACL is used to determine DATAREAD and DATAWRITE
	projectDataDir = DataTransferConstants.projectDataDirectory(databaseDirectory, projectName)
	dataDirACLDefault = posix1e.ACL(filedef = projectDataDir)
	dataDirACL = posix1e.ACL(file = projectDataDir)
	
	for curEntry in projectDirACL:
		perms = curEntry.permset
		if curEntry.tag_type == posix1e.ACL_USER:
			# this will be a user id
			curUserName = UNIXUtilities.getUserNameOrID(curEntry.qualifier)
			# every user that has any access will have read and execute permissions on the project directory
			# so initialise the access mask to zero if they have 
			if perms.test(posix1e.ACL_READ) == True and perms.test(posix1e.ACL_EXECUTE) == True:
				ACLs['users'][curUserName] = 0
		elif curEntry.tag_type == posix1e.ACL_GROUP:
			curGroupName = UNIXUtilities.getGroupNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ) == True and perms.test(posix1e.ACL_EXECUTE) == True:
				ACLs['group'][curGroupName] = 0

	# set up ACLs, as in get the ACL entries from the files and directories we are testing for
	# metadata access is determined by rx access to projects

	if os.path.isfile(DataTransferConstants.metaDataFileName(databaseDirectory, projectName, 'namehashmapping.txt')):
		nameHashACL = posix1e.ACL(file = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, 'namehashmapping.txt'))
	else:
		nameHashACL = posix1e.ACL()
	# projectXMLACL is used to determine whether a user had read or write access to the metadata
	if os.path.isfile(DataTransferConstants.metaDataFileName(databaseDirectory, projectName, 'projects.xml')):
		projectXMLACL = posix1e.ACL(file = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, 'projects.xml'))
	else:
		projectXMLACL = posix1e.ACL()

	# now check for metadata access
	for curEntry in projectXMLACL:
		perms = curEntry.permset
		if curEntry.tag_type == posix1e.ACL_USER:
			curUserName = UNIXUtilities.getUserNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ):
				if curUserName in ACLs['users']:
					ACLs['users'][curUserName] |= ACCESS_METAREAD

			if perms.test(posix1e.ACL_WRITE):
				if curUserName in ACLs['users']:
					ACLs['users'][curUserName] |= ACCESS_METAWRITE
					# write access implies read access, so disable it
					ACLs['users'][curUserName] &= ~ACCESS_METAREAD

		elif curEntry.tag_type == posix1e.ACL_GROUP:
			curGroupName = UNIXUtilities.getGroupNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ):
				if curGroupName in ACLs['groups']:
					ACLs['groups'][curGroupName] |= ACCESS_METAREAD

			if perms.test(posix1e.ACL_WRITE):
				if curGroupName in ACLs['groups']:
					ACLs['groups'][curGroupName] |= ACCESS_METAWRITE
					# write access implies read access, so disable it
					ACLs['groups'][curGroupName] &= ~ACCESS_METAREAD

	# now check for encread access, must have r access on namehashmapping.txt file
	for curEntry in nameHashACL:
		perms = curEntry.permset
		if curEntry.tag_type == posix1e.ACL_USER:
			curUserName = UNIXUtilities.getUserNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ):
				if curUserName in ACLs['users']:
					ACLs['users'][curUserName] |= ACCESS_ENCREAD
		elif curEntry.tag_type == posix1e.ACL_GROUP:
			curGroupName = UNIXUtilities.getGroupNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ):
				if curGroupName in ACLs['groups']:
					ACLs['groups'][curGroupName] |= ACCESS_ENCREAD
	
	# do the data access permissions by looking at the data directory ACL


	for curEntry in dataDirACL:
		perms = curEntry.permset
		if curEntry.tag_type == posix1e.ACL_USER:
			curUserName = UNIXUtilities.getUserNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ) and perms.test(posix1e.ACL_EXECUTE):
				if curUserName in ACLs['users']:
					ACLs['users'][curUserName] |= ACCESS_DATAREAD
					# data read access implies metadata read access, so disable it
					ACLs['users'][curUserName] &= ~ACCESS_METAREAD

			if perms.test(posix1e.ACL_WRITE) and perms.test(posix1e.ACL_EXECUTE):
				if curUserName in ACLs['users']:
					ACLs['users'][curUserName] |= ACCESS_DATAWRITE
					# write access implies read access, so disable it
					ACLs['users'][curUserName] &= ~ACCESS_METAREAD
					ACLs['users'][curUserName] &= ~ACCESS_METAWRITE
					ACLs['users'][curUserName] &= ~ACCESS_ENCREAD
					ACLs['users'][curUserName] &= ~ACCESS_DATAREAD
		elif curEntry.tag_type == posix1e.ACL_GROUP:
			curGroupName = UNIXUtilities.getGroupNameOrID(curEntry.qualifier)
			if perms.test(posix1e.ACL_READ):
				if curGroupName in ACLs['groups']:
					ACLs['groups'][curGroupName] |= ACCESS_DATAREAD
					ACLs['groups'][curGroupName] &= ~ACCESS_METAREAD

			if perms.test(posix1e.ACL_WRITE):
				if curGroupName in ACLs['groups']:
					ACLs['groups'][curGroupName] |= ACCESS_DATAWRITE
					# write access implies read access, so disable it
					ACLs['groups'][curGroupName] &= ~ACCESS_METAREAD
					ACLs['groups'][curGroupName] &= ~ACCESS_METAWRITE
					ACLs['groups'][curGroupName] &= ~ACCESS_ENCREAD
					ACLs['groups'][curGroupName] &= ~ACCESS_DATAREAD
		#if curEntry.tag_type == posix1e.ACL_USER:
	return ACLs


#	dataAccessPerms = getDataReadWritePermissions(databaseDirectory = databaseDirectory, projectName = projectName)
#
#	for curUserName in dataAccessPerms['userRead']:
#		if curUserName in ACLs['users']:
#			ACLs['users'][curUserName] |= ACCESS_DATAREAD
#	for curUserName in dataAccessPerms['userWrite']:
#		if curUserName in ACLs['users']:
#			ACLs['users'][curUserName] |= ACCESS_DATAWRITE
#
#	for curGroupName in dataAccessPerms['groupRead']:
#		if curGroupName in ACLs['groups']:
#			ACLs['groups'][curGroupName] |= ACCESS_DATAREAD
#	for curGroupName in dataAccessPerms['groupWrite']:
#		if curGroupName in ACLs['groups']:
#			ACLs['groups'][curGroupName] |= ACCESS_DATAWRITE
#
	#if databaseDirectory == None:
#def getProjectACLs(databaseDirectory = None):

#databaseACLsToString(ACLs = None, databaseDirectory = None, sortBy = 'permissions'):
# if sortBy == "permissions" then each permission will be deisplayed with a list of users
#Project: project1
#Owner: projectowner, Group: projectowner
#Users (metadata read access): user1, user2
#Users (encryption reversal): user2
#
# if sortBy == "users" then each user and their permissions will be displayed
#	

def getDatabaseACLs(databaseDirectory = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	if databaseDirectory == None:
		return None
	else:
		projectList = DataTransferConstants.projectList(databaseDirectory)
		#print projectList
		#processUID = os.getuid()
		
		ACLs = {}
		
		for curProject in projectList:
			ACLs[curProject] = getProjectACLs(databaseDirectory = databaseDirectory, projectName = curProject)
		return ACLs

def ACLPermissionsToString(ACLPermissions = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	if not isinstance(ACLPermissions, int):
		raise Exceptions.InvalidType()

	if ACLPermissions == 0:
		retString = "all permissions denied"
	else:
		firstPermPrinted = False
		retString = ""
		if ACLPermissions & ACCESS_METAREAD:
			if firstPermPrinted == True:
				retString = retString + ", "
			else:
				firstPermPrinted = True
			retString = retString + "metadata read"
		if ACLPermissions & ACCESS_METAWRITE:
			if firstPermPrinted == True:
				retString = retString + ", "
			else:
				firstPermPrinted = True
			retString = retString + "metadata read/write"
		if ACLPermissions &	ACCESS_ENCREAD:
			if firstPermPrinted == True:
				retString = retString + ", "
			else:
				firstPermPrinted = True
			retString = retString + "encryption reversal"
		if ACLPermissions & ACCESS_DATAREAD:
			if firstPermPrinted == True:
				retString = retString + ", "
			else:
				firstPermPrinted = True
			retString = retString + "data read"
		if ACLPermissions & ACCESS_DATAWRITE:
			if firstPermPrinted == True:
				retString = retString + ", "
			else:
				firstPermPrinted = True
			retString = retString + "data read/write"
	return retString


def databaseACLsToString(ACLs = None, databaseDirectory = None, sortBy = 'permissions'):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE

	retString = ''
	if databaseDirectory == None:
		return None
	else:
		projectList = DataTransferConstants.projectList(databaseDirectory)

		for curProject in ACLs.iterkeys():
			curProjectDir = DataTransferConstants.projectDirectory(databaseDirectory, curProject)
			(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(curProjectDir)

			projectOwner = UNIXUtilities.getUserNameOrID(st_uid)
			projectGroup = UNIXUtilities.getGroupNameOrID(st_gid)
			retString = retString + "Project: " + curProject + "\nOwner: " + str(projectOwner) + ", Group: " + str(projectGroup) + "\n"
			
			if sortBy.lower() == 'permissions':
				categories = ['users', 'groups']
				for curCategory in categories:
					firstNamePrinted = False
					for curName in ACLs[curProject][curCategory].iterkeys():
						if ACLs[curProject][curCategory][curName] & ACCESS_METAREAD:
							if firstNamePrinted == True:
								retString = retString + ", " + str(curName)
							else:
								retString = retString + curCategory.capitalize() + " (metadata read access): " + str(curName)
								firstNamePrinted = True
					if firstNamePrinted == True:
						retString = retString + "\n"
					
					firstNamePrinted = False
					for curName in ACLs[curProject][curCategory].iterkeys():
						if ACLs[curProject][curCategory][curName] & ACCESS_METAWRITE:
							if firstNamePrinted == True:
								retString = retString + ", " + str(curName)
							else:
								retString = retString + curCategory.capitalize() + " (metadata read/write access): " + str(curName)
								firstNamePrinted = True
					if firstNamePrinted == True:
						retString = retString + "\n"
						
					firstNamePrinted = False
					for curName in ACLs[curProject][curCategory].iterkeys():
						if ACLs[curProject][curCategory][curName] & ACCESS_ENCREAD:
							if firstNamePrinted == True:
								retString = retString + ", " + str(curName)
							else:
								retString = retString + curCategory.capitalize() + " (encryption reversal): " + str(curName)
								firstNamePrinted = True
					if firstNamePrinted == True:
						retString = retString + "\n"
					
					firstNamePrinted = False
					for curName in ACLs[curProject][curCategory].iterkeys():
						if ACLs[curProject][curCategory][curName] & ACCESS_DATAREAD:
							if firstNamePrinted == True:
								retString = retString + ", " + str(curName)
							else:
								retString = retString + curCategory.capitalize() + " (data read access): " + str(curName)
								firstNamePrinted = True
					if firstNamePrinted == True:
						retString = retString + "\n"
					
					firstNamePrinted = False
					for curName in ACLs[curProject][curCategory].iterkeys():
						if ACLs[curProject][curCategory][curName] & ACCESS_DATAWRITE:
							if firstNamePrinted == True:
								retString = retString + ", " + str(curName)
							else:
								retString = retString + curCategory.capitalize() + " (data read/write access): " + str(curName)
								firstNamePrinted = True
					if firstNamePrinted == True:
						retString = retString + "\n"
				#for curCategory in categories:
			elif sortBy.lower() == 'users':
				categories = ['users', 'groups']
				for curCategory in categories:
					if len(ACLs[curProject][curCategory]) > 0:
						retString = retString + curCategory.capitalize() + "\n"
						for curUserName in ACLs[curProject][curCategory].iterkeys():
							retString = retString + "\t" + str(curUserName) + ": " + ACLPermissionsToString(ACLPermissions = ACLs[curProject][curCategory][curUserName])
							retString = retString + "\n"
			#if sortBy.lower() == 'permissions':
			retString = retString + "\n"
		#for curProject in ACLs.iterkeys():
	return retString

# checks to see if the format of this array is correct, i.e. returned by getProjectACLs
# ACLs must be a dict
# each key of ACLs must be a string, a project that exists
# each value of ACLs must be a dict, with keys 'groupReadOnly', 'userFull', 'groupReadOnly', 'groupFull'
# then each value needs to be an array of user names
def checkDatabaseACLs(ACLs = None, databaseDirectory = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	if databaseDirectory == None:
		return None
	else:
		projectList = DataTransferConstants.projectList(databaseDirectory)
		
		if not isinstance(ACLs, dict):
			return False
		else:
			for curProject in ACLs.iterkeys():
				if not curProject in projectList:
					return False
				else:
					# check for the keys
					if not ('users' in ACLs[curProject] and 'groups' in ACLs[curProject]):
						return False
					else:
						if not (isinstance(ACLs[curProject]['users'], dict) and isinstance(ACLs[curProject]['groups'], dict)):
							return False
						else:
							for curUserName in ACLs[curProject]['users'].iterkeys():
								try:
									pwd.getpwnam(curUserName)
								except KeyError, e:
									return False

							for curGroupName in ACLs[curProject]['groups'].iterkeys():
								try:
									grp.getgrnam(curGroupName)
								except KeyError, e:
									return False
							
							for curUserName in ACLs[curProject]['users'].iterkeys():
								if not isinstance(ACLs[curProject]['users'][curUserName], int):
									return False
								else:
									# go through all the cases where implied permissions are used
									if ACLs[curProject]['users'][curUserName] & ACCESS_METAWRITE and ACLs[curProject]['users'][curUserName] & ACCESS_METAREAD:
										return False
									if ACLs[curProject]['users'][curUserName] & ACCESS_DATAREAD and ACLs[curProject]['users'][curUserName] & ACCESS_METAREAD:
										return False
									if ACLs[curProject]['users'][curUserName] & ACCESS_DATAWRITE and (ACLs[curProject]['users'][curUserName] & ACCESS_METAREAD or ACLs[curProject]['users'][curUserName] & ACCESS_METAWRITE or ACLs[curProject]['users'][curUserName] & ACCESS_ENCREAD):
										return False

							#for curUserName in ACLs[curProject]['users'].iterkeys():
									
							for curGroupName in ACLs[curProject]['groups'].iterkeys():
								if not isinstance(ACLs[curProject]['groups'][curUserName], int):
									return False
								else:
									# go through all the cases where implied permissions are used
									if ACLs[curProject]['groups'][curGroupName] & ACCESS_METAWRITE and ACLs[curProject]['groups'][curGroupName] & ACCESS_METAREAD:
										return False
									if ACLs[curProject]['groups'][curGroupName] & ACCESS_DATAREAD and ACLs[curProject]['groups'][curGroupName] & ACCESS_METAREAD:
										return False
									if ACLs[curProject]['groups'][curGroupName] & ACCESS_DATAWRITE and (ACLs[curProject]['groups'][curGroupName] & ACCESS_METAREAD or ACLs[curProject]['groups'][curGroupName] & ACCESS_METAWRITE or ACLs[curProject]['groups'][curGroupName] & ACCESS_ENCREAD):
										return False
							#for curGroupName in ACLs[curProject]['groups'].iterkeys():

						#if not (isinstance(ACLs[curProject]['users'], dict) and isinstance(ACLs[curProject]['groups'], dict)):
					#if not ('users' in ACLs[curProject] and 'groups' in ACLs[curProject]]):
				#if not curProject in projectList:
			#for curProject in ACLs.iterkeys():
		#if not isinstance(ACLs, dict):
		return True
	#if databaseDirectory == None:
#def applyProjectACLs(ACLs = None, databaseDirectory = None):

def applyProjectACLs(ACLs = None, databaseDirectory = None, projectName = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	curProjectDir = os.path.join(databaseDirectory, projectName)
	
	processOwner = os.getuid()
	try:
		ownerRecord = pwd.getpwnam(ACLs['owner'])
		projectOwnerID = ownerRecord[2]
	except Exception, e:
		projectOwnerID = ACLs['owner']
	
	try:
		groupRecord = grp.getgrnam(ACLs['maingroup'])
		projectGroupID = groupRecord[2]
	except Exception, e:
		projectGroupID = ACLs['maingroup'] 
	
	if processOwner == projectOwnerID or processOwner == 0: # the second clause means that root is running the program

		#print ACLs
		# files in project directory
		# datapermissions.txt
		# privatekey.txt
		# publickey.txt
		# namehashmapping.txt
		# projects.xml
		# userfields.txt

		# owner write access
		# privatekey.txt, datapermissions.txt (read/write)

		# metadata access
		# projects.xml, userfields.txt

		# data access
		# projects.xml, userfields.txt, datapermissions.txt (read), publickey.txt

		# make the project's directory ACL
		# this has to give read and execute permission to ALL users who have access, regardless of their level

		# projectDirectoryACL, by default, contains the main project owner and group
		# projectDirectoryACL is rwxrwx---
		projectDirectoryACL = posix1e.ACL()
		
		projectDirectoryUserObjEntry = projectDirectoryACL.append()
		projectDirectoryUserObjEntry.tag_type = posix1e.ACL_USER_OBJ
		#print projectOwnerID
		#projectDirectoryUserObjEntry.qualifier = projectOwnerID
		projectDirectoryUserObjPerms = posix1e.Permset(projectDirectoryUserObjEntry)
		projectDirectoryUserObjPerms.add(posix1e.ACL_READ)
		projectDirectoryUserObjPerms.add(posix1e.ACL_WRITE)
		projectDirectoryUserObjPerms.add(posix1e.ACL_EXECUTE)
		
		projectDirectoryGroupObjEntry = projectDirectoryACL.append()
		projectDirectoryGroupObjEntry.tag_type = posix1e.ACL_GROUP_OBJ
		#projectDirectoryGroupObjEntry.qualifier = projectGroupID
		#print projectDirectoryGroupObjEntry.qualifier
		projectDirectoryGroupObjPerms = posix1e.Permset(projectDirectoryGroupObjEntry)
		projectDirectoryGroupObjPerms.add(posix1e.ACL_READ)
		projectDirectoryGroupObjPerms.add(posix1e.ACL_WRITE)
		projectDirectoryGroupObjPerms.add(posix1e.ACL_EXECUTE)
		
		projectDirectoryOtherEntry = projectDirectoryACL.append()
		projectDirectoryOtherEntry.tag_type = posix1e.ACL_OTHER
		projectDirectoryOtherPerms = posix1e.Permset(projectDirectoryOtherEntry)
		projectDirectoryOtherPerms.delete(posix1e.ACL_READ)
		projectDirectoryOtherPerms.delete(posix1e.ACL_WRITE)
		projectDirectoryOtherPerms.delete(posix1e.ACL_EXECUTE)
		
		projectDirectoryMaskEntry = projectDirectoryACL.append()
		projectDirectoryMaskEntry.tag_type = posix1e.ACL_MASK
		projectDirectoryMaskPerms = posix1e.Permset(projectDirectoryMaskEntry)
		projectDirectoryMaskPerms.add(posix1e.ACL_READ)
		projectDirectoryMaskPerms.add(posix1e.ACL_WRITE)
		projectDirectoryMaskPerms.add(posix1e.ACL_EXECUTE)

		# fileACL is rw-rw----
		fileACL = posix1e.ACL()

		fileUserObjEntry = fileACL.append()
		fileUserObjEntry.tag_type = posix1e.ACL_USER_OBJ
		#fileUserObjEntry.qualifier = projectOwnerID
		fileUserObjPerms = posix1e.Permset(fileUserObjEntry)
		fileUserObjPerms.add(posix1e.ACL_READ)
		fileUserObjPerms.add(posix1e.ACL_WRITE)
		fileUserObjPerms.delete(posix1e.ACL_EXECUTE)

		fileGroupObjEntry = fileACL.append()
		fileGroupObjEntry.tag_type = posix1e.ACL_GROUP_OBJ
		fileGroupObjPerms = posix1e.Permset(fileGroupObjEntry)
		#fileGroupEntry.qualifier = projectGroupID
		fileGroupObjPerms.add(posix1e.ACL_READ)
		fileGroupObjPerms.add(posix1e.ACL_WRITE)
		fileGroupObjPerms.delete(posix1e.ACL_EXECUTE)

		fileOtherEntry = fileACL.append()
		fileOtherEntry.tag_type = posix1e.ACL_OTHER
		fileOtherPerms = posix1e.Permset(fileOtherEntry)
		fileOtherPerms.delete(posix1e.ACL_READ)
		fileOtherPerms.delete(posix1e.ACL_WRITE)
		fileOtherPerms.delete(posix1e.ACL_EXECUTE)

		fileMaskEntry = fileACL.append()
		fileMaskEntry.tag_type = posix1e.ACL_MASK
		fileMaskPerms = posix1e.Permset(fileMaskEntry)
		fileMaskPerms.add(posix1e.ACL_READ)
		fileMaskPerms.add(posix1e.ACL_WRITE)
		fileMaskPerms.delete(posix1e.ACL_EXECUTE)

		metaDataFiles = ['projects.xml', 'userfields.txt']
		metaDataFilesACL = posix1e.ACL(acl = fileACL)

		# for data importing, the ReadFiles are those files that need to be READ only for data importing
		# the WriteFiles are those that need read/write access for data importing
		dataImportReadFiles = ['publickey.txt']
		dataImportReadFilesACL = posix1e.ACL(acl = fileACL)
		dataImportWriteFiles = ['namehashmapping.txt']
		dataImportWriteFilesACL = posix1e.ACL(acl = fileACL)
		
		encReadFiles = ['privatekey.txt']
		encReadACL = posix1e.ACL(acl = fileACL)
		
		#privateKeyACL = fileACL
		#publicKeyACL = fileACL
		#nameHashMappingACL = fileACL
		#projectsXMLACL = fileACL
		#userFields = fileACL
		
		# these are for directories and files that are rooted in projectName/data...
		dataDirectoryACL = posix1e.ACL(acl = projectDirectoryACL)
		dataDirectoryDefaultACL = posix1e.ACL(acl = projectDirectoryACL)
		
		# add the main user and group to the default ACL for the data directory
		curEntry = dataDirectoryDefaultACL.append()
		curEntry.tag_type = posix1e.ACL_USER
		curEntry.qualifier = projectOwnerID
		curPerms = posix1e.Permset(curEntry)
		curPerms.add(posix1e.ACL_READ)
		curPerms.add(posix1e.ACL_WRITE)
		curPerms.add(posix1e.ACL_EXECUTE)
		
		curEntry = dataDirectoryDefaultACL.append()
		curEntry.tag_type = posix1e.ACL_GROUP
		curEntry.qualifier = projectGroupID
		curPerms = posix1e.Permset(curEntry)
		curPerms.add(posix1e.ACL_READ)
		curPerms.add(posix1e.ACL_WRITE)
		curPerms.add(posix1e.ACL_EXECUTE)

		dataFileACL = posix1e.ACL(acl = fileACL)
		
		curEntry = dataFileACL.append()
		curEntry.tag_type = posix1e.ACL_USER
		curEntry.qualifier = projectOwnerID
		curPerms = posix1e.Permset(curEntry)
		curPerms.add(posix1e.ACL_READ)
		curPerms.add(posix1e.ACL_WRITE)
		curPerms.delete(posix1e.ACL_EXECUTE)
		
		# these files are controlled by the database owner
		databaseWideFiles = ['userfields.txt', 'protocols.xml']
		

		for curFile in databaseWideFiles:
			if os.path.isfile(os.path.join(databaseDirectory, curFile)):
				(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(os.path.join(databaseDirectory, curFile))
				if processOwner == st_uid:
					os.chmod(os.path.join(databaseDirectory, curFile), stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
				else:
					print "the following file is not owned by the user who ran the process, cannot change its mode: " + os.path.join(databaseDirectory, curFile)
			#if os.path.isfile(os.path.join(databaseDirectory, curFile)):
		#for curFile in databaseWideFiles:
			
		categories = ['users', 'groups']

		for curCategory in categories:
			for curName in ACLs[curCategory].iterkeys():
				if curCategory == 'users':
					curPWEntry = pwd.getpwnam(curName)
					curID = curPWEntry[2]
					tagType = posix1e.ACL_USER
				elif curCategory == 'groups':
					curPWEntry = pwd.getgrnam(curName)
					curID = curPWEntry[2]
					tagType = posix1e.ACL_GROUP
				
				# do the project directory ACL,
				# the user will have read and execute permissions since they have SOME level of access
				# so create and append a rule, and set the permissions to r-x
				
				curEntry = projectDirectoryACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.add(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.add(posix1e.ACL_EXECUTE)

				# do the metadata files, metaDataFiles

				# do projects.xml, users will have read access to this 
				# set the initial user access to ---
				curEntry = metaDataFilesACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
			
				# check to see if we have metaread, metawrite, dataread or datawrite permissions
				# if we have one of those, then we need read access to the metadata file
				if ACLs[curCategory][curName] & ACCESS_METAREAD or ACLs[curCategory][curName] & ACCESS_METAWRITE or ACLs[curCategory][curName] & ACCESS_DATAREAD or ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_READ)
				
				# check to see if we have metawrite or  datawrite permissions
				# if we have one of those, then we need write access to the metadata file
				if ACLs[curCategory][curName] & ACCESS_METAWRITE or ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_WRITE)
				

				# now do the data import files
				curEntry = dataImportReadFilesACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
			
				# check to see if we have metaread, metawrite, dataread or datawrite permissions
				# if we have one of those, then we need read access to the metadata file
				if ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_READ)

				# now do the data import write files
				curEntry = dataImportWriteFilesACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
			
				# check to see if we have metaread, metawrite, dataread or datawrite permissions
				# if we have one of those, then we need read access to the metadata file
				if ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_READ)
					curPermset.add(posix1e.ACL_WRITE)
				
				# now do the data files
				# first do the data directory ACLs
				curEntry = dataDirectoryACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
				
				if (ACLs[curCategory][curName] & ACCESS_DATAREAD or ACLs[curCategory][curName] & ACCESS_DATAWRITE):
					curPermset.add(posix1e.ACL_READ)
					curPermset.add(posix1e.ACL_EXECUTE)

				if ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_WRITE)
				
				curEntry = dataDirectoryDefaultACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
				
				if (ACLs[curCategory][curName] & ACCESS_DATAREAD or ACLs[curCategory][curName] & ACCESS_DATAWRITE):
					curPermset.add(posix1e.ACL_READ)
					curPermset.add(posix1e.ACL_EXECUTE)

				if ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_WRITE)
				
				curEntry = dataFileACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
				
				if (ACLs[curCategory][curName] & ACCESS_DATAREAD or ACLs[curCategory][curName] & ACCESS_DATAWRITE):
					curPermset.add(posix1e.ACL_READ)

				if ACLs[curCategory][curName] & ACCESS_DATAWRITE:
					curPermset.add(posix1e.ACL_WRITE)
				
				# encReadACL is for the private key file
				
				curEntry = encReadACL.append()
				curEntry.tag_type = tagType
				curEntry.qualifier = curID
				curPermset = posix1e.Permset(curEntry)
				curPermset.delete(posix1e.ACL_READ)
				curPermset.delete(posix1e.ACL_WRITE)
				curPermset.delete(posix1e.ACL_EXECUTE)
				
				if (ACLs[curCategory][curName] & ACCESS_ENCREAD or ACLs[curCategory][curName] & ACCESS_DATAWRITE):
					curPermset.add(posix1e.ACL_READ)

			#for curName in ACLs[curCategory].iterkeys():
		#for curCategory in categories:
		
		#print "deleting default ACL from project directory, since we don't need it"
		posix1e.delete_default(DataTransferConstants.projectDirectory(databaseDirectory, projectName))
		#print "applying project directory ACL"
		projectDirectoryACL.applyto(DataTransferConstants.projectDirectory(databaseDirectory, projectName))

#	print "metaDataFilesACL"
#	print metaDataFilesACL
#	print "dataImportReadFilesACL"
#	print dataImportReadFilesACL
#	print "dataImportWriteFilesACL"
#	print dataImportWriteFilesACL
#	print "dataDirectoryACL"
#	print dataDirectoryACL
#	print "dataDirectoryDefaultACL"
#	print dataDirectoryDefaultACL
#	print "dataFileACL"
#	print dataFileACL
#
		for curMetaDataFile in metaDataFiles:
			curFileName = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, curMetaDataFile)
			#print "applying ACL to " + curFileName
			try:
				metaDataFilesACL.applyto(curFileName)
			except IOError, e:
				if e.errno == errno.ENOENT:
					print "file: " + curFileName + " does not exist"
				else:
					raise
		for curDataImportReadFile in dataImportReadFiles:
			curFileName = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, curDataImportReadFile)
			#print "applying ACL to " + curFileName
			try:
				dataImportReadFilesACL.applyto(curFileName)
			except IOError, e:
				if e.errno == errno.ENOENT:
					print "file: " + curFileName + " does not exist"
				else:
					raise
		
		for curDataImportWriteFile in dataImportWriteFiles:
			curFileName = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, curDataImportWriteFile)
			#print "applying ACL to " + curFileName
			try:
				dataImportWriteFilesACL.applyto(curFileName)
			except IOError, e:
				if e.errno == errno.ENOENT:
					print "file: " + curFileName + " does not exist"
				else:
					raise
		for curEncReadFile in encReadFiles:
			curFileName = DataTransferConstants.metaDataFileName(databaseDirectory, projectName, curEncReadFile)
			#print "applying ACL to " + curFileName
			try:
				metaDataFilesACL.applyto(curFileName)
			except IOError, e:
				if e.errno == errno.ENOENT:
					print "file: " + curFileName + " does not exist"
				else:
					raise
		
		#now do the data directory
		projectDataDirectory = DataTransferConstants.projectDataDirectory(databaseDirectory, projectName)
		if not os.path.isdir(projectDataDirectory):
			try:
				os.mkdir(projectDataDirectory)
			except OSError, e:
				if e.errno != errno.EEXIST:
					raise
		
		#print "applying to data directory"
		dataDirectoryACL.applyto(projectDataDirectory)
		dataDirectoryDefaultACL.applyto(projectDataDirectory, posix1e.ACL_TYPE_DEFAULT)
		# set the group sticky bit on the data directory
		os.chmod(projectDataDirectory, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_ISGID)

		otherOwnerDirectoryList = []
		otherOwnerFileList = []

		for root, dirs, files in os.walk(projectDataDirectory, topdown=True):
			for name in files:
				(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(os.path.join(root, name))
				if st_uid != projectOwnerID or st_gid != projectGroupID:
					otherOwnerFileList.append(os.path.join(root, name))
			for name in dirs:
				(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(os.path.join(root, name))
				if st_uid != projectOwnerID or st_gid != projectGroupID:
					otherOwnerDirectoryList.append(os.path.join(root, name))

		if len(otherOwnerFileList) > 0 or len(otherOwnerDirectoryList) > 0:
			tempDirectory = tempfile.mkdtemp(dir=curProjectDir)
		
			# change ownership hack
			# copy files into a temporary directory, changing their ownership
			# delete original and then move back
			for curOtherOwnerFile in otherOwnerFileList:
				(head, tail) = os.path.split(curOtherOwnerFile)
				shutil.copyfile(curOtherOwnerFile, os.path.join(tempDirectory, tail))
				os.remove(curOtherOwnerFile)
				shutil.move(os.path.join(tempDirectory, tail), curOtherOwnerFile)
				if processOwner == 0:
					os.chown(curOtherOwnerFile, projectOwnerID, projectGroupID)
			
			#print otherOwnerFileList
			for curOtherOwnerDirectory in otherOwnerDirectoryList:
				(head, tail) = os.path.split(curOtherOwnerDirectory)
				os.mkdir(os.path.join(tempDirectory, tail))
				fileList = os.listdir(curOtherOwnerDirectory)
				for curFile in fileList:
					shutil.move(os.path.join(curOtherOwnerDirectory, curFile), os.path.join(tempDirectory, tail, curFile))
				os.rmdir(curOtherOwnerDirectory)
				shutil.move(os.path.join(tempDirectory, tail), curOtherOwnerDirectory)
				if processOwner == 0:
					os.chown(curOtherOwnerDirectory, projectOwnerID, projectGroupID)
			os.rmdir(tempDirectory)
		#if len(otherOwnerFileList) > 0 or len(otherOwnerDirectoryList) > 0:

		# walk through the directories and apply the ACLs
		for root, dirs, files in os.walk(projectDataDirectory, topdown=True):
			for name in files:
				dataFileACL.applyto(os.path.join(root, name))
			for name in dirs:
				dataDirectoryACL.applyto(os.path.join(root, name))
				dataDirectoryDefaultACL.applyto(os.path.join(root, name), posix1e.ACL_TYPE_DEFAULT)
				# FINALLY! change the mode of the directory to include group sticky
				os.chmod(os.path.join(root, name), stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IRGRP | stat.S_IWGRP | stat.S_IXGRP | stat.S_ISGID)
	#if processOwner == projectOwnerID:
#def applyProjectACLs(ACLs = None, databaseDirectory = None, projectName = None):

	
#def applyProjectACLs(ACLs = None, databaseDirectory = None, projectName = None):
#user::rwx
#group::r-x
#mask::r-x
#other::---
# DEFAULT ACL
#default:user::rwx
#default:group::r-x
#default:mask::r-x
#default:other::r-x


def applyDatabaseACLs(ACLs = None, databaseDirectory = None):
	global ACCESS_METAREAD
	global ACCESS_METAWRITE
	global ACCESS_ENCREAD
	global ACCESS_DATAREAD
	global ACCESS_DATAWRITE
	
	if checkDatabaseACLs(ACLs = ACLs, databaseDirectory = databaseDirectory) == True:
		for curProject in ACLs.iterkeys():
			applyProjectACLs(databaseDirectory = databaseDirectory, projectName = curProject, ACLs = ACLs[curProject])
	#if checkDatabaseACLs(ACLs = ACLs, databaseDirectory = databaseDirectory) == True:
#def applyDatabaseACLs(ACLs = None, databaseDirectory = None):

def applyDatabaseACLsRecurse(fileACL = None, directoryACL = None, directory = None):
	if os.path.isdir(directory):
		# apply the default and access ACLs to the current directory
		directoryACL.applyto(curFileName, posix1e.ACL_TYPE_DEFAULT)
		directoryACL.applyto(curFileName, posix1e.ACL_TYPE_ACCESS)
		fileList = os.listdir(directory)
		for curFile in FileList:
			curFileName = os.path.join(directory, curFile)
			if os.path.isfile(curFileName):
				fileACL.applyto(curFileName, posix1e.ACL_TYPE_ACCESS)
			elif os.path.isdir(curFileName):
				applyDatabaseACLsRecurse(fileACL = fileACL, directoryACL = directoryACL, directory = curFileName)

