padZerosLength = 4
databaseDirectory = '####'

import os
import stat
import subprocess
import re
import hashlib
import datetime

import ezPyCrypto

import Exceptions

def metaDataFileName(databaseDirectory, projectName, fileName):
	return os.path.join(databaseDirectory, projectName, fileName)

def projectDataDirectory(databaseDirectory, projectName):
	return os.path.join(databaseDirectory, projectName, 'data')

def acquisitionFileDirectory(databaseDirectory, projectName, subjectID, visitID, studyID, episodeID, acquisitionID):
	global padZerosLength
	return os.path.join(databaseDirectory, projectName, 'data', str(subjectID).zfill(padZerosLength), visitID, studyID, episodeID, str(acquisitionID).zfill(padZerosLength))

def projectDirectory(databaseDirectory, projectName):
	return os.path.join(databaseDirectory, projectName)
	

# returns the database directory
# firstly it looks for it on the command line
# then it looks for it on the environment variable
# then it returns the value specified in the above constant

floatingPointRegex = '[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?'

def getFileList(rootDirectory):
	fileList = []
	if os.path.isdir(rootDirectory):
		for root, dirs, files in os.walk(rootDirectory):
			for curFile in files:
				fileList.append(os.path.join(root, curFile))
	return fileList

def getDatabaseDirectory(databaseDirectoryCommandLine):
	databaseDirectoryEnv = os.environ.get("DT_DATABASE_DIR", None)
	if databaseDirectoryCommandLine != None:
		if not os.path.isdir(databaseDirectoryCommandLine):
			return None
		else:
			return databaseDirectoryCommandLine
	elif databaseDirectoryEnv != None:
		if not os.path.isdir(databaseDirectoryEnv):
			return None
		else:
			return databaseDirectoryEnv
	else:
		if not os.path.isdir(databaseDirectory):
			return None
		else:
			return databaseDirectory
		
def encryptTextWithProject(databaseDirectory, projectName, Text):
	encryptPipe = subprocess.Popen(["dt_encrypt_text.pl", "-p", projectName, "-d", databaseDirectory], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
	encryptPipe.stdin.write(Text)
	(textEnc, stderrdata) = encryptPipe.communicate()
	return textEnc		

# returns the location of a command returned by "which", if it can't find the command it returns None
def findCommand(commandString):
	# find the command command
	commandPipe = os.popen("which " + commandString, 'r')
	command = commandPipe.readline()
	commandPipe.close()
	command = command.strip()
	if os.path.isfile(command):
		# check to see if it is executable
		(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(command)
		if st_mode & stat.S_IXUSR and st_mode & stat.S_IXGRP and st_mode & stat.S_IXOTH:
			return command
		else:
			return None
	else:
		return None

# bash doesnt like backslashes as the last character of the data resource ID so 
# this routine simply adds a second backslash wherever it sees a single backslash or space
# so that Bash likes it
def escapeDataResourceIDForBash(dataResourceID):
	s = dataResourceID
	s = s.replace('\\', '\\\\')
	s = s.replace(' ', '\\ ')
	s = s.replace('&', '\\&')
	s = s.replace(':', '\\:')
	return s

def isValidDate(dateString):
	pat = re.compile("^\d\d\d\d\d\d\d\d$")
	if re.match(pat, dateString) == None:
		return False
	else:
		return True

def isProjectExist(databaseDirectory, projectName):
	if not os.path.isdir(os.path.join(databaseDirectory, projectName)):
		#print "The project directory does not exist"
		return False
	else:
		if not os.path.isfile(metaDataFileName(databaseDirectory, projectName, 'projects.xml')):
			#print "The project directory exists but there is no projects.xml file"
			return False
		else:
			return True

def projectList(databaseDirectory):
	projects = []
	for curFile in os.listdir(databaseDirectory):
		if isProjectExist(databaseDirectory, curFile):
			projects.append(curFile)
	return projects

def getSubjectHash(name, DOB):
	return hashlib.sha256(str(name) + str(DOB)).hexdigest()

def getAgeAtVisit(visitDate, birthDate):
	if isValidDate(visitDate) == True and isValidDate(birthDate) == True:
		mat = re.match("(\d\d\d\d)(\d\d)(\d\d)", birthDate);
		birthYear = int(mat.group(1))
		birthMonth = int(mat.group(2))
		birthDay = int(mat.group(3))

		mat = re.match("(\d\d\d\d)(\d\d)(\d\d)", visitDate);
		visitYear = int(mat.group(1))
		visitMonth = int(mat.group(2))
		visitDay = int(mat.group(3))

		date2 = datetime.date(visitYear, visitMonth, visitDay)
		date1 = datetime.date(birthYear, birthMonth, birthDay)
		dateChange = date2 - date1
		return dateChange.days
	else:
		raise Exceptions.InvalidValue()

def supportedOutputFormats():
	return ['nifti', 'mgh', 'dicom']

def isOutputFormatSupported(outputFormat):
	if not outputFormat.lower() in supportedOutputFormats():
	#if outputFormat.lower() != "nifti" and outputFormat.lower() != "minc" and outputFormat.lower() != "mgh" and outputFormat.lower() != "dicom":
	#if outputFormat.lower() != "nifti" and outputFormat.lower() != "mgh" and outputFormat.lower() != "dicom":
		return False
	else:
		return True

class encryptionModule:

	""" Reads in the public or public/private keys from the files previously saved
	    when setting up the project """
	def __init__(self, projectName=None, databaseDirectory=None, publicOrPrivate="public"):
		self.projectName = projectName
		self.databaseDirectory = databaseDirectory
		self.publicOrPrivate = publicOrPrivate

		if self.publicOrPrivate.lower() == "public":
			keyFile = file(os.path.join(self.databaseDirectory, self.projectName, 'publickey.txt'), 'r')
		elif self.publicOrPrivate.lower() == "private":
			keyFile = file(os.path.join(self.databaseDirectory, self.projectName, 'privatekey.txt'), 'r')
		else:
			print "publicOrPrivate must be \"public\" or \"private\""
			keyFile = None

		if keyFile != None:
			keyString = keyFile.read()
			keyFile.close()
			self.Key = ezPyCrypto.key(something=1024, algoPub="RSA", algoSess=None)
			#self.Key = ezPyCrypto.key(keyobj=keyString)
			self.Key.importKey(keyString)
		else:
			self.Key = None

	def encryptText(self, textToEncrypt):
		return self.Key.encStringToAscii(textToEncrypt)

	def decryptText(self, textToDecrypt):
		return self.Key.decStringFromAscii(textToDecrypt)
