import time
import fnmatch
import DataTransferConstants

import DataTransferUserFields
import DataTransferProject

from lxml import etree
import os
# represents a data resource that has been previously matched, so it exists, by dt_checkout_pattern.py
# idx is the current index 
############ DEPRECATED ###################
#class MatchedDataResource:
#	def __init__(self, projectID=None, subjectType=None, subjectID=None, visitID=None, studyID=None, episodeID=None, acquisitionID=None, dataResourceID=None):
#		self.projectID = projectID
#		self.subjectType = subjectType
#		self.subjectID = subjectID
#		self.visitID = visitID
#		self.studyID = studyID
#		self.episodeID = episodeID
#		self.acquisitionID = acquisitionID
#		self.dataResourceID = dataResourceID

#	def __str__(self):
#		return str(self.projectID) + "\t" + str(self.subjectType) + "\t" + str(self.subjectID) + "\t" + str(self.visitID) + "\t" + str(self.studyID) + "\t" + str(self.episodeID) + "\t" + str(self.acquisitionID) + "\t" + str(self.dataResourceID)

def matchStringPattern(matchString, matchPattern):
	# fnmatch performs unix style matching which is what we want, its simpler than regular expressions and people will want that
	return fnmatch.fnmatch(matchString.lower(), matchPattern.lower())

# matches patterns for numeric fields
# modes:
#	pattern is XX
#	XX may be
#		YY: match that number only
#		YY:ZZ match the range between YY and ZZ inclusive
#		YY:Inc:ZZ match values between YY and ZZ but incremented by Inc, so 1:2:5 will match 1, 3, 5
#	multiple patterns
#		AA,BB,CC match AA or BB or CC
#		
def matchNumericPattern(matchString, matchPattern):
	if matchPattern.find(',') != -1:
		individualPatterns = matchPattern.split(",")
	else:
		individualPatterns = []
		individualPatterns.append(matchPattern)
	
	for curMatchPattern in individualPatterns:
		patternArray = curMatchPattern.split(":")
		if len(patternArray) == 1:
			#so --subject-pattern=21 will select subjects with ID 21
			if patternArray[0] == "*":
				return True
			elif int(patternArray[0]) == int(matchString):
				return True
			#else:
			#	return False
		elif len(patternArray) == 2:
			#print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
			if patternArray[0] != "*" and patternArray[1] != "*":
				if int(patternArray[0]) < int(patternArray[1]):
					if int(matchString) >= int(patternArray[0]) and int(matchString) <= int(patternArray[1]):
						return True
					#else:
					#	return False
				else:
					print "Invalid pattern array, the left integer should be less than the right integer"
			elif patternArray[0] == "*" and patternArray[1] != "*":
				if int(matchString) <= int(patternArray[1]):
					return True
				#else:
				#	return False
			elif patternArray[0] != "*" and patternArray[1] == "*":
				if int(matchString) >= int(patternArray[0]):
					return True
				#else:
				#	return False
		elif len(patternArray) == 3:
			# make sure that the increment gets us from the left number to the right number
			# so if left < right then increment must be +ve
			# so if left > right then increment must be -ve
			if int(patternArray[0]) < int(patternArray[2]) and int(patternArray[1]) <= 0:
				print "Invalid numeric pattern: " + curMatchPattern
				#quit()
			elif int(patternArray[0]) > int(patternArray[2]) and int(patternArray[1]) >= 0:
				print "Invalid numeric pattern: " + curMatchPattern
				#quit()
			elif int(patternArray[0]) != int(patternArray[2]) and int(patternArray[1]) == 0:
				print "Invalid numeric pattern: " + curMatchPattern
				#quit()
			else:
				for curIdx in range(int(patternArray[0]), int(patternArray[2]) + 1, int(patternArray[1])):
					if curIdx == int(matchString):
						return True
				# we havent found a match in the for loop so this match is not valid
				#return False
		else:
			print "Invalid numeric pattern: " + curMatchPattern
			#quit()
	# A match hasn't succeeded after looking through all options
	# return false
	return False
	
def matchBooleanPattern(matchString, matchPattern):
	if not (isinstance(matchPattern, str) and isinstance(matchString, str)):
		print "matchBooleanPattern: one of the arguments was not a string, returning False"
		return False
	else:
		if matchPattern.lower() == "please edit":
			if matchString.lower() == "please edit":
				return True
		elif matchPattern.lower() == "yes" or matchPattern.lower() == "true":
			if matchString.lower() == "yes" or matchString.lower() == "true":
				return True
			else:
				return False
		elif matchPattern.lower() == "no" or matchPattern.lower() == "false":
			if matchString.lower() == "no" or matchString.lower() == "false":
				return True
			else:
				return False
		else:
			print "matchBooleanPattern: the matchPattern was not one of the accepted words, using False"
			return False
	
def matchPatternsInXML(projectName = None, databaseDirectory = None, subjectPattern = None, subjectTypePattern = None, visitPattern = None, studyPattern = None, episodePattern = None, acquisitionPattern = None, dataResourcePattern = None, userFieldsPattern = None):
	userFields = DataTransferUserFields.UserFields(projectName = projectName, databaseDirectory = databaseDirectory)

	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	matchedDataResourceArray = []
	
	# an array of dict elements
	# each element will have 'level', 'name', 'pattern'
	userFieldsToSearchFor = []
	if userFieldsPattern != None:
		userFieldsPatternElements = userFieldsPattern.split(',')
		curIdx = 0
		while(curIdx < len(userFieldsPatternElements)):
			if(curIdx < len(userFieldsPatternElements) - 2): # there has to be two elements remaining after the current one
				curLevel = userFieldsPatternElements[curIdx]
				curName = userFieldsPatternElements[curIdx + 1]
				curPattern = userFieldsPatternElements[curIdx + 2]
				if curLevel == 'subjectInfo' or curLevel == 'visit' or curLevel == 'study' or curLevel == 'episode' or curLevel == 'acquisition':
					if userFields.fieldScope(level = curLevel, name = curName) != None:
						curSearchField = {}
						curSearchField['level'] = curLevel
						curSearchField['name'] = curName
						curSearchField['pattern'] = curPattern
						curSearchField['format'] = userFields.fieldFormat(level = curLevel, name = curName)
						userFieldsToSearchFor.append(curSearchField)
					else:
						print "warning: the field (" + curLevel + ", " + curName + ") is not in the collection of user fields"
				else:
					print "warning: the level should be subjectInfo, visit, study, episode, acquisition"
			else:
				print "warning: number of elements remaining less than three, ignoring last few elements"
			curIdx = curIdx + 3
		
	# since all the ID values are stored in the DataResource nodes, we search directly through them
	dataResourceNodes = projectXML.xpath("/XCEDE/dataResource")
	curDataResourceIdx = 0
	idx = 0
	for curDataResourceNode in dataResourceNodes:
	# if the data resource passes all our ID tests then we check it out
	# so we are looking for conditions that are not met by the patterns
		checkOutCurrentDataResource = True
		if subjectTypePattern != None or subjectPattern != None or visitPattern != None or studyPattern != None or episodePattern != None or acquisitionPattern != None or dataResourcePattern != None or userFieldsPattern != None:
			if subjectTypePattern != None:
				if matchStringPattern(curDataResourceNode.get('subjectGroupID'), subjectTypePattern) == False:
					checkOutCurrentDataResource = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and checkOutCurrentDataResource == True:
				if matchNumericPattern(curDataResourceNode.get('subjectID'), subjectPattern) == False:
					checkOutCurrentDataResource = False
			
			if visitPattern != None and checkOutCurrentDataResource == True:
				if matchNumericPattern(curDataResourceNode.get('visitID'), visitPattern) == False:
					checkOutCurrentDataResource = False
			
			if studyPattern != None and checkOutCurrentDataResource == True:
				if matchStringPattern(curDataResourceNode.get('studyID'), studyPattern) == False:
					checkOutCurrentDataResource = False

			if episodePattern != None and checkOutCurrentDataResource == True:
				if matchStringPattern(curDataResourceNode.get('episodeID'), episodePattern) == False:
					checkOutCurrentDataResource = False
			
			if acquisitionPattern != None and checkOutCurrentDataResource == True:
				if matchNumericPattern(curDataResourceNode.get('acquisitionID'), acquisitionPattern) == False:
					checkOutCurrentDataResource = False
			
			if dataResourcePattern != None and checkOutCurrentDataResource == True:
				if matchStringPattern(curDataResourceNode.get('ID'), dataResourcePattern) == False:
					checkOutCurrentDataResource = False

			if userFieldsPattern != None and checkOutCurrentDataResource == True:
				for curUserField in userFieldsToSearchFor:
					# the node that we are looking for is the child node of the node at the given level
					# here we assign the variable "curParentNode" to be searched later with the given pattern
					if curUserField['level'] == 'subjectInfo':
						curSubjectNode = projectXML.xpath("/XCEDE/subject[@ID='" + curDataResourceNode.get('subjectID') + "']")
						if len(curSubjectNode) != 1:
							curParentNode = None
						else:
							curSubjectNode = curSubjectNode[0]
							for curSubjectInfoNodes in curSubjectNode.iterchildren(tag = "subjectInfo"):
								curParentNode = curSubjectInfoNodes
					elif curUserField['level'] == 'visit':
						curVisitNode = projectXML.xpath("/XCEDE/visit[@ID='" + curDataResourceNode.get('visitID') + "' and @subjectID='" + curDataResourceNode.get('subjectID') + "']")
						if len(curVisitNode) != 1:
							curParentNode = None
						else:
							curParentNode = curVisitNode[0]
					elif curUserField['level'] == 'study':
						curStudyNode = projectXML.xpath("/XCEDE/study[@ID='" + curDataResourceNode.get('studyID') + "' and @visitID='" + curDataResourceNode.get('visitID') + "' and @subjectID='" + curDataResourceNode.get('subjectID') + "']")
						if len(curStudyNode) != 1:
							curParentNode = None
						else:
							curParentNode = curStudyNode[0]
					elif curUserField['level'] == 'episode':
						curEpisodeNode = projectXML.xpath("/XCEDE/episode[@ID='" + curDataResourceNode.get('episodeID') + "' and @studyID='" + curDataResourceNode.get('studyID') + "' and @visitID='" + curDataResourceNode.get('visitID') + "' and @subjectID='" + curDataResourceNode.get('subjectID') + "']")
						if len(curEpisodeNode) != 1:
							curParentNode = None
						else:
							curParentNode = curEpisodeNode[0]
					elif curUserField['level'] == 'acquisition':
						curAcquisitionNode = projectXML.xpath("/XCEDE/acquisition[@ID='" + curDataResourceNode.get('acquisitionID') + "' and @episodeID='" + curDataResourceNode.get('episodeID') + "' and @studyID='" + curDataResourceNode.get('studyID') + "' and @visitID='" + curDataResourceNode.get('visitID') + "' and @subjectID='" + curDataResourceNode.get('subjectID') + "']")
						if len(curAcquisitionNode) != 1:
							curParentNode = None
						else:
							curParentNode = curAcquisitionNode[0]
					elif curUserField['level'] == 'dataResource':
						curParentNode = curDataResourceNode

					for curDetailNode in curParentNode.iterchildren(tag = curUserField['name']):
						if curUserField['format'] == 'string':
							if matchStringPattern(curDetailNode.text, curUserField['pattern']) == False:
								checkOutCurrentDataResource = False
						elif curUserField['format'] == 'numeric':
							if matchNumericPattern(curDetailNode.text, curUserField['pattern']) == False:
								checkOutCurrentDataResource = False
						elif curUserField['format'] == 'boolean':
							if matchBooleanPattern(curDetailNode.text, curUserField['pattern']) == False:
								checkOutCurrentDataResource = False
		
		#if subjectTypePattern != None or subjectPattern != None or visitPattern != None or episodePattern != None or acquisitionPattern != None or dataResourcePattern != None:
		if checkOutCurrentDataResource == True:
			idx = idx + 1

			curMatchedDataResource = DataTransferProject.DataResource(projectName = projectName, \
			subjectType = curDataResourceNode.get('subjectGroupID'), \
			subjectID = int(curDataResourceNode.get('subjectID')), \
			visitID = curDataResourceNode.get('visitID'), \
			studyID = curDataResourceNode.get('studyID'), \
			episodeID = curDataResourceNode.get('episodeID'), \
			acquisitionID = int(curDataResourceNode.get('acquisitionID')), \
			dataResourceID = curDataResourceNode.get('ID'))
			# set all the user fields
			curMatchedDataResource.setDataResourceFieldsFromXML(dataResourceNode = curDataResourceNode)
			curMatchedDataResource.getOtherFieldsFromXML(projectXML = projectXML)
			#print curMatchedDataResource.getSubjectInfoFields()
			#print curMatchedDataResource.getVisitFields()
			#print curMatchedDataResource.getStudyFields()
			#print curMatchedDataResource.getEpisodeFields()
			#print curMatchedDataResource.getAcquisitionFields()
			#print curMatchedDataResource
			matchedDataResourceArray.append(curMatchedDataResource)
	
	return matchedDataResourceArray
