import os
import tempfile
import sys
import re
import errno
import shutil
import time 

from lxml import etree

import Exceptions
import DataTransferConstants
import DataTransferUserFields

import csv

class Subject:

	# subject has
	#	name
	#	subjectType
	#	subjectID

	def __init__(self, name = None, address = None, DOB = None, species = None, sex = None, hash = None, subjectType = None, nameEnc = None, addressEnc = None):
		self.setName(name)
		self.setAddress(address)
		self.setDOB(DOB)
		self.setSpecies(species)
		self.setSex(sex)
		self.setSubjectType(subjectType)
		self.setNameEnc(nameEnc)
		self.setAddressEnc(addressEnc)

		# we need to handle the situation where we don't have the name, so if we are reading from an XML file
		# here we are expecting that we are given a hash, if we are given a hash, use it, otherwise
		# generate the hash here
		if hash != None:
			self.setHash(hash)
		else:
			self.setHash(DataTransferConstants.getSubjectHash(self.getName(), self.getDOB()))

	def setName(self, name = None):
		if isinstance(name, str):
			self.__name = name
		elif name == None:
			self.__name = ""
		else:
			raise Exceptions.InvalidType()
	
	def getName(self):
		return self.__name

	def setAddress(self, address = None):
		if isinstance(address, str):
			self.__address = address
		elif address == None:
			self.__address = ""
		else:
			raise Exceptions.InvalidType()
	
	def getAddress(self):
		return self.__address

	def setDOB(self, DOB = None):
		if isinstance(DOB, str):
			mat = re.match('(\d\d\d\d)(\d\d)(\d\d)', DOB)
			if mat != None:
				if int(mat.group(2)) > 0 and int(mat.group(2)) < 13 and int(mat.group(3)) > 0 and int(mat.group(3)) < 32:
					self.__DOB = DOB
				else:
					raise Exceptions.InvalidValue()
			else:
				raise Exceptions.InvalidValue()
		elif DOB == None:
			DOB = ""
		else:
			raise Exceptions.InvalidType()
	
	def getDOB(self):
		return self.__DOB

	def setSpecies(self, species = None):
		if isinstance(species, str):
			self.__species = species
		elif species == None:
			self.__species = ""
		else:
			raise Exceptions.InvalidType()
	
	def getSpecies(self):
		return self.__species

	def setSex(self, sex = None):
		if sex == 'm' or sex == 'f' or sex == 'M' or sex == 'F':
			self.__sex = sex
		elif isinstance(sex, str):
			print "warning, gender " + sex + " invalid"
			self.__sex = sex
		elif sex == None:
			self.__sex = ""
		else:
			raise Exceptions.InvalidType()
	
	def getSex(self):
		return self.__sex

	def setSubjectType(self, subjectType = None):
		if subjectType == 'patients' or subjectType == 'controls':
			self.__subjectType = subjectType
		elif subjectType == None:
			self.__subjectType = ""
		else:
			raise Exceptions.InvalidType()
	
	def getSubjectType(self):
		return self.__subjectType

	# equality operator
	def isEqual(self, otherSUbject = None):
		if isinstance(otherSubject, Subject):
			#if self.__name == otherSubject.getName() and self.__DOB == otherSubject.getDOB():
			if self.__hash == otherSubject.getHash():
				return True
			else:
				return False
		elif otherSubject == None:
			return False
		else:
			raise Exceptions.InvalidType()
	
	def setHash(self, hash = None):
		self.__hash = hash
	
	def getHash(self):
		return self.__hash

	def setNameEnc(self, nameEnc = None):
		if isinstance(nameEnc, str):
			self.__nameEnc = nameEnc
		elif nameEnc == None:
			self.__nameEnc = None
		else:
			raise Exceptions.InvalidType()
	
	def getNameEnc(self):
		return self.__nameEnc

	def setAddressEnc(self, addressEnc = None):
		if isinstance(addressEnc, str):
			self.__addressEnc = addressEnc
		elif addressEnc == None:
			self.__addressEnc = None
		else:
			raise Exceptions.InvalidType()

	def getAddressEnc(self):
		return self.__addressEnc

	def __str__(self):
		return 	"[" + \
			self.getName() + ", " + \
			self.getDOB() + ", " + \
			self.getSex() + ", " + \
			self.getAddress() + ", " + \
			self.getSubjectType() + \
			"]"
			#str(self.getNameEnc()) + ", " + \
			#str(self.getAddressEnc()) + \
#class SubjectInfo:
	# address
	# DOB
	# hash
	# species
	# other fields (dict)


#	def __init__(self, 

# 8/9/2010 add the next user ID to the name hash file, that way IDs don't get reused
class NameHashMapping:

	def __init__(self, databaseDirectory = None, projectName = None):
		self.__quoteChar = "\""
		self.__delimiterChar = ","
		if not isinstance(databaseDirectory, str) or not isinstance(projectName, str):
			raise Exceptions.InvalidType()
		else:
			self.__databaseDirectory = databaseDirectory
			self.__projectName = projectName

			self.__IDSubjectMapping = {}
			self.__hashIDMapping = {}
			self.__nextSubjectID = 1
			nextSubjectIDInFile = None
			maxSubjectID = 0

			''' Check to see if the project exists '''
			if os.path.isfile(os.path.join(self.__databaseDirectory, self.__projectName, "namehashmapping.txt")):
				# read the first character to see if it has been converted to CSV already
				# i.e. the first character is the quote character
				nameHashMappingFile = file(os.path.join(self.__databaseDirectory, self.__projectName, "namehashmapping.txt"), 'r')
				testChar = nameHashMappingFile.read(1)
				nameHashMappingFile.seek(0)
				
				if testChar == self.__quoteChar:
					CSVReader = csv.reader(nameHashMappingFile, delimiter = self.__delimiterChar, quotechar = self.__quoteChar, quoting = csv.QUOTE_ALL)
					for row in CSVReader:
						#print str(len(row))
						if len(row) == 6:
							(curSubjectID, subjectHash, subjectName, subjectSex, subjectDOB, subjectAddress) = row
							curSubject = Subject(name = subjectName, address = subjectAddress, DOB = subjectDOB, sex = subjectSex, hash = subjectHash)
							curSubjectID = int(curSubjectID)
							self.__IDSubjectMapping[curSubjectID] = curSubject
							self.__hashIDMapping[subjectHash] = curSubjectID
							if maxSubjectID < curSubjectID:
								maxSubjectID = curSubjectID

						elif len(row) == 2:
							if row[0] == "nextID":
								nextSubjectIDInFile = int(row[1])
						#curSubject = {}
						#curSubject['name'] = subjectName
						#curSubject['ID'] = int(curSubjectID)
						#curSubject['hash'] = subjectHash
						#curSubject['dob'] = subjectDOB
						#curSubject['sex'] = subjectSex
						#curSubject['address'] = subjectAddress
						#print "Reading, adding subject to name hash mapping:"
						#print curSubject

				else:
					print "file for project " + projectName + " is in old format, will write in new format when written"
					#if os.path.isfile(os.path.join(self.__databaseDirectory, self.__projectName, "namehashmapping.txt")):
					#nameHashMappingFile = file(os.path.join(self.__databaseDirectory, self.__projectName, "namehashmapping.txt"), 'r')
					for curLine in nameHashMappingFile.xreadlines():
						curLine = curLine.strip()
						testArray = curLine.split(',')
						if len(testArray) > 6:
							(curSubjectID, subjectHash, subjectName, subjectSex, subjectDOB) = testArray[0:5]
							subjectAddress = ",".join(testArray[5:])
						else:
							(curSubjectID, subjectHash, subjectName, subjectSex, subjectDOB, subjectAddress) = testArray
						#(curSubjectID, subjectHash, subjectName, subjectSex, subjectDOB, subjectAddress) = curLine.split(',')
						curSubject = Subject(name = subjectName, address = subjectAddress, DOB = subjectDOB, sex = subjectSex, hash = subjectHash)
						#curSubject = {}
						#curSubject['name'] = subjectName
						#curSubject['ID'] = int(curSubjectID)
						#curSubject['hash'] = subjectHash
						#curSubject['dob'] = subjectDOB
						#curSubject['sex'] = subjectSex
						#curSubject['address'] = subjectAddress
						#print "Reading, adding subject to name hash mapping:"
						#print curSubject
						curSubjectID = int(curSubjectID)
						self.__IDSubjectMapping[curSubjectID] = curSubject
						self.__hashIDMapping[subjectHash] = curSubjectID
						if maxSubjectID < curSubjectID:
							maxSubjectID = curSubjectID
					#for curLine in nameHashMappingFile.xreadlines():
				#if testChar == self.__quoteChar:
				if nextSubjectIDInFile == None:
					self.__nextSubjectID = maxSubjectID + 1
				else:
					self.__nextSubjectID = nextSubjectIDInFile

				nameHashMappingFile.close()
			
	def write(self):
		# write in new csv format

		nameHashMappingFile = file(os.path.join(self.__databaseDirectory, self.__projectName, "namehashmapping.txt"), 'w')
		
		CSVWriter = csv.writer(nameHashMappingFile, delimiter = self.__delimiterChar, quotechar = self.__quoteChar, quoting = csv.QUOTE_ALL)

		# sort the keys and print out the file
		for curSubjectID in self.__IDSubjectMapping:
			CSVWriter.writerow([str(curSubjectID), 
			self.__IDSubjectMapping[curSubjectID].getHash(), \
			self.__IDSubjectMapping[curSubjectID].getName(), \
			self.__IDSubjectMapping[curSubjectID].getSex(), \
			self.__IDSubjectMapping[curSubjectID].getDOB(), \
			self.__IDSubjectMapping[curSubjectID].getAddress()])

			#nameHashMappingFile.write(str(curSubjectID) + \
			#"," + self.__IDSubjectMapping[curSubjectID].getHash() + \
			#"," + self.__IDSubjectMapping[curSubjectID].getName() + \
			#"," + self.__IDSubjectMapping[curSubjectID].getSex() + \
			#"," + self.__IDSubjectMapping[curSubjectID].getDOB() + \
			#"," + self.__IDSubjectMapping[curSubjectID].getAddress() + "\n")
		# write the next ID
		CSVWriter.writerow(["nextID", str(self.__nextSubjectID)])
		nameHashMappingFile.close()
	
	def getIDByHash(self, hash = None):
		if not hash in self.__hashIDMapping:
			return None
		else:
			return self.__hashIDMapping[hash]

	def getIDs(self):
		retArray = []
		for curID in self.__hashIDMapping.itervalues():
			retArray.append(curID)
		return retArray
		

	def getSubjectByHash(self, hash = None):
		if not hash in self.__hashIDMapping:
			return None
		else:
			CurID = self.__hashIDMapping[hash]
			return self.__IDSubjectMapping[CurID]

	def getSubjectByID(self, subjectID = None):
		if not subjectID in self.__IDSubjectMapping:
			return None
		else:
			return self.__IDSubjectMapping[subjectID]
	
	def isHashInMapping(self, hash = None):
		if hash in self.__hashIDMapping:
			return True
		else:
			return False
	
	def isIDInMapping(self, subjectID = None):
		if subjectID in self.__IDSubjectMapping:
			return True
		else:
			return False
	
	def addSubject(self, subjectHash = None, subjectName = None, subjectSex = None, subjectDOB = None, subjectAddress = None):
		if subjectHash == None:
			realHash = DataTransferConstants.getSubjectHash(subjectName, subjectDOB)
		else:
			realHash = subjectHash

		if realHash in self.__hashIDMapping:
			return self.__hashIDMapping[realHash]
		else:
			curSubject = Subject(name = subjectName, address = subjectAddress, DOB = subjectDOB, sex = subjectSex, hash = realHash)

			#nextSubjectID = 1
			#for curID in self.__IDSubjectMapping:
			#	if curID >= nextSubjectID:
			#		nextSubjectID = curID + 1
			print "Adding new subject to name hash mapping:"
			print curSubject
			curSubjectID = self.__nextSubjectID
			self.__nextSubjectID = self.__nextSubjectID + 1
			self.__IDSubjectMapping[curSubjectID] = curSubject
			self.__hashIDMapping[realHash] = curSubjectID

			return curSubjectID
	
	def deleteSubjectByID(self, subjectID = None):
		if self.isIDInMapping(subjectID = subjectID) == True:
			curHash = self.__IDSubjectMapping[subjectID].getHash()
			del self.__IDSubjectMapping[subjectID]
			del self.__hashIDMapping[curHash]
	
	#def changeSubjectID(self, oldSubjectID = None, newSubjectID = None):

class Project:
	def __init__(self, databaseDirectory = None, projectName = None, subjectType = None, studyID = None, species = None):
		self.__databaseDirectory = databaseDirectory
		self.__projectName = projectName

		self.__species = species
		self.__encryptionObject = DataTransferConstants.encryptionModule(projectName = projectName, databaseDirectory = databaseDirectory, publicOrPrivate="public")
		self.__nameHashMapping = NameHashMapping(projectName = projectName, databaseDirectory = databaseDirectory)
		self.__projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
		self.__projectXMLRoot = self.__projectXML.getroot()

		self.__subjects = {}
		self.__dataResources = []
		
		if isinstance(studyID, str):
			self.__studyID = studyID
		else:
			raise Exceptions.InvalidType()

		if subjectType == 'controls' or subjectType == 'patients':
			self.__subjectType = subjectType
		else:
			raise Exceptions.InvalidValue()

		# get all the existing subjects from the XML file
		subjectNodes = self.__projectXMLRoot.xpath("/XCEDE/subject")

		for curSubjectNode in subjectNodes:
			#for curSubjectInfoNode in curSubjectNode.iterchildren(tag = "subjectInfo"):
			#	for infoElement in curSubjectInfoNode.iterchildren(tag = "hash"):
			#		curSubjectHash = infoElement.text
			curSubjectID = int(curSubjectNode.get("ID"))
			
			curSubject = self.__nameHashMapping.getSubjectByID(curSubjectID)
			curSubject.setSubjectType(curSubjectNode.get("subjectGroupID"))
			#(self, name = None, address = None, DOB = None, species = None, sex = None, hash = None, infoFields = None, subjectType = None):
			
			#self.__subjects[curSubjectHash] = {}
			#self.__subjects[curSubjectHash]['subjectID'] = int(curSubjectNode.get("ID"))
			#self.__subjects[curSubjectHash]['subjectType'] = curSubjectNode.get("subjectGroupID")
			#self.__subjects[curSubjectHash]['subjectInfo'] = {}
			
			#for curSubjectInfoNode in curSubjectNode.iterchildren(tag = "subjectInfo"):
			#	for infoElement in curSubjectInfoNode.iterchildren():
			#		if infoElement.tag != "hash":
			#			self.__subjects[curSubjectHash]['subjectInfo'][infoElement.tag] = infoElement.text
		#print self.__subjects

	def addDataResourcesToXML(self):
		PIGZ = DataTransferConstants.findCommand("pigz")
		if PIGZ == None:
			compressionProgram = 'gzip'
		else:
			compressionProgram = 'pigz'

		projectURI = self.__projectName
		for curDataResource in self.__dataResources:
			curSubjectNode = self.__projectXMLRoot.xpath("/XCEDE/subject[@ID='" + str(curDataResource.getSubjectID()) + "']")
			curSubjectURI = os.path.join(projectURI, str(curDataResource.getSubjectID()).zfill(DataTransferConstants.padZerosLength))

			if len(curSubjectNode) == 0:
 				print "subject not in database"
			else:
				# create the visit node if it's not already there
				curVisitNode = self.__projectXMLRoot.xpath("/XCEDE/visit[@ID='" + curDataResource.getVisitID() + "' and @subjectID='" + str(curDataResource.getSubjectID()) + "']")
				curVisitURI = os.path.join(curSubjectURI, curDataResource.getVisitID())
				if len(curVisitNode) == 0:
					curVisitNode = etree.SubElement(self.__projectXMLRoot, "visit", \
					ID = curDataResource.getVisitID(), \
					subjectID = str(curDataResource.getSubjectID()), \
					subjectGroupID = self.__subjectType, \
					subjectURI = curSubjectURI, \
					projectID = self.__projectName, \
					projectURI = projectURI)
					
					curVisitInfoNode = etree.SubElement(curVisitNode, "visitInfo")
					curVisitDescriptionNode = etree.SubElement(curVisitInfoNode, "description")

					if curDataResource.getVisitFields().getDescription() != None:
						curVisitDescriptionNode.text = curDataResource.getVisitFields().getDescription()
				else:
					curVisitNode = curVisitNode[0]
				#if len(curVisitNode) == 0:
				curDataResource.getVisitFields().addToXML(curVisitNode)
				
				# check to see if the "study" node is already there
				curStudyURI = os.path.join(curVisitURI, curDataResource.getStudyID())
				curStudyNode = self.__projectXMLRoot.xpath("/XCEDE/study[@ID='" + curDataResource.getStudyID() + \
					"' and @visitID='" + curDataResource.getVisitID() + \
					"' and @subjectID='" + str(curDataResource.getSubjectID()) + \
					"' and @subjectGroupID='" + self.__subjectType + \
					"' and @projectID='" + self.__projectName + "']")
				if len(curStudyNode) == 0:
					curStudyNode = etree.SubElement(self.__projectXMLRoot, "study", \
						ID = curDataResource.getStudyID(), \
						projectID = self.__projectName, \
						projectURI = projectURI, \
						subjectID = str(curDataResource.getSubjectID()), \
						subjectGroupID = self.__subjectType, \
						subjectURI = curSubjectURI, \
						visitID = curDataResource.getVisitID(), \
						visitURI = curVisitURI)
					curStudyInfoNode = etree.SubElement(curStudyNode, "studyInfo")
					curStudyDescriptionNode = etree.SubElement(curStudyInfoNode, 'description')
					
					if curDataResource.getStudyFields().getDescription() != None:
						curStudyDescriptionNode.text = curDataResource.getStudyFields().getDescription()
				else:
					curStudyNode = curStudyNode[0]
				curDataResource.getStudyFields().addToXML(curStudyNode)
				
				curEpisodeURI = os.path.join(curStudyURI, curDataResource.getEpisodeID())
				curEpisodeNode = self.__projectXMLRoot.xpath("/XCEDE/study[@ID='" + curDataResource.getEpisodeID() + \
					"' + @studyID='" + curDataResource.getStudyID() + \
					"' and @visitID='" + curDataResource.getVisitID() + \
					"' and @subjectID='" + str(curDataResource.getSubjectID()) + \
					"' and @subjectGroupID='" + curDataResource.getSubjectType() + \
					"' and @projectID='" + self.__projectName + "']")
				if len(curEpisodeNode) == 0:	
					curEpisodeNode = etree.SubElement(self.__projectXMLRoot, "episode", ID = curDataResource.getEpisodeID(), \
						projectID = self.__projectName, projectURI = projectURI, \
						subjectID = str(curDataResource.getSubjectID()), subjectGroupID = self.__subjectType, subjectURI = curSubjectURI, \
						visitID = curDataResource.getVisitID(), visitURI = curVisitURI, \
						studyID = curDataResource.getStudyID(), studyURI = curStudyURI)
					curEpisodeInfoNode = etree.SubElement(curEpisodeNode, "episodeInfo")
					curEpisodeDescriptionNode = etree.SubElement(curEpisodeInfoNode, 'description')
					if curDataResource.getEpisodeFields().getDescription() != None:
						curEpisodeDescriptionNode.text = curDataResource.getEpisodeFields().getDescription()
				else:
					curEpisodeNode = curEpisodeNode[0]
				curDataResource.getEpisodeFields().addToXML(curEpisodeNode)

				curAcquisitions = self.__projectXMLRoot.xpath("/XCEDE/acquisition[@episodeID='" + curDataResource.getEpisodeID() + \
								"' and @projectID='" + self.__projectName + \
								"' and @subjectID='" + str(curDataResource.getSubjectID()) + \
								"' and @visitID='" + curDataResource.getVisitID() + \
								"' and @studyID='" + curDataResource.getStudyID() + "']")

				##### MUST CHECK FOR DUPLICATES HERE #####
				maxAcquisitionID = -1
				for curAcquisitionIdx in range(len(curAcquisitions)):
					if int(curAcquisitions[curAcquisitionIdx].get("ID")) >= maxAcquisitionID:
						maxAcquisitionID = int(curAcquisitions[curAcquisitionIdx].get("ID")) + 1
				del curAcquisitions
				# increment that number by one
				#curAcquisitionID = len(curAcquisitions) + 1
				if maxAcquisitionID == -1:
					maxAcquisitionID = 1
				curAcquisitionID = maxAcquisitionID
				#print curAcquisitionID
				curAcquisitionURI = os.path.join(curEpisodeURI, str(curAcquisitionID).zfill(DataTransferConstants.padZerosLength))

				# add the acquisition node
				curAcquisitionNode = etree.SubElement(self.__projectXMLRoot, "acquisition", \
					ID = curAcquisitionID, \
					episodeID = curDataResource.getEpisodeID(), episodeURI = curEpisodeURI, 
					projectID = self.__projectName, projectURI = projectURI, \
					subjectID = str(curDataResource.getSubjectID()), subjectGroupID = self.__subjectType, subjectURI = curSubjectURI, \
					visitID = curDataResource.getVisitID(), visitURI = curVisitURI, \
					studyID = curDataResource.getStudyID(), studyURI = curStudyURI)
				curAcquisitionInfoNode = etree.SubElement(curAcquisitionNode, "acquisitionInfo")
				curAcquisitionDescriptionNode = etree.SubElement(curAcquisitionInfoNode, "description")
				
				if curDataResource.getAcquisitionFields().getDescription() != None:
					curAcquisitionDescriptionNode.text = curDataResource.getAcquisitionFields().getDescription()
				
				# add the acquisition nodes
				curDataResource.getAcquisitionFields().addToXML(curAcquisitionNode)

				# add the data resource node
				curDataResourceNode = etree.SubElement(self.__projectXMLRoot, "dataResource", \
					ID = curDataResource.getDataResourceID(), \
					projectID = self.__projectName, projectURI = projectURI, \
					subjectID = str(curDataResource.getSubjectID()), subjectGroupID = self.__subjectType, subjectURI = curSubjectURI, \
					visitID = curDataResource.getVisitID(), visitURI = curVisitURI, \
					studyID = curDataResource.getStudyID(), studyURI = curStudyURI, \
					episodeID = curDataResource.getEpisodeID(), episodeURI = curEpisodeURI, \
					acquisitionID = str(curAcquisitionID), acquisitionURI = curAcquisitionURI, \
					format = curDataResource.getFileFormat(), \
					type = curDataResource.getXCEDEDataResourceType())
				#print curDataResource.getXCEDEDataResourceType()
				#curDataResourceInfoNode = etree.SubElement(curDataResourceNode, "DataResourceInfo")
				#etree.SubElement(DataResourceInfoNode, "description")
				elementTypeNode = etree.SubElement(curDataResourceNode, "elementType")
				elementTypeNode.text = curDataResource.getElementType()
				byteOrderNode = etree.SubElement(curDataResourceNode, "byteOrder")
				byteOrderNode.text = curDataResource.getByteOrder()
				#print curDataResource.getDimensions()
				# add the dimensions
				for curDimension in curDataResource.getDimensions():
					curDimension.addToXML(curDataResourceNode)

				(fileList, sizes, offsets) = curDataResource.getFileListSizesAndOffsets()
				for idx in range(len(fileList)):
					curURINode = etree.SubElement(curDataResourceNode, "uri", size = str(sizes[idx]), offsets = str(offsets[idx]))
					(head, tail) = os.path.split(fileList[idx])
					curURINode.text = tail

				curSubject = self.__nameHashMapping.getSubjectByID(curDataResource.getSubjectID())
				#print curSubject	
				temporaryDirectory = tempfile.mkdtemp()
			
				# filesToCompress is a format-specific list of files to compress
				# so GENESIS files may have subdirectories associated with them
				filesToCompress = curDataResource.anonymiseFiles(temporaryDirectory = temporaryDirectory, subjectNameEnc = curSubject.getNameEnc(), subjectAddressEnc = curSubject.getAddressEnc())
				
				TarListFile = tempfile.NamedTemporaryFile(mode='w+t')
				TarListFile.write("\n".join(["%s" % (v) for v in filesToCompress]))
				TarListFile.flush()
				
				acquisitionDir = DataTransferConstants.acquisitionFileDirectory(self.__databaseDirectory, self.__projectName, curDataResource.getSubjectID(), curDataResource.getVisitID(), curDataResource.getStudyID(), curDataResource.getEpisodeID(), curAcquisitionID)
				
				TarBallName = os.path.join(acquisitionDir, curDataResource.getFileFormat().lower() + "files.tar.gz")

				try:
					os.makedirs(acquisitionDir)
					#print "making " + os.path.join(databaseDirectory, curAcquisitionURI)
				except os.error, e:
					if e.errno == errno.EEXIST:
						print "directory " + curAcquisitionURI + " already exists, overwriting files underneath"
					if e.errno != errno.EEXIST:
						raise
				print curAcquisitionURI

				commandString = "tar c -C \"" + temporaryDirectory + "\" -T " + TarListFile.name + " | " + compressionProgram + " -1 > \"" + TarBallName + "\""
				os.system(commandString)
				TarListFile.close() # NamedTemporaryFile will automatically delete the file
				shutil.rmtree(temporaryDirectory)	
				#os.removedirs(temporaryDirectory)

				# get the DataResource to anonymise the files into a temporary directory
			#if len(curSubjectNode) == 0:
		#for curDataResource in self.__dataResources:
	#def addDataResourcesToXML(self):

	def saveXML(self):
		userFields = DataTransferUserFields.UserFields(projectName = self.__projectName, databaseDirectory = self.__databaseDirectory)
		userFields.addFieldsInXML(projectXMLRoot = self.__projectXMLRoot)
		
		self.__projectXML.write(os.path.join(self.__databaseDirectory, self.__projectName, "projects.xml"), pretty_print = True)
	
	def saveNameHashMapping(self):
		self.__nameHashMapping.write()
	#def getSubjectsFromData(self):
	#	for curFileReader in self.__fileReaders:
	#		curSubjectDetails = curFileReader.getSubjectDetails()
	#		for curSubject in curSubjectDetails.iterkeys():
	#			self.addSubject(name = curSubjectDetails[curSubject]['name'], DOB = curSubjectDetails[curSubject]['DOB'], sex = curSubjectDetails[curSubject]['sex'], address = curSubjectDetails[curSubject]['address'], species = curSubjectDetails[curSubject]['species'])
		#print self.__subjects
	#def addSubject(self, subjectHash = None, subjectName = None, subjectSex = None, subjectDOB = None, subjectAddress = None):
	
	# adds a new subject to the list of subjects, including insertion into the XML file
	# returns the subjectID
	def addSubject(self, name = None, DOB = None, sex = None, address = None, species = None):
		curHash = DataTransferConstants.getSubjectHash(name, DOB)
		if self.__nameHashMapping.isHashInMapping(curHash) == False:
			curSubjectID = self.__nameHashMapping.addSubject(subjectHash = curHash, subjectName = name, subjectSex = sex, subjectDOB = DOB, subjectAddress = address)

			curSubjectList = self.__projectXMLRoot.xpath("/XCEDE/project/projectInfo/subjectGroupList/subjectGroup[@ID='" + self.__subjectType + "']")
			curSubjectIDNode = etree.SubElement(curSubjectList[0], "subjectID")
			curSubjectIDNode.text = str(curSubjectID)

			curSubjectNode = etree.SubElement(self.__projectXMLRoot, "subject", ID = str(curSubjectID))

			curSubjectInfoNode = etree.SubElement(curSubjectNode, "subjectInfo")
			speciesNode = etree.SubElement(curSubjectInfoNode, "species")
			if species == None:
				speciesNode.text = "human"
			else:
				speciesNode.text = species

			sexNode = etree.SubElement(curSubjectInfoNode, "sex")
			sexNode.text = sex
			DOBNode = etree.SubElement(curSubjectInfoNode, "birthdate")
			DOBNode.text = DOB
			hashNode = etree.SubElement(curSubjectInfoNode, "hash")
			hashNode.text = curHash
		else:
			curSubjectID = self.__nameHashMapping.getIDByHash(curHash)
		curSubject = self.__nameHashMapping.getSubjectByHash(curHash)

		if curSubject.getNameEnc() == None or curSubject.getNameEnc() == '':
			if curSubject.getName() == None:
				textToEncrypt = ''
			else:
				textToEncrypt = curSubject.getName()
			curSubject.setNameEnc(self.__encryptionObject.encryptText(textToEncrypt))
			curSubject.setNameEnc(curSubject.getNameEnc().replace("\n", ""))
			
			if curSubject.getAddress() == None:
				textToEncrypt = ''
			else:
				textToEncrypt = curSubject.getAddress()
			curSubject.setAddressEnc(self.__encryptionObject.encryptText(textToEncrypt))
			curSubject.setAddressEnc(curSubject.getAddressEnc().replace("\n", ""))
		return curSubjectID
	#def addSubject(self, name = None, DOB = None, sex = None, address = None, species = None):
	
	def fileReadersGetFiles(self, baseDirectory = None):
		if not isinstance(baseDirectory, str):
			raise Exceptions.InvalidType()
		else:
			fileList = DataTransferConstants.getFileList(baseDirectory)
		self.__fileReaders = []

		for curFileReader in fileReaders:
			self.__fileReaders.append(curFileReader())

		for curFile in fileList:
			for curFileReader in self.__fileReaders:
				curFileReader.addFile(curFile)
	
	def fileReadersMakeDataResources(self):
		#subjectHashIDMapping = {}
		#for curSubject in self.__subjects.iterkeys():
		#	subjectHashIDMapping[curSubject] = self.__subjects[curSubject]['subjectID']
		#print subjectHashIDMapping
		for curFileReader in self.__fileReaders:
			tmp = curFileReader.makeDataResources(subjectType = self.__subjectType, studyID = self.__studyID)
			for curDataResource in tmp:
				# add the subject to the XML file and then replace the ID by the ID that we get when we add the subject
				curSubject = curDataResource.getSubjectObject()
				subjectID = self.addSubject(name = curSubject.getName(), DOB = curSubject.getDOB(), sex = curSubject.getSex(), address = curSubject.getAddress(), species = self.__species)
				curDataResource.setSubjectID(subjectID)
				self.__dataResources.append(curDataResource)

# DataTransferDataResource
#
# DESCRIPTION
#	Generic container class used to insert a dataset into a project's XML file
#	It also creates the directory structure needed to put in the data and performs data compression.
#	It is intended that format-specific classes be written that overload the functions defined here.
#
# The following fields need to be set by an implementing class.
#
# subjectID (int): the numeric ID for the subject
# visitID (date) YYYYMMDD: the visit date for this image
# studyID (string): the overall study name, this is user-supplied
# episodeID (string): the sequence name for the image, this is generally obtained from the meta-data
# DataResourceID (string): for DICOM files this is the image type, so for example PRIMARY\M\ND\MOSAIC, for other image modalities it could be different, this is generally obtained from the meta-data
#
#DataResource generic fields
# 	fileFormat: the format of the files, this will be put into the DataResource attribute "format", this will be used to form the compressed tarball of the data, so the files will be compressed to "format"files.tar.gz
# 	elementType: the element size of the dataset, takes on string values that are matlab-like in syntax, so: uint8, int16, single, double etc...
# 	fileList: the list of files to import
# 	offsets: a list of offsets for each file to say where the data in each file starts and the header, or meta-data, ends
#	byteOrder: little endian "lsbfirst" or big endian "msbfirst"
#	compression: whether the file is compressed
#	originCoords: perhaps this is "scanner" or "image" or something, don't know, but it's a string
#
# Other fields (not in XCEDE)
# acquisitionFields (dict) [tag: text]: image-specific fields inserted from the meta-data, these will generally be sequence parameters
# acquisitionDescription (string): the description field for the acquisition, this is generally None

class DataResource(object):

	def __str__(self):
		outputString = ''
		outputString = outputString + "subjectID: " + str(self.__subjectID) + ", "
		outputString = outputString + "subjectType: " + str(self.__subjectType) + ", "
		outputString = outputString + "visitID: " + str(self.__visitID) + ", "
		outputString = outputString + "studyID: " + str(self.__studyID) + ", "
		outputString = outputString + "episodeID: " + str(self.__episodeID) + ", "
		outputString = outputString + "DataResourceID: " + str(self.__dataResourceID) + ", "
		outputString = outputString + "compression: " + str(self.__compression) + ", "
		outputString = outputString + "byteOrder: " + str(self.__byteOrder) + ", "
		if self.getType() == "mappedBinaryDataResource_t" or self.getType() == "dimensionedBinaryDataResource_t":
			(imageDimensions, imageScales) = self.getDimensionsAndScales()
			
			outputString = outputString + "Dimensions: [" + ", ".join(["%s" % (v) for v in imageDimensions]) + "], "
			outputString = outputString + "Scales: [" + ", ".join(["%s" % (v) for v in imageScales]) + "]"
		else:
			outputString = outputString[:-2]
		#if self.getType() == "mappedBinaryDataResource_t":
		#for curDim in self.__dimensions:
		#	outputString = outputString + "[" + str(curDim) + "]"

		return outputString

	def getDimensionsAndScales(self):
		
		if self.getType() == "mappedBinaryDataResource_t" or self.getType() == "dimensionedBinaryDataResource_t":
			imageDimensions = [None] * 4
			imageScales = [None] * 4
			for curDimension in self.__dimensions:
				if curDimension.getLabel() == "x":
					dimensionIdx = 0
				if curDimension.getLabel() == "y":
					dimensionIdx = 1
				if curDimension.getLabel() == "z":
					dimensionIdx = 2
				if curDimension.getLabel() == "t":
					dimensionIdx = 3
				imageDimensions[dimensionIdx] = curDimension.getSize()
				imageScales[dimensionIdx] = curDimension.getSpacing()

			#for curDimensionAttribute in curDimension.iterchildren():
			if imageDimensions[3] == None:
				imageDimensions = imageDimensions[:3]
				imageScales = imageScales[:3]

			# for 2D images!
			if imageDimensions[2] == None:
				imageDimensions = imageDimensions[:2]
				imageScales = imageScales[:2]
		return (imageDimensions, imageScales)

	def __init__(self, databaseDirectory = None, projectName = None, \
	subjectID = None, subjectType = None, visitID = None, studyID = None, episodeID = None, acquisitionID = None, dataResourceID = None, \
	subjectInfoDescription = None, visitDescription = None, studyDescription = None, episodeDescription = None, acquisitionDescription = None, \
	dataResourceFormat = None, dataResourceType = None, \
	fileList = None, sizes = None, offsets = None, elementType = None, byteOrder = None, compression = None, originCoords = None, \
	dimensions = None):
		self.setProjectID(projectName)
		self.setSubjectID(subjectID)
		self.setSubjectType(subjectType)
		self.setVisitID(visitID)
		self.setStudyID(studyID)
		self.setEpisodeID(episodeID)
		self.setAcquisitionID(acquisitionID)
		self.setDataResourceID(dataResourceID)

		self.setType(dataResourceType)
		self.setFormat(dataResourceFormat)

		self.setElementType(elementType)
		self.setByteOrder(byteOrder)
		self.setFileListSizesAndOffsets(fileList = fileList, sizes = sizes, offsets = offsets)
		self.setCompression(compression)
		self.setOriginCoords(originCoords)
		self.setDimensions(dimensions)
		
		self.__subjectInfoFields = OtherFields(description = subjectInfoDescription)
		self.__visitFields = OtherFields(description = visitDescription)
		self.__studyFields = OtherFields(description = studyDescription)
		self.__episodeFields = OtherFields(description = episodeDescription)
		self.__acquisitionFields = OtherFields(description = acquisitionDescription)

	def setDataResourceFieldsFromXML(self, dataResourceNode = None):
		if isinstance(dataResourceNode, etree._Element):
			self.setFormat(dataResourceNode.get('format'))
			self.setType(dataResourceNode.get('type'))
			fileListSizesAndOffsets = {}
			fileListSizesAndOffsets['files'] = []
			fileListSizesAndOffsets['sizes'] = []
			fileListSizesAndOffsets['offsets'] = []

			for curChildNode in dataResourceNode.iterchildren():
				if curChildNode.tag == "elementType":
					self.setElementType(curChildNode.text)
				elif curChildNode.tag == "byteOrder":
					self.setByteOrder(curChildNode.text)
				elif curChildNode.tag == "dimension":
					
					curDimensionProps = {}
					curDimensionProps['label'] = curChildNode.get("label")
					curDimensionProps['size'] = None
					curDimensionProps['origin'] = None 
					curDimensionProps['spacing'] = None 
					curDimensionProps['gap'] = None 
					curDimensionProps['datapoints'] = None 
					curDimensionProps['direction'] = None 
					curDimensionProps['units'] = None 
					curDimensionProps['measurementframe'] = None 
					curDimensionProps['mosaicSize'] = None

					# mosaic check, if label == z and we don't have outputSelect, then ignore this dimension since the real size will be in another dimension
					# if splitRank is there and outputSelect is not, then we don't need to add this dimension
					if not (curDimensionProps['label'] == 'z' and curChildNode.get("splitRank") != None and curChildNode.get("outputSelect") == None):
						for curDimensionChildNode in curChildNode.iterchildren():
							if curDimensionChildNode.tag == 'size':
								if curChildNode.get("outputSelect") != None:
									curDimensionProps['size'] = len(curChildNode.get("outputSelect").split(" "))
									curDimensionProps['mosaicSize'] = int(curDimensionChildNode.text)
								else:
									curDimensionProps['size'] = int(curDimensionChildNode.text)
							elif curDimensionChildNode.tag == "spacing":
								curDimensionProps['spacing'] = float(curDimensionChildNode.text)
							elif curDimensionChildNode.tag == "gap":
								curDimensionProps['gap'] = float(curDimensionChildNode.text)
							elif curDimensionChildNode.tag == "units":
								curDimensionProps['units'] = curDimensionChildNode.text
							elif curDimensionChildNode.tag == "direction":
								curDimensionProps['direction'] = curDimensionChildNode.text.split(" ")
								for idx in range(len(curDimensionProps['direction'])):
									curDimensionProps['direction'][idx] = float(curDimensionProps['direction'][idx])
							elif curDimensionChildNode.tag == "datapoints":
								curDimensionProps['datapoints'] = curDimensionChildNode.text.split(" ")
								for idx in range(len(curDimensionProps['datapoints'])):
									curDimensionProps['datapoints'][idx] = float(curDimensionProps['datapoints'][idx])
							elif curDimensionChildNode.tag == "measurementframe":
								curDimensionProps['measurementframe'] = curDimensionChildNode.text
					self.addDimension(DataResourceDimension( label = curDimensionProps['label'], size = curDimensionProps['size'], origin = curDimensionProps['origin'], spacing = curDimensionProps['spacing'], gap = curDimensionProps['gap'], datapoints = curDimensionProps['datapoints'], direction = curDimensionProps['direction'], units = curDimensionProps['units'], measurementframe = curDimensionProps['measurementframe'], mosaicSize = curDimensionProps['mosaicSize']))
				elif curChildNode.tag == "uri":
					fileListSizesAndOffsets['files'].append(curChildNode.text)
					fileListSizesAndOffsets['sizes'].append(curChildNode.get("size"))
					fileListSizesAndOffsets['offsets'].append(curChildNode.get("offset"))
				elif curChildNode.tag == "compression":
					self.setCompression(curChildNode.text)
				#elif curChildNode.tag == "compression":
				#	self.setOriginCoords(originCoords)
					
			self.setFileListSizesAndOffsets(fileList = fileListSizesAndOffsets['files'], sizes = fileListSizesAndOffsets['sizes'], offsets = fileListSizesAndOffsets['offsets'])

	def getXCEDEDataResourceType(self):
		if len(self.__dimensions) == 0:
			return "binaryDataResource_t"
		else:
			return self.__dimensions[0].getXCEDENodeType()

#	this code is used to extract the dimensions from the XML file, put this in the "extract everything" from the XML file for the dataresource
#	if self.curDataResourceNodesXML[curDataResourceIdx].get('type') == "mappedbinaryDataResource_t":
#		imageDimensions = [None] * 4
#		imageScales = [None] * 4
#		dimensionDirections = [None] * 3
#		ZSplit = False
#		for curDimension in self.curDataResourceNodesXML[curDataResourceIdx].iterchildren(tag='dimension'):
#			if curDimension.get("label") == "x":
#				dimensionIdx = 0
#			if curDimension.get("label") == "y":
#				dimensionIdx = 1
#			if curDimension.get("label") == "z":
#				if curDimension.get("splitRank") != None:
#					ZSplit = True
#					if not curDimension.get("outputSelect") == None:
#						imageDimensions[2] = len(curDimension.get("outputSelect").split(" "))
#				dimensionIdx = 2
#			if curDimension.get("label") == "t":
#				dimensionIdx = 3
#			for curDimensionAttribute in curDimension.iterchildren():
#				if curDimensionAttribute.tag == "size" and not (dimensionIdx == 2 and ZSplit == True):
#					imageDimensions[dimensionIdx] = int(curDimensionAttribute.text)
#				if curDimensionAttribute.tag == "spacing":
#					imageScales[dimensionIdx] = float(curDimensionAttribute.text)
#				if curDimensionAttribute.tag == "direction" and dimensionIdx != 3:
#					dimensionDirections[dimensionIdx] = "[" + curDimensionAttribute.text + "]"
#		for curDimensionAttribute in curDimension.iterchildren():
#		if imageScales[3] == None:
#			imageScales = imageScales[:3]
#			imageDimensions = imageDimensions[:3]

	def getOtherFieldsFromXML(self, projectXML = None):
		subjectNode = projectXML.xpath("/XCEDE/subject[@ID='" + str(self.getSubjectID()) + "']")
		if len(subjectNode) != 1:
			print "Could not find a single SUBJECT node, " + str(len(subjectNode)) + " found, with subjectID = " + str(self.getSubjectID())
		else:
			subjectNode = subjectNode[0]
			for subjectInfoNodes in subjectNode.iterchildren(tag = "subjectInfo"):
				curParentNode = subjectInfoNodes
			for curInfoNode in curParentNode.iterchildren():
				self.__subjectInfoFields.setField(key = curInfoNode.tag, value = curInfoNode.text)

		visitNode = projectXML.xpath("/XCEDE/visit[@ID='" + self.getVisitID() + "' and @subjectID='" + str(self.getSubjectID()) + "']")
		if len(visitNode) != 1:
			print "Could not find a single VISIT node, " + str(len(visitNode)) + " found, visitID = " + self.getVisitID() + ",  with subjectID = " + str(self.getSubjectID())
		else:
			visitNode = visitNode[0]
			for curChildNode in visitNode.iterchildren():
				if curChildNode.tag != 'visitInfo':
					self.__visitFields.setField(key = curChildNode.tag, value = curChildNode.text)
				else:
					for curDescriptionNode in curChildNode.iterchildren():
						self.__visitFields.setDescription(description = curDescriptionNode.text)

		studyNode = projectXML.xpath("/XCEDE/study[@ID='" + self.getStudyID() + "' and @visitID='" + self.getVisitID() + "' and @subjectID='" + str(self.getSubjectID()) + "']")
		if len(studyNode) != 1:
			print "Could not find a single STUDY node, " + str(len(visitNode)) + " found, studyID = " + self.getStudyID() + ", visitID = " + self.getVisitID() + ",  with subjectID = " + str(self.getSubjectID())
		else:
			studyNode = studyNode[0]
			for curChildNode in studyNode.iterchildren():
				if curChildNode.tag != 'studyInfo':
					self.__studyFields.setField(key = curChildNode.tag, value = curChildNode.text)
				else:
					for curDescriptionNode in curChildNode.iterchildren():
						self.__studyFields.setDescription(description = curDescriptionNode.text)
		
		episodeNode = projectXML.xpath("/XCEDE/episode[@ID='" + self.getEpisodeID() + "' and @studyID='" + self.getStudyID() + "' and @visitID='" + self.getVisitID() + "' and @subjectID='" + str(self.getSubjectID()) + "']")
		if len(episodeNode) != 1:
			print "Could not find a single EPISODE node, " + str(len(visitNode)) + " found, episodeID = " + self.getEpisodeID() + ", studyID = " + self.getStudyID() + ", visitID = " + self.getVisitID() + ",  with subjectID = " + str(self.getSubjectID())
		else:
			episodeNode = episodeNode[0]
			for curChildNode in episodeNode.iterchildren():
				if curChildNode.tag != 'episodeInfo':
					self.__episodeFields.setField(key = curChildNode.tag, value = curChildNode.text)
				else:
					for curDescriptionNode in curChildNode.iterchildren():
						self.__episodeFields.setDescription(description = curDescriptionNode.text)
		
		acquisitionNode = projectXML.xpath("/XCEDE/acquisition[@ID='" + str(self.getAcquisitionID()) + "' and @episodeID='" + self.getEpisodeID() + "' and @studyID='" + self.getStudyID() + "' and @visitID='" + self.getVisitID() + "' and @subjectID='" + str(self.getSubjectID()) + "']")
		if len(acquisitionNode) != 1:
			print "Could not find a single ACQUISITION node, " + str(len(visitNode)) + " found, acquisitionID = " + str(self.getAcquisitionID()) + ", episodeID = " + self.getEpisodeID() + ", studyID = " + self.getStudyID() + ", visitID = " + self.getVisitID() + ",  with subjectID = " + str(self.getSubjectID())
		else:
			acquisitionNode = acquisitionNode[0]
			for curChildNode in acquisitionNode.iterchildren():
				if curChildNode.tag != 'acquisitionInfo':
					self.__acquisitionFields.setField(key = curChildNode.tag, value = curChildNode.text)
				else:
					for curDescriptionNode in curChildNode.iterchildren():
						self.__acquisitionFields.setDescription(description = curDescriptionNode.text)
		return

	# intended to be executed by subclasses when creating the dataresource
	def createSubjectObject(self, name = None, address = None, DOB = None, sex = None):
		self.__subjectObject = Subject(name = name, address = address, DOB = DOB, sex = sex)

	def getType(self):
		return self.__dataResourceType

	def setType(self, dataResourceType = None):
		if isinstance(dataResourceType, str):
			self.__dataResourceType = dataResourceType 
		elif dataResourceType == None:
			self.__dataResourceType = None
		else:
			raise Exceptions.InvalidType()
	
	def getFormat(self):
		return self.__dataResourceFormat

	def setFormat(self, dataResourceFormat = None):
		if isinstance(dataResourceFormat, str):
			self.__dataResourceFormat = dataResourceFormat 
		elif dataResourceFormat == None:
			self.__dataResourceFormat = None
		else:
			raise Exceptions.InvalidType()


	def getSubjectObject(self):
		return self.__subjectObject

	def setDimensions(self, dimensions = None):
		if isinstance(dimensions, list):
			for curDim in dimensions:
				if not isinstance(curDim, DataResourceDimension):
					raise Exceptions.InvalidType()
			self.__dimensions = dimensions
		elif dimensions == None:
			self.__dimensions = []
		else:
			raise Exceptions.InvalidType()

	def addDimension(self, dimension = None):
		if not isinstance(dimension, DataResourceDimension):
			raise Exceptions.InvalidType()
		else:
			self.__dimensions.append(dimension)
	
	def getDimensions(self):
		return self.__dimensions
	
	def setSubjectInfoField(self, key, value, **kwargs):
		self.__subjectInfoFields.setField(key, value, **kwargs)

	def setVisitField(self, key, value, **kwargs):
		self.__visitFields.setField(key, value, **kwargs)
	
	def setStudyField(self, key, value, **kwargs):
		self.__studyFields.setField(key, value, **kwargs)
		
	def setEpisodeField(self, key, value, **kwargs):
		self.__episodeFields.setField(key, value, **kwargs)
		
	def setAcquisitionField(self, key, value, **kwargs):
		self.__acquisitionFields.setField(key, value, **kwargs)
	
	def getSubjectInfoFields(self):
		return self.__subjectInfoFields
		
	def getVisitFields(self):
		return self.__visitFields

	def getStudyFields(self):
		return self.__studyFields

	def getEpisodeFields(self):
		return self.__episodeFields

	def getAcquisitionFields(self):
		return self.__acquisitionFields

	def setProjectID(self, projectID = None):
		if isinstance(projectID, str):
			self.__projectID = projectID
		elif projectID == None:
			self.__projectID = None
		else:
			raise Exceptions.InvalidType()

	def getProjectID(self):
		return self.__projectID

	def setSubjectID(self, subjectID = None):
		if isinstance(subjectID, int):
			self.__subjectID = subjectID
		elif subjectID == None:
			self.__subjectID = None
		else:
			raise Exceptions.InvalidType()

	def getSubjectID(self):
		return self.__subjectID

	def getSubjectType(self):
		return self.__subjectType

	def setSubjectType(self, subjectType = None):
		if subjectType == 'controls' or subjectType == 'patients':
			self.__subjectType = subjectType
		else:
			print subjectType
			raise Exceptions.InvalidValue()

	def setVisitID(self, visitID = None):
		if isinstance(visitID, str):
			if len(visitID) == 8 and visitID.isdigit() == True:
				Month = int(visitID[4:6])
				Day = int(visitID[6:8])
				if Month >= 1 and Month <= 12 and Day >= 1 and Day <= 31:
					self.__visitID = visitID
				else:
					raise Exceptions.InvalidValue()
			else:
				raise Exceptions.InvalidValue()
		elif visitID == None:
			self.__visitID = None
		else:
			raise Exceptions.InvalidType()

	def getVisitID(self):
		return self.__visitID

	def setStudyID(self, studyID = None):
		if isinstance(studyID, str):
			self.__studyID = studyID
		elif studyID == None:
			self.__studyID = None
		else:
			raise Exceptions.InvalidType()
	
	def getStudyID(self):
		return self.__studyID
	
	def setEpisodeID(self, episodeID = None):
		if isinstance(episodeID, str):
			self.__episodeID = episodeID 
		elif episodeID == None:
			self.__episodeID = None
		else:
			raise Exceptions.InvalidType()

	def getEpisodeID(self):
		return self.__episodeID
	
	def setAcquisitionID(self, acquisitionID = None):
		if isinstance(acquisitionID, int):
			self.__acquisitionID = acquisitionID 
		elif acquisitionID == None:
			self.__acquisitionID = None
		else:
			raise Exceptions.InvalidType()

	def getAcquisitionID(self):
		return self.__acquisitionID
		
	def setDataResourceID(self, dataResourceID = None):
		if isinstance(dataResourceID, str):
			self.__dataResourceID = dataResourceID 
		elif dataResourceID == None:
			self.__dataResourceID = None
		else:
			raise Exceptions.InvalidType()

	def getDataResourceID(self):
		return self.__dataResourceID

	def setElementType(self, elementType = None):
		if isinstance(elementType, str):
			if elementType == "uint8" or \
				elementType == "int8" or \
				elementType == "uint16" or \
				elementType == "int16" or \
				elementType == "uint32" or \
				elementType == "int32" or \
				elementType == "single" or \
				elementType == "double" or \
				elementType == "complex" or \
				elementType == "rgb":
					self.__elementType = elementType
			else:
				raise Exceptions.InvalidValue()
		elif elementType == None:
			self.__elementType = None
		else:
			raise Exceptions.InvalidType()
				
	def getElementType(self):
		return self.__elementType

	def getFileFormat(self):
		return None
	
	def getDataResourceID(self):
		return self.__dataResourceID
	
	def setCompression(self, compression = None):
		if isinstance(compression, str):
			self.__compression = compression 
		elif compression == None:
			self.__compression = None
		else:
			raise Exceptions.InvalidType()

	def getCompression(self):
		return self.__compression
		
	def setFileListSizesAndOffsets(self, fileList = None, sizes = None, offsets = None):
		if not isinstance(fileList, list) or not isinstance(offsets, list) or not isinstance(sizes, list):
			if fileList == None and offsets == None:
				self.__fileList = None
				self.__sizes = None
				self.__offsets = None
			else:
				raise Exceptions.InvalidType()
		else:
			if len(fileList) == len(offsets) and len(fileList) == len(sizes):
				self.__fileList = fileList
				self.__sizes = sizes
				self.__offsets = offsets
			else:
				raise Exceptions.InvalidValue()
	
	def getFileListSizesAndOffsets(self):
		return self.__fileList, self.__sizes, self.__offsets
	
	def setOriginCoords(self, originCoords = None):
		if isinstance(originCoords, str):
			self.__originCoords = originCoords 
		elif originCoords == None:
			self.__originCoords = None
		else:
			raise Exceptions.InvalidType()

	def getOriginCoords(self):
		return self.__originCoords
	
	def setByteOrder(self, byteOrder = None):
		if isinstance(byteOrder, str):
			if byteOrder == 'lsbfirst' or byteOrder == 'msbfirst':
				self.__byteOrder = byteOrder 
			else:
				raise Exceptions.InvalidValue()
		elif byteOrder == None:
			self.__byteOrder = None
		else:
			raise Exceptions.InvalidType()

	def getByteOrder(self):
		return self.__byteOrder
	
	def anonymiseFiles(self, temporaryDirectory = None, subjectNameEnc = None, subjectAddressEnc = None):
		pass
	
	def setSubjectID(self, subjectID = None):
		if isinstance(subjectID, int):
			self.__subjectID = subjectID
		elif subjectID == None:
			subjectID = None
		else:
			raise Exceptions.InvalidType()

	def getSubjectID(self):
		return self.__subjectID
	
	## These are functions to be used by the pattern matcher
	# We absorbed these functions into here because
	def metaDataDirectoryString(self, pattern = None):
		patternStrings = []
		for curPattern in pattern.split(','):
			patternStrings.append(self.metaDataPatternString(pattern = curPattern))
		
		sepString = os.sep
		return sepString.join(patternStrings)

	# idx is the number of the current file to be matched
	def metaDataPatternString(self, pattern = None, idx = None):
		outputString = ''
		curIdx = 0
		missingFieldError = False
		while curIdx < len(pattern):
			if pattern[curIdx] == '%':
				if len(pattern) >= curIdx:
					# "%p: project name"
					# "%t: subject type"
					# "%s: subject ID"
					# "%v: visit ID (date)"
					# "%e: episode ID"
					# "%a: acquisition ID"
					# "%d: today's date
					# "%n: the index of the file in the current set of checked out files
					# "%%: the character %"
					# "%,: the character ,"
					# find the string to place in herre based on the next character
			
					if pattern[curIdx + 1] == 'p':
						outputString = outputString + self.getProjectID()
					elif pattern[curIdx + 1] == 't':
						outputString = outputString + self.getSubjectType()
					elif pattern[curIdx + 1] == 's':
						outputString = outputString + str(self.getSubjectID()).zfill(DataTransferConstants.padZerosLength)
					elif pattern[curIdx + 1] == 'v':
						outputString = outputString + self.getVisitID()
					elif pattern[curIdx + 1] == 'e':
						outputString = outputString + self.getEpisodeID()
					elif pattern[curIdx + 1] == 'a':
						outputString = outputString + str(self.getAcquisitionID()).zfill(DataTransferConstants.padZerosLength)
					elif pattern[curIdx + 1] == 'd':
						outputString = outputString + time.strftime("%Y%m%d")
					elif pattern[curIdx + 1] == 'n':
						outputString = outputString + str(idx)#str(curDataResourceIdx).zfill(DataTransferConstants.padZerosLength)
					elif pattern[curIdx + 1] == '%':
						outputString = outputString + '%'
					elif pattern[curIdx + 1] == ',':
						outputString = outputString + ','
					elif pattern[curIdx + 1] == '#':
						outputString = outputString + '#'
					elif pattern[curIdx + 1] == ':':
						outputString = outputString + ':'
					else:
						print "format converter not supported, ignored: " + pattern[curIdx + 1]
					curIdx = curIdx + 1
				#if len(pattern) >= curIdx:
				else:
					print "A percent sign was the last character of the output format string, this is invalid"
			elif pattern[curIdx] == '#':
				# we have a user-specified field
				if len(pattern) >= curIdx:
					# look for the next hash after this one
					nextHash = pattern[curIdx + 1:]
					nextHashIdx = nextHash.find('#')
					if nextHashIdx == -1:
						print "I can't find a hash to terminate the user-supplied field, skipping the character and parsing the rest"
					else:
						subString = pattern[curIdx + 1:curIdx + 1 + nextHashIdx]
						# split the two strings inbetween by the comma
						fieldElements = subString.split(':')
						# check that the "level" is valid
						if len(fieldElements) != 2:
							print "Pattern incorrect: #" + subString + "#, should be #level:name#, skipping the pattern and parsing the rest"
						else:
							curLevel = fieldElements[0]
							curName = fieldElements[1]
							if curLevel == 'subjectInfo':
								curOtherFields = self.__subjectInfoFields
							elif curLevel == 'visit':
								curOtherFields = self.__visitFields
							elif curLevel == 'study':
								curOtherFields = self.__studyFields
							elif curLevel == 'episode':
								curOtherFields = self.__episodeFields
							elif curLevel == 'acquisition':
								curOtherFields = self.__acquisitionFields
							elif curLevel == 'dataResource':
								curOtherFields = self.__dataResourceFields
							else:
								curOtherFields = None
								print "level identifier not recognised: " + curLevel
							
							if curOtherFields != None:
								if curName == 'description':
									outputString = outputString + curOtherFields.getDescription()
								else:
									if curOtherFields.getFieldText(key = curName) == None:
										missingFieldError = True
									else:
										outputString = outputString + curOtherFields.getFieldText(key = curName)
						#if len(fieldElements) != 2:

						curIdx = curIdx + nextHashIdx + 1
					#if nextHashIdx == -1:
				#if len(outputPattern) >= curIdx:
			else:
				outputString = outputString + pattern[curIdx]
			#if outputPattern[curIdx] == '%':
			curIdx = curIdx + 1
		#while curIdx < len(outputPattern):
		if missingFieldError == True:
			return None
		else:
			return outputString
	#def metaDataPatternString(self, pattern, idx=None)i

	''' This returns the string needed to go into the pattern match file produced by dt_checkout_pattern.py'''
	def patternMatchString(self):
		return str(self.getProjectID()) + "\t" + str(self.getSubjectType()) + "\t" + str(self.getSubjectID()) + "\t" + str(self.getVisitID()) + "\t" + str(self.getStudyID()) + "\t" + str(self.getEpisodeID()) + "\t" + str(self.getAcquisitionID()) + "\t" + str(self.getDataResourceID())

class OtherFields:
	def __str__(self):
		return str(self.__userFields)

	def __init__(self, description = None, userFields = None):
		self.setDescription(description)
		self.__userFields = []
		self.__keyMapping = {}

	def setDescription(self, description = None):
		if isinstance(description, str):
			self.__description = description 
		elif description == None:
			self.__description = None
		else:
			raise Exceptions.InvalidType()

	def getDescription(self):
		return self.__description

	def setField(self, key = None, value = None, **kwargs):
		if key in self.__keyMapping:
			self.__userFields[self.__keyMapping[key]] = Field(key, value, **kwargs)
		else:
			self.__keyMapping[key] = len(self.__userFields)
			self.__userFields.append(Field(key, value, **kwargs))

	def getField(self, key = None):
		return self.__userFields[self.__keyMapping[key]]

	def getFieldText(self, key = None):
		return self.__userFields[self.__keyMapping[key]].getText()

	def addToXML(self, XMLRoot = None):
		if not isinstance(XMLRoot, etree._Element):
			raise Exceptions.InvalidType()
		else:
			# check to see if the description is already there
			#ChildFound = False
			#for Children in XMLRoot.iterchildren(tag = "description"):
			#	ChildFound = True
			#if ChildFound == False:
			#	DescriptionNode = etree.SubElement(XMLRoot, 'description')
			#	if self.__description != None:
			#		DescriptionNode.text = self.__description

			for curField in self.__userFields:
				ChildFound = False
				for Children in XMLRoot.iterchildren(tag = curField.getKey()):
					ChildFound = True
				if ChildFound == False:
					curNode = etree.SubElement(XMLRoot, curField.getKey())
					attributes = curField.getAttributes()
					if curField.getText() != None:
						if not isinstance(curField.getText(), str):
							print curField.getText()
						curNode.text = curField.getText()

					for curAttribute in attributes.iterkeys():
						curNode.set(curAttribute, attributes[curAttribute])

class Field:

	def __str__(self):
		return "(" + self.getKey() + ", " + self.getText() + "): " + str(self.__attributes)

	def __init__(self, key, text, **kwargs):
		self.__key = key
		self.__text = text
		self.setAllAttributes(kwargs)
	
	def setAllAttributes(self, attributes):
		# check all types
		if not isinstance(attributes, dict):
			raise Exceptions.InvalidType()
		else:
			for curKey in attributes.iterkeys():
				if not isinstance(curKey, str):
					raise Exceptions.InvalidType()
				if not isinstance(attributes[curKey], str):
					raise Exceptions.InvalidType()
			self.__attributes = attributes
	
	def setAttribute(self, key = None, value = None):
		if not isinstance(key, str) or not isinstance(value, str):
			raise InvalidType()
		else:
			self.__attributes[key] = value

	def getAttributes(self):
		return self.__attributes
	
	def setKey(self, key = None):
		if not isinstance(key, str):
			raise InvalidType()
		else:
			self.__key = key

	def getKey(self):
		return self.__key

	def setText(self, text = None):
		if not isinstance(text, str):
			raise InvalidType()
		else:
			self.__text = text

	def getText(self):
		return self.__text


# For image data
#	DataResourceDimension: the keys of this will be 'x', 'y', 'z', 't' and each of these will have a dict with the following elements
#		label: the label of the dimension, should be 'x', 'y', 'z', 't'
# 		size: the size of the dimension
#		origin: the origin of the dimension
# 		pixelSpacing: (float) the physical dimension in mm of the pixels
#		gap: any gap in the dimension
#		dataPoints (list): generally only used for the fourth dimension, time
#		direction (list): the direction for this dimension
#		mosaicSize: for SIEMENS mosaic data, if this is not None then this dimension will be split across two nodes using splitRank and this value will be used for the size, and the number in size will be used for the outputSelect attribute


class DataResourceDimension:
	def __init__(self, label = None, size = None, origin = None, spacing = None, gap = None, datapoints = None, direction = None, units = None, measurementframe = None, mosaicSize = None):
		self.setLabel(label)
		self.setSize(size)
		self.setOrigin(origin)
		self.setSpacing(spacing)
		self.setGap(gap)
		self.setDatapoints(datapoints)
		self.setDirection(direction)
		self.setUnits(units)
		self.setMeasurementframe(measurementframe)
		self.setMosaicSize(mosaicSize)

	def __str__(self):
		outputString = "label: " + self.__label + ", "
		outputString = outputString + "size: " + str(self.__size) + ", "
		outputString = outputString + "origin: " + str(self.__origin) + ", "
		outputString = outputString + "spacing: " + str(self.__spacing) + ", "
		outputString = outputString + "gap: " + str(self.__gap) + ", "
		outputString = outputString + "units: " + str(self.__units) + ", "
		outputString = outputString + "mosaicSize: " + str(self.__mosaicSize)
		return outputString
	
	# XCEDE has three main types of data node depending on whether the file is an image (dimensionedBinaryDataResource_t), image with orientation info (mappedBinaryDataResource_t) or not an image (binaryDataResource_t)
	# this function returns the string that represents the correct type (in XCEDE), based on the non-None fields that we have
	def getXCEDENodeType(self):
		if self.__size == None:
			return "binaryDataResource_t"
		elif self.__origin != None or self.__spacing != None or self.__gap != None or self.__datapoints != None or self.__direction != None or self.__units != None or self.__measurementframe != None:
			return "mappedBinaryDataResource_t"
		else:
			return "dimensionedBinaryDataResource_t"

	def addToXML(self, dataElementNode = None):
		if self.__mosaicSize == None: # then we don't need to split
			dimensionNode = etree.SubElement(dataElementNode, "dimension", label = self.__label)
			
			if self.__size != None:
				sizeNode = etree.SubElement(dimensionNode, "size")
				sizeNode.text = str(self.__size)
			
			if self.__origin != None:
				originNode = etree.SubElement(dimensionNode, "origin")
				originNode.text = str(self.__origin)
		
			if self.__spacing != None:
				spacingNode = etree.SubElement(dimensionNode, "spacing")
				spacingNode.text = str(self.__spacing)

			if self.__gap != None:
				gapNode = etree.SubElement(dimensionNode, "gap")
				gapNode.text = str(self.__gap)
			
			if self.__datapoints != None:
				datapointsNode = etree.SubElement(dimensionNode, "datapoints")
				datapointsNode.text = " ".join(str(v) for v in self.__datapoints)

			if self.__direction != None:
				directionNode = etree.SubElement(dimensionNode, "direction")
				directionNode.text = " ".join(str(v) for v in self.__direction)

			if self.__units != None:
				unitsNode = etree.SubElement(dimensionNode, "units")
				unitsNode.text = self.__units

			if self.__measurementframe != None:
				measurementframeNode = etree.SubElement(dimensionNode, "measurementFrame")
				measurementframeNode.text = self.__measurementframe
		else:
			for idx in range(2):
				dimensionNode = etree.SubElement(dataElementNode, "dimension", label = self.__label, splitRank = str(idx + 1))
				if idx == 0:
					dimensionNode.set('outputSelect', " ".join(str(v) for v in range(self.__size)))

				if self.__size != None:
					sizeNode = etree.SubElement(dimensionNode, "size")
					sizeNode.text = str(self.__mosaicSize)
				
				if self.__origin != None:
					originNode = etree.SubElement(dimensionNode, "origin")
					originNode.text = str(self.__origin)
			
				if self.__spacing != None:
					spacingNode = etree.SubElement(dimensionNode, "spacing")
					spacingNode.text = str(self.__spacing)

				if self.__gap != None:
					gapNode = etree.SubElement(dimensionNode, "gap")
					gapNode.text = str(self.__gap)
				
				if self.__datapoints != None:
					datapointsNode = etree.SubElement(dimensionNode, "datapoints")
					datapointsNode.text = " ".join(str(v) for v in self.__datapoints)

				if self.__direction != None:
					directionNode = etree.SubElement(dimensionNode, "direction")
					directionNode.text = " ".join(str(v) for v in self.__direction)

				if self.__units != None:
					unitsNode = etree.SubElement(dimensionNode, "units")
					unitsNode.text = self.__units

				if self.__measurementframe != None:
					measurementframeNode = etree.SubElement(dimensionNode, "measurementFrame")
					measurementframeNode.text = self.__measurementframe
		#if self.__mosaicSize == None: # then we don't need to split
	#def addToXML(self, dataElementNode = None):

	def setLabel(self, label = None):
		if label == 'x' or label == 'y' or label == 'z' or label == 't':
			self.__label = label
		else:
			raise Exceptions.InvalidValue()
	
	def getLabel(self):
		return self.__label

	def setSize(self, size = None):
		if isinstance(size, int):
			if size > 0:
				self.__size = size
		elif size == None:
			self.__size = None
		else:
			raise Exceptions.InvalidType()

	def getSize(self):
		return self.__size
	
	def setOrigin(self, origin = None):
		if isinstance(origin, float):
			self.__origin = origin
		elif origin == None:
			self.__origin = None
		else:
			raise Exceptions.InvalidType()

	def getOrigin(self):
		return self.__origin
	
	def setSpacing(self, spacing = None):
		if isinstance(spacing, float) or isinstance(spacing, int):
			self.__spacing = float(spacing)
		elif spacing == None:
			self.__spacing = None
		else:
			raise Exceptions.InvalidType()

	def getSpacing(self):
		return self.__spacing
	
	def setGap(self, gap = None):
		if isinstance(gap, float) or isinstance(gap, int):
			self.__gap = float(gap)
		elif gap == None:
			self.__gap = None
		else:
			raise Exceptions.InvalidType()

	def getGap(self):
		return self.__gap
		
	def setDatapoints(self, datapoints = None):
		if isinstance(datapoints, list):
			allFloats = True
			for curPoint in datapoints:
				if not isinstance(curPoint, float):
					allFloats = False
					break
			if allFloats == True:
				self.__datapoints = datapoints
			else:
				raise Exceptions.InvalidType()
		elif datapoints == None:
			self.__datapoints = None
		else:
			raise Exceptions.InvalidType()
				

	def getDatapoints(self):
		return self.__datapoints
	
	def setDirection(self, direction = None):
		if isinstance(direction, list):
			if len(direction) == 3:
				allFloats = True
				for curDir in direction:
					if not isinstance(curDir, float):
						allFloats = False
						break
			if allFloats == True:
				self.__direction = direction
			else:
				print direction
				raise Exceptions.InvalidType()
		elif direction == None:
			self.__direction = None
		else:
			raise Exceptions.InvalidType()

	def getDataPoints(self):
		return self.__direction
	
	def setUnits(self, units = None):
		if isinstance(units, str):
			if units == 'mm' or units == 'cm' or units == 'm' or units == 's' or units == 'ms' or units == 'sec' or units == 'msec':
				self.__units = units
			else:
				raise Exceptions.InvalidValue()
		elif units == None:
			self.__units = None
		else:
			raise Exceptions.InvalidType()
	
	def getUnits(self):
		return self.__units
	
	def setMeasurementframe(self, measurementframe = None):
		if isinstance(measurementframe, str):
			self.__measurementframe = measurementframe
		elif measurementframe == None:
			self.__measurementframe = None
		else:
			raise Exceptions.InvalidType()

	def getMeasurementframe(self):
		return self.__measurementframe
	
	def setMosaicSize(self, mosaicSize = None):
		if isinstance(mosaicSize, int):
			if mosaicSize > 0:
				self.__mosaicSize = mosaicSize
		elif mosaicSize == None:
			self.__mosaicSize = None
		else:
			raise Exceptions.InvalidType()

	def getMosaicSize(self):
		return self.__mosaicSize

class FileReader(object):
	def __init__(self):
		self.subjectDetails = {} # this is used to store birthdate, name, address and sex, hash
		print "FileReader"

	################### All inheriting classes must implement the methods below ################################
	# all FileReaders must implement this method, it asks the reader to look at the file and see if it is the right format
	# if it has imported the file
	def addFile(self, curFile = None, species = None):
		pass
	
	# this goes through the files obtained using addFile and forms dataresource objects from these files
	# inheriting classes must implement this method
	def makeDataResources(self, subjectHashIDMapping = None, subjectType = None, studyID = None):
		pass
	
# this needs to go down here because we have to parse all the classes above so that they are available to the inherited classes
import DICOMObjects

fileReaders = [DICOMObjects.DICOMFileReader]
