from lxml import etree

import dicom
import os
import tempfile # to get a temporary directory

import DataTransferConstants

# rewrites the name and address fields and recompresses the files underneath 
# does not move the files, just edits their names and addresses
def rewritePrivateTags(projectDirectory=None, projectXMLRoot=None, subjectNameEnc=None, subjectAddressEnc=None, subjectID=None, visitID=None, studyID=None, episodeID=None, acquisitionID=None):
	
	# project directory, projectXMLRoot, the encoded strings and we won't do this for more than one subject so subjectID must be non-None
	if projectDirectory == None or projectXMLRoot == None or subjectNameEnc == None or subjectAddressEnc == None or subjectID == None:
		return

	PIGZ = DataTransferConstants.findCommand("pigz")

	temporaryDirectory = tempfile.mkdtemp()

	XPathString = "/XCEDE/dataResource[@subjectID='" + subjectID + "'"
	if visitID != None:
		XPathString = XPathString + " and @visitID='" + visitID + "'"
	if studyID != None:
		XPathString = XPathString + " and @studyID='" + studyID + "'"
	if episodeID != None:
		XPathString = XPathString + " and @episodeID='" + episodeID + "'"
	if acquisitionID != None:
		XPathString = XPathString + " and @acquisitionID='" + acquisitionID + "'"
	XPathString = XPathString + "]"

	dataResourceNodes = projectXMLRoot.xpath(XPathString)

	for curDataResourceNode in dataResourceNodes:
		# check the format of the data
		if curDataResourceNode.get("format") == "DICOM":
			# extract the dicomfiles.tar.gz tarball to the temporary directory
			TarBall = os.path.join(projectDirectory, 'data', \
			curDataResourceNode.get("subjectID").zfill(DataTransferConstants.padZerosLength), \
			curDataResourceNode.get("visitID"), \
			curDataResourceNode.get("studyID"), \
			curDataResourceNode.get("episodeID"), \
			curDataResourceNode.get("acquisitionID").zfill(DataTransferConstants.padZerosLength), \
			"dicomfiles.tar.gz")
			#print TarBall
			if not os.path.isfile(TarBall):
				print "Tar file: " + TarBall + " not found"
			else:
				commandString = "tar -xz -C " + temporaryDirectory + " -f " + TarBall
				#print commandString
				os.system(commandString)
				# go through each file and replace the name and address fields
				FilesInTar = []
				for curURINode in curDataResourceNode.iterchildren(tag="uri"):
					curFile = curURINode.text
					(head, tail) = os.path.split(curFile)
					curFile = os.path.join(temporaryDirectory, tail)
					FilesInTar.append(tail)
					del head
					del tail
					try:
						ds = dicom.filereader.read_file(curFile)
					except Exception, e:
						ds = None
					if not (0x10,0x10) in ds:
						ds.AddNew(0x00100010, "PN", subjectNameEnc)
					else:
						ds[0x10,0x10].value = subjectNameEnc # subject name

					if not (0x10,0x1040) in ds:
						ds.AddNew(0x00101040, "LO", subjectAddressEnc)
					else:
						ds[0x10,0x1040].value = subjectAddressEnc # subject address
					#dicom.filewriter.write_file(os.path.join(databaseDirectory, curAcquisitionURI, tail), ds)
					dicom.filewriter.write_file(curFile, ds)
					del ds
				#for curURINode in curDataResourceNode.iterchildren(tag="uri"):
				commandString = "tar c -C " + temporaryDirectory + " " + " ".join(FilesInTar) + " | "
				if PIGZ == None:
					commandString = commandString + "gzip -9 > " + TarBall
				else:
					commandString = commandString + "pigz -9 > " + TarBall
				#print commandString
				os.system(commandString)
				for curFile in FilesInTar:
					os.remove(os.path.join(temporaryDirectory, curFile))
			#if not os.path.isfile(TarBall):
		elif curDataResourceNode.get("format") == "GENESIS":
			print "GENESIS files not supported yet"
			pass
		else:
			print "Arbitrary data, not changing anything"
		#if curDataResourceNode.get("format") == "DICOM":
	#for curDataResourceNode in dataResourceNodes:
	os.rmdir(temporaryDirectory)
#def rewritePrivateTags(projectDirectory=None, projectXMLRoot=None, subjectNameEnc=None, subjectAddressEnc=None, subjectID=None, visitID=None, studyID=None, episodeID=None, acquisitionID=None):
