#!/usr/bin/python

import getopt
import os
import sys

import stat

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import pwd
import grp

import DataTransferConstants
import DataTransferACL

import UNIXUtilities

import Exceptions

class ModifyUserWindow:

	def __init__(self):
		#                              user/group
		self.listStore = gtk.ListStore(gobject.TYPE_STRING)

	def showWindow(self, mainOwnerAndGroup = None, category = None, existingNamesAndPerms = None, excludedNames = None):
		self.curCategory = category
		self.mainOwnerAndGroup = mainOwnerAndGroup
		self.existingNamesAndPerms = existingNamesAndPerms
		# excludedNames only applies when we are adding new users/groups
		# contains a list of users/groups who already have permissions in the project
		self.excludedNames = excludedNames
		
		if not (self.curCategory == "users" or self.curCategory == "groups"):
			raise Exceptions.InvalidValue()
		
		# self.initialisingPreventRadioClicks is True when we are modifying the access levels in the program
		# clicked signals seem to be sent if the buttons change state via a function call as well as a real click
		self.initialisingPreventRadioClicks = True

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_modal(True)
		self.window.set_size_request(640, 480)
		self.window.set_border_width(10)
		self.window.set_title('Edit ' + self.curCategory[:-1] + ' permissions')
		self.mainTable = gtk.Table(rows=4, columns=2, homogeneous=False)
		self.window.add(self.mainTable)
		
		if self.curCategory == "users":
			self.nameList = UNIXUtilities.getUserList()
		else:
			self.nameList = UNIXUtilities.getGroupList()

		if not isinstance(self.existingNamesAndPerms, dict):
			self.scrolled_window = gtk.ScrolledWindow()
			self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			self.mainTable.attach(self.scrolled_window, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			self.mainTable.show()

			self.listBox = gtk.TreeView(self.listStore)
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn(self.curCategory[:-1].capitalize() + " ID", curRenderer, text = 0)
			curColumn.set_sort_order(gtk.SORT_ASCENDING)
			self.listBox.append_column(curColumn)
			self.scrolled_window.add(self.listBox)
			
			self.listBoxTreeSelections = self.listBox.get_selection()
			self.listBoxTreeSelections.set_mode(gtk.SELECTION_MULTIPLE)
			
			self.pathNameMapping = {}
			sortedNameList = []
				
			for curNameID in self.nameList:
				if curNameID >= 1000 and not self.nameList[curNameID] in self.excludedNames:
					if self.curCategory == 'users' and self.nameList[curNameID] != self.mainOwnerAndGroup['owner']:
						sortedNameList.append(self.nameList[curNameID])
					if self.curCategory == 'groups' and self.nameList[curNameID] != self.mainOwnerAndGroup['group']:
						sortedNameList.append(self.nameList[curNameID])

			self.listStore.clear()
			for curName in sorted(sortedNameList):
				curTreeIter = self.listStore.append([curName])
				self.pathNameMapping[self.listStore.get_path(curTreeIter)] = curName
			
			self.listBox.show()
			self.curAccessLevel = 0

		else:
			str = "Editing permissions for " + self.curCategory + ":\n" + "\n".join(self.existingNamesAndPerms.keys())

			self.editingPermsLabel = gtk.Label(str = str)
			self.mainTable.attach(self.editingPermsLabel, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			
			tmpNames = self.existingNamesAndPerms.values()
			self.curAccessLevel = tmpNames[0]
			del tmpNames

		self.metaFrame = gtk.Frame(label = "Metadata access")
		self.mainTable.attach(self.metaFrame, 1, 2, 0, 1, gtk.FILL, gtk.FILL, 10, 10)
		self.metaTable = gtk.Table(2, 3)
		self.metaNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.metaTable.attach(self.metaNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaReadButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read")
		self.metaTable.attach(self.metaReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaWriteButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read/Write")
		self.metaTable.attach(self.metaWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.metaActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.metaTable.attach(self.metaActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaFrame.add(self.metaTable)
		
		self.metaNoAccessButton.connect("toggled", self.accessbutton_cb, "meta no access")
		self.metaReadButton.connect("toggled", self.accessbutton_cb, "meta read")
		self.metaWriteButton.connect("toggled", self.accessbutton_cb, "meta write")
		
		self.dataFrame = gtk.Frame(label = "Data access")
		self.mainTable.attach(self.dataFrame, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 10, 10)
		self.dataTable = gtk.Table(2, 3)
		self.dataNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.dataTable.attach(self.dataNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataReadButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read")
		self.dataTable.attach(self.dataReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataWriteButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read/Write")
		self.dataTable.attach(self.dataWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.dataActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		
		self.dataNoAccessButton.connect("toggled", self.accessbutton_cb, "data no access")
		self.dataReadButton.connect("toggled", self.accessbutton_cb, "data read")
		self.dataWriteButton.connect("toggled", self.accessbutton_cb, "data write")

		self.dataTable.attach(self.dataActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataFrame.add(self.dataTable)
		
		self.encFrame = gtk.Frame(label = "Encryption reversal")
		self.mainTable.attach(self.encFrame, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 10, 10)

		self.encVBox = gtk.VBox()
		self.encFrame.add(self.encVBox)

		self.encButton = gtk.CheckButton("Encryption reversal permitted")
		self.encVBox.pack_start(self.encButton)
		self.encActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.encActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.encVBox.pack_start(self.encActualAccessLabel, True, True, 0)
		
		self.encButton.connect("toggled", self.accessbutton_cb, "enc")

		self.mainButtonBox = gtk.HButtonBox()
		self.mainButtonBox.set_layout(gtk.BUTTONBOX_SPREAD)
		
		self.mainOKButton = gtk.Button(stock = gtk.STOCK_OK)
		self.mainOKButton.connect("clicked", self.mainbutton_cb, "OK")
		self.mainButtonBox.add(self.mainOKButton)
		self.mainCancelButton = gtk.Button(stock = gtk.STOCK_CANCEL)
		self.mainCancelButton.connect("clicked", self.mainbutton_cb, "Cancel")
		self.mainButtonBox.add(self.mainCancelButton)

		self.mainTable.attach(self.mainButtonBox, 1, 2, 3, 4, gtk.FILL, gtk.FILL, 10, 10)
		
		self.window.show_all()
		self.updateButtonsFromAccessLevel()
		self.initialisingPreventRadioClicks = False
		
		if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: none")
			self.dataActualAccessLabel.set_text("Actual access level: none")
		
		if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: read")
			self.dataActualAccessLabel.set_text("Actual access level: none")
		
		if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: read/write")
			self.dataActualAccessLabel.set_text("Actual access level: none")
		
		if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: read/write")
			self.dataActualAccessLabel.set_text("Actual access level: read")
		
		if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: read (implied)")
			self.dataActualAccessLabel.set_text("Actual access level: read")

		if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaActualAccessLabel.set_text("Actual access level: read/write (implied)")
			self.dataActualAccessLabel.set_text("Actual access level: read/write")
			self.encActualAccessLabel.set_text("Actual access level: read (implied)")
		
		if not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataActualAccessLabel.set_text("Actual access level: none")
			
		if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.encActualAccessLabel.set_text("Actual access level: read")
		elif not self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.encActualAccessLabel.set_text("Actual access level: none")

	def updateButtonsFromAccessLevel(self):
		self.initialisingPreventRadioClicks = True
		if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaReadButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.metaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaWriteButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.metaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
			( self.curAccessLevel & DataTransferACL.ACCESS_METAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE ):
			self.encButton.set_active(True)
			self.encButton.set_sensitive(True)
		
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataReadButton.set_active(True)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)
			self.encButton.set_sensitive(True)
			
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataWriteButton.set_active(True)
			self.encButton.set_active(True)
			self.encButton.set_sensitive(False)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)

		if self.curAccessLevel == 0:
			self.mainOKButton.set_sensitive(False)
		else:
			self.mainOKButton.set_sensitive(True)
		#self.updateActualAccessLabels()
		self.initialisingPreventRadioClicks = False
	
	def accessbutton_cb(self, widget, data=None):
#		print data + " " + str(widget.get_active())
#		pass
		if self.initialisingPreventRadioClicks == False:
			
			if data == "meta no access":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAWRITE

			elif data == "meta read":
				if widget.get_active():
					# activate METAREAD attribute
					self.curAccessLevel |= DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAWRITE
			elif data == "meta write":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAREAD
					# activate METAWRITE attribute
					self.curAccessLevel |= DataTransferACL.ACCESS_METAWRITE
			elif data == "data no access":
				if widget.get_active():
					# deactivate DATAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_DATAWRITE
					# find out what the state of the META permissions are and change the access level and button states
					if self.metaReadButton.get_active():
						self.curAccessLevel |= DataTransferACL.ACCESS_METAREAD
					elif self.metaWriteButton.get_active():
						self.curAccessLevel |= DataTransferACL.ACCESS_METAWRITE

					self.metaNoAccessButton.set_sensitive(True)
					self.metaReadButton.set_sensitive(True)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data read":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribut
					self.curAccessLevel |= DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_DATAWRITE
					# make all buttons in the meta frame disabled since access is implied by data write
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data write":
				if widget.get_active():
					# make all buttons in the meta frame disabled since access is implied by data write
					# deactivate METAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribute
					self.curAccessLevel &= ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel |= DataTransferACL.ACCESS_DATAWRITE
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(False)

					self.encButton.set_sensitive(False)
			elif data == "enc":
				if widget.get_active():
					self.curAccessLevel |= DataTransferACL.ACCESS_ENCREAD
				else:
					self.curAccessLevel &= ~DataTransferACL.ACCESS_ENCREAD
			
			if self.curAccessLevel == 0:
				self.mainOKButton.set_sensitive(False)
			else:
				self.mainOKButton.set_sensitive(True)

			if widget.get_active() or widget == self.encButton:
				#print DataTransferACL.ACLPermissionsToString(self.curAccessLevel)
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: none")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: read")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read")

				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read/write")
					self.encActualAccessLabel.set_text("Actual access level: read (implied)")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.dataActualAccessLabel.set_text("Actual access level: none")
					
				if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.encActualAccessLabel.set_text("Actual access level: read")
				elif not self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.encActualAccessLabel.set_text("Actual access level: none")
			#if widgetActive:
		#if self.initialisingPreventRadioClicks == False:
	#def accessbutton_cb(self, widget, data=None):
		
	def mainbutton_cb(self, widget, data=None):
		if data == "OK":
			CancelBecauseNoAccess = False
			# this shouldnt happen, but it is in here anyway
			if self.curAccessLevel == 0:
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_YES_NO, message_format="All permissions are denied, this would normally mean that the users are deleted from or not added to the access list. Do you want this to happen?")
				response = dialog.run()
				dialog.destroy()
				if response == gtk.RESPONSE_YES:
					CancelBecauseNoAccess = True
				else:
					CancelBecauseNoAccess = False

			if not isinstance(self.existingNamesAndPerms, dict):
				# adding new users
				curSelectedRows = self.listBox.get_selection().get_selected_rows()[1]

				if len(curSelectedRows) == 0:
					dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="No users/groups were selected")
					response = dialog.run()
					dialog.destroy()
				else:
					nameList = []
					for curSelectedRow in curSelectedRows:
						nameList.append(self.pathNameMapping[curSelectedRow])

					modProjectWindow.modifyUserFromWindow(curCategory = self.curCategory, nameList = nameList, permissions = self.curAccessLevel)
					self.window.destroy()
			else:
				# editing existing users
				modProjectWindow.modifyUserFromWindow(curCategory = self.curCategory, nameList = self.existingNamesAndPerms.keys(), permissions = self.curAccessLevel)
				self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()


class ModifyProjectWindow:

	def __init__(self):
		UsersAndGroupsArray = ['users', 'groups']
		
		self.listStores = {}

		for curCategory in UsersAndGroupsArray: 
			#							                 user/group         , permission description
			self.listStores[curCategory] = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)

	def modifyUserFromWindow(self, curCategory = None, nameList = None, permissions = None):
		#print nameList
		#print self.__ACLs
		for curName in nameList:
			if curName in self.__ACLs[curCategory]:
				# find the path that refers to the current user
				self.__ACLs[curCategory][curName] = permissions
				curIter = self.listBoxTreeRowReferences[curCategory][curName].get_model().get_iter(self.listBoxTreeRowReferences[curCategory][curName].get_path())
				self.listStores[curCategory].set(curIter, 1, DataTransferACL.ACLPermissionsToString(permissions))
			else:
				self.__ACLs[curCategory][curName] = permissions
				curTreeIter = self.listStores[curCategory].append([curName, DataTransferACL.ACLPermissionsToString(self.__ACLs[curCategory][curName])])
				self.listBoxTreeRowReferences[curCategory][curName] = gtk.TreeRowReference(self.listStores[curCategory], self.listStores[curCategory].get_path(curTreeIter))

	def showWindow(self, projectName = None, ACLs = None):

		self.__ACLs = ACLs
		self.__projectName = projectName

		self.window = gtk.Window()
		self.window.set_size_request(640, 480)
		self.window.set_modal(True)
		self.window.set_border_width(10)
		self.window.set_title('Edit project permissions: ' + self.__projectName)
		self.mainVBox = gtk.VBox(spacing=10)
		
		self.window.add(self.mainVBox)

		UsersAndGroupsArray = ['users', 'groups']

		self.backFrame = {}

		self.backVBoxes = {}
		self.listButtonHBoxes = {}
		
		self.listBoxWindows = {}
		self.listBoxes = {}
		self.listBoxRenderers = {}
		self.listBoxColumns = {}
		self.listBoxTreeSelections = {}
		self.listBoxTreeRowReferences = {}

		self.addButtons = {}
		self.removeButtons = {}
		self.editButtons = {}
		
		for curCategory in UsersAndGroupsArray:

			self.backFrame[curCategory] = gtk.Frame()
			self.mainVBox.pack_start(self.backFrame[curCategory])
			self.backFrame[curCategory].set_label(curCategory.capitalize())
			self.backFrame[curCategory].set_label_align(0.0, 0.0)
			self.backFrame[curCategory].set_shadow_type(gtk.SHADOW_ETCHED_OUT)
			self.backFrame[curCategory].show()
			
			self.backVBoxes[curCategory] = gtk.VBox(homogeneous = False)
			self.backVBoxes[curCategory].show()
			self.backFrame[curCategory].add(self.backVBoxes[curCategory])
			self.listBoxes[curCategory] = gtk.TreeView(self.listStores[curCategory])
			self.listBoxes[curCategory].show()
			self.listBoxes[curCategory].set_rules_hint(True)
			self.listBoxTreeSelections[curCategory] = self.listBoxes[curCategory].get_selection()
			self.listBoxTreeSelections[curCategory].set_mode(gtk.SELECTION_MULTIPLE)
			self.listBoxTreeSelections[curCategory].set_select_function(self.listbox_selection_changed, full = True, data = curCategory)
			
			self.listBoxWindows[curCategory] = gtk.ScrolledWindow()
			self.listBoxWindows[curCategory].set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			#self.mainTable.attach(self.listBoxWindws[curCategory], 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			self.listBoxWindows[curCategory].add(self.listBoxes[curCategory])	
			self.backVBoxes[curCategory].pack_start(self.listBoxWindows[curCategory], True, True)

			self.listBoxRenderers[curCategory] = []
			self.listBoxColumns[curCategory] = []
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn(curCategory[:-1].capitalize() + " ID", curRenderer, text = 0)
			curColumn.set_sort_order(gtk.SORT_ASCENDING)
			curColumn.set_sort_column_id(0)
			curColumn.set_sort_indicator(True)
			#curColumn.set_clickable(False)
			self.listBoxes[curCategory].append_column(curColumn)

			self.listBoxRenderers[curCategory].append(curRenderer)
			self.listBoxColumns[curCategory].append(curColumn)
			
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn("Permissions", curRenderer, text = 1)
			curColumn.set_expand(True)
			self.listBoxes[curCategory].append_column(curColumn)
			self.listBoxRenderers[curCategory].append(curRenderer)
			self.listBoxColumns[curCategory].append(curColumn)
			
			# self.listBoxTreeRowReferences[curCategory][nameID] = the gtk.TreeRowReference pointing to the row that that name is placed in the listBox
			self.listBoxTreeRowReferences[curCategory] = {}
			self.listStores[curCategory].clear()
			for curName in sorted(self.__ACLs[curCategory].iterkeys()):
				curTreeIter = self.listStores[curCategory].append([curName, DataTransferACL.ACLPermissionsToString(self.__ACLs[curCategory][curName])])
				self.listBoxTreeRowReferences[curCategory][curName] = gtk.TreeRowReference(self.listStores[curCategory], self.listStores[curCategory].get_path(curTreeIter))
				#print curName + " " + DataTransferACL.ACLPermissionsToString(ACLs[curCategory][curName])
				
			self.listButtonHBoxes[curCategory] = gtk.HBox()
			self.listButtonHBoxes[curCategory].show()
			self.backVBoxes[curCategory].pack_start(self.listButtonHBoxes[curCategory], False, False)
			
			self.addButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_ADD, use_underline=True)
			self.removeButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_DELETE, use_underline=True)
			self.editButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_EDIT, use_underline=True)
			
			self.addButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "add")
			self.removeButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "remove")
			self.editButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "edit")

			self.addButtons[curCategory].set_sensitive(True)
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)

			self.listButtonHBoxes[curCategory].pack_start(self.addButtons[curCategory])
			self.listButtonHBoxes[curCategory].pack_start(self.editButtons[curCategory])
			self.listButtonHBoxes[curCategory].pack_start(self.removeButtons[curCategory])
			self.listStores[curCategory].set_sort_column_id(0, gtk.SORT_ASCENDING)
		
		self.bottomHBox = gtk.HBox()

		self.mainVBox.pack_start(self.bottomHBox, False, False)
		self.mainOKButton = gtk.Button(label = None, stock = gtk.STOCK_SAVE)
		self.mainCancelButton = gtk.Button(label = None, stock = gtk.STOCK_CANCEL)
		self.bottomHBox.pack_end(self.mainCancelButton, False, False)
		self.bottomHBox.pack_end(self.mainOKButton, False, False)
		self.mainOKButton.connect("clicked", self.mainbuttons_cb, "OK")
		self.mainCancelButton.connect("clicked", self.mainbuttons_cb, "Cancel")

		#	gtk.TreeViewColumn(
			
		self.window.show_all()

	def listbox_selection_changed(self, selection, model, path, is_selected, data = None):
		curCategory = data

		selectedRows = selection.get_selected_rows()[1][:]
		#selectedNames = []
		# for some reason is_selected means that it is not selected, or rather it is inactive
		# so remove it from the selectedRows
		if is_selected:
			for idx in range(0, len(selectedRows)):
				if selectedRows[idx] == path:
					del selectedRows[idx]
					break
		else:
			selectedRows.append(path)

		if len(selectedRows) == 0:
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)
		else:
			#for curPath in selectedRows:
			#	for curName in self.listBoxTreeRowReferences[curCategory]:
			#		if curPath == self.listBoxTreeRowReferences[curCategory][curName].get_path():
			#			selectedNames.append(curName)
			self.removeButtons[curCategory].set_sensitive(True)
			self.editButtons[curCategory].set_sensitive(True)

		
		#print "----"
		#print selection.get_selected_rows()[1]
		#print model
		#print path
		#print is_selected
		#print data
		#print selectedNames
		#print "----"
		return True

	def mainbuttons_cb(self, widget, data = None):
		if data == "OK":
			mainWindow.updateProjectACLs(ACLs = self.__ACLs)
			self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()

	def buttons_cb(self, widget, curCategory = None, action = None):
		# this contains a list of paths (rows) that are currently selected
		curSelectionPaths = self.listBoxTreeSelections[curCategory].get_selected_rows()[1]

		curPathUserMapping = {}

		for curName in self.listBoxTreeRowReferences[curCategory]:
			curPathUserMapping[self.listBoxTreeRowReferences[curCategory][curName].get_path()] = curName

		existingNamesAndPerms = {}
		for curSelection in curSelectionPaths:
			if curSelection in curPathUserMapping:
				curName = curPathUserMapping[curSelection]
				existingNamesAndPerms[curName] = self.__ACLs[curCategory][curName]

		#rint curSelectionPaths
		#self.window.set_modal(True)

		#print action
		#print existingNamesAndPerms
		mainOwnerAndGroup = {}
		mainOwnerAndGroup['owner'] = self.__ACLs['owner']
		mainOwnerAndGroup['group'] = self.__ACLs['maingroup']
		excludedNames = self.__ACLs[curCategory].keys()
		if action == "add":
			modUserWindow.showWindow(mainOwnerAndGroup = mainOwnerAndGroup, category = curCategory, existingNamesAndPerms = None, excludedNames = excludedNames)
		elif action == "edit":
			modUserWindow.showWindow(mainOwnerAndGroup = mainOwnerAndGroup, category = curCategory, existingNamesAndPerms = existingNamesAndPerms, excludedNames = excludedNames)
		elif action == "remove":
			for curName in existingNamesAndPerms:
				del self.__ACLs[curCategory][curName]
				# find the rows in the tree store and delete them, as well as a the TreeRowReferences
				curIter = self.listStores[curCategory].get_iter(self.listBoxTreeRowReferences[curCategory][curName].get_path())
				self.listStores[curCategory].remove(curIter)
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)
				
		#print self.__ACLs

#class InfoModel:
#
#	""" The model class holds the information we want to display """
#	def __init__(self, databaseDirectory=None):
#		""" Sets up and populates our gtk.ListStore """
#		""" The format of the widget is dictated by the type of store we have, since this will be a liststore it will give us a tabular output """ 
#		
#			
#	def __str__(self):
#		return str(self.userFields)
#
#	def get_model(self):
#		""" Returns the model """
#		if self.listStore:
#			return self.listStore
#		else:
#			return None
#
#		def commit(self):
#		pass
#		#print DataTransferACL.databaseACLsToString(ACLs = ACLs, databaseDirectory = self.__databaseDirectory)
#		#DataTransferACL.applyDatabaseACLs(ACLs = ACLs, databaseDirectory = self.__databaseDirectory)
#
class DTACLGUI:

	WIDTH = 1024
	HEIGHT = 600
	def layout_resize(self, widget, event):
		x, y, width, height = widget.get_allocation()
		if width > self.lwidth or height > self.lheight:
			self.lwidth = max(width, self.lwidth)
			self.lheight = max(height, self.lheight)
			widget.set_size(self.lwidth, self.lheight)

	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="ActivatePermissions"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
		<menu action="Edit">
			<menuitem action="ModifyPermissions"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="ActivatePermissions"/>
		<separator/>
		<toolitem action="ModifyPermissions"/>
	</toolbar>
	</ui>'''
	def updateProjectACLs(self, ACLs = None):

		curProject = self.__projectNameEditing
		self.__projectChanged[curProject] = True
		curProjectACLs = {}
		curProjectACLs[curProject] = ACLs
		curPermissions = self.ACLsToPermissions(ACLs = curProjectACLs)

		self.__userMetaRead[curProject] = curPermissions['userMetaRead'][curProject]
		self.__userMetaWrite[curProject] = curPermissions['userMetaWrite'][curProject]
		self.__userEncRead[curProject] = curPermissions['userEncRead'][curProject]
		self.__userDataRead[curProject] = curPermissions['userDataRead'][curProject]
		self.__userDataWrite[curProject] = curPermissions['userDataWrite'][curProject]
		
		self.__groupMetaRead[curProject] = curPermissions['groupMetaRead'][curProject]
		self.__groupMetaWrite[curProject] = curPermissions['groupMetaWrite'][curProject]
		self.__groupEncRead[curProject] = curPermissions['groupEncRead'][curProject]
		self.__groupDataRead[curProject] = curPermissions['groupDataRead'][curProject]
		self.__groupDataWrite[curProject] = curPermissions['groupDataWrite'][curProject]
		
		curPermStringArray = self.projectPermissionStrings( \
			userMetaRead = self.__userMetaRead[curProject], \
			userMetaWrite = self.__userMetaWrite[curProject], \
			userEncRead = self.__userEncRead[curProject], \
			userDataRead = self.__userDataRead[curProject], \
			userDataWrite = self.__userDataWrite[curProject], \
			groupMetaRead = self.__groupMetaRead[curProject], \
			groupMetaWrite = self.__groupMetaWrite[curProject], \
			groupEncRead = self.__groupEncRead[curProject], \
			groupDataRead = self.__groupDataRead[curProject], \
			groupDataWrite = self.__groupDataWrite[curProject])
			
		curArray = [curProject, self.__owner[curProject], self.__mainGroup[curProject]]
		for curString in curPermStringArray:
			curArray.append(curString)
		for idx in range(0, len(curArray)):
			self.listStore.set(self.listStore.get_iter(self.__projectRowMapping[curProject].get_path()), idx, curArray[idx])
		self.actiongroup.get_action("ActivatePermissions").set_sensitive(True)

	def ACLsToPermissions(self, ACLs = None):
		owner = {}
		mainGroup = {}
		userMetaRead = {}
		userMetaWrite = {}
		userEncRead = {}
		userDataRead = {}
		userDataWrite = {}
		
		groupMetaRead = {}
		groupMetaWrite = {}
		groupEncRead = {}
		groupDataRead = {}
		groupDataWrite = {}

		for curProject in sorted(ACLs.iterkeys()):
			# get the ACL entries for the project directory
			
			userMetaRead[curProject] = []
			userMetaWrite[curProject] = []
			userEncRead[curProject] = []
			userDataRead[curProject] = []
			userDataWrite[curProject] = []
			
			groupMetaRead[curProject] = []
			groupMetaWrite[curProject] = []
			groupEncRead[curProject] = []
			groupDataRead[curProject] = []
			groupDataWrite[curProject] = []

			for curUserName in ACLs[curProject]['users'].iterkeys():
				if ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_METAREAD and not ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_METAWRITE:
					userMetaRead[curProject].append(curUserName)
				elif not ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_METAREAD and ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_METAWRITE:
					userMetaWrite[curProject].append(curUserName)
				
				if ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_ENCREAD:
					userEncRead[curProject].append(curUserName)
				
				if ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_DATAREAD and not ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_DATAWRITE:
					userDataRead[curProject].append(curUserName)
				elif not ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_DATAREAD and ACLs[curProject]['users'][curUserName] & DataTransferACL.ACCESS_DATAWRITE:
					userDataWrite[curProject].append(curUserName)

			for curGroupName in ACLs[curProject]['groups'].iterkeys():
				if ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_METAREAD and not ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_METAWRITE:
					groupMetaRead[curProject].append(curGroupName)
				elif not ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_METAREAD and ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_METAWRITE:
					groupMetaWrite[curProject].append(curGroupName)
				
				if ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_ENCREAD:
					groupEncRead[curProject].append(curGroupName)
				
				if ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_DATAREAD and not ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_DATAWRITE:
					groupDataRead[curProject].append(curGroupName)
				elif not ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_DATAREAD and ACLs[curProject]['groups'][curGroupName] & DataTransferACL.ACCESS_DATAWRITE:
					groupDataWrite[curProject].append(curGroupName)

			owner[curProject] = ACLs[curProject]['owner']
			mainGroup[curProject] = ACLs[curProject]['maingroup']
		permissions = {}
		permissions['owner'] = owner
		permissions['mainGroup'] = mainGroup
		permissions['userMetaRead'] = userMetaRead
		permissions['userMetaWrite'] = userMetaWrite
		permissions['userEncRead'] = userEncRead
		permissions['userDataRead'] = userDataRead
		permissions['userDataWrite'] = userDataWrite
				
		permissions['groupMetaRead'] = groupMetaRead
		permissions['groupMetaWrite'] = groupMetaWrite
		permissions['groupEncRead'] = groupEncRead
		permissions['groupDataRead'] = groupDataRead
		permissions['groupDataWrite'] = groupDataWrite
		return permissions	

	def permissionsToACLs(self):
		ACLs = {}
		for curProject in self.__userMetaRead.iterkeys():
			ACLs[curProject] = {}
			ACLs[curProject]['users'] = {}
			ACLs[curProject]['groups'] = {}
			# do the users
			for curUserName in self.__userMetaRead[curProject]:
				if curUserName in ACLs[curProject]['users']:
					ACLs[curProject]['users'][curUserName] |= DataTransferACL.ACCESS_METAREAD
				else:
					ACLs[curProject]['users'][curUserName] = DataTransferACL.ACCESS_METAREAD
			
			for curUserName in self.__userMetaWrite[curProject]:
				if curUserName in ACLs[curProject]['users']:
					ACLs[curProject]['users'][curUserName] |= DataTransferACL.ACCESS_METAWRITE
				else:
					ACLs[curProject]['users'][curUserName] = DataTransferACL.ACCESS_METAWRITE

			for curUserName in self.__userEncRead[curProject]:
				if curUserName in ACLs[curProject]['users']:
					ACLs[curProject]['users'][curUserName] |= DataTransferACL.ACCESS_ENCREAD
				else:
					ACLs[curProject]['users'][curUserName] = DataTransferACL.ACCESS_ENCREAD
			
			for curUserName in self.__userDataRead[curProject]:
				if curUserName in ACLs[curProject]['users']:
					ACLs[curProject]['users'][curUserName] |= DataTransferACL.ACCESS_DATAREAD
				else:
					ACLs[curProject]['users'][curUserName] = DataTransferACL.ACCESS_DATAREAD
			
			for curUserName in self.__userDataWrite[curProject]:
				if curUserName in ACLs[curProject]['users']:
					ACLs[curProject]['users'][curUserName] |= DataTransferACL.ACCESS_DATAWRITE
				else:
					ACLs[curProject]['users'][curUserName] = DataTransferACL.ACCESS_DATAWRITE
			### do the groups
			for curGroupName in self.__groupMetaRead[curProject]:
				if curGroupName in ACLs[curProject]['groups']:
					ACLs[curProject]['groups'][curGroupName] |= DataTransferACL.ACCESS_METAREAD
				else:
					ACLs[curProject]['groups'][curGroupName] = DataTransferACL.ACCESS_METAREAD
			
			for curGroupName in self.__groupMetaWrite[curProject]:
				if curGroupName in ACLs[curProject]['groups']:
					ACLs[curProject]['groups'][curGroupName] |= DataTransferACL.ACCESS_METAWRITE
				else:
					ACLs[curProject]['groups'][curGroupName] = DataTransferACL.ACCESS_METAWRITE

			for curGroupName in self.__groupEncRead[curProject]:
				if curGroupName in ACLs[curProject]['groups']:
					ACLs[curProject]['groups'][curGroupName] |= DataTransferACL.ACCESS_ENCREAD
				else:
					ACLs[curProject]['groups'][curGroupName] = DataTransferACL.ACCESS_ENCREAD
			
			for curGroupName in self.__groupDataRead[curProject]:
				if curGroupName in ACLs[curProject]['groups']:
					ACLs[curProject]['groups'][curGroupName] |= DataTransferACL.ACCESS_DATAREAD
				else:
					ACLs[curProject]['groups'][curGroupName] = DataTransferACL.ACCESS_DATAREAD
			
			for curGroupName in self.__groupDataWrite[curProject]:
				if curGroupName in ACLs[curProject]['groups']:
					ACLs[curProject]['groups'][curGroupName] |= DataTransferACL.ACCESS_DATAWRITE
				else:
					ACLs[curProject]['groups'][curGroupName] = DataTransferACL.ACCESS_DATAWRITE

			ACLs[curProject]['owner'] = self.__owner[curProject]	
			ACLs[curProject]['maingroup'] = self.__mainGroup[curProject]	
		return ACLs	

	def projectPermissionStrings(self, \
	userMetaRead = None, \
	userMetaWrite = None, \
	userEncRead = None, \
	userDataRead = None, \
	userDataWrite = None, \
	groupMetaRead = None, \
	groupMetaWrite = None, \
	groupEncRead = None, \
	groupDataRead = None, \
	groupDataWrite = None):

		metaReadString = ""
		metaWriteString = ""
		encReadString = ""
		dataReadString = ""
		dataWriteString = ""
		
		if len(userMetaRead) > 0:
			metaReadString = metaReadString + "Users: " + ", ".join(userMetaRead)
		
		if len(userMetaWrite) > 0:
			metaWriteString = metaWriteString + "Users: " + ", ".join(userMetaWrite)
		
		if len(userEncRead) > 0:
			encReadString = encReadString + "Users: " + ", ".join(userEncRead)

		if len(userDataRead) > 0:
			dataReadString = dataReadString + "Users: " + ", ".join(userDataRead)
		
		if len(userDataWrite) > 0:
			dataWriteString = dataWriteString + "Users: " + ", ".join(userDataWrite)
		
		if len(groupMetaRead) > 0:
			if len(metaReadString) > 0:
				metaReadString = metaReadString + "\n"
			metaReadString = metaReadString + "Groups: " + ", ".join(groupMetaRead)
		
		if len(groupMetaWrite) > 0:
			if len(metaWriteString) > 0:
				metaWriteString = metaWriteString + "\n"
			metaWriteString = metaWriteString + "Groups: " + ", ".join(groupMetaWrite)
		
		if len(groupEncRead) > 0:
			if len(encReadString) > 0:
				encReadString = encReadString + "\n"
			encReadString = encReadString + "Groups: " + ", ".join(groupEncRead)

		if len(groupDataRead) > 0:
			if len(dataReadString) > 0:
				dataReadString = dataReadString + "\n"
			dataReadString = dataReadString + "Groups: " + ", ".join(groupDataRead)
		
		if len(groupDataWrite) > 0:
			if len(dataWriteString) > 0:
				dataWriteString = dataWriteString + "\n"
			dataWriteString = dataWriteString + "Groups: " + ", ".join(groupDataWrite)
		
		return [metaReadString, metaWriteString, encReadString, dataReadString, dataWriteString]
	
	def __init__(self, databaseDirectory = None):
		# Create the toplevel window
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBIdb: ACL permission manager")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT
		
		#                                           project,               owner,               group, users who have meta, users who have read, users who have write, groups who have meta, groups who have read, groups who have write
		#self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,  gobject.TYPE_STRING,  gobject.TYPE_STRING,  gobject.TYPE_STRING,   gobject.TYPE_STRING)
		# more compact layout
		#                                          project,               owner,               group,            metaread,           metawrite,  decryption and namehashmapping,            data read,            data full
		self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,             gobject.TYPE_STRING,  gobject.TYPE_STRING,  gobject.TYPE_STRING)
		#self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN)

		self.__databaseDirectory = databaseDirectory
		ACLs = DataTransferACL.getDatabaseACLs(databaseDirectory = self.__databaseDirectory) 
		
		#print ACLs
		#projectList = DataTransferConstants.projectList(databaseDirectory)
		#print DataTransferACL.databaseACLsToString(ACLs = ACLs, databaseDirectory = self.__databaseDirectory)
		
		processOwner = os.getuid()
		
		# make a list of projects
		self.__projectChanged = {}
		projectsToDeleteFromACLs = []
		for curProject in ACLs.iterkeys():
			(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(DataTransferConstants.projectDirectory(self.__databaseDirectory, curProject))
			if processOwner == st_uid or processOwner == 0:
				self.__projectChanged[curProject] = False
			else:
				projectsToDeleteFromACLs.append(curProject)

		for curProject in projectsToDeleteFromACLs:
			del ACLs[curProject]
		print ACLs
		permissions = self.ACLsToPermissions(ACLs)
		self.__owner = permissions['owner']
		self.__mainGroup = permissions['mainGroup']
		self.__userMetaRead = permissions['userMetaRead']
		self.__userMetaWrite = permissions['userMetaWrite']
		self.__userEncRead = permissions['userEncRead']
		self.__userDataRead = permissions['userDataRead']
		self.__userDataWrite = permissions['userDataWrite']
		
		self.__groupMetaRead = permissions['groupMetaRead']
		self.__groupMetaWrite = permissions['groupMetaWrite']
		self.__groupEncRead = permissions['groupEncRead']
		self.__groupDataRead = permissions['groupDataRead']
		self.__groupDataWrite = permissions['groupDataWrite']
		del permissions
		
		self.__projectRowMapping = {}
		for curProject in sorted(ACLs.iterkeys()):
			curPermStringArray = self.projectPermissionStrings( \
			userMetaRead = self.__userMetaRead[curProject], \
			userMetaWrite = self.__userMetaWrite[curProject], \
			userEncRead = self.__userEncRead[curProject], \
			userDataRead = self.__userDataRead[curProject], \
			userDataWrite = self.__userDataWrite[curProject], \
			groupMetaRead = self.__groupMetaRead[curProject], \
			groupMetaWrite = self.__groupMetaWrite[curProject], \
			groupEncRead = self.__groupEncRead[curProject], \
			groupDataRead = self.__groupDataRead[curProject], \
			groupDataWrite = self.__groupDataWrite[curProject])
			
			curArray = [curProject, self.__owner[curProject], self.__mainGroup[curProject]]
			for curString in curPermStringArray:
				curArray.append(curString)
			#print curArray
			curIter = self.listStore.append(curArray)
			del curArray
			self.__projectRowMapping[curProject] = gtk.TreeRowReference(self.listStore, self.listStore.get_path(curIter))

			#self.listStore.append([curProject, self.__owner[curProject], self.__mainGroup[curProject], ", ".join(self.__userMetaData[curProject]), ", ".join(self.__userReadOnly[curProject]), ", ".join(self.__userFull[curProject]), ", ".join(self.__groupMetaData[curProject]), ", ".join(self.__groupReadOnly[curProject]), ", ".join(self.__groupFull[curProject])])

			#if processUID == st_uid or processUID == 0: # second clause: if I am the root user
			#	self.listStore.append([curProject, userIDStats[0], groupIDStats[0], stat.S_IRGRP & st_mode, stat.S_IROTH & st_mode and stat.S_IXOTH & st_mode])
				#self.columnTitles = ["Project", "Owner", "Group", "Group: write (allow changes)", "Everyone: read (allow export)"]
				# the user owns this project directory, add it to the list and make it editable

			#print st_uid
			#print st_gid
		# A gtk.VBox is a container that allows you to insert objects
		# vertically
		vbox = gtk.VBox()
		window.add(vbox)
		
		# Create a UIManager instance
		uimanager = gtk.UIManager()

		# Add the accelerator group to the toplevel window
		accelgroup = uimanager.get_accel_group()
		window.add_accel_group(accelgroup)

		# Create an ActionGroup
		self.actiongroup = gtk.ActionGroup('UIManagerExample')

		# Create actions
		#(action, ?, menu test, ?, tooltiptext, callback)
		#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
		self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
					('ActivatePermissions', gtk.STOCK_SAVE, '_Apply permissions', None, 'Apply permissions to files and directories', self.applyACLs_cb),
					('ModifyPermissions', gtk.STOCK_EDIT, '_Modify project permissions', None, 'Modify selected project\' permissions', self.modifyACLs_cb),
					('Edit', None, '_Edit'),
					('File', None, '_File')])
		self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

		self.actiongroup.get_action("ModifyPermissions").set_sensitive(False)
		if gtk.pygtk_version >= (2, 16, 0):
			self.actiongroup.get_action("ModifyPermissions").set_is_important(True)
		else:
			self.actiongroup.get_action("ModifyPermissions").set_property("is-important", True)
		self.actiongroup.get_action("ActivatePermissions").set_sensitive(False)
		# Add the actiongroup to the uimanager
		uimanager.insert_action_group(self.actiongroup, 0)

		# Add a UI description
		uimanager.add_ui_from_string(self.ui)

		# Create a MenuBar
		menubar = uimanager.get_widget('/MenuBar')
		vbox.pack_start(menubar, False)

		# Create a Toolbar
		toolbar = uimanager.get_widget('/Toolbar')
		vbox.pack_start(toolbar, False)
        	
		self.view = gtk.TreeView( self.listStore )

		self.renderers = []
		self.treeViewColumns = []
		#self.columnTitles = ["Project", 	"Owner", 	"Group", 	"Group: write (allow changes)", 	"Everyone: read (allow export)"]
		#self.columnTypes = 	['text',		"text",		"text",		"toggle",							"toggle"]
		#self.columnTitles = ["Project", 	"Owner", 	"Group", 	"Users\n(metadata only)", "Users\n(read only)", 	"Users\n(full access)",	"Groups\n(metadata only)", 	"Groups\n(read only)",	"Groups\n(full access)"]
		#self.columnTypes = 	['text',		"text",		"text",		"text",					  "text", 					"text",					"text",						"text",					"text"]
		self.columnTitles = ["Project", 	"Owner", 	"Group", 	"Metadata read",	 "Metadata read/write", 	"Decrypt/reverse anonymisation",	"Data read", 	"Data read/write"]
		self.columnTypes = 	['text',		"text",		"text",		"text",			 "text", 				"text",								"text",			"text"]

		for i in range(len(self.columnTitles)):
			if self.columnTypes[i] == 'text':
				curRenderer = gtk.CellRendererText()
				curRenderer.set_property('editable', False)
				self.renderers.append(curRenderer)
				curColumn = gtk.TreeViewColumn(self.columnTitles[i], self.renderers[i], text=i)
				curColumn.set_alignment(0.5)
				curColumn.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
				curColumn.set_resizable(False)
				if i == 0:
					curColumn.set_sort_order(gtk.SORT_ASCENDING)
					curColumn.set_sort_column_id(i)
				#curColumn.set_sort_indicator(False)
				#curColumn.set_sensitive(False)
				curColumn.set_expand(True)
				self.treeViewColumns.append(curColumn)

			elif self.columnTypes[i] == 'toggle':
				curRenderer = gtk.CellRendererToggle()
				curRenderer.set_property('activatable', True)
				self.renderers.append(curRenderer)
				self.treeViewColumns.append(gtk.TreeViewColumn(self.columnTitles[i], self.renderers[i]))

		self.listStore.set_sort_column_id(0, gtk.SORT_ASCENDING)
		for i in range(len(self.renderers)):
			if self.columnTypes[i] == 'toggle':
				self.treeViewColumns[i].add_attribute(self.renderers[i], "active", i)
			self.view.append_column(self.treeViewColumns[i])
		self.view.set_rules_hint(True)

		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		tree_selection = self.view.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		#tree_selection.select_path(0)	
		#tree_selection.set_select_function(self.selectionChanged, full=False, data=None)
		window.connect("delete-event", self.window_deleting_cb)
		tree_selection.connect("changed", self.selectionChanged)
		scrolled_window.add(self.view)
		vbox.pack_start(scrolled_window, True)
		window.connect("delete-event", self.window_deleting_cb)
		window.set_geometry_hints(None, min_width=1000, min_height=600, max_width=-1, max_height=-1, base_width=-1, base_height=-1, width_inc=1, height_inc=1, min_aspect=-1.0, max_aspect=-1.0)
		window.show_all()
	
	def anyProjectsModified(self):
		anyProjectsChanged = False
		for curProject in self.__projectChanged.iterkeys():
			if self.__projectChanged[curProject] == True:
				anyProjectsChanged = True
		return anyProjectsChanged

	def applyACLs_cb(self, w):
		self.applyModifiedACLs()
		#print ACLs
		#DataTransferACL.applyProjectACLs(ACLs = ACLs, databaseDirectory = self.__databaseDirectory, projectName = self.
		#Store.commit()

	def modifyACLs_cb(self, w):
		treeSelection = self.view.get_selection()
		curIter = treeSelection.get_selected()[1]

		if curIter != None:
			projectName = self.listStore.get(treeSelection.get_selected()[1], 0)[0]
			self.__projectNameEditing = projectName
			ACLs = self.permissionsToACLs()
			modProjectWindow.showWindow(projectName = projectName, ACLs = ACLs[projectName])
		
	def selectionChanged(self, Selection):
		# see if this field is in the current project's field list, if it is, then enable the migrate option
		curSelection = Selection.get_selected()

		if curSelection[1] == None:
			self.actiongroup.get_action("ModifyPermissions").set_sensitive(False)
		else:
			self.actiongroup.get_action("ModifyPermissions").set_sensitive(True)

		return True
	
	def applyModifiedACLs(self):
		ACLs = self.permissionsToACLs()
		for curProject in self.__projectChanged.iterkeys():
			if self.__projectChanged[curProject] == True:
				DataTransferACL.applyProjectACLs(ACLs = ACLs[curProject], databaseDirectory = self.__databaseDirectory, projectName = curProject)
				self.__projectChanged[curProject] = False
		# make the apply permissions box not sensitive
		self.actiongroup.get_action("ActivatePermissions").set_sensitive(False)

		
	def ask_and_save(self):
		if self.anyProjectsModified() == True:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO, message_format="The permissions have been modified. Do you wish to save the changes?")
			response = dialog.run()
			dialog.destroy()
			if response == gtk.RESPONSE_YES:
				self.applyModifiedACLs()
		return False

	def quit_cb(self, b):
		self.ask_and_save()
		gtk.main_quit()

	def window_deleting_cb(self, widget, event, data=None):
		print "windowdeleting"
		response = self.ask_and_save()
		return response
#class DTACLGUI:
		
if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "", ['database-directory='])
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
	#for o, a in opts:

	#databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	databaseDirectory = '/data/addo/database'
	if not DataTransferACL.isACLSupported(databaseDirectory = databaseDirectory):
		print "For database directory: " + databaseDirectory + ", ACL is not supported, use dt_permissions_gui.py"
	else:
		projectList = DataTransferConstants.projectList(databaseDirectory)
	
		modProjectWindow = ModifyProjectWindow()
		modUserWindow = ModifyUserWindow()

		mainWindow = DTACLGUI(databaseDirectory = databaseDirectory)	
		gtk.main()
