#!/usr/bin/python

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import pwd
import grp

import getopt
import sys

import UNIXUtilities

import DataTransferACL
import DataTransferConstants

class ModifyUserWindow:

	def __init__(self):
		#                              user/group
		self.listStore = gtk.ListStore(gobject.TYPE_STRING)

	def showWindow(self, mainOwnerAndGroup = None, category = None, existingNamesAndPerms = None, excludedNames = None):
		self.curCategory = category
		self.mainOwnerAndGroup = mainOwnerAndGroup
		self.existingNamesAndPerms = existingNamesAndPerms
		# excludedNames only applies when we are adding new users/groups
		# contains a list of users/groups who already have permissions in the project
		self.excludedNames = excludedNames
		
		if not (self.curCategory == "users" or self.curCategory == "groups"):
			raise Exceptions.InvalidValue()
		
		# self.initialisingPreventRadioClicks is True when we are modifying the access levels in the program
		# clicked signals seem to be sent if the buttons change state via a function call as well as a real click
		self.initialisingPreventRadioClicks = True

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_modal(True)
		self.window.set_size_request(640, 480)
		self.window.set_border_width(10)
		self.window.set_title('Edit ' + self.curCategory[:-1] + ' permissions')
		self.mainTable = gtk.Table(rows=4, columns=2, homogeneous=False)
		self.window.add(self.mainTable)
		
		if self.curCategory == "users":
			self.nameList = UNIXUtilities.getUserList()
		else:
			self.nameList = UNIXUtilities.getGroupList()

		if not isinstance(self.existingNamesAndPerms, dict):
			self.scrolled_window = gtk.ScrolledWindow()
			self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			self.mainTable.attach(self.scrolled_window, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			self.mainTable.show()

			self.listBox = gtk.TreeView(self.listStore)
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn(self.curCategory[:-1].capitalize() + " ID", curRenderer, text = 0)
			curColumn.set_sort_order(gtk.SORT_ASCENDING)
			self.listBox.append_column(curColumn)
			self.scrolled_window.add(self.listBox)
			
			self.listBoxTreeSelections = self.listBox.get_selection()
			self.listBoxTreeSelections.set_mode(gtk.SELECTION_MULTIPLE)
			
			self.pathNameMapping = {}
			sortedNameList = []
				
			for curNameID in self.nameList:
				if curNameID >= 1000 and not self.nameList[curNameID] in self.excludedNames:
					if self.curCategory == 'users' and self.nameList[curNameID] != self.mainOwnerAndGroup['owner']:
						sortedNameList.append(self.nameList[curNameID])
					if self.curCategory == 'groups' and self.nameList[curNameID] != self.mainOwnerAndGroup['group']:
						sortedNameList.append(self.nameList[curNameID])

			self.listStore.clear()
			for curName in sorted(sortedNameList):
				curTreeIter = self.listStore.append([curName])
				self.pathNameMapping[self.listStore.get_path(curTreeIter)] = curName
			
			self.listBox.show()
			self.curAccessLevel = 0

		else:
			str = "Editing permissions for " + self.curCategory + ":\n" + "\n".join(self.existingNamesAndPerms.keys())

			self.editingPermsLabel = gtk.Label(str = str)
			self.mainTable.attach(self.editingPermsLabel, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			
			tmpNames = self.existingNamesAndPerms.values()
			self.curAccessLevel = tmpNames[0]
			del tmpNames

		self.metaFrame = gtk.Frame(label = "Metadata access")
		self.mainTable.attach(self.metaFrame, 1, 2, 0, 1, gtk.FILL, gtk.FILL, 10, 10)
		self.metaTable = gtk.Table(2, 3)
		self.metaNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.metaTable.attach(self.metaNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaReadButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read")
		self.metaTable.attach(self.metaReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaWriteButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read/Write")
		self.metaTable.attach(self.metaWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.metaActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.metaTable.attach(self.metaActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaFrame.add(self.metaTable)
		
		self.metaNoAccessButton.connect("toggled", self.accessbutton_cb, "meta no access")
		self.metaReadButton.connect("toggled", self.accessbutton_cb, "meta read")
		self.metaWriteButton.connect("toggled", self.accessbutton_cb, "meta write")
		
		self.dataFrame = gtk.Frame(label = "Data access")
		self.mainTable.attach(self.dataFrame, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 10, 10)
		self.dataTable = gtk.Table(2, 3)
		self.dataNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.dataTable.attach(self.dataNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataReadButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read")
		self.dataTable.attach(self.dataReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataWriteButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read/Write")
		self.dataTable.attach(self.dataWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.dataActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		
		self.dataNoAccessButton.connect("toggled", self.accessbutton_cb, "data no access")
		self.dataReadButton.connect("toggled", self.accessbutton_cb, "data read")
		self.dataWriteButton.connect("toggled", self.accessbutton_cb, "data write")

		self.dataTable.attach(self.dataActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataFrame.add(self.dataTable)
		
		self.encFrame = gtk.Frame(label = "Encryption reversal")
		self.mainTable.attach(self.encFrame, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 10, 10)

		self.encVBox = gtk.VBox()
		self.encFrame.add(self.encVBox)

		self.encButton = gtk.CheckButton("Encryption reversal permitted")
		self.encVBox.pack_start(self.encButton)
		self.encActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.encActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.encVBox.pack_start(self.encActualAccessLabel, True, True, 0)
		
		self.encButton.connect("toggled", self.accessbutton_cb, "enc")

		self.mainButtonBox = gtk.HButtonBox()
		self.mainButtonBox.set_layout(gtk.BUTTONBOX_SPREAD)
		
		self.mainOKButton = gtk.Button(stock = gtk.STOCK_OK)
		self.mainOKButton.connect("clicked", self.mainbutton_cb, "OK")
		self.mainButtonBox.add(self.mainOKButton)
		self.mainCancelButton = gtk.Button(stock = gtk.STOCK_CANCEL)
		self.mainCancelButton.connect("clicked", self.mainbutton_cb, "Cancel")
		self.mainButtonBox.add(self.mainCancelButton)

		self.mainTable.attach(self.mainButtonBox, 1, 2, 3, 4, gtk.FILL, gtk.FILL, 10, 10)
		
		self.window.show_all()
		self.updateButtonsFromAccessLevel()
		self.initialisingPreventRadioClicks = False

	def updateButtonsFromAccessLevel(self):
		self.initialisingPreventRadioClicks = True
		if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaReadButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.metaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaWriteButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.metaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
			( self.curAccessLevel & DataTransferACL.ACCESS_METAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE ):
			self.encButton.set_active(True)
			self.encButton.set_sensitive(True)
		
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataReadButton.set_active(True)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)
			self.encButton.set_sensitive(True)
			
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataWriteButton.set_active(True)
			self.encButton.set_active(True)
			self.encButton.set_sensitive(False)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)

		if self.curAccessLevel == 0:
			self.mainOKButton.set_sensitive(False)
		else:
			self.mainOKButton.set_sensitive(True)
		#self.updateActualAccessLabels()
		self.initialisingPreventRadioClicks = False
	
	def accessbutton_cb(self, widget, data=None):
#		print data + " " + str(widget.get_active())
#		pass
		if self.initialisingPreventRadioClicks == False:
			
			if data == "meta no access":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE

			elif data == "meta read":
				if widget.get_active():
					# activate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
			elif data == "meta write":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# activate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAWRITE
			elif data == "data no access":
				if widget.get_active():
					# deactivate DATAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAWRITE
					# find out what the state of the META permissions are and change the access level and button states
					if self.metaReadButton.get_active():
						self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAREAD
					elif self.metaWriteButton.get_active():
						self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAWRITE

					self.metaNoAccessButton.set_sensitive(True)
					self.metaReadButton.set_sensitive(True)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data read":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribut
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAWRITE
					# make all buttons in the meta frame disabled since access is implied by data write
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data write":
				if widget.get_active():
					# make all buttons in the meta frame disabled since access is implied by data write
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_DATAWRITE
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(False)

					self.encButton.set_sensitive(False)
			elif data == "enc":
				if widget.get_active():
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_ENCREAD
				else:
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_ENCREAD
			
			if self.curAccessLevel == 0:
				self.mainOKButton.set_sensitive(False)
			else:
				self.mainOKButton.set_sensitive(True)

			if widget.get_active() or widget == self.encButton:
				#print DataTransferACL.ACLPermissionsToString(self.curAccessLevel)
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: none")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: read")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read")

				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read/write")
					self.encActualAccessLabel.set_text("Actual access level: read (implied)")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.dataActualAccessLabel.set_text("Actual access level: none")
					
				if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.encActualAccessLabel.set_text("Actual access level: read")
				elif not self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.encActualAccessLabel.set_text("Actual access level: none")
			#if widgetActive:
		#if self.initialisingPreventRadioClicks == False:
	#def accessbutton_cb(self, widget, data=None):
		
	def mainbutton_cb(self, widget, data=None):
		if data == "OK":
			CancelBecauseNoAccess = False
			# this shouldnt happen, but it is in here anyway
			if self.curAccessLevel == 0:
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_YES_NO, message_format="All permissions are denied, this would normally mean that the users are deleted from or not added to the access list. Do you want this to happen?")
				response = dialog.run()
				dialog.destroy()
				if response == gtk.RESPONSE_YES:
					CancelBecauseNoAccess = True
				else:
					CancelBecauseNoAccess = False

			if not isinstance(self.existingNamesAndPerms, dict):
				# adding new users
				curSelectedRows = self.listBox.get_selection().get_selected_rows()[1]

				if len(curSelectedRows) == 0:
					dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="No users/groups were selected")
					response = dialog.run()
					dialog.destroy()
				else:
					nameList = []
					for curSelectedRow in curSelectedRows:
						nameList.append(self.pathNameMapping[curSelectedRow])

					modProjectWindow.modifyUserFromWindow(curCategory = self.curCategory, nameList = nameList, permissions = self.curAccessLevel)
					self.window.destroy()
			else:
				# editing existing users
				modProjectWindow.modifyUserFromWindow(curCategory = self.curCategory, nameList = self.existingNamesAndPerms.keys(), permissions = self.curAccessLevel)
				self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()


class ModifyProjectWindow:

	def __init__(self):
		UsersAndGroupsArray = ['users', 'groups']
		
		self.listStores = {}

		for curCategory in UsersAndGroupsArray: 
			#							                 user/group         , permission description
			self.listStores[curCategory] = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)

	def modifyUserFromWindow(self, curCategory = None, nameList = None, permissions = None):
		#print nameList
		#print self.__ACLs
		for curName in nameList:
			if curName in self.__ACLs[curCategory]:
				# find the path that refers to the current user
				self.__ACLs[curCategory][curName] = permissions
				curIter = self.listBoxTreeRowReferences[curCategory][curName].get_model().get_iter(self.listBoxTreeRowReferences[curCategory][curName].get_path())
				self.listStores[curCategory].set(curIter, 1, DataTransferACL.ACLPermissionsToString(permissions))
			else:
				self.__ACLs[curCategory][curName] = permissions
				curTreeIter = self.listStores[curCategory].append([curName, DataTransferACL.ACLPermissionsToString(self.__ACLs[curCategory][curName])])
				self.listBoxTreeRowReferences[curCategory][curName] = gtk.TreeRowReference(self.listStores[curCategory], self.listStores[curCategory].get_path(curTreeIter))

	def showWindow(self, projectName = None, ACLs = None):

		self.__ACLs = ACLs
		self.__projectName = projectName

		self.window = gtk.Window()
		self.window.set_size_request(640, 480)
		self.window.set_modal(True)
		self.window.set_border_width(10)
		self.window.set_title('Edit project permissions')
		self.mainVBox = gtk.VBox(spacing=10)
		
		self.window.add(self.mainVBox)

		UsersAndGroupsArray = ['users', 'groups']

		self.backFrame = {}

		self.backVBoxes = {}
		self.listButtonHBoxes = {}
		
		self.listBoxWindows = {}
		self.listBoxes = {}
		self.listBoxRenderers = {}
		self.listBoxColumns = {}
		self.listBoxTreeSelections = {}
		self.listBoxTreeRowReferences = {}

		self.addButtons = {}
		self.removeButtons = {}
		self.editButtons = {}
		
		for curCategory in UsersAndGroupsArray:

			self.backFrame[curCategory] = gtk.Frame()
			self.mainVBox.pack_start(self.backFrame[curCategory])
			self.backFrame[curCategory].set_label(curCategory.capitalize())
			self.backFrame[curCategory].set_label_align(0.0, 0.0)
			self.backFrame[curCategory].set_shadow_type(gtk.SHADOW_ETCHED_OUT)
			self.backFrame[curCategory].show()
			
			self.backVBoxes[curCategory] = gtk.VBox(homogeneous = False)
			self.backVBoxes[curCategory].show()
			self.backFrame[curCategory].add(self.backVBoxes[curCategory])
			self.listBoxes[curCategory] = gtk.TreeView(self.listStores[curCategory])
			self.listBoxes[curCategory].show()
			self.listBoxes[curCategory].set_rules_hint(True)
			self.listBoxTreeSelections[curCategory] = self.listBoxes[curCategory].get_selection()
			self.listBoxTreeSelections[curCategory].set_mode(gtk.SELECTION_MULTIPLE)
			self.listBoxTreeSelections[curCategory].set_select_function(self.listbox_selection_changed, full = True, data = curCategory)
			
			self.listBoxWindows[curCategory] = gtk.ScrolledWindow()
			self.listBoxWindows[curCategory].set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			#self.mainTable.attach(self.listBoxWindws[curCategory], 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			self.listBoxWindows[curCategory].add(self.listBoxes[curCategory])	
			self.backVBoxes[curCategory].pack_start(self.listBoxWindows[curCategory], True, True)

			self.listBoxRenderers[curCategory] = []
			self.listBoxColumns[curCategory] = []
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn(curCategory[:-1].capitalize() + " ID", curRenderer, text = 0)
			curColumn.set_sort_order(gtk.SORT_ASCENDING)
			curColumn.set_sort_column_id(0)
			curColumn.set_sort_indicator(True)
			#curColumn.set_clickable(False)
			self.listBoxes[curCategory].append_column(curColumn)

			self.listBoxRenderers[curCategory].append(curRenderer)
			self.listBoxColumns[curCategory].append(curColumn)
			
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn("Permissions", curRenderer, text = 1)
			curColumn.set_expand(True)
			self.listBoxes[curCategory].append_column(curColumn)
			self.listBoxRenderers[curCategory].append(curRenderer)
			self.listBoxColumns[curCategory].append(curColumn)
			
			# self.listBoxTreeRowReferences[curCategory][nameID] = the gtk.TreeRowReference pointing to the row that that name is placed in the listBox
			self.listBoxTreeRowReferences[curCategory] = {}
			self.listStores[curCategory].clear()
			for curName in sorted(self.__ACLs[curCategory].iterkeys()):
				curTreeIter = self.listStores[curCategory].append([curName, DataTransferACL.ACLPermissionsToString(self.__ACLs[curCategory][curName])])
				self.listBoxTreeRowReferences[curCategory][curName] = gtk.TreeRowReference(self.listStores[curCategory], self.listStores[curCategory].get_path(curTreeIter))
				#print curName + " " + DataTransferACL.ACLPermissionsToString(ACLs[curCategory][curName])
				
			self.listButtonHBoxes[curCategory] = gtk.HBox()
			self.listButtonHBoxes[curCategory].show()
			self.backVBoxes[curCategory].pack_start(self.listButtonHBoxes[curCategory], False, False)
			
			self.addButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_ADD, use_underline=True)
			self.removeButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_DELETE, use_underline=True)
			self.editButtons[curCategory] = gtk.Button(label=None, stock=gtk.STOCK_EDIT, use_underline=True)
			
			self.addButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "add")
			self.removeButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "remove")
			self.editButtons[curCategory].connect("clicked", self.buttons_cb, curCategory, "edit")

			self.addButtons[curCategory].set_sensitive(True)
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)

			self.listButtonHBoxes[curCategory].pack_start(self.addButtons[curCategory])
			self.listButtonHBoxes[curCategory].pack_start(self.editButtons[curCategory])
			self.listButtonHBoxes[curCategory].pack_start(self.removeButtons[curCategory])
			self.listStores[curCategory].set_sort_column_id(0, gtk.SORT_ASCENDING)
		
		self.bottomHBox = gtk.HBox()

		self.mainVBox.pack_start(self.bottomHBox, False, False)
		self.mainOKButton = gtk.Button(label = None, stock = gtk.STOCK_SAVE)
		self.mainCancelButton = gtk.Button(label = None, stock = gtk.STOCK_CANCEL)
		self.bottomHBox.pack_end(self.mainCancelButton, False, False)
		self.bottomHBox.pack_end(self.mainOKButton, False, False)

		#	gtk.TreeViewColumn(
			
		self.window.show_all()

	def listbox_selection_changed(self, selection, model, path, is_selected, data = None):
		curCategory = data

		selectedRows = selection.get_selected_rows()[1][:]
		#selectedNames = []
		# for some reason is_selected means that it is not selected, or rather it is inactive
		# so remove it from the selectedRows
		if is_selected:
			for idx in range(0, len(selectedRows)):
				if selectedRows[idx] == path:
					del selectedRows[idx]
					break
		else:
			selectedRows.append(path)

		if len(selectedRows) == 0:
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)
		else:
			#for curPath in selectedRows:
			#	for curName in self.listBoxTreeRowReferences[curCategory]:
			#		if curPath == self.listBoxTreeRowReferences[curCategory][curName].get_path():
			#			selectedNames.append(curName)
			self.removeButtons[curCategory].set_sensitive(True)
			self.editButtons[curCategory].set_sensitive(True)

		
		#print "----"
		#print selection.get_selected_rows()[1]
		#print model
		#print path
		#print is_selected
		#print data
		#print selectedNames
		#print "----"
		return True

	def buttons_cb(self, widget, curCategory = None, action = None):
		# this contains a list of paths (rows) that are currently selected
		curSelectionPaths = self.listBoxTreeSelections[curCategory].get_selected_rows()[1]

		curPathUserMapping = {}

		for curName in self.listBoxTreeRowReferences[curCategory]:
			curPathUserMapping[self.listBoxTreeRowReferences[curCategory][curName].get_path()] = curName

		existingNamesAndPerms = {}
		for curSelection in curSelectionPaths:
			if curSelection in curPathUserMapping:
				curName = curPathUserMapping[curSelection]
				existingNamesAndPerms[curName] = self.__ACLs[curCategory][curName]

		#rint curSelectionPaths
		#self.window.set_modal(True)

		#print action
		#print existingNamesAndPerms
		mainOwnerAndGroup = {}
		mainOwnerAndGroup['owner'] = self.__ACLs['owner']
		mainOwnerAndGroup['group'] = self.__ACLs['maingroup']
		excludedNames = self.__ACLs[curCategory].keys()
		if action == "add":
			modUserWindow.showWindow(mainOwnerAndGroup = mainOwnerAndGroup, category = curCategory, existingNamesAndPerms = None, excludedNames = excludedNames)
		elif action == "edit":
			modUserWindow.showWindow(mainOwnerAndGroup = mainOwnerAndGroup, category = curCategory, existingNamesAndPerms = existingNamesAndPerms, excludedNames = excludedNames)
		elif action == "remove":
			for curName in existingNamesAndPerms:
				del self.__ACLs[curCategory][curName]
				# find the rows in the tree store and delete them, as well as a the TreeRowReferences
				curIter = self.listStores[curCategory].get_iter(self.listBoxTreeRowReferences[curCategory][curName].get_path())
				self.listStores[curCategory].remove(curIter)
			self.removeButtons[curCategory].set_sensitive(False)
			self.editButtons[curCategory].set_sensitive(False)
				
		#print self.__ACLs

class MainWindow:
	WIDTH = 320
	HEIGHT = 200
	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="ActivatePermissions"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
		<menu action="Edit">
			<menuitem action="ModifyPermissions"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="ActivatePermissions"/>
		<separator/>
		<toolitem action="ModifyPermissions"/>
	</toolbar>
	</ui>'''
	def __init__(self, databaseDirectory = None):
# Create the toplevel window

		self.__databaseDirectory = databaseDirectory
		self.__ACLs = DataTransferACL.getDatabaseACLs(databaseDirectory = self.__databaseDirectory) 
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBIdb: ACL permission manager")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

# A gtk.VBox is a container that allows you to insert objects
# vertically
		vbox = gtk.VBox()
		window.add(vbox)

# Create a UIManager instance
		uimanager = gtk.UIManager()

# Add the accelerator group to the toplevel window
		#accelgroup = uimanager.get_accel_group()
		#window.add_accel_group(accelgroup)

# Create an ActionGroup
#self.actiongroup = gtk.ActionGroup('UIManagerExample')

# Create actions
#(action, ?, menu test, ?, tooltiptext, callback)
#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
#self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
#			('ActivatePermissions', gtk.STOCK_SAVE, '_Apply permissions', None, 'Apply permissions to files and directories', self.applyACLs_cb),
#			('ModifyPermissions', gtk.STOCK_EDIT, '_Modify project permissions', None, 'Modify selected project\' permissions', self.modifyACLs_cb),
#			('Edit', None, '_Edit'),
#			('File', None, '_File')])
#self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

# Add the actiongroup to the uimanager
#uimanager.insert_action_group(self.actiongroup, 0)

# Add a UI description
#uimanager.add_ui_from_string(self.ui)

# Create a MenuBar
#menubar = uimanager.get_widget('/MenuBar')
#vbox.pack_start(menubar, False)

# Create a Toolbar
#toolbar = uimanager.get_widget('/Toolbar')
#vbox.pack_start(toolbar, False)
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		vbox.pack_start(self.ButtonOK, False);
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, 'OK')
		window.show_all()
		return

	def callback(self, widget, data=None):
		modProjectWindow.showWindow(ACLs = self.__ACLs['CC_agenesis'])
		return

if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "", ['database-directory='])
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
	#for o, a in opts:

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	#databaseDirectory = '/home/addo/datatransfer/database';
	
	ACLSupport = DataTransferACL.isACLSupported(databaseDirectory)

	print ACLSupport

	projectList = DataTransferConstants.projectList(databaseDirectory)

	#Store = InfoModel(databaseDirectory = databaseDirectory)	
	#Display = DisplayModel()
	#UIExample = UIManagerExample()
	Pw = UNIXUtilities.getUserList()
	Gr = UNIXUtilities.getGroupList()
	#print "Users"
	#for curUser in Pw:
	#	print str(curUser) + " => " + Pw[curUser]
	
	#print "Groups"
	#for curGroup in Gr:
	#	print str(curGroup) + " => " + Gr[curGroup]

	modProjectWindow = ModifyProjectWindow()
	modUserWindow = ModifyUserWindow()
	Main = MainWindow(databaseDirectory = databaseDirectory)
	gtk.main()
