#!/usr/bin/python

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import pwd
import grp

import getopt
import sys

import UNIXUtilities

import DataTransferACL
import DataTransferConstants

import Exceptions

class ModifyUserWindow:

	def __init__(self):
		#                              user/group
		self.listStore = gtk.ListStore(gobject.TYPE_STRING)

	def showWindow(self, category = None, existingNamesAndPerms = None, excludedNames = None):
		self.category = category
		self.existingNamesAndPerms = existingNamesAndPerms
		# excludedNames only applies when we are adding new users/groups
		# contains a list of users/groups who already have permissions in the project
		self.excludedNames = excludedNames
		
		if not (self.category == "users" or self.category == "groups"):
			raise Exceptions.InvalidValue()
		
		# self.initialisingPreventRadioClicks is True when we are modifying the access levels in the program
		# clicked signals seem to be sent if the buttons change state via a function call as well as a real click
		self.initialisingPreventRadioClicks = True

		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.set_modal(True)
		self.window.set_size_request(640, 480)
		self.window.set_border_width(10)
		self.window.set_title('Edit ' + self.category[:-1] + ' permissions')
		self.mainTable = gtk.Table(rows=4, columns=2, homogeneous=False)
		self.window.add(self.mainTable)
		
		if self.category == "users":
			self.nameList = UNIXUtilities.getUserList()
		else:
			self.nameList = UNIXUtilities.getGroupList()

		if not isinstance(self.existingNamesAndPerms, dict):
			self.scrolled_window = gtk.ScrolledWindow()
			self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
			self.mainTable.attach(self.scrolled_window, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			self.mainTable.show()

			self.listBox = gtk.TreeView(self.listStore)
			curRenderer = gtk.CellRendererText()
			curRenderer.set_property('editable', False)
			curColumn = gtk.TreeViewColumn(self.category[:-1].capitalize() + " ID", curRenderer, text = 0)
			curColumn.set_sort_order(gtk.SORT_ASCENDING)
			self.listBox.append_column(curColumn)
			self.scrolled_window.add(self.listBox)

			for curNameID in self.nameList:
				if curNameID >= 1000 and not self.nameList[curNameID] in self.excludedNames :
					self.listStore.append([self.nameList[curNameID]])
			self.listBox.show()
			self.curAccessLevel = 0

		else:
			str = "Editing permissions for " + self.category + ":\n" + "\n".join(self.existingNamesAndPerms.keys())

			self.editingPermsLabel = gtk.Label(str = str)
			self.mainTable.attach(self.editingPermsLabel, 0, 1, 0, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 10, 10)
			
			tmpNames = self.existingNamesAndPerms.values()
			self.curAccessLevel = tmpNames[0]
			del tmpNames

		self.metaFrame = gtk.Frame(label = "Metadata access")
		self.mainTable.attach(self.metaFrame, 1, 2, 0, 1, gtk.FILL, gtk.FILL, 10, 10)
		self.metaTable = gtk.Table(2, 3)
		self.metaNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.metaTable.attach(self.metaNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaReadButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read")
		self.metaTable.attach(self.metaReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaWriteButton = gtk.RadioButton(group = self.metaNoAccessButton, label = "Read/Write")
		self.metaTable.attach(self.metaWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.metaActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.metaTable.attach(self.metaActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.metaFrame.add(self.metaTable)
		
		self.metaNoAccessButton.connect("toggled", self.accessbutton_cb, "meta no access")
		self.metaReadButton.connect("toggled", self.accessbutton_cb, "meta read")
		self.metaWriteButton.connect("toggled", self.accessbutton_cb, "meta write")
		
		self.dataFrame = gtk.Frame(label = "Data access")
		self.mainTable.attach(self.dataFrame, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 10, 10)
		self.dataTable = gtk.Table(2, 3)
		self.dataNoAccessButton = gtk.RadioButton(group = None, label = "None")
		self.dataTable.attach(self.dataNoAccessButton, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataReadButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read")
		self.dataTable.attach(self.dataReadButton, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataWriteButton = gtk.RadioButton(group = self.dataNoAccessButton, label = "Read/Write")
		self.dataTable.attach(self.dataWriteButton, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.dataActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		
		self.dataNoAccessButton.connect("toggled", self.accessbutton_cb, "data no access")
		self.dataReadButton.connect("toggled", self.accessbutton_cb, "data read")
		self.dataWriteButton.connect("toggled", self.accessbutton_cb, "data write")

		self.dataTable.attach(self.dataActualAccessLabel, 0, 3, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.dataFrame.add(self.dataTable)
		
		self.encFrame = gtk.Frame(label = "Encryption reversal")
		self.mainTable.attach(self.encFrame, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 10, 10)

		self.encVBox = gtk.VBox()
		self.encFrame.add(self.encVBox)

		self.encButton = gtk.CheckButton("Encryption reversal permitted")
		self.encVBox.pack_start(self.encButton)
		self.encActualAccessLabel = gtk.Label(str = "Actual access level: ")
		self.encActualAccessLabel.set_justify(gtk.JUSTIFY_LEFT)
		self.encVBox.pack_start(self.encActualAccessLabel, True, True, 0)
		
		self.encButton.connect("toggled", self.accessbutton_cb, "enc")

		self.mainButtonBox = gtk.HButtonBox()
		self.mainButtonBox.set_layout(gtk.BUTTONBOX_SPREAD)
		
		self.mainOKButton = gtk.Button(stock = gtk.STOCK_OK)
		self.mainOKButton.connect("clicked", self.mainbutton_cb, "OK")
		self.mainButtonBox.add(self.mainOKButton)
		self.mainCancelButton = gtk.Button(stock = gtk.STOCK_CANCEL)
		self.mainCancelButton.connect("clicked", self.mainbutton_cb, "Cancel")
		self.mainButtonBox.add(self.mainCancelButton)

		self.mainTable.attach(self.mainButtonBox, 1, 2, 3, 4, gtk.FILL, gtk.FILL, 10, 10)
		
		self.window.show_all()
		self.updateButtonsFromAccessLevel()
		self.initialisingPreventRadioClicks = False

	def updateButtonsFromAccessLevel(self):
		self.initialisingPreventRadioClicks = True
		if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaReadButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.metaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.metaWriteButton.set_active(True)
			self.dataNoAccessButton.set_active(True)
			self.encButton.set_active(False)
			
			# make the data permission buttons sensitive so that users can change them
			self.met3yyaNoAccessButton.set_sensitive(True)
			self.metaReadButton.set_sensitive(True)
			self.metaWriteButton.set_sensitive(True)
			self.dataNoAccessButton.set_sensitive(True)
			self.dataReadButton.set_sensitive(True)
			self.dataWriteButton.set_sensitive(True)
			self.encButton.set_sensitive(True)

		if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
			( self.curAccessLevel & DataTransferACL.ACCESS_METAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD or \
			  self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE ):
			self.encButton.set_active(True)
			self.encButton.set_sensitive(True)
		
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
			not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataReadButton.set_active(True)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)
			self.encButton.set_sensitive(True)
			
		if self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
			self.dataWriteButton.set_active(True)
			self.encButton.set_active(True)
			self.encButton.set_sensitive(False)
			
			# make all buttons in the meta frame disabled since access is implied by data write
			self.metaNoAccessButton.set_sensitive(False)
			self.metaReadButton.set_sensitive(False)
			self.metaWriteButton.set_sensitive(False)

		self.updateActualAccessLabels()
		self.initialisingPreventRadioClicks = False
	
	def accessbutton_cb(self, widget, data=None):
#		print data + " " + str(widget.get_active())
#		pass
		if self.initialisingPreventRadioClicks == False:
			
			if data == "meta no access":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE

			elif data == "meta read":
				if widget.get_active():
					# activate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
			elif data == "meta write":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# activate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAWRITE
			elif data == "data no access":
				if widget.get_active():
					# deactivate DATAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAWRITE
					# find out what the state of the META permissions are and change the access level and button states
					if self.metaReadButton.get_active():
						self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAREAD
					elif self.metaWriteButton.get_active():
						self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_METAWRITE

					self.metaNoAccessButton.set_sensitive(True)
					self.metaReadButton.set_sensitive(True)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data read":
				if widget.get_active():
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribut
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAWRITE
					# make all buttons in the meta frame disabled since access is implied by data write
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(True)
					self.encButton.set_sensitive(True)
			elif data == "data write":
				if widget.get_active():
					# make all buttons in the meta frame disabled since access is implied by data write
					# deactivate METAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAREAD
					# deactivate METAWRITE attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_METAWRITE
					# activate DATAREAD attribute
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_DATAREAD
					# deactivate DATAWRITE attribute
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_DATAWRITE
					self.metaNoAccessButton.set_sensitive(False)
					self.metaReadButton.set_sensitive(False)
					self.metaWriteButton.set_sensitive(False)

					self.encButton.set_sensitive(False)
			elif data == "enc":
				if widget.get_active():
					self.curAccessLevel = self.curAccessLevel | DataTransferACL.ACCESS_ENCREAD
				else:
					self.curAccessLevel = self.curAccessLevel & ~DataTransferACL.ACCESS_ENCREAD

			if widget.get_active() or widget == self.encButton:
				print DataTransferACL.ACLPermissionsToString(self.curAccessLevel)
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: none")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: none")
				
				if self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write")
					self.dataActualAccessLabel.set_text("Actual access level: read")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read")

				if not self.curAccessLevel & DataTransferACL.ACCESS_METAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_METAWRITE and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.metaActualAccessLabel.set_text("Actual access level: read/write (implied)")
					self.dataActualAccessLabel.set_text("Actual access level: read/write")
					self.encActualAccessLabel.set_text("Actual access level: read (implied)")
				
				if not self.curAccessLevel & DataTransferACL.ACCESS_DATAREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.dataActualAccessLabel.set_text("Actual access level: none")
					
				if self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD and \
					not self.curAccessLevel & DataTransferACL.ACCESS_DATAWRITE:
					self.encActualAccessLabel.set_text("Actual access level: read")
				elif not self.curAccessLevel & DataTransferACL.ACCESS_ENCREAD:
					self.encActualAccessLabel.set_text("Actual access level: none")
			#if widgetActive:
		#if self.initialisingPreventRadioClicks == False:
	#def accessbutton_cb(self, widget, data=None):
		
	def mainbutton_cb(self, widget, data=None):
		if data == "OK":
			pass
		elif data == "Cancel":
			self.window.destroy()

	def updateActualAccessLabels(self):
		pass

class MainWindow:
	WIDTH = 320
	HEIGHT = 200
	def __init__(self, databaseDirectory = None):
# Create the toplevel window

		self.__databaseDirectory = databaseDirectory
		#self.__ACLs = DataTransferACL.getDatabaseACLs(databaseDirectory = self.__databaseDirectory) 
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBIdb: ACL permission manager")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

# A gtk.VBox is a container that allows you to insert objects
# vertically
		vbox = gtk.VBox()
		window.add(vbox)

# Create a UIManager instance
		uimanager = gtk.UIManager()

# Add the accelerator group to the toplevel window
		#accelgroup = uimanager.get_accel_group()
		#window.add_accel_group(accelgroup)

# Create an ActionGroup
#self.actiongroup = gtk.ActionGroup('UIManagerExample')

# Create actions
#(action, ?, menu test, ?, tooltiptext, callback)
#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
#self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
#			('ActivatePermissions', gtk.STOCK_SAVE, '_Apply permissions', None, 'Apply permissions to files and directories', self.applyACLs_cb),
#			('ModifyPermissions', gtk.STOCK_EDIT, '_Modify project permissions', None, 'Modify selected project\' permissions', self.modifyACLs_cb),
#			('Edit', None, '_Edit'),
#			('File', None, '_File')])
#self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

# Add the actiongroup to the uimanager
#uimanager.insert_action_group(self.actiongroup, 0)

# Add a UI description
#uimanager.add_ui_from_string(self.ui)

# Create a MenuBar
#menubar = uimanager.get_widget('/MenuBar')
#vbox.pack_start(menubar, False)

# Create a Toolbar
#toolbar = uimanager.get_widget('/Toolbar')
#vbox.pack_start(toolbar, False)
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		vbox.pack_start(self.ButtonOK, False);
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, 'OK')
		window.show_all()
		return

	def callback(self, widget, data=None):
		#modProjectWindow.showWindow(self.__ACLs['CC_agenesis'])
		existingNamesAndPerms = {}
		existingNamesAndPerms['addo'] = DataTransferACL.ACCESS_DATAWRITE | DataTransferACL.ACCESS_ENCREAD
		#modProjectWindow.showWindow(category = 'groups', existingNamesAndPerms = existingNamesAndPerms)
		modProjectWindow.showWindow(category = 'groups', existingNamesAndPerms = None)
		return

if __name__ == '__main__':
	#opts, args = getopt.getopt(sys.argv[1:], "", ['database-directory='])
	#databaseDirectory = None

	#for o, a in opts:
	#	if o == '--database-directory':
	#		databaseDirectory = a
	#for o, a in opts:

	#databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	#projectList = DataTransferConstants.projectList(databaseDirectory)

	#Store = InfoModel(databaseDirectory = databaseDirectory)	
	#Display = DisplayModel()
	#UIExample = UIManagerExample()
	#Pw = UNIXUtilities.getUserList()
	#Gr = UNIXUtilities.getGroupList()
	#print "Users"
	#for curUser in Pw:
	#	print str(curUser) + " => " + Pw[curUser]
	
	#print "Groups"
	#for curGroup in Gr:
#		print str(curGroup) + " => " + Gr[curGroup]

	modProjectWindow = ModifyUserWindow()
	Main = MainWindow(databaseDirectory = '/home/addo/datatransfer/database')
	gtk.main()
