#!/usr/bin/python

# this script is used to print out the participant's ages at scan time
# the output is in tabular form
# <subject id> <birth date> <scan date> <age (years,months,days)> <age (months,days)> <age (days)>

from lxml import etree
import sys
import os
import getopt
import DataTransferConstants
import DataTransferPatterns
import re

import datetime
def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] [--subject-type-pattern=<> --subject-pattern=<> --visit-pattern=<>]"

	print 
	print "For each visit, prints the age of the participant in years, months and days"
	print "By default, all visits for each subject are printed"
	print "The collection of subjects and visits can be optionally restricted by using"
	print "the --subject-type-pattern, --subject-pattern and --visit-pattern options"
	print "These options have the same syntax as used in dt_checkout_pattern.py"
	print ""
	print "-p: the project name, this directory must exist: " + os.path.join("<project directory>", "<project name>")
	print "--database-directory: the root directory of the database"
	print "-h: prints this help message, program exits"
	print ""
	print "Input field patterns:"
	print "--subject-type-pattern can either be: controls or patients"
	print ""
	print "--subject-pattern and --visit-pattern are numeric fields"
	print "Numeric patterns use MATLAB-like syntax"
	print "\t--subject-pattern=lowerLimit:increment:upperLimit, lowerLimit:*, *:upperlimit"
	print "\t\t so --subject-pattern=1:5:21 will select subjects with IDs 1, 6, 11, 16, 21"
	print "\t\t so --subject-pattern=*:21 will select subjects with IDs below and including 21"
	print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
	print "Use commas to specify a series of patterns: 1,4:5,7,9,2 ..."
	
def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "subject-type-pattern=", "subject-pattern=", "visit-pattern="])

	# options to implement --3d-only and --4d-only --images-only --non-images-only
	projectName = None
	databaseDirectory = None
	subjectTypePattern = None
	subjectPattern = None
	visitPattern = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-type-pattern':
			subjectTypePattern = a
		if o == '--subject-pattern':
			subjectPattern = a
		if o == '--visit-pattern':
			visitPattern = a
		if o == '-h':
			usage()
			quit()
	#for o, a in opts:
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "The specified project does not exist"
		quit()
	
	# read in project XML file
	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	subjectNodes = projectXML.xpath("/XCEDE/subject")

	subjectDOB = {}

	for curSubjectNode in subjectNodes:
		for curSubjectInfoNode in curSubjectNode.iterchildren(tag = "subjectInfo"):
			for DOBNode in curSubjectInfoNode.iterchildren(tag = "birthdate"):
				subjectDOB[curSubjectNode.get("ID")] = DOBNode.text
	
	visitNodes = projectXML.xpath("/XCEDE/visit")

	for curVisitNode in visitNodes:
		printCurrentVisit = True
		if subjectTypePattern != None or subjectPattern != None or visitPattern != None:
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curVisitNode.get('subjectGroupID'), subjectTypePattern) == False:
					printCurrentVisit = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and printCurrentVisit == True:
				if DataTransferPatterns.matchNumericPattern(curVisitNode.get('subjectID'), subjectPattern) == False:
					printCurrentVisit = False
			
			if visitPattern != None and printCurrentVisit == True:
				if DataTransferPatterns.matchNumericPattern(curVisitNode.get('visitID'), visitPattern) == False:
					printCurrentVisit = False
			
		if printCurrentVisit == True:
			visitDate = curVisitNode.get("ID")
			birthDate = subjectDOB[curVisitNode.get('subjectID')]

			mat = re.match('(\d\d\d\d)(\d\d)(\d\d)', visitDate)
			visitYear = int(mat.group(1))
			visitMonth = int(mat.group(2))
			visitDay = int(mat.group(3))

			mat = re.match('(\d\d\d\d)(\d\d)(\d\d)', birthDate)
			birthYear = int(mat.group(1))
			birthMonth = int(mat.group(2)) 
			birthDay = int(mat.group(3))
			
			date1 = datetime.date(birthYear, birthMonth, birthDay)
			date2 = datetime.date(visitYear, visitMonth, visitDay)

			dateDiff = date2 - date1
			print curVisitNode.get('subjectID') + "\t" + curVisitNode.get("ID") + "\t" + subjectDOB[curVisitNode.get('subjectID')] + "\t" + str(dateDiff.days)#years) + "\t" + str(months) + "\t" + str(days)
#def main():


if __name__ == "__main__":
	main()
