#!/usr/bin/python

# this program deletes subjects, visits, episodes, acquisitions
# it uses the pattern matching routines used in dt_checkout_pattern.py
# will delete at the level of the lowest pattern given
# for example if subject id, acquisition id is given, it will delete acquisitions
# if subject id is given then it will delete subjects
# except for subject types, it will not delete the subject type list

from lxml import etree
import sys
import os
import errno
import getopt
import re
import shutil

import DataTransferConstants
import DataTransferPatterns
import DataTransferProject

import Exceptions

def deleteSubject(databaseDirectory, projectXMLRoot, projectName, subjectID, dryRun=True, verbose=False):
	if not isinstance(subjectID, int):
		raise Exceptions.InvalidType()
	if verbose == True:
		print "deleting subject: " + str(subjectID)
	visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + str(subjectID) + "']")
	for curNode in visitNodes:
		deleteVisit(databaseDirectory, projectXMLRoot, projectName, subjectID, curNode.get("ID"))
	subjectNode = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + str(subjectID) + "']")
	subjectNode = subjectNode[0]

	subjectDirectory = os.path.join(DataTransferConstants.projectDataDirectory(databaseDirectory, projectName), str(subjectID).zfill(DataTransferConstants.padZerosLength))
	# remove the subject's directory on the filesystem
	if verbose == True:
		print "rm -fr " + subjectDirectory
	if dryRun == False:
		shutil.rmtree(subjectDirectory, ignore_errors = True)
	# find the subject's entry in the subject list and delete it from there

	subjectGroupRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='" + subjectNode.get("subjectGroupID") + "']")
	subjectGroupRoot = subjectGroupRoot[0]
	for curSubjectIDNode in subjectGroupRoot.iterchildren():
		if int(curSubjectIDNode.text) == subjectID:
			subjectGroupRoot.remove(curSubjectIDNode)
	
	nameHashMapping = DataTransferProject.NameHashMapping(databaseDirectory = databaseDirectory, projectName = projectName)

	if dryRun == False:
		nameHashMapping.deleteSubjectByID(subjectID)
		nameHashMapping.write()
		
#		nameHashMappingFile = file(os.path.join(databaseDirectory, projectName, "namehashmapping.txt"), 'r')
#		nameHashMapping = {}
#		for curLine in nameHashMappingFile.xreadlines():
#			curLine = curLine.strip()
#			(curSubjectID, subjectHash, subjectName, subjectSex, subjectDOB, subjectAddress) = curLine.split(',')
#			curSubject = {}
#			curSubject['name'] = subjectName
#			curSubject['hash'] = subjectHash
#			curSubject['dob'] = subjectDOB
#			curSubject['sex'] = subjectSex
#			curSubject['address'] = subjectAddress
#			nameHashMapping[int(curSubjectID)] = curSubject
#		#for curLine in nameHashMappingFile.xreadlines():
#		nameHashMappingFile.close()
#		
#		del nameHashMapping[int(subjectID)]
#		nameHashMappingFile = file(os.path.join(databaseDirectory, projectName, "namehashmapping.txt"), 'w')
#		for curSubjectID in sorted(nameHashMapping.keys()):
#			nameHashMappingFile.write(str(curSubjectID) + "," + nameHashMapping[curSubjectID]['hash'] + "," + nameHashMapping[curSubjectID]['name'] + "," + nameHashMapping[curSubjectID]['sex'] + "," + nameHashMapping[curSubjectID]['dob'] + "," + nameHashMapping[curSubjectID]['address'] + "\n")
#		nameHashMappingFile.close()
#
	projectXMLRoot.remove(subjectNode)


def deleteVisit(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, dryRun=True, verbose=False):
	#if verbose == True:
	#	print "deleting visit: " + str(subjectID) + " "  + visitID
	
	if not isinstance(subjectID, int):
		raise Exceptions.InvalidType()

	if not isinstance(visitID, str):
		raise Exceptions.InvalidType()
	
	studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "']")
	for curNode in studyNodes:
		deleteStudy(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, curNode.get("ID"))
	visitNode = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + str(subjectID) + "' and @ID='" + visitID + "']")
	visitNode = visitNode[0]
	projectXMLRoot.remove(visitNode)
	
	visitDirectory = os.path.join(DataTransferConstants.projectDataDirectory(databaseDirectory, projectName), str(subjectID).zfill(DataTransferConstants.padZerosLength), visitID)
	if verbose == True:
		print "rm -fr " + visitDirectory
	if dryRun == False:
		shutil.rmtree(visitDirectory, ignore_errors = True)

def deleteStudy(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, studyID, dryRun=True, verbose=False):
	if not isinstance(subjectID, int):
		raise Exceptions.InvalidType()

	if not isinstance(visitID, str):
		raise Exceptions.InvalidType()

	if not isinstance(studyID, str):
		raise Exceptions.InvalidType()
	
	#if verbose == True:
	#	print "deleting study: " + str(subjectID) + " "  + visitID + " " + studyID
	episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "']")
	for curNode in episodeNodes:
		deleteEpisode(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, studyID, curNode.get("ID"))
	studyNode = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @ID='" + studyID + "']")
	studyNode = studyNode[0]
	projectXMLRoot.remove(studyNode)

	studyDirectory = os.path.join(DataTransferConstants.projectDataDirectory(databaseDirectory, projectName), str(subjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID)
	if verbose == True:
		print "rm -fr " + studyDirectory
	if dryRun == False:
		shutil.rmtree(studyDirectory, ignore_errors = True)

def deleteEpisode(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, studyID, episodeID, dryRun=True, verbose=False):
	if not isinstance(subjectID, int):
		raise Exceptions.InvalidType()

	if not isinstance(visitID, str):
		raise Exceptions.InvalidType()

	if not isinstance(studyID, str):
		raise Exceptions.InvalidType()
	
	if not isinstance(episodeID, str):
		raise Exceptions.InvalidType()
		#if verbose == True:
	#	print "deleting episode: " + str(subjectID) + " "  + visitID + " " + episodeID
	acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "']")
	for curNode in acquisitionNodes:
		deleteAcquisition(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, studyID, episodeID, int(curNode.get("ID")))
	episodeNode = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @ID='" + episodeID + "']")
	episodeNode = episodeNode[0]
	projectXMLRoot.remove(episodeNode)

	episodeDirectory = os.path.join(DataTransferConstants.projectDataDirectory(databaseDirectory, projectName), str(subjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID)
	if verbose == True:
		print "rm -fr " + episodeDirectory
	if dryRun == False:
		shutil.rmtree(episodeDirectory, ignore_errors = True)

def deleteAcquisition(databaseDirectory, projectXMLRoot, projectName, subjectID, visitID, studyID, episodeID, acquisitionID, dryRun=True, verbose=False):
	if not isinstance(subjectID, int):
		raise Exceptions.InvalidType()

	if not isinstance(visitID, str):
		raise Exceptions.InvalidType()

	if not isinstance(studyID, str):
		raise Exceptions.InvalidType()
	
	if not isinstance(episodeID, str):
		raise Exceptions.InvalidType()
	
	if not isinstance(acquisitionID, int):
		raise Exceptions.InvalidType()
	#if verbose == True:
	#	print "deleting acquisition: " + str(subjectID) + " "  + visitID + " " + studyID + " " + episodeID + " " + acquisitionID 
	dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "' and @acquisitionID='" + str(acquisitionID) + "']")
	for curNode in dataResourceNodes:
		deleteDataResource(projectXMLRoot, curNode)
	acquisitionNode = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + str(subjectID) + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @episodeID='" + episodeID + "' and @ID='" + str(acquisitionID) + "']")
	acquisitionNode = acquisitionNode[0]
	projectXMLRoot.remove(acquisitionNode)

	acquisitionDirectory = os.path.join(DataTransferConstants.projectDataDirectory(databaseDirectory, projectName), str(subjectID).zfill(DataTransferConstants.padZerosLength), visitID, studyID, episodeID, str(acquisitionID).zfill(DataTransferConstants.padZerosLength))

	if verbose == True:
		print "rm -fr " + acquisitionDirectory
	if dryRun == False:
		shutil.rmtree(acquisitionDirectory, ignore_errors = True)

# this just deletes the XML for the data resource
def deleteDataResource(projectXMLRoot, dataResourceNode):
	projectXMLRoot.remove(dataResourceNode)

def usage():
	print "Usage: " + sys.argv[0] + " [-h] -p <project name> [--database-directory=<>] [--subject-type-pattern=<> --subject-pattern=<> --visit-pattern=<> --study-pattern=<> --episode-pattern=<> --acquisition-pattern=<>] [--dry-run]"

	print
	print "Deletes subjects/visits/episodes/acquisitions"
	print
	print "The user inputs \"input\" patterns that match subject types/subject ids/visit ids/episode ids/acquisition ids as per dt_checkout_pattern.py"
	print "The files at the lowest level of pattern input will be deleted"
	print "i.e. the levels go subject -> visit -> episode -> acquisition"
	print "so if the acquisition pattern is given then only the acquisitions will be deleted"
	print "if the acquisition pattern is omitted and the episode pattern is given then the episodes and all acquisitions underneath will be deleted"
	print
	print "-p: the project name, this directory must exist: " + os.path.join("<project directory>", "<project name>")
	print "--database-directory: the root directory of the database"
	print "-h: prints this help message, program exits"
	print "-v: verbose mode, tells you what directories it is deleting"
	print
	print "Input field patterns:"
	print
	print "String fields (study-pattern, episode-pattern) permits the use of wildcard patterns in the same way as used in the shell for file matching"
	print ""
	print "Numeric patterns (subject-pattern, visit-pattern, acquisition-pattern) uses MATLAB-like syntax"
	print "\t--subject-pattern=lowerLimit:increment:upperLimit, lowerLimit:*, *:upperlimit"
	print "\t\t so --subject-pattern=1:5:21 will select subjects with IDs 1, 6, 11, 16, 21"
	print "\t\t so --subject-pattern=*:21 will select subjects with IDs below and including 21"
	print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
	print ""
	print "IMPORTANT:"
	print "\tThe wildcard character, *, is interpreted by the shell prior to program execution and thus may return a list of files in the current directory"
	print "\tTo avoid this, escape any wildcards with two backslashes, \\\\*, so that bash does not perform file matching"
	print ""
	print "--database-directory NOTES"
	print "\tThe project directory is the root directory of the database and it is an optional command line argument."
	print "\tIf the project directory given on the command line then that value will be used."
	print "\tIf it is not specified on the command line it will search the environment variable DT_PROJECT_DIR"
	print "\tIf the environment variable is empty, the value in DataTransferConstants.databaseDirectory will be used: " + DataTransferConstants.databaseDirectory
	print
	print "--dry-run will not actually delete the files, but just print out the list of files it found"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "fp:hv", ["database-directory=", "subject-type-pattern=", "subject-pattern=", "visit-pattern=", "study-pattern=", "episode-pattern=", "acquisition-pattern=", "dry-run"])

	projectName = None
	databaseDirectory = None
	subjectTypePattern = None
	subjectPattern = None
	visitPattern = None
	studyPattern = None
	episodePattern = None
	acquisitionPattern = None
	dryRun = False
	verbose = False
	forceDelete = False

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-type-pattern':
			subjectTypePattern = a
		if o == '--subject-pattern':
			subjectPattern = a
		if o == '--visit-pattern':
			visitPattern = a
		if o == '--study-pattern':
			studyPattern = a
		if o == '--episode-pattern':
			episodePattern = a
		if o == '--acquisition-pattern':
			acquisitionPattern = a
		if o == '--dry-run':
			dryRun = True
		if o == '-f':
			forceDelete = True
		if o == '-h':
			usage()
			quit()
		if o == '-v':
			verbose = True
	#for o, a in opts:
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None:
		print "Project name not given"
		usage()
		quit()
	projectName = projectName.strip('/')
		
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "Project does not exist"
		quit()

	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()

	nodesToDelete = []

	lowestLevel = 'subject'

	if visitPattern != None:
		lowestLevel = 'visit'
	
	if studyPattern != None:
		lowestLevel = 'study'
	
	if episodePattern != None:
		lowestLevel = 'episode'
	
	if acquisitionPattern != None:
		lowestLevel = 'acquisition'

	if lowestLevel == 'subject': # deleting entire subjects
		subjectNodes = projectXMLRoot.xpath('/XCEDE/subject')
		for curNode in subjectNodes:
			deleteCurNode = True
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curNode.get('subjectGroupID'), subjectTypePattern) == False:
					deleteCurNode = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('ID'), subjectPattern) == False:
					deleteCurNode = False
			
			if deleteCurNode == True:
				deleteConfirmation = False
				if not forceDelete:
					confirmKey = raw_input("Delete SUBJECT: ID = " + curNode.get("ID") + " ? ")
					if confirmKey == 'y' or confirmKey == 'Y':
						deleteConfirmation = True

				if forceDelete or deleteConfirmation:
					nodesToDelete.append(curNode)
		# put the deletion here so that if the user presses Ctrl-C, and has already deleted some, thats bad, so wait until all the decisions have been made, then delete
		for curNode in nodesToDelete:
			deleteSubject(databaseDirectory, projectXMLRoot, projectName, int(curNode.get("ID")), dryRun=dryRun, verbose=verbose)

	elif lowestLevel == 'visit': # deleting entire visit
		visitNodes = projectXMLRoot.xpath('/XCEDE/visit')
		for curNode in visitNodes:
			deleteCurNode = True
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curNode.get('subjectGroupID'), subjectTypePattern) == False:
					deleteCurNode = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('subjectID'), subjectPattern) == False:
					deleteCurNode = False

			if visitPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('ID'), visitPattern) == False:
					deleteCurNode = False

			if deleteCurNode == True:
				deleteConfirmation = False
				if not forceDelete:
					confirmKey = raw_input("Delete VISIT: subject ID = " + curNode.get("subjectID") + ", visit ID = " + curNode.get("ID") + " ? ")
					if confirmKey == 'y' or confirmKey == 'Y':
						deleteConfirmation = True

				if forceDelete or deleteConfirmation:
					nodesToDelete.append(curNode)
		for curNode in nodesToDelete:
			deleteVisit(databaseDirectory, projectXMLRoot, projectName, int(curNode.get("subjectID")), curNode.get("ID"), dryRun=dryRun, verbose=verbose)

	elif lowestLevel == 'study': # deleting entire visit
		studyNodes = projectXMLRoot.xpath('/XCEDE/study')
		for curNode in studyNodes:
			deleteCurNode = True
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curNode.get('subjectGroupID'), subjectTypePattern) == False:
					deleteCurNode = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('subjectID'), subjectPattern) == False:
					deleteCurNode = False

			if visitPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('visitID'), visitPattern) == False:
					deleteCurNode = False
			
			if studyPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchStringPattern(curNode.get('ID'), studyPattern) == False:
					deleteCurNode = False

			if deleteCurNode == True:
				deleteConfirmation = False
				if not forceDelete:
					confirmKey = raw_input("Delete STUDY: subject ID = " + curNode.get("subjectID") + ", visit ID = " + curNode.get("visitID") + ", study ID = " + curNode.get("ID") + " ? ")
					if confirmKey == 'y' or confirmKey == 'Y':
						deleteConfirmation = True
				if forceDelete or deleteConfirmation:
					nodesToDelete.append(curNode)
		for curNode in nodesToDelete:
			deleteStudy(databaseDirectory, projectXMLRoot, projectName, int(curNode.get("subjectID")), curNode.get("visitID"), curNode.get("ID"), dryRun=dryRun, verbose=verbose)

	elif lowestLevel == 'episode':
		episodeNodes = projectXMLRoot.xpath('/XCEDE/episode')
		for curNode in episodeNodes:
			deleteCurNode = True
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curNode.get('subjectGroupID'), subjectTypePattern) == False:
					deleteCurNode = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('subjectID'), subjectPattern) == False:
					deleteCurNode = False

			if visitPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('visitID'), visitPattern) == False:
					deleteCurNode = False
			
			if studyPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchStringPattern(curNode.get('studyID'), studyPattern) == False:
					deleteCurNode = False
			
			if episodePattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchStringPattern(curNode.get('ID'), episodePattern) == False:
					deleteCurNode = False

			if deleteCurNode == True:
				deleteConfirmation = False
				if not forceDelete:
					confirmKey = raw_input("Delete EPISODE: subject ID = " + curNode.get("subjectID") + ", visit ID = " + curNode.get("visitID") + ", study ID = " + curNode.get("studyID") + ", episode ID = " + curNode.get("ID") + " ? ")
					if confirmKey == 'y' or confirmKey == 'Y':
						deleteConfirmation = True
				if forceDelete or deleteConfirmation:
					nodesToDelete.append(curNode)
		for curNode in nodesToDelete:
			deleteEpisode(databaseDirectory, projectXMLRoot, projectName, int(curNode.get("subjectID")), curNode.get("visitID"), curNode.get("studyID"), curNode.get("ID"), dryRun=dryRun, verbose=verbose)

	elif lowestLevel == 'acquisition':
		acquisitionNodes = projectXMLRoot.xpath('/XCEDE/acquisition')
		for curNode in acquisitionNodes:
			deleteCurNode = True
			if subjectTypePattern != None:
				if DataTransferPatterns.matchStringPattern(curNode.get('subjectGroupID'), subjectTypePattern) == False:
					deleteCurNode = False

			# the subject patterns are numeric so we try and match it 
			if subjectPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('subjectID'), subjectPattern) == False:
					deleteCurNode = False

			if visitPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('visitID'), visitPattern) == False:
					deleteCurNode = False
			
			if studyPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchStringPattern(curNode.get('studyID'), studyPattern) == False:
					deleteCurNode = False
			
			if episodePattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchStringPattern(curNode.get('episodeID'), episodePattern) == False:
					deleteCurNode = False

			if acquisitionPattern != None and deleteCurNode == True:
				if DataTransferPatterns.matchNumericPattern(curNode.get('ID'), acquisitionPattern) == False:
					deleteCurNode = False

			if deleteCurNode == True:
				deleteConfirmation = False
				if not forceDelete:
					confirmKey = raw_input("Delete ACQUISITION: subject ID = " + curNode.get("subjectID") + ", visit ID = " + curNode.get("visitID") + ", study ID = " + curNode.get("studyID") + ", episode ID = " + curNode.get("episodeID") + ", acquisition ID = " + curNode.get("ID") + " ? ")
					if confirmKey == 'y' or confirmKey == 'Y':
						deleteConfirmation = True
				if forceDelete or deleteConfirmation:
					nodesToDelete.append(curNode)
		for curNode in nodesToDelete:
			deleteAcquisition(databaseDirectory, projectXMLRoot, projectName, int(curNode.get("subjectID")), curNode.get("visitID"), curNode.get("studyID"), curNode.get("episodeID"), int(curNode.get("ID")), dryRun=dryRun, verbose=verbose)
		#if subjectTypePattern != None or subjectPattern != None or visitPattern != None or episodePattern != None or acquisitionPattern != None or dataResourcePattern != None:
	if dryRun == False:
		projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
#def main():

if __name__ == "__main__":
	main()
