#!/usr/bin/python

# this script is used to insert the participants ages at scan time
# this is primarily a migration script

from lxml import etree
import sys
import os
import getopt
import DataTransferConstants
import DataTransferPatterns
import re

import datetime

def usage():
	print "Usage: " + sys.argv[0] + " [--database-directory=<>]"

	print 
	print "For each visit, inserts the age of the subject at scan time at the visit level"
	print "This is a migration script"
	
def main():
	opts, args = getopt.getopt(sys.argv[1:], "h", ["database-directory="])

	# options to implement --3d-only and --4d-only --images-only --non-images-only
	projectName = None
	databaseDirectory = None
	subjectTypePattern = None
	subjectPattern = None
	visitPattern = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-h':
			usage()
			quit()
	#for o, a in opts:
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)

	for projectName in projectList:
		# read in project XML file
		projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
		projectXMLRoot = projectXML.getroot()
		
		subjectNodes = projectXML.xpath("/XCEDE/subject")

		subjectDOB = {}

		for curSubjectNode in subjectNodes:
			for curSubjectInfoNode in curSubjectNode.iterchildren(tag = "subjectInfo"):
				for DOBNode in curSubjectInfoNode.iterchildren(tag = "birthdate"):
					subjectDOB[curSubjectNode.get("ID")] = DOBNode.text
		
		visitNodes = projectXML.xpath("/XCEDE/visit")

		for curVisitNode in visitNodes:
			ageNodeFound = False
			for ageNodes in curVisitNode.iterchildren(tag = "subjectAgeAtVisit"):
				ageNodeFound = True
				
			if ageNodeFound == False:
				visitDate = curVisitNode.get("ID")
				birthDate = subjectDOB[curVisitNode.get('subjectID')]

				mat = re.match('(\d\d\d\d)(\d\d)(\d\d)', visitDate)
				visitYear = int(mat.group(1))
				visitMonth = int(mat.group(2))
				visitDay = int(mat.group(3))

				mat = re.match('(\d\d\d\d)(\d\d)(\d\d)', birthDate)
				birthYear = int(mat.group(1))
				birthMonth = int(mat.group(2)) 
				birthDay = int(mat.group(3))
				
				date1 = datetime.date(birthYear, birthMonth, birthDay)
				date2 = datetime.date(visitYear, visitMonth, visitDay)

				dateDiff = date2 - date1
				ageAtScanNode = etree.SubElement(curVisitNode, 'subjectAgeAtVisit')
				ageAtScanNode.text = str(dateDiff.days)
				print projectName + "\t" + curVisitNode.get('subjectID') + "\t" + curVisitNode.get("ID") + "\t" + subjectDOB[curVisitNode.get('subjectID')] + "\t" + str(dateDiff.days)#years) + "\t" + str(months) + "\t" + str(days)
		#for curVisitNode in visitNodes:
		projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
	#for projectName in projectList:
#def main():


if __name__ == "__main__":
	main()
