#!/usr/bin/python -u

from lxml import etree

import sys
import os
import errno
import getopt
import hashlib

import re
#from Crypto.Util import randpool
import DataTransferConstants
#from Crypto.PublicKey import RSA
class contributor:
	role = ""
	salutation = ""
	givenName = ""
	middleName = ""
	surname = ""
	institution = ""
	academicTitles = ""
	department = ""
	def getID(self):
		#return hashlib.sha256(self.givenName + self.middleName + self.surname).hexdigest()
		return (self.givenName + " " + self.middleName + " " + self.surname)
	def pushOntoContributorList(self, contributorListRoot):
		# push this contributor onto the given contributorList
		selfRoot = etree.SubElement(contributorListRoot, "contributor", ID=self.getID(), role=self.role)
		curElement = etree.SubElement(selfRoot, "salutation")
		curElement.text = self.salutation
		curElement = etree.SubElement(selfRoot, "givenName")
		curElement.text = self.givenName
		curElement = etree.SubElement(selfRoot, "middleName")
		curElement.text = self.middleName
		curElement = etree.SubElement(selfRoot, "surname")
		curElement.text = self.surname
		curElement = etree.SubElement(selfRoot, "institution")
		curElement.text = self.institution
		curElement = etree.SubElement(selfRoot, "academicTitles")
		curElement.text = self.academicTitles
		curElement = etree.SubElement(selfRoot, "department")
		curElement.text = self.department
		return

def main(): 
	# Memory debug specific
	#libxml2.debugMemory(1)

	# this file adds investigator(s) to the project xml file
	# the investigators are given in a text file that must be provided in the program arguments
	# the investigator ID will be generated as the SHA256 hash of the investigator's name
	
	opts, args = getopt.getopt(sys.argv[1:], "p:f:ade", ['database-directory='])
	
	# execution mode is add, edit or delete
	executionMode = None
	projectName = None
	databaseDirectory = None
	contributorListFileName = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-f':
			contributorListFileName = a
		if o == '-a':
			executionMode = "Add"
		if o == '-d':
			executionMode = "Delete"
		if o == '-e':
			executionMode = "Edit"
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)

	if projectName == None or contributorListFileName == None:
		print "Usage: " + sys.argv[0] + " -p <project name> -f <contributor list filename> [--database-directory=<>]"
		quit()
	projectName = projectName.strip('/')
	
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory was not found"
		quit()
	elif executionMode == None:
		print "Execution mode was not set, must use -a -d or -e"
		quit()
	else:
		if os.path.isfile(contributorListFileName):
			# parse the project xml file
			xmltree = etree.parse(os.path.join(databaseDirectory, projectName, 'projects.xml'))
				
			# find the contributorList root element
			contributorListRoot = xmltree.xpath("/XCEDE/project[@ID='" + projectName + "']/contributorList")
			if len(contributorListRoot) == 0:
				projectRoot = xmltree.xpath("/XCEDE/project[@ID='" + projectName + "']")
				if len(projectRoot) == 1:
					contributorListRoot = etree.SubElement(projectRoot[0], "contributorList")
			elif len(contributorListRoot) == 1:
				contributorListRoot = contributorListRoot[0]
			else:
				print "More than one contributorList in the XML file, using first one"
				contributorListRoot = contributorListRoot[0]
			#if len(contributorListRoot) == 0:
			# parse the contributor file

			# flow control
			# initialise curContributor = Null
			# for each line in file
			# 	if the line is "contributor"
			#		if curContributor != Null
			#			set AddContributor = true
			#			reinitialise curContributor
			#		else
			#			initialise curContributor
			#	else perform a regular expression match for ([^:])+<key>: .*$<value>
			#		if we have a match
			#			case <key>
			#				'role': curContributor.role = <value>
			#				'salutation': curContributor.salutation = <value>
			#				'Given Name': curContributor.givenName = <value>
			#				'Middle Name': curContributor.middleName = <value>
			#				'Surname': curContributor.surname = <value>
			#				'Institution': curContributor.institution = <value>
			#				'Academic Titles': curContributor.institution = <value>
			#				'Department': curContributor.department = <value>
			# 	read next line in file
			#	if next line is None or AddContributor == True
			#		if executionMode = "Add"
			#			if curContributor does not already exist in XML
			#				push curContributor onto curContributorList with ID = SHA-256 hash of firstName + middleName + surname
			#			else
			#				warning "Not adding, already there"
			#		else if executionMode = "Edit"
			#			if curContributor does not already exist in XML
			#				push curContributor onto curContributorList with ID = SHA-256 hash of firstName + middleName + surname
			#			else
			#				curElement = find contributor element based on ID of curCcontributor
			#				for each field in curContributor
			#					if field is not empty
			#						replace child's text with value in field
			
			contributorFile = open(contributorListFileName, 'r')
			
			line = contributorFile.readline()
			curContributor = None
			while line:
				# equivalent of chomp
				line = line.rstrip()
				if line == 'contributor':
					if curContributor == None:
						curContributor = contributor()
						addContributor = False
					else:
						addContributor = True
				else:
					pat = re.compile('^([^:]+): (.*)$')
					mat = pat.match(line)
					
					if mat != None:
						if mat.group(1) == 'Role':
							curContributor.role = mat.group(2)
						elif mat.group(1) == 'Salutation':
							curContributor.salutation = mat.group(2)
						elif mat.group(1) == 'Given Name':
							curContributor.givenName = mat.group(2)
						elif mat.group(1) == 'Middle Name':
							curContributor.middleName = mat.group(2)
						elif mat.group(1) == 'Surname':
							curContributor.surname = mat.group(2)
						elif mat.group(1) == 'Institution':
							curContributor.institution = mat.group(2)
						elif mat.group(1) == 'Academic Titles':
							curContributor.academicTitles = mat.group(2)
						elif mat.group(1) == 'Department':
							curContributor.department = mat.group(2)
						else:
							print "Unsupported field: " + mat.group(1)
					addContributor = False
				#if line == 'contributor':
				line = contributorFile.readline()
				if not line or addContributor == True:
					# flush the last contributor to the lis
					curContributorAlreadyInProject = xmltree.xpath("/XCEDE/project[@ID='" + projectName + "']/contributorList/contributor[@ID='" + curContributor.getID() + "']");
					if executionMode == "Add":
						if len(curContributorAlreadyInProject) == 0:
							curContributor.pushOntoContributorList(contributorListRoot)
						else:
							print "Not adding entry, already in project"
						#if len(curContributorAlreadyInProject) == 0:
					#if executionMode == "Add":
					elif executionMode == "Edit":
						if len(curContributorAlreadyInProject) == 1:
							if curContributor.role:
								curContributorAlreadyInProject[0].set('role', curContributor.role)
							for element in curContributorAlreadyInProject[0].iterchildren():
								if element.tag == 'salutation':
									if curContributor.salutation:
										element.text = curContributor.salutation
								elif element.tag == 'institution':
									if curContributor.institution:
										element.text = curContributor.institution
								elif element.tag == 'academicTitles':
									if curContributor.academicTitles:
										element.text = curContributor.academicTitles
								elif element.tag == 'department':
									if curContributor.department:
										element.text = curContributor.department
						else:	
							curContributor.pushOntoContributorList(contributorListRoot)
					#elif executionMode == "Edit":
					elif executionMode == "Delete":
						if len(curContributorAlreadyInProject) == 1:
							contributorListRoot.remove(curContributorAlreadyInProject[0])
						
					curContributor = contributor()
				#if not line
			#while line:
			contributorFile.close()
			xmltree.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
			# open up the contributor list given by the user
		#if not os.path.isfile(contributorListFileName):
	#if not os.path.isfile(projectName + '/projects.xml'):
#def main(): 

if __name__ == "__main__":
    main()

