#!/usr/bin/python

from lxml import etree

import re
import os
import errno
import sys
import getopt
import subprocess
import hashlib

from math import sqrt

import Numeric

import DataTransferConstants
import shutil

class subject:
	name = ''
	address = ''
	DOB = ''
	sex = ''
	species = ''
	visits = {}

	def __init__(self, DOB, sex, species, name, address):
		self.DOB = DOB
		self.sex = sex
		self.species = species
		self.name = name
		self.address = address
		self.visits = {}

class visit:
	series = []
	date = ''
	def __init__(self, hash):
		self.hash = hash
	
class series:
	files = []
	seriesID = ''

def usage():
	print sys.argv[0] + ": checks in non-DICOM, non-GENESIS files into the database"
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --subject-type=<patients or controls> --subject-id=<subject id> --visit-id=<visit> --study-id=<study> --episode-id=<episode> [--file-format=<format>] files..."
	print ""
	print "All the files listed on the command line will be placed into one acquisition node, so they are considered to be grouped together"
	print "If --file-format is not given then the format will be obtained from the UNIX \"file\" command on the first specified file"


##################### database_insert_arbitrary_data.py ###################################
# Inserts non MR data into the database
# automatically generates acquisition numbers but episode and study name have to be entered

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "subject-type=", 'subject-id=', 'visit-id=', 'study-id=', 'episode-id=', 'file-format='])
	
	projectName = None
	databaseDirectory = None
	fileFormat = None
	subjectType = None
	subjectID = None
	visitID = None
	studyID = None
	episodeID = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--file-format':
			fileFormat = a
		if o == '--subject-type':
			subjectType = a
		if o == '--subject-id':
			subjectID = a
		if o == '--visit-id':
			visitID = a
		if o == '--study-id':
			studyID = a
		if o == '--episode-id':
			episodeID = a
		if o == '-h':
			usage()
			quit()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)

	if projectName == None or subjectType == None or subjectID == None or visitID == None or studyID == None or episodeID == None:
		print "Mandatory field not given"
		usage()
		quit()
	projectName = projectName.split('/')

	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "Project does not exist"
		quit()
	
	# look for bad characters in curStudyID
	pat = re.compile('^\w+$')
	mat = pat.match(studyID)
	
	if mat == None:
		print "the study id contains illegal characters, it must not have any spaces in it"
		quit()

	if len(args) == 0:
		print "No files were specified"
		quit()

	for curArg in args:
		if os.path.isfile(curArg) == False:
			print "One of the file names specified on the command line was not found or was not a regular file: " + curArg
			quit()

	# read in project XML file
	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	# firstly look for the subject ID in the subjectList
	projectPatientsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='patients']")
	if len(projectPatientsListRoot) == 0:
		print "No subjects subject group list, should not happen"
		quit()

	projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
	if len(projectControlsListRoot) == 0:
		print "No controls subject group list, should not happen"
		quit()

	if subjectType == 'controls':
		curSubjectListRoot = projectControlsListRoot[0]
	elif subjectType == 'patients':
		curSubjectListRoot = projectPatientsListRoot[0]
	else:
		print "--subject-type must be either \"patients\" or \"controls\""
		quit()
	
# find the subject in the subjectList
	subjectFound = False
	for curElement in curSubjectListRoot.iterchildren():
		if curElement.text == subjectID:
			subjectFound = True

	if subjectFound == False:
		print "The subject specified on the command line was not found in the XML file: " + subjectID
		quit()

#	pseudocode for data insertion procedure
#	if the visit doesnt exist in the database
#		create new visit node
#		create new study node
#		create new episode node
#
#	create new acquisition node
#	create new dataResource node
#
#	create the directories
#	copy the file
#
#	save the XML file
	
	visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@projectID='" + projectName + "' and @subjectID='" + subjectID + "' and @ID='" + visitID + "']")
	studyNodes = projectXMLRoot.xpath("/XCEDE/study[@projectID='" + projectName + "' and @subjectID='" + subjectID + "' and @visitID='" + visitID + "' and @ID='" + studyID + "']")
	episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@projectID='" + projectName + "' and @subjectID='" + subjectID + "' and @visitID='" + visitID + "' and @studyID='" + studyID + "' and @ID='" + episodeID + "']")

	projectURI = projectName
	subjectURI = os.path.join(projectURI, subjectID.zfill(DataTransferConstants.padZerosLength))
	visitURI = os.path.join(subjectURI, visitID)
	studyURI = os.path.join(visitURI, studyID)
	episodeURI = os.path.join(studyURI, episodeID)
	
	if len(visitNodes) == 0:
		visitNode = etree.SubElement(projectXMLRoot, "visit", projectID=projectName, subjectGroupID=subjectType,  subjectID=subjectID, ID=visitID, projectURI=projectURI, subjectURI=subjectURI)
		visitInfoNode = etree.SubElement(visitNode, "visitInfo")
		etree.SubElement(visitInfoNode, "description")
	
	if len(studyNodes) == 0:
		studyNode = etree.SubElement(projectXMLRoot, "study", projectID=projectName, subjectGroupID=subjectType, subjectID=subjectID, visitID=visitID, ID=studyID, projectURI=projectURI, subjectURI=subjectURI, visitURI=visitURI)
		studyInfoNode = etree.SubElement(studyNode, "studyInfo")
		etree.SubElement(studyInfoNode, "description")
		invoiceNode = etree.SubElement(studyNode, "invoiceNumber")
		invoiceNode.text = "Please edit"
		reportReceivedNode = etree.SubElement(studyNode, "reportReceived")
		reportReceivedNode.text = "no"
	
	if len(episodeNodes) == 0:
		episodeNode = etree.SubElement(projectXMLRoot, "episode", projectID=projectName, subjectGroupID=subjectType, subjectID=subjectID, visitID=visitID, studyID=studyID, ID=episodeID, projectURI=projectURI, subjectURI=subjectURI, visitURI=visitURI, studyURI=studyURI)
		episodeInfoNode = etree.SubElement(episodeNode, "episodeInfo")
		etree.SubElement(episodeInfoNode, "description")
		
		FollowupNode = etree.SubElement(episodeNode, 'followUpRequired')
		FollowupNode.text = "Please edit"
		FollowupCompletedNode = etree.SubElement(episodeNode, 'followUpCompleted')
		FollowupCompletedNode.text = "Please edit"

	acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@projectID='" + projectName + "' and @subjectID='" + subjectID + "' and @visitID='" + visitID + "' and @episodeID='" + episodeID + "']")
	
	nextAcquisitionID = 1
	for curAcquisitionNode in acquisitionNodes:
		if nextAcquisitionID <= int(curAcquisitionNode.get('ID')):
			nextAcquisitionID = int(curAcquisitionNode.get('ID')) + 1
	
	acquisitionID = str(nextAcquisitionID)
	acquisitionURI = os.path.join(episodeURI, acquisitionID.zfill(DataTransferConstants.padZerosLength))

	acquisitionNode = etree.SubElement(projectXMLRoot, "acquisition", projectID=projectName, subjectGroupID=subjectType, subjectID=subjectID, visitID=visitID, studyID=studyID, episodeID=episodeID, ID=acquisitionID, projectURI=projectURI, subjectURI=subjectURI, visitURI=visitURI, studyURI=studyURI, episodeURI=episodeURI)
	acquisitionInfoNode = etree.SubElement(acquisitionNode, "acquisitionInfo")
	etree.SubElement(acquisitionInfoNode, "description")

# now add the dataResource node
# if the file format is not given, use the UNIX "file" command to obtain the file format
# use the format of the first file given
	if fileFormat == None:
		FILEPipe = os.popen("file -b " + args[0], 'r')
		fileFormatString = FILEPipe.readline()
		FILEPipe.close()
		fileFormatString = fileFormatString.rstrip()
	else:
		fileFormatString = fileFormat

	dataResourceNode = etree.SubElement(projectXMLRoot, "DataResource", projectID=projectName, subjectGroupID=subjectType, subjectID=subjectID, visitID=visitID, studyID=studyID, episodeID=episodeID, acquisitionID=acquisitionID, ID="1", projectURI=projectURI, subjectURI=subjectURI, visitURI=visitURI, studyURI=studyURI, episodeURI=episodeURI, format=fileFormatString, type="binaryDataResource_t")
	elementTypeNode = etree.SubElement(dataResourceNode, 'elementType')
	elementTypeNode.text = 'uint8'
	byteOrderNode = etree.SubElement(dataResourceNode, 'byteOrder')
# assuming little-endian since this machine is little-endian
	byteOrderNode.text = 'lsbfirst'
	for curArg in args:
		(head, tail) = os.path.split(curArg)
		(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(curArg)

		URINode = etree.SubElement(dataResourceNode, 'uri', size=str(st_size), offset='0')
		URINode.text = tail

	# create the acquisition directories
	try:
		os.makedirs(acquisitionURI)
	except os.error, e:
		if e.errno == errno.EEXIST:
			print "directory " + acquisitionURI + " already exists, overwriting files underneath"
		if e.errno != errno.EEXIST:
			raise
	
	for curArg in args:
		(head, tail) = os.path.split(curArg)
		shutil.copyfile(curArg, os.path.join(databaseDirectory, acquisitionURI, tail))

	projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
#def main():

if __name__ == "__main__":
	main()
