#!/usr/bin/python

from lxml import etree

import re
import os
import errno
import sys
import getopt
import subprocess
import hashlib
import math
import tempfile
import errno
from struct import unpack
from StringIO import StringIO

import dicom

import DataTransferConstants
from DataTransferProject import *
import DataTransferUserFields


class subject:
	name = ''
	address = ''
	DOB = ''
	sex = ''
	species = ''
	nameEnc = ''
	addressEnc = ''
	
	visits = {}

	def __init__(self, DOB = None, sex = None, species = None, name = None, address = None, nameEnc = None, addressEnc = None):
		self.DOB = DOB
		self.sex = sex
		self.species = species
		self.name = name
		self.address = address
		self.nameEnc = nameEnc
		self.addressEnc = addressEnc

		if self.address == None:
			self.address = ''
		self.visits = {}

class visit:
	series = []
	date = ''
	def __init__(self, hash):
		self.hash = hash
	
class series:
	files = []
	seriesID = ''

#class DICOMKeys
#	def __init__(self):
#		pass

################## SPECIAL FIELDS AS REQUESTED BY AMANDA ##################################
#Report received
#Follow up required I'm putting this in at the episode level, since it may be related to particular scans
#If yes, f/up completed?
#Invoice number I'm putting this in at the study level
#Chief Investigator already in there because we have contributor info
#Consent for Future Control (yes/no) At patient level, in patientInfo

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> -d <DICOM directory> --subject-type=<patients or controls> --study-id=<study> [--database-directory=<>] [--subject-species=<species>]"
	print
	print "Mandatory arguments:"
	print "-p <project name>: the project name, the project's directory must exist and be created by dt_create_project.py"
	print "-d <DICOM directory>: the root directory of the DICOM files to be imported"
	print "--subject-type=<patients or controls>: in each data import you must select whether the subjects are part of the patients or controls group"
	print "--study-id=<study id>: the study id, this will generally be something like MR_STUDY or CT_STUDY"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "--database-directory=<database directory>: the root directory of where the database is situated"
	print "--subject-species=<species>: the speices of the subject, defaults to \"human\""

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:d:h", ["database-directory=", "subject-type=", 'subject-species=', 'study-id='])

	#DCCP = DataTransferConstants.findCommand("dccp")
	#if DCCP == None:
	#	print "Could not find the command dccp"
	#	quit()
	
	PIGZ = DataTransferConstants.findCommand("pigz")

	if PIGZ == None:
		print "Could not find the command pigz, doing slow serial compression"
	else:
		print "Found pigz, doing fast parallel compression"

	# all of the numeric, incremented IDs will have their corresponding directories padded with zeros to this length

	projectName = None
	databaseDirectory = None
	baseDirectory = None
	subjectType = None
	subjectSpecies = None
	studyID = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '-h':
			usage()
			quit()
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-d':
			baseDirectory = a
		if o == '--subject-type':
			subjectType = a
		if o == '--subject-species':
			subjectSpecies = a
		if o == '--study-id':
			studyID = a
	#for o, a in opts:

	temporaryDirectory = tempfile.mkdtemp()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or baseDirectory == None or subjectType == None or studyID == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	projectName = projectName.strip('/')
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory could not be found"
		quit()

	encryptionObject = DataTransferConstants.encryptionModule(projectName = projectName, databaseDirectory = databaseDirectory, publicOrPrivate="public")

	# look for bad characters in curStudyID
	pat = re.compile('^\w+$')
	mat = pat.match(studyID)
	
	if mat == None:
		print "the study name contains illegal characters, it must not have any spaces in it"
		quit()

	if subjectSpecies == None:
		subjectSpecies = "human"

	theProject = Project(projectName = projectName, databaseDirectory = databaseDirectory, subjectType = subjectType, studyID = studyID, species = subjectSpecies)
	theProject.fileReadersGetFiles(baseDirectory)
	#theProject.getSubjectsFromData()
	theProject.fileReadersMakeDataResources()		
	theProject.addDataResourcesToXML() # this makes the data archives as well
	theProject.saveXML()

if __name__ == "__main__":
	main()
