#!/usr/bin/python

''' 
Our original strategy for anonymisation was to place the patient names into the fields themselves, however
this will result in problems with field length restrictions. So this searches through the database and replaces all the encrypted fields in DICOM with blanks and writes the encrypted fields out to a file
Each file foo will have its encrypted fields written out to .foo
'''

# this script is used to print out the participant's ages at scan time
# the output is in tabular form
# <subject id> <birth date> <scan date> <age (years,months,days)> <age (months,days)> <age (days)>

# this is a migration script

import dicom

import sys
import os
import getopt
import DataTransferConstants
import DataTransferPatterns

import tempfile
import shutil

def usage():
	print "Usage: " + sys.argv[0] + " [--database-directory=<>]"

	print 
	print "For each visit, inserts the age of the subject at scan time at the visit level"
	print "This is a migration script"
	
def main():
	opts, args = getopt.getopt(sys.argv[1:], "h", ["database-directory="])
	
	PIGZ = DataTransferConstants.findCommand("pigz")
	if PIGZ == None:
		compressionCommand = 'gzip'
	else:
		compressionCommand = 'pigz'

	# options to implement --3d-only and --4d-only --images-only --non-images-only
	projectName = None
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-h':
			usage()
			quit()
	#for o, a in opts:
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
	
	temporaryDirectory = tempfile.mkdtemp()

	for projectName in projectList:
		# traverse the directories, looking for dicom tarballs
		for root, dirs, files in os.walk(os.path.join(databaseDirectory, projectName)):
			for curFile in files:
				if curFile == "dicomfiles.tar.gz":
					commandString = 'tar xzf "' + root + os.sep + curFile + "\" -C " + temporaryDirectory
					print commandString
					os.system(commandString)
					# we know these guys are dicom files so just get the files in the current directory
					DICOMFiles = dict([(f, None) for f in os.listdir(temporaryDirectory)])
					DICOMAndEncryptedFiles = []
					for curDICOMFile in DICOMFiles.iterkeys():
						#fp = file(os.path.join(temporaryDirectory, curDICOMFile), 'rb')
						#fp.seek(128, os.SEEK_SET)
						#header = fp.read(4)
						#fp.close()
						#if header == "DICM":
						fp = file(os.path.join(temporaryDirectory, "." + curDICOMFile), 'w')
						#print name
						ds = dicom.filereader.ReadFile(os.path.join(temporaryDirectory, curDICOMFile))
						
						fp.write(ds[0x10,0x10].value + "\n")
						fp.write(ds[0x10,0x1040].value + "\n")
						ds[0x10,0x10].value = ""
						ds[0x10,0x1040].value = ""

						dicom.filewriter.WriteFile(os.path.join(temporaryDirectory, curDICOMFile), ds)
						fp.close()
						DICOMAndEncryptedFiles.append(curDICOMFile)
						DICOMAndEncryptedFiles.append("." + curDICOMFile)
					#for curDICOMFile in DICOMFiles.iterkeys():
					TarListFileName = os.tempnam()
					TarListFile = file(TarListFileName, 'w')
					TarListFile.write("\n".join(["%s" % (v) for v in DICOMAndEncryptedFiles]))
					TarListFile.close()
					commandString = "tar c -C \"" + temporaryDirectory + "\" -T " + TarListFile.name + " | " + compressionCommand + " -9 > \"" + root + os.sep + curFile + "\""
					print commandString
					os.system(commandString)
					for curDICOMFile in DICOMAndEncryptedFiles:
						os.remove(os.path.join(temporaryDirectory, curDICOMFile))
					os.remove(TarListFile.name)
					#shutil.rmtree(temporaryDirectory)
				#if curFile == "dicomfiles.tar.gz":
			#for curFile in files:
		#for root, dirs, files in os.walk(os.path.join(databaseDirectory, projectName)):
	#for projectName in projectList:
#def main():


if __name__ == "__main__":
	main()
