#!/usr/bin/env python

import getopt
import sys
import os
import stat

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import DataTransferConstants
import pwd
import grp

class InfoModel:

	""" The model class holds the information we want to display """
	def __init__(self, databaseDirectory=None):
		""" Sets up and populates our gtk.ListStore """
		""" The format of the widget is dictated by the type of store we have, since this will be a liststore it will give us a tabular output """ 
		self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN)
		self.protocols = None

		projectList = DataTransferConstants.projectList(databaseDirectory)

		processUID = os.getuid()
		userIDStats = pwd.getpwuid(processUID)
		for curProject in projectList:
			(st_mode, st_ino, st_dev, st_nlink, st_uid, st_gid, st_size, st_atime, st_mtime, st_ctime) = os.stat(os.path.join(databaseDirectory, curProject))
			groupIDStats = grp.getgrgid(st_gid)
			
			if processUID == st_uid or processUID == 0: # second clause: if I am the root user
				self.listStore.append([curProject, userIDStats[0], groupIDStats[0], stat.S_IRGRP & st_mode, stat.S_IROTH & st_mode and stat.S_IXOTH & st_mode])
				#self.columnTitles = ["Project", "Owner", "Group", "Group: write (allow changes)", "Everyone: read (allow export)"]
				# the user owns this project directory, add it to the list and make it editable

			
	def __str__(self):
		return str(self.userFields)

	def get_model(self):
		""" Returns the model """
		if self.listStore:
			return self.listStore
		else:
			return None

class UIManagerExample:

	WIDTH = 800
	HEIGHT = 600
	def layout_resize(self, widget, event):
		x, y, width, height = widget.get_allocation()
		if width > self.lwidth or height > self.lheight:
			self.lwidth = max(width, self.lwidth)
			self.lheight = max(height, self.lheight)
			widget.set_size(self.lwidth, self.lheight)

	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="ActivatePermissions"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="ActivatePermissions"/>
	</toolbar>
	</ui>'''
	def __init__(self):
		# Create the toplevel window
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBIdb: standard (no ACL) permission manager")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

		# A gtk.VBox is a container that allows you to insert objects
		# vertically
		vbox = gtk.VBox()
		window.add(vbox)
		
		# Create a UIManager instance
		uimanager = gtk.UIManager()

		# Add the accelerator group to the toplevel window
		accelgroup = uimanager.get_accel_group()
		window.add_accel_group(accelgroup)

		# Create an ActionGroup
		self.actiongroup = gtk.ActionGroup('UIManagerExample')

		# Create actions
		#(action, ?, menu test, ?, tooltiptext, callback)
		#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
		self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
					('ActivatePermissions', None, '_Apply permissions', None, 'Apply permissions to files and directories', self.apply_permissions_cb),
					('File', None, '_File')])
		self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

		# Add the actiongroup to the uimanager
		uimanager.insert_action_group(self.actiongroup, 0)

		# Add a UI description
		uimanager.add_ui_from_string(self.ui)

		# Create a MenuBar
		menubar = uimanager.get_widget('/MenuBar')
		vbox.pack_start(menubar, False)

		# Create a Toolbar
		toolbar = uimanager.get_widget('/Toolbar')
		vbox.pack_start(toolbar, False)
        	
		self.mdl = Store.get_model()
		self.view = Display.make_view( self.mdl )
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		tree_selection = self.view.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		#tree_selection.select_path(0)	
		#tree_selection.set_select_function(self.selectionChanged, full=False, data=None)
		tree_selection.connect("changed", self.selectionChanged)
		scrolled_window.add(self.view)
		vbox.pack_start(scrolled_window, True)
		window.connect("delete-event", self.window_deleting_cb)
	
		window.show_all()
		return
	
	def open_field_files_cb(self, w):
		pass
	
	def save_field_files_cb(self, w):
		Store.commit()
		pass

	def insert_field_cb(self, w):
		addFieldWindow.showWindow()
	
	def edit_field_cb(self, path):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getFieldFromPath(path)

		addFieldWindow.showWindow(level = selectedEntry['level'], name = selectedEntry['name'])
		return True

	def delete_field_cb(self, w):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getFieldFromPath(path)

		# if it is a database-wide field, then we have to queue up deletion for all projects
		if Store.projectName == None:
			Store.deleteFieldDatabase(level=selectedEntry['level'], name=selectedEntry['name'], path=path)
		else:
			Store.deleteFieldProject(level=selectedEntry['level'], name=selectedEntry['name'], path=path)
		print Store

	def migrate_field_cb(self):
		pass
	
	def selectionChanged(self, Selection):
		# see if this field is in the current project's field list, if it is, then enable the migrate option
		#election = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		if parentIter != None:
			path = Model.get_path(parentIter)
			selectedEntry = Store.getFieldFromPath(path)
			curScope = Store.fieldScope(level = selectedEntry['level'], name = selectedEntry['name'])
			if Store.readOnly == False:
				if Store.projectName == None and curScope == 'project':
					self.actiongroup.get_action("EditField").set_sensitive(False)
					self.actiongroup.get_action("DeleteField").set_sensitive(False)
				elif Store.projectName != None and curScope == 'project':
					self.actiongroup.get_action("EditField").set_sensitive(True)
					self.actiongroup.get_action("DeleteField").set_sensitive(True)
				elif Store.projectName == None and curScope == 'database':
					self.actiongroup.get_action("EditField").set_sensitive(True)
					self.actiongroup.get_action("DeleteField").set_sensitive(True)
				elif Store.projectName != None and curScope == 'database':
					self.actiongroup.get_action("EditField").set_sensitive(False)
					self.actiongroup.get_action("DeleteField").set_sensitive(False)

		return True
	
	def ask_and_save(self):
		if Store.needToCommit == True:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_YES_NO, message_format="The fields have been modified. Do you wish to save the changes?")
			response = dialog.run()
			dialog.destroy()
			if response == gtk.RESPONSE_YES:
				Store.commit()
				return False
			else:
				return True

	def quit_cb(self, b):
		self.ask_and_save()
		gtk.main_quit()

	def apply_permissions_cb(self, b):
		pass

	def window_deleting_cb(self, widget, event, data=None):
		response = self.ask_and_save()
		return response


# This contains the TreeView widget and handles callbacks to do with the tree view
class DisplayModel:
	""" Displays the Info_Model model in a view """
	def make_view( self, model ):
		""" Form a view for the Tree Model """
		self.view = gtk.TreeView( model )

		self.renderers = []
		self.treeViewColumns = []
		self.columnTitles = ["Project", 	"Owner", 	"Group", 	"Group: write (allow changes)", 	"Everyone: read (allow export)"]
		self.columnTypes = 	['text',		"text",		"text",		"toggle",							"toggle"]

		for i in range(len(self.columnTitles)):
			if self.columnTypes[i] == 'text':
				curRenderer = gtk.CellRendererText()
				curRenderer.set_property('editable', False)
				self.renderers.append(curRenderer)
				self.treeViewColumns.append(gtk.TreeViewColumn(self.columnTitles[i], self.renderers[i], text=i))
			elif self.columnTypes[i] == 'toggle':
				curRenderer = gtk.CellRendererToggle()
				curRenderer.set_property('activatable', True)
				self.renderers.append(curRenderer)
				self.treeViewColumns.append(gtk.TreeViewColumn(self.columnTitles[i], self.renderers[i]))


		for i in range(len(self.renderers)):
			if self.columnTypes[i] == 'toggle':
				self.treeViewColumns[i].add_attribute(self.renderers[i], "active", i)
			self.view.append_column(self.treeViewColumns[i])
		self.view.set_rules_hint(True)

		return self.view


if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "", ['database-directory='])
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
	#for o, a in opts:

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	projectList = DataTransferConstants.projectList(databaseDirectory)

	Store = InfoModel(databaseDirectory = databaseDirectory)	
	Display = DisplayModel()
	UIExample = UIManagerExample()
	gtk.main()
