#!/usr/bin/env python
import getopt
import sys
import os

from lxml import etree

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import DataTransferConstants

# this class allows a path element as added to the TreeView to know what subjectGroup, subjectIdx, visitIdx, studyIdx, episodeIdx, acqusitionIdx it belongs to
class reversePathLookup:
	# isEditable means that the field's value is editable
	# isDescription is true if the field is part of the visitInfo, episodeInfo, acqusitionInfo, description fields, these should always be in the XML files, even if empty
	# isAppendable means that it can have children
	# isDeletable means that the element can be deleted
	def __init__(self, subjectGroup="", subjectIdx = -1, visitIdx = -1, studyIdx = -1, episodeIdx = -1, acquisitionIdx = -1, dataResourceIdx = -1, XMLElement = None, XMLAppendElement = None, isEditable = False, isContributorRoot = False, isAppendable = False, isDeletable = False):
		self.subjectGroup = subjectGroup
		self.subjectIdx = subjectIdx
		self.visitIdx = visitIdx
		self.studyIdx = studyIdx
		self.episodeIdx = episodeIdx
		self.acquisitionIdx = acquisitionIdx
		self.dataResourceIdx = dataResourceIdx
		# this is for convenience, it allows us to easily set/change an element in the XML tree based on the rows in the table
		self.XMLElement = XMLElement
		# the append element is the element that we are going to be appending other elements to
		# it is different for
#		subject
#			subjectInfo
#				arbitrary
#		visit
#			arbitrary
#			visitInfo
#				timestamp
#				description
#		study
#			arbitrary
#			studyInfo
#				description
#		episode
#			arbitrary
#			episodeInfo
#				description
#		acquisition
#			arbitrary
#			acquisitionInfo
#				description
#		dataResource
#			not appendable only provenance

		self.XMLAppendElement = XMLAppendElement
		self.isEditable = isEditable
		self.isContributorRoot = isContributorRoot
		self.isAppendable = isAppendable
		self.isDeletable = isDeletable

	def __str__(self):
		return "[subjectGroup: " + self.subjectGroup + ", subjectIdx: " + str(self.subjectIdx) + ", visitIdx: " + str(self.visitIdx) + ", studyIdx: " + str(self.studyIdx) + ", episodeIdx: " + str(self.episodeIdx) + ", acqusitionIdx: " + str(self.acquisitionIdx) + ", dataResourceIdx: " + str(self.dataResourceIdx) + ", isEditable: " + str(self.isEditable) + ", isDescription: " + str(self.isDescription) + ", isAppendable: " + str(self.isAppendable) + ", isDeletable: " + str(self.isDeletable) + "]"

# this holds the information for each scientist working on the project
class Contributor:
	def __init__(self, role="", salutation="", givenName="", middleName="", surname="", institution="", academicTitles="", department="", XMLElement=None, XMLElementSalutation=None, XMLElementGivenName=None, XMLElementMiddleName=None, XMLElementSurname=None, XMLElementInstitution=None, XMLElementAcademicTitles=None, XMLElementDepartment=None, TreeElement=None, TreeElementSalutation=None, TreeElementGivenName=None, TreeElementMiddleName=None, TreeElementSurname=None, TreeElementInstitution=None, TreeElementAcademicTitles=None, TreeElementDepartment=None):
		self.role = role
		self.salutation = salutation
		self.givenName = givenName
		self.middleName = middleName
		self.surname = surname
		self.institution = institution 
		self.academicTitles = academicTitles
		self.department = department
		self.role = role
		self.XMLElement = XMLElement
		self.XMLElementSalutation = XMLElementSalutation
		self.XMLElementGivenName = XMLElementGivenName
		self.XMLElementMiddleName = XMLElementMiddleName
		self.XMLElementSurname = XMLElementSurname
		self.XMLElementInstitution = XMLElementInstitution 
		self.XMLElementAcademicTitles = XMLElementAcademicTitles
		self.XMLElementDepartment = XMLElementDepartment
		self.TreeElement = TreeElement
		self.TreeElementSalutation = TreeElementSalutation
		self.TreeElementGivenName = TreeElementGivenName
		self.TreeElementMiddleName = TreeElementMiddleName
		self.TreeElementSurname = TreeElementSurname
		self.TreeElementInstitution = TreeElementInstitution 
		self.TreeElementAcademicTitles = TreeElementAcademicTitles
		self.TreeElementDepartment = TreeElementDepartment
		
	def getID(self):
		return (self.givenName + " " + self.middleName + " " + self.surname)
		
class subjectGroup:
	def __init__(self, ID):
		self.ID = ID
		self.subjects = []
		self.rootNode = [];
	def setRootNode(self, Node):
		self.rootNode = Node
	
class subject:
	def __init__(self, ID):
		rootNode = []
		self.visits = []
		self.values = {}
		self.valueNodes = {}
		self.ID = ID
	
	def setRootNode(self, Node):
		self.rootNode = Node

class visit:
	def __init__(self, ID):
		self.rootNode = []
		self.studies = []
		self.values = {}
		self.valueNodes = {}
		self.ID = ID
	def setRootNode(self, Node):
		self.rootNode = Node

class study:
	def __init__(self, ID):
		self.rootNode = []
		self.episodes = []
		self.values = {}
		self.valueNodes = {}
		self.ID = ID
	
	def setRootNode(self, Node):
		self.rootNode = Node

class episode:
	def __init__(self, ID):
		self.rootNode = []
		self.acquisitions = []
		self.values = {}
		self.valueNodes = {}
		self.ID = ID
	
	def setRootNode(self, Node):
		self.rootNode = Node

class acquisition:
	def __init__(self, ID):
		self.rootNode = []
		self.values = {}
		self.valueNodes = {}
		self.dataResources = []
		self.ID = ID
	
	def setRootNode(self, Node):
		self.rootNode = Node

class dataResource:
	def __init__(self, ID):
		self.rootNode = []
		self.valueNodes = {}
		self.ID = ID
	
	def setRootNode(self, Node):
		self.rootNode = Node



#class GUI_Controller:
#	
#	def print_hello(self, w, data):
#		print "Hello, World!"
#
#	""" The GUI class is the controller for our application """
#	def __init__(self):
#		# setup the main window
#		#self.root = gtk.Window(type=gtk.WINDOW_TOPLEVEL)
#		self.root = gtk.Dialog()
#		self.root.set_title("CellRenderer Example")
#		self.root.connect("destroy", self.destroy_cb)
#		self.root.set_size_request(640, 480)
#		window = gtk.Window()
#		window.set_border_width(10)
#
#		# the policy is one of POLICY AUTOMATIC, or POLICY_ALWAYS.
#		# POLICY_AUTOMATIC will automatically decide whether you need
#		# scrollbars, whereas POLICY_ALWAYS will always leave the scrollbars
#		# there. The first one is the horizontal scrollbar, the second, the
#		# vertical.
#		#scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
#		menubar = self.get_main_menu(window)
#
#		# The dialog window is created with a vbox packed into it.
#		self.root.vbox.pack_start(scrolled, True, True, 0)
#		self.root.vbox.pack_start(menubar, False, True, 0)
#		scrolled_window.show()
#
#		# Get the model and attach it to the view
#		self.mdl = Store.get_model()
#		self.view = Display.make_view( self.mdl )
#		# Add our view into the main window
#		tree_selection = self.view.get_selection()
#		tree_selection.set_mode(gtk.SELECTION_SINGLE)
#		tree_selection.set_select_function(self.selectCategory, Full=False, data=None)
#		tree_selection.connect("changed", self.selectionChanged) 
#		print "here"
#		scrolled_window.add(self.view)
#
#		self.root.show_all()
#		return

# NOTE 
# When appending to the list store, there are 3 entries
# [first column, second column, True or False depending on whether the element is editable or not]

class InfoModel:

	""" Deallocates the subjectGroups and pathIterators arrays, used for loading a new project """
	#def deallocateArrays(self):
#		print self.subjectGroups
#		for curSubjectGroup in ("controls", "patients"):
#			for curSubjectIdx in range(len(self.subjectGroups[curSubjectGroup].subjects)):
#				for curValue in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values.iterkeys():
#					del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values[curValue]
#				for curValueNode in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes.iterkeys():
#					del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes[curValueNode]
#				del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx]
#				for curVisitIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits)):
#					for curValue in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].values.iterkeys():
#						del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].values[curValue]
#					for curValueNode in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes.iterkeys():
#						del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[curValueNode]
#					del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx]
#					for curEpisodeIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes)):
#						for curValue in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].values.iterkeys():
#							del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].values[curValue]
#						for curValueNode in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].valueNodes.iterkeys():
#							del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].valueNodes[curValueNode]
#						del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx]
#						for curAcquisitionIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions)):
#							for curValue in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].values.iterkeys():
#								del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].values[curValue]
#							for curValueNode in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes.iterkeys():
#								del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curValueNode]
#							del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx]
#							for curDataResourceIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources)):
#								for curValue in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].values.iterkeys():
#									del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].values[curValue]
#								for curValueNode in self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes.iterkeys():
#									del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes[curValueNode]
#								del self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx]
#
							#for curDataResourceIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx])):
						#for curAcquisitionIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].episodes[curEpisodeIdx])):
					#for curEpisodeIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx])):
				#for curVisitIdx in range(len(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx])):
			#for curSubjectIdx in range(len(self.subjectGroups[curSubjectGroup])):
		#for curSubjectGroup in ("controls", "patients"):
		#for curPathIterator in self.pathIterators.iterkeys():
		#	del self.pathIterators[curPathIterator]
		
	#def deallocateArrays():
		
	def loadProject(self, projectName):
		if self.subjectGroups != None:
			#self.deallocateArrays()
			self.tree_store.clear()

		self.contributors = []
		self.subjectGroups = {}
		self.pathIterators = {}
		self.projectName = projectName
		print os.path.join(databaseDirectory, projectName, 'projects.xml')
		self.projectXML = etree.parse(os.path.join(databaseDirectory, projectName, 'projects.xml'))
		self.projectXMLRoot = self.projectXML.getroot()

		self.XMLEdited = False

		self.treeRoot = self.tree_store.append(None, [projectName, "", False])
		self.mdl = self.get_model()	
		
		# find the contributorList root element
		contributorListRoot = self.projectXML.xpath("/XCEDE/project[@ID='" + projectName + "']/contributorList")
		if len(contributorListRoot) > 0:
			contributorListRoot = contributorListRoot[0]
			self.treeContributorRoot = self.tree_store.append(self.treeRoot, ["contributors", "", False])
			self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.treeContributorRoot))] = reversePathLookup(XMLElement=contributorListRoot, isContributorRoot=False)

			for curContributorRoot in contributorListRoot.iterchildren():
				curID = curContributorRoot.get("ID")
				curRole = curContributorRoot.get("role")
				
				curContributorRootTree = self.tree_store.append(self.treeContributorRoot, [curID, "", False])
				self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(curContributorRootTree))] = reversePathLookup(XMLElement=curContributorRoot, isContributorRoot=True)

				for curContributorAttribute in curContributorRoot.iterchildren():
					if curContributorAttribute.tag == "salutation":
						curSalutation = curContributorAttribute.text
						curXMLElementSalutation = curContributorAttribute
						curTreeElementSalutation = self.tree_store.append(curContributorRootTree, ['Salutation', curContributorAttribute.text, True])
						curTreeElementSalutation = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementSalutation))
						self.pathIterators[curTreeElementSalutation] = reversePathLookup(XMLElement=curXMLElementSalutation, isEditable=True)
					elif curContributorAttribute.tag == "givenName":
						curGivenName = curContributorAttribute.text
						curXMLElementGivenName = curContributorAttribute
						curTreeElementGivenName = self.tree_store.append(curContributorRootTree, ['GivenName', curContributorAttribute.text, True])
						curTreeElementGivenName = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementGivenName))				
						self.pathIterators[curTreeElementGivenName] = reversePathLookup(XMLElement=curXMLElementGivenName, isEditable=True)
					elif curContributorAttribute.tag == "middleName":
						curMiddleName = curContributorAttribute.text
						curXMLElementMiddleName = curContributorAttribute
						curTreeElementMiddleName = self.tree_store.append(curContributorRootTree, ['MiddleName', curContributorAttribute.text, True])
						curTreeElementMiddleName = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementMiddleName))				
						self.pathIterators[curTreeElementMiddleName] = reversePathLookup(XMLElement=curXMLElementMiddleName, isEditable=True)
					elif curContributorAttribute.tag == "surname":
						curSurname = curContributorAttribute.text
						curXMLElementSurname = curContributorAttribute
						curTreeElementSurname = self.tree_store.append(curContributorRootTree, ['Surname', curContributorAttribute.text, True])
						curTreeElementSurname = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementSurname))				
						self.pathIterators[curTreeElementSurname] = reversePathLookup(XMLElement=curXMLElementSurname, isEditable=True)
					elif curContributorAttribute.tag == "academicTitles":
						curAcademicTitles = curContributorAttribute.text
						curXMLElementAcademicTitles = curContributorAttribute
						curTreeElementAcademicTitles = self.tree_store.append(curContributorRootTree, ['AcademicTitles', curContributorAttribute.text, True])
						curTreeElementAcademicTitles = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementAcademicTitles))				
						self.pathIterators[curTreeElementAcademicTitles] = reversePathLookup(XMLElement=curXMLElementAcademicTitles, isEditable=True)
					elif curContributorAttribute.tag == "institution":
						curInstitution = curContributorAttribute.text
						curXMLElementInstitution = curContributorAttribute
						curTreeElementInstitution = self.tree_store.append(curContributorRootTree, ['Institution', curContributorAttribute.text, True])
						curTreeElementInstitution = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementInstitution))				
						self.pathIterators[curTreeElementInstitution] = reversePathLookup(XMLElement=curXMLElementInstitution, isEditable=True)
					elif curContributorAttribute.tag == "department":
						curDepartment = curContributorAttribute.text
						curXMLElementDepartment = curContributorAttribute
						curTreeElementDepartment = self.tree_store.append(curContributorRootTree, ['Department', curContributorAttribute.text, True])
						curTreeElementDepartment = gtk.TreeRowReference(self.mdl, self.mdl.get_path(curTreeElementDepartment))				
						self.pathIterators[curTreeElementDepartment] = reversePathLookup(XMLElement=curXMLElementDepartment, isEditable=True)
				#for curContributorAttribute in curContributorRoot.iterchildren():
				# add this contributor to the Store and pathIterators
				self.contributors.append(Contributor(role=curRole, salutation=curSalutation, givenName=curGivenName, middleName=curMiddleName, surname=curSurname, institution=curInstitution, academicTitles=curAcademicTitles, department=curDepartment, XMLElement=curContributorRoot, XMLElementSalutation=curXMLElementSalutation, XMLElementGivenName=curXMLElementGivenName, XMLElementMiddleName=curXMLElementMiddleName, XMLElementSurname=curXMLElementSurname, XMLElementInstitution=curXMLElementInstitution, XMLElementAcademicTitles=curXMLElementAcademicTitles, XMLElementDepartment=curXMLElementDepartment, TreeElement=self.treeContributorRoot, TreeElementSalutation=curTreeElementSalutation, TreeElementGivenName=curTreeElementGivenName, TreeElementMiddleName=curTreeElementMiddleName, TreeElementSurname=curTreeElementSurname, TreeElementInstitution=curTreeElementInstitution, TreeElementAcademicTitles=curTreeElementAcademicTitles, TreeElementDepartment=curTreeElementDepartment))
			#for curContributorRoot in contributorListRoot.iterchildren():
		else:
			contributorListRoot = etree.SubElement(self.projectXMLRoot, 'contributorList')
			self.treeContributorRoot = self.tree_store.append(self.treeRoot, ["contributors", "", False])
			self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.treeContributorRoot))] = reversePathLookup(XMLElement=contributorListRoot, isContributorRoot=False)

		for curSubjectGroup in ("controls", "patients"):
			curSubjectGroupXMLRoot = self.projectXML.xpath("//subjectGroup[@ID='" + curSubjectGroup + "']")
			self.subjectGroups[curSubjectGroup] = subjectGroup(curSubjectGroup)
			self.subjectGroups[curSubjectGroup].setRootNode(self.tree_store.append(self.treeRoot, [curSubjectGroup, "", False]))
			self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].rootNode))] = reversePathLookup(XMLElement=curSubjectGroupXMLRoot[0], subjectGroup=curSubjectGroup)
			subjectList = curSubjectGroupXMLRoot[0].getchildren()
				
			for curSubjectIdx in range(len(subjectList)):
				curSubjectID = subjectList[curSubjectIdx].text
				self.subjectGroups[curSubjectGroup].subjects.append(subject(curSubjectID))
				self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].rootNode, [curSubjectID, "", False]))
				# find this subject in the database
				curSubjectNode = self.projectXMLRoot.xpath("//subject[@ID='" + curSubjectID + "']")
				if len(curSubjectNode) == 1:
					curSubjectInfoNode = curSubjectNode[0].find("subjectInfo")
					if not curSubjectInfoNode == None:
						self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, XMLElement=curSubjectNode[0], XMLAppendElement=curSubjectInfoNode, isAppendable=True)
						for curDetail in curSubjectInfoNode.iterchildren():
							#if curDetail.tag == 'sex' or curDetail.tag == 'hash' or curDetail.tag == 'species' or curDetail.tag == 'birthdate':
							self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values[curDetail.tag] = curDetail.text
							if curDetail.tag == "hash" or curDetail.tag == 'birthdate':
								canEditThisField = False
							else:
								canEditThisField = True

							if curDetail.tag == "hash" or curDetail.tag == 'species' or curDetail.tag == 'birthdate' or curDetail.tag == 'sex':
								canDelete = False
							else:
								canDelete = True
							self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].rootNode, [curDetail.tag, curDetail.text, canEditThisField])
							self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, XMLElement=curDetail, isEditable=canEditThisField, isAppendable=False, isDeletable=canDelete)
						#for curDetail in curSubjectInfoNodes[0].iterchildren():
					#if len(curSubjectInfoNodes) == 1:
					#if not self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values["sex"]:
					#	print "Subject has no sex field"
					#if not self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values['hash']:
					#	print "Subject has no hash field"
					#if not self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values['species']:
					#	print "Subject has no species field"
					#if not self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].values['birthdate']:
					#	print "Subject has no birthdate field"
					# get the visits for the subject
					#print "//visit[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "']"
					self.curVisitNodesXML = self.projectXMLRoot.xpath("//visit[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "']")
					if len(self.curVisitNodesXML) > 0:
						for curVisitIdx in range(len(self.curVisitNodesXML)):
							curVisitID = self.curVisitNodesXML[curVisitIdx].get("ID")
							curVisitInfoNode = self.curVisitNodesXML[curVisitIdx].find("visitInfo")
							self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits.append(visit(curVisitID))
							self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].rootNode, ["Visit: " + curVisitID, "", False]))
							#self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].rootNode, ["visit " + curVisitID, ""])
							self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, XMLElement=self.curVisitNodesXML[curVisitIdx], XMLAppendElement=self.curVisitNodesXML[curVisitIdx], isEditable=False, isAppendable=True, isDeletable=False)
							if not curVisitInfoNode == None:
								for curDetail in curVisitInfoNode.iterchildren():
									# get the description and the date and time
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].values[curDetail.tag] = curDetail.text
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].rootNode, [curDetail.tag, curDetail.text, True])
									self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=False)
								#for curDetail in curVisitInfoNodes[].iterchildren():
							#if len(curVisitInfoNodes) == 1:
							curVisitValueNodes = self.curVisitNodesXML[curVisitIdx].iterchildren()
							for curVisitValue in curVisitValueNodes:
								if curVisitValue.tag != "visitInfo":
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].values[curVisitValue.tag] = curVisitValue.text
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[curVisitValue.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].rootNode, [curVisitValue.tag, curVisitValue.text, True])
									self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[curVisitValue.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, XMLElement=curVisitValue, isEditable=True, isAppendable=False, isDeletable=True)
									
							# iterate through the children
							# get the studies for the visit
							self.curStudyNodesXML = self.projectXMLRoot.xpath("//study[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "' and @visitID='" + curVisitID + "']")
							if len(self.curStudyNodesXML) > 0:
								for curStudyIdx in range(len(self.curStudyNodesXML)):
									curStudyID = self.curStudyNodesXML[curStudyIdx].get("ID")
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies.append(study(curStudyID))
									self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].rootNode, ["Study: " + curStudyID, "", False]))
									self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, XMLElement=self.curStudyNodesXML[curStudyIdx], XMLAppendElement=self.curStudyNodesXML[curStudyIdx], isEditable=False, isAppendable=True, isDeletable=False)
									curStudyInfoNode = self.curStudyNodesXML[curStudyIdx].find("studyInfo")
									if not curStudyInfoNode == None:
										for curDetail in curStudyInfoNode.iterchildren():
											# get the description and the date and time
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].values[curDetail.tag] = curDetail.text
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].rootNode, [curDetail.tag, curDetail.text, True])
											self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=False)
										#for curDetail in curStudyInfoNodes[0].iterchildren():
									#if not curStudyInfoNodes == None:
									for curDetail in self.curStudyNodesXML[curStudyIdx].iterchildren():
										if curDetail.tag != "studyInfo":
											# get the description and the date and time
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].values[curDetail.tag] = curDetail.text
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].rootNode, [curDetail.tag, curDetail.text, True])
											self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=True)
										#for curDetail in curStudyInfoNodes[0].iterchildren():

									self.curEpisodeNodesXML = self.projectXMLRoot.xpath("//episode[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "' and @visitID='" + curVisitID + "' and @studyID='" + curStudyID + "']")
									#print str(len(self.curEpisodeNodesXML))
									# get the episodes for the visit
									if len(self.curEpisodeNodesXML) > 0:
										for curEpisodeIdx in range(len(self.curEpisodeNodesXML)):
											curEpisodeID = self.curEpisodeNodesXML[curEpisodeIdx].get("ID")
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes.append(episode(curEpisodeID))
											self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].rootNode, ["Episode: " + curEpisodeID, "", False]))
											self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, XMLElement=self.curEpisodeNodesXML[curEpisodeIdx], XMLAppendElement=self.curEpisodeNodesXML[curEpisodeIdx], isEditable=False, isAppendable=True, isDeletable=False)
											curEpisodeInfoNode = self.curEpisodeNodesXML[curEpisodeIdx].find("episodeInfo")
											if not curEpisodeInfoNode == None:

												for curDetail in curEpisodeInfoNode.iterchildren():
													# get the description and the date and time
													self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].values[curDetail.tag] = curDetail.text
													self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].rootNode, [curDetail.tag, curDetail.text, True])
													if curDetail.tag == "description":
														canDelete = False
													else:
														canDelete = True

													self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=canDelete)
												#for curDetail in self.curEpisodeNodesXML[curStudyIdx].iterchildren():
											#if not curEpisodeInfoNode == None:
											for curDetail in self.curEpisodeNodesXML[curEpisodeIdx].iterchildren():
												if curDetail.tag != 'episodeInfo':
													# get the description and the date and time
													self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].values[curDetail.tag] = curDetail.text
													self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].rootNode, [curDetail.tag, curDetail.text, True])
													self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=True)
												#if curDetail.tag != 'episodeInfo':
											#for curDetail in self.curEpisodeNodesXML[curEpisodeIdx].iterchildren():
											self.curAcquisitionNodesXML = self.projectXMLRoot.xpath("//acquisition[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "' and @visitID='" + curVisitID + "' and @studyID='" + curStudyID + "' and @episodeID='" + curEpisodeID + "']")
											if len(self.curAcquisitionNodesXML) > 0:
													for curAcquisitionIdx in range(len(self.curAcquisitionNodesXML)):
														curAcquisitionID = self.curAcquisitionNodesXML[curAcquisitionIdx].get("ID")
														self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions.append(acquisition(curAcquisitionID))
														self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].rootNode, ["Acquisition: " + curAcquisitionID, "", False]))
														self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, XMLElement=self.curAcquisitionNodesXML[curAcquisitionIdx], XMLAppendElement=self.curAcquisitionNodesXML[curAcquisitionIdx], isEditable=False, isAppendable=True, isDeletable=False)
														curAcquisitionInfoNode = self.curAcquisitionNodesXML[curAcquisitionIdx].find("acquisitionInfo")
														if not curAcquisitionInfoNode == None:
															for curDetail in curAcquisitionInfoNode.iterchildren():
																# get the description and the date and time
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].values[curDetail.tag] = curDetail.text
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].rootNode, [curDetail.tag, curDetail.text, True])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=False)
															#for curDetail in curAcquisitionInfoNodes[0].iterchildren():
														#if not curAcquisitionInfoNodes == None:
														for curDetail in self.curAcquisitionNodesXML[curAcquisitionIdx].iterchildren():
															if curDetail.tag != "acquisitionInfo":
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].values[curDetail.tag] = curDetail.text
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].rootNode, [curDetail.tag, curDetail.text, True])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curDetail.tag]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, XMLElement=curDetail, isEditable=True, isAppendable=False, isDeletable=False)

														# get the data resources for the acquisition
														self.curDataResourceNodesXML = self.projectXMLRoot.xpath("//dataResource[@subjectID='" + curSubjectID + "' and @projectID='" + projectName + "' and @subjectGroupID='" + curSubjectGroup + "' and @visitID='" + curVisitID + "' and @studyID='" + curStudyID + "' and @episodeID='" + curEpisodeID + "' and @acquisitionID='" + curAcquisitionID + "']")
														for curDataResourceIdx in range(len(self.curDataResourceNodesXML)):
															curDataResourceID = self.curDataResourceNodesXML[curDataResourceIdx].get("ID")
															self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources.append(dataResource(curDataResourceID))
															
															self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].setRootNode(self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].rootNode, ["Data Resource: " + curDataResourceID, "", False]))
															self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, XMLElement=self.curDataResourceNodesXML[curDataResourceIdx], isEditable=False, isAppendable=True, isDeletable=False)
															
															# get the format of the data and append that to the table
															self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["format"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Format", self.curDataResourceNodesXML[curDataResourceIdx].get("format"), False])
															
															self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["format"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, isEditable=False, isAppendable=False, isDeletable=False)
															self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, XMLElement=self.curDataResourceNodesXML[curDataResourceIdx], isEditable=False, isAppendable=True, isDeletable=False)
															
															for curElementType in self.curDataResourceNodesXML[curDataResourceIdx].iterchildren(tag="elementType"):
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["elementType"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Element Type", curElementType.text, False])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["elementType"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, XMLElement=curElementType, isEditable=False, isAppendable=False, isDeletable=False)
															for curElementType in self.curDataResourceNodesXML[curDataResourceIdx].iterchildren(tag="byteOrder"):
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["byteOrder"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Byte Order", curElementType.text, False])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["byteOrder"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, XMLElement=curElementType, isEditable=False, isAppendable=False, isDeletable=False)

															# get the dimensions of the image, if it is an image with known dimensions
															if self.curDataResourceNodesXML[curDataResourceIdx].get('type') == "mappedBinaryDataResource_t":
																imageDimensions = [None] * 4
																imageScales = [None] * 4
																dimensionDirections = [None] * 3
																ZSplit = False
																for curDimension in self.curDataResourceNodesXML[curDataResourceIdx].iterchildren(tag='dimension'):
																	if curDimension.get("label") == "x":
																		dimensionIdx = 0
																	if curDimension.get("label") == "y":
																		dimensionIdx = 1
																	if curDimension.get("label") == "z":
																		if curDimension.get("splitRank") != None:
																			ZSplit = True
																			if not curDimension.get("outputSelect") == None:
																				imageDimensions[2] = len(curDimension.get("outputSelect").split(" "))
																		dimensionIdx = 2
																	if curDimension.get("label") == "t":
																		dimensionIdx = 3
																	for curDimensionAttribute in curDimension.iterchildren():
																		if curDimensionAttribute.tag == "size" and not (dimensionIdx == 2 and ZSplit == True):
																			imageDimensions[dimensionIdx] = int(curDimensionAttribute.text)
																		if curDimensionAttribute.tag == "spacing":
																			imageScales[dimensionIdx] = float(curDimensionAttribute.text)
																		if curDimensionAttribute.tag == "direction" and dimensionIdx != 3:
																			dimensionDirections[dimensionIdx] = "[" + curDimensionAttribute.text + "]"


																#for curDimensionAttribute in curDimension.iterchildren():
																if imageScales[3] == None:
																	imageScales = imageScales[:3]
																	imageDimensions = imageDimensions[:3]

																# for 2D images!
																if imageScales[2] == None:
																	imageScales = imageScales[:2]
																	imageDimensions = imageDimensions[:2]

																#for curDimension in self.curDataResourceNodesXML[curDataResourceIdx].iterchildren(tag='dimension'):
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["dimensions"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Dimensions", " x ".join(str(v) for v in imageDimensions), False])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["dimensions"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, isEditable=False, isAppendable=False, isDeletable=False)
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["scales"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Scales", " x ".join(str(v) for v in imageScales), False])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["scales"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, isEditable=False, isAppendable=False, isDeletable=False)
																self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["dimensionDirections"] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].rootNode, ["Dimension Directions", " x ".join(str(v) for v in dimensionDirections), False])
																self.pathIterators[gtk.TreeRowReference(self.mdl, self.mdl.get_path(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes["dimensionDirections"]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, isEditable=False, isAppendable=False, isDeletable=False)
															#if self.curDataResourceNodesXML[curDataResourceIdx].get('type') == "mappedbinaryDataResource_t":
														#for curDataResourceIdx in range(len(self.curDataResourceNodesXML)):
															#self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[curDetail.tag] = self.tree_store.append(self.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].acquisitions[curAcquisitionIdx].rootNode, [curDetail.tag, curDetail.text])
														# get the imaging parameters, volume size and any relevant parameters from the episode 
													#for curAcquisitionIdx in range(len(self.curAcquisitionNodesXML)):
											#if len(self.curAcquisitionNodesXML) > 0:
										#for curEpisodeIdx in range(len(self.curEpisodeNodesXML)):
									#if len(self.curEpisodeNodesXML) > 0:
								#for curStudyIdx in range(len(self.curStudyNodesXML)):
							#if len(self.curStudyNodesXML) > 0:
						#for curDetail in self.curVisitNodesXML[curVisitIdx].iterchildren():
					#if len(self.curVisitNodesXML) > 0:
				#if len(curSubjectNode) == 1:
			#for curSubject in self.subjectGroups[curSubjectGroup].iterchildren():
		#for curSubjectGroup in ("controls", "patients"):
	
	""" The model class holds the information we want to display """
	def __init__(self, projectName):
		""" Sets up and populates our gtk.TreeStore """
		self.tree_store = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
		self.subjectGroups = None
		self.pathIterators = None
		self.loadProject(projectName)
		
	def get_model(self):
		""" Returns the model """
		if self.tree_store:
			return self.tree_store
		else:
			return None
	def getEntryFromPath(self, path):
		curRowRef = gtk.TreeRowReference(self.mdl, path)
		#if isinstance(path, str):
		#	searchPath = path
		#elif isinstance(path, tuple):
		#	searchPath = ":".join(str(v) for v in path)
		#print "###"
		#print searchPath
		#print "###"
		selectedEntry = None
		for curEntry in self.pathIterators.iterkeys():
			#curPath = ":".join(str(v) for v in curEntry.get_path())
			#if curPath == searchPath:
			if curRowRef.get_path() == curEntry.get_path():
				selectedEntry = curEntry
				break
		return selectedEntry
	
	def getContributorFromPath(self, path):
		if isinstance(path, str):
			searchPath = path
		elif isinstance(path, tuple):
			searchPath = ":".join(str(v) for v in path)
		#print searchPath
		selectedEntry = None
		for curEntry in self.contributors:
			curPath = ":".join(str(v) for v in self.mdl.get_string_from_iter(curEntry.TreeElement))
			if curPath == searchPath:
				#print curPath
				selectedEntry = curEntry
				break
		return selectedEntry


class AddContributorWindow:
	def __init__(self):
		pass
	
	def showWindow(self):
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(5, 3, False)
	   	self.table.show()
		
		self.roleLabel = gtk.Label("Role:")
		self.table.attach(self.roleLabel, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.roleLabel.show()
		
		self.salutationLabel = gtk.Label("Salutation:")
		self.table.attach(self.salutationLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.salutationLabel.show()
		
		self.givenNameLabel = gtk.Label("Given Name:")
		self.table.attach(self.givenNameLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.givenNameLabel.show()
		
		self.middleNameLabel = gtk.Label("Middle Name:")
		self.table.attach(self.middleNameLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.middleNameLabel.show()
		
		self.surnameLabel = gtk.Label("Surname:")
		self.table.attach(self.surnameLabel, 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.surnameLabel.show()
		
		self.academicTitlesLabel = gtk.Label("Academic Titles:")
		self.table.attach(self.academicTitlesLabel, 0, 1, 5, 6, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.academicTitlesLabel.show()
		
		self.institutionLabel = gtk.Label("Institution:")
		self.table.attach(self.institutionLabel, 0, 1, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.institutionLabel.show()

		self.departmentLabel = gtk.Label("Department:")
		self.table.attach(self.departmentLabel, 0, 1, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.departmentLabel.show()

		self.roleEntry = gtk.Entry()
		self.table.attach(self.roleEntry, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.roleEntry.show()
		
		self.salutationEntry = gtk.Entry()
		self.table.attach(self.salutationEntry, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.salutationEntry.show()
		
		self.givenNameEntry = gtk.Entry()
		self.table.attach(self.givenNameEntry, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.givenNameEntry.show()
		
		self.middleNameEntry = gtk.Entry()
		self.table.attach(self.middleNameEntry, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.middleNameEntry.show()
		
		self.surnameEntry = gtk.Entry()
		self.table.attach(self.surnameEntry, 1, 2, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.surnameEntry.show()
		
		self.academicTitlesEntry = gtk.Entry()
		self.table.attach(self.academicTitlesEntry, 1, 2, 5, 6, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.academicTitlesEntry.show()
		
		self.institutionEntry = gtk.Entry()
		self.table.attach(self.institutionEntry, 1, 2, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.institutionEntry.show()

		self.departmentEntry = gtk.Entry()
		self.table.attach(self.departmentEntry, 1, 2, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.departmentEntry.show()

		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 8, 9, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 8, 9, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
	
	def callback(self, widget, data=None):
		if data == "OK":
			self.salutationEntry.get_text()
			self.givenNameEntry.get_text()
			self.middleNameEntry.get_text()
			self.surnameEntry.get_text()
			self.institutionEntry.get_text()
			self.academicTitlesEntry.get_text()
			self.roleEntry.get_text()
			self.departmentEntry.get_text()

			# so we have to have a role, first name and surname
			if self.roleEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Role must be entered")
				dialog.run()
				dialog.destroy()
			elif self.salutationEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Salutation must be entered")
				dialog.run()
				dialog.destroy()
			elif self.givenNameEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Given name must be entered")
				dialog.run()
				dialog.destroy()
			elif self.surnameEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Surname must be entered")
				dialog.run()
				dialog.destroy()
			elif self.institutionEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Insititution must be entered")
				dialog.run()
				dialog.destroy()
			elif self.academicTitlesEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Academic titles must be entered")
				dialog.run()
				dialog.destroy()
			elif self.departmentEntry.get_text() == "":
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Department must be entered")
				dialog.run()
				dialog.destroy()
			else:
				# add to the tree
				# find the contributorList root element
				contributorListRoot = Store.projectXML.xpath("/XCEDE/project[@ID='" + Store.projectName + "']/contributorList")
				contributorListRoot = contributorListRoot[0]
				curContributor = Contributor(role=self.roleEntry.get_text(), salutation=self.salutationEntry.get_text(), givenName=self.givenNameEntry.get_text(), middleName=self.middleNameEntry.get_text(), surname=self.surnameEntry.get_text(), institution=self.institutionEntry.get_text(), academicTitles=self.academicTitlesEntry.get_text(), department=self.departmentEntry.get_text())
				# add to the XML
				
				curContributor.XMLElement = etree.SubElement(contributorListRoot, "contributor")
				curContributor.XMLElement.set("ID", curContributor.getID())
				curContributor.XMLElement.set("role", self.roleEntry.get_text())
				curContributor.TreeElement = Store.tree_store.append(Store.treeContributorRoot, [curContributor.getID(), "", False])
				curTreeElementRoot = curContributor.TreeElement
				curContributor.TreeElement = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curContributor.TreeElement))
				Store.pathIterators[curContributor.TreeElement] = reversePathLookup(XMLElement=curContributor.XMLElement, isEditable=False)

				curContributor.XMLElementGivenName = etree.SubElement(curContributor.XMLElement, "givenName")
				curContributor.XMLElementGivenName.text = self.givenNameEntry.get_text()
				curTreeElementRootGivenName = Store.tree_store.append(curTreeElementRoot, ['Given Name', self.givenNameEntry.get_text(), True])
				curTreeElementRootGivenName = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootGivenName))				
				Store.pathIterators[curTreeElementRootGivenName] = reversePathLookup(XMLElement=curContributor.XMLElementGivenName, isEditable=True)
				
				curContributor.XMLElementMiddleName = etree.SubElement(curContributor.XMLElement, "middleName")
				curContributor.XMLElementMiddleName.text = self.middleNameEntry.get_text()
				curTreeElementRootMiddleName = Store.tree_store.append(curTreeElementRoot, ['Middle Name', self.middleNameEntry.get_text(), True])
				curTreeElementRootMiddleName = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootMiddleName))				
				Store.pathIterators[curTreeElementRootMiddleName] = reversePathLookup(XMLElement=curContributor.XMLElementMiddleName, isEditable=True)
				
				curContributor.XMLElementSurname = etree.SubElement(curContributor.XMLElement, "surname")
				curContributor.XMLElementSurname.text = self.surnameEntry.get_text()
				curTreeElementRootSurname = Store.tree_store.append(curTreeElementRoot, ['Surname', self.surnameEntry.get_text(), True])
				curTreeElementRootSurname = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootSurname))				
				Store.pathIterators[curTreeElementRootSurname] = reversePathLookup(XMLElement=curContributor.XMLElementSurname, isEditable=True)
				
				curContributor.XMLElementAcademicTitles = etree.SubElement(curContributor.XMLElement, "academicTitles")
				curContributor.XMLElementAcademicTitles.text = self.academicTitlesEntry.get_text()
				curTreeElementRootAcademicTitles = Store.tree_store.append(curTreeElementRoot, ['Academic Titles', self.academicTitlesEntry.get_text(), True])
				curTreeElementRootAcademicTitles = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootAcademicTitles))				
				Store.pathIterators[curTreeElementRootAcademicTitles] = reversePathLookup(XMLElement=curContributor.XMLElementAcademicTitles, isEditable=True)
				
				curContributor.XMLElementDepartment = etree.SubElement(curContributor.XMLElement, "department")
				curContributor.XMLElementDepartment.text = self.departmentEntry.get_text()
				curTreeElementRootDepartment = Store.tree_store.append(curTreeElementRoot, ['Department', self.departmentEntry.get_text(), True])
				curTreeElementRootDepartment = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootDepartment))				
				Store.pathIterators[curTreeElementRootDepartment] = reversePathLookup(XMLElement=curContributor.XMLElementDepartment, isEditable=True)
				
				curContributor.XMLElementInstitution = etree.SubElement(curContributor.XMLElement, "institution")
				curContributor.XMLElementInstitution.text = self.institutionEntry.get_text()
				curTreeElementRootInstitution = Store.tree_store.append(curTreeElementRoot, ['Institution', self.institutionEntry.get_text(), True])
				curTreeElementRootInstitution = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curTreeElementRootInstitution))				
				Store.pathIterators[curTreeElementRootInstitution] = reversePathLookup(XMLElement=curContributor.XMLElementInstitution, isEditable=True)
				Store.XMLEdited = True

				#Store.treeContributorRoot
				#curSurname = curContributorAttribute.text
				#curXMLElementSurname = curContributorAttribute

				#commandString = "../datatransfer/trunk/database_insert_data_multiple_subjects_multiple_vists.py -p " + Store.projectName + " -d " + self.directoryEntry.get_text() + " --subject-type=" + subjectTypeText + " --subject-species=" + subjectSpeciesText + " --study-name=" + studyNameText 
				#os.system(commandString)
				#Store.loadProject(Store.projectName)
				self.window.destroy()
			#if self.salutationEntry.get_text() == None:
		#if data == "OK":
		elif data == "Cancel":
			self.window.destroy()
		elif data == "Browse":
			#self.FileWindow = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=None, backend=None)
			# Create a new file selection widget
			self.filew = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK), backend=None)
		
			#self.filew.connect("destroy", self.destroy)
			# Connect the ok_button to file_ok_sel method
			#self.filew.ok_button.connect("clicked", self.file_ok_sel)
		    
			# Connect the cancel_button to destroy the widget
			#self.filew.cancel_button.connect("clicked", lambda w: self.filew.destroy())
		    
			# Lets set the filename, as if this were a save dialog,
			# and we are giving a default filename
			#self.filew.set_filename("penguin.png")
		   	response = self.filew.run()
			if response == gtk.RESPONSE_OK:
				#oldProjectName = projectName
				if os.path.isdir(self.filew.get_filename()):
					self.directoryEntry.set_text(self.filew.get_filename())
					self.filew.destroy()
				else:
					dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The file you selected is not a directory")
					dialog.run()
					dialog.destroy()
			elif response == gtk.RESPONSE_CANCEL:
				print 'Closed, no files selected'
				self.filew.destroy()

class CheckinArbitraryDataWindow:
	def __init__(self):
		pass
	
	def showWindow(self):
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.window.set_title("Checkin arbitrary data")
		self.vbox.show()
		self.table = gtk.Table(10, 3, False)
	   	self.table.show()

		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		
		self.subjectTypeLabel = gtk.Label("Subject Type:")
		self.table.attach(self.subjectTypeLabel, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.subjectTypeLabel.show()
		
		self.subjectIDLabel = gtk.Label("Subject ID:")
		self.table.attach(self.subjectIDLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.subjectIDLabel.show()

		self.visitLabel = gtk.Label("Visit:")
		self.table.attach(self.visitLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.visitLabel.show()

		self.studyNameLabel = gtk.Label("Study:")
		self.table.attach(self.studyNameLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.studyNameLabel.show()
		
		self.episodeNameLabel = gtk.Label("Episode:")
		self.table.attach(self.episodeNameLabel, 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.episodeNameLabel.show()
		
		self.acquisitionNameLabel = gtk.Label("Acquisition:")
		self.table.attach(self.acquisitionNameLabel, 0, 1, 5, 6, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.acquisitionNameLabel.show()

		self.fileLabel = gtk.Label("File:")
		self.table.attach(self.fileLabel, 0, 1, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.fileLabel.show()
		
		self.formatLabel = gtk.Label("format:")
		self.table.attach(self.formatLabel, 0, 1, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.formatLabel.show()
		
		self.subjectTypeCombo = gtk.combo_box_new_text()
		self.subjectTypeCombo.append_text("patients")
		self.subjectTypeCombo.append_text("controls")
		self.subjectTypeCombo.show()
		self.subjectTypeCombo.connect('changed', self.comboChanged, "Subject Type") 
		self.table.attach(self.subjectTypeCombo, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.subjectIDCombo = gtk.combo_box_new_text()
		self.subjectIDCombo.show()
		self.subjectIDCombo.connect('changed', self.comboChanged, "Subject ID") 
		self.table.attach(self.subjectIDCombo, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.visitCombo = gtk.combo_box_new_text()
		self.visitCombo.connect('changed', self.comboChanged, "Visit") 
		self.table.attach(self.visitCombo, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 0, 0)
		self.visitCombo.show()
		
		self.studyCombo = gtk.combo_box_new_text()
		self.studyCombo.connect('changed', self.comboChanged, "Study") 
		self.table.attach(self.studyCombo, 1, 2, 3, 4, gtk.FILL, gtk.FILL, 0, 0)
		self.studyCombo.show()

		self.episodeCombo = gtk.combo_box_new_text()
		self.episodeCombo.connect('changed', self.comboChanged, "Episode") 
		self.table.attach(self.episodeCombo, 1, 2, 4, 5, gtk.FILL, gtk.FILL, 0, 0)
		self.episodeCombo.show()
	
		self.acquisitionCombo = gtk.combo_box_new_text()
		self.acquisitionCombo.connect('changed', self.comboChanged, "Acquisition") 
		self.table.attach(self.acquisitionCombo, 1, 2, 5, 6, gtk.FILL, gtk.FILL, 0, 0)
		self.acquisitionCombo.show()
		
		self.formatEntry = gtk.Entry()
		self.table.attach(self.formatEntry, 1, 2, 6, 7, gtk.FILL, gtk.FILL, 0, 0)
		self.formatEntry.show()
		
		self.fileEntry = gtk.Entry()
		self.table.attach(self.fileEntry, 1, 2, 7, 8, gtk.FILL, gtk.FILL, 0, 0)
		self.fileEntry.show()

		# find out what element is selected and sensitise the entry boxes as appropriate
		if Store.pathIterators[selectedEntry].subjectGroup != "":
			curSubjectGroup = Store.pathIterators[selectedEntry].subjectGroup
			if curSubjectGroup == "patients":
				self.subjectTypeCombo.set_active(0)
			else:
				self.subjectTypeCombo.set_active(1)
			for iterSubjectIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects)):
				self.subjectIDCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[iterSubjectIdx].ID)
			self.subjectTypeCombo.set_sensitive(False)
			#self.subjectTypeCombo.set_button_sensitivity(gtk.SENSITIVITY_OFF)
			if Store.pathIterators[selectedEntry].subjectIdx != -1:
				curSubjectIdx = Store.pathIterators[selectedEntry].subjectIdx
				#self.subjectIDCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].ID)
				for iterVisitIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits)):
						self.visitCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[iterVisitIdx].ID)
				self.subjectIDCombo.set_active(curSubjectIdx)
				self.subjectIDCombo.set_sensitive(False)
				if Store.pathIterators[selectedEntry].visitIdx != -1:
					curVisitIdx = Store.pathIterators[selectedEntry].visitIdx
					self.visitCombo.set_sensitive(False)
					self.visitCombo.set_active(curVisitIdx)
					for iterStudyIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies)):
						self.studyCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[iterStudyIdx].ID)
					#Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID
					if Store.pathIterators[selectedEntry].studyIdx != -1:
						curStudyIdx = Store.pathIterators[selectedEntry].studyIdx
						self.studyCombo.set_sensitive(False)
						self.studyCombo.set_active(curStudyIdx)
						#Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID
						for iterEpisodeIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes)):
							self.episodeCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[iterEpisodeIdx].ID)
						if Store.pathIterators[selectedEntry].episodeIdx != -1:
							curEpisodeIdx = Store.pathIterators[selectedEntry].episodeIdx
							self.episodeCombo.set_sensitive(False)
							self.episodeCombo.set_active(curEpisodeIdx)
							for iterAcquisitionIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions)):
								self.acquisitionCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[iterAcquisitionIdx].ID)
							#Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID
							if Store.pathIterators[selectedEntry].acquisitionIdx != -1:
								curAcquisitionIdx = Store.pathIterators[selectedEntry].acquisitionIdx
								self.acquisitionCombo.set_sensitive(False)
								self.acquisitionCombo.set_active(curAcquisitionIdx)
								#Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID
								#if Store.pathIterators[selectedEntry].dataResourceIdx != -1:
								#	curDataResourceIdx = Store.pathIterators[selectedEntry].dataResourceIdx
								#else: # dataResourceIdx == -1
								#	pass
							else: # acquisitionIdx == -1
								pass
						else: # episodeIdx == -1
							pass
					else: # studyIdx == -1
						pass
				else: # visitIdx == -1
					pass
			else: # subjectIdx == -1
				pass
			# set the subject grup
		else: # subjectGroup == None
			pass

		self.ButtonBrowse = gtk.Button(label="Browse...", stock=gtk.STOCK_OPEN)
		self.table.attach(self.ButtonBrowse, 2, 3, 7, 8, gtk.FILL, gtk.FILL, 0, 0)
		self.ButtonBrowse.show()
		self.ButtonBrowse.connect("clicked", self.callback, "Browse")
	
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
	        self.table.attach(self.ButtonOK, 0, 1, 8, 9, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
	        self.table.attach(self.ButtonCancel, 1, 2, 8, 9, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
	
	def callback(self, widget, data=None):
		if data == "OK":
			subjectTypeText = self.subjectTypeCombo.get_active_text()
			subjectSpeciesText = self.subjectSpeciesEntry.get_text()
			studyNameText = self.studyNameEntry.get_text()
			
			if subjectTypeText == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Subject type must be selected")
				dialog.run()
				dialog.destroy()
			elif self.studyNameEntry.get_text() == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="A studyName must be entered")
				dialog.run()
				dialog.destroy()
			elif self.directoryEntry.get_text() == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="A directory must be entered")
				dialog.run()
				dialog.destroy()
			elif subjectSpeciesText == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK, message_format="Species was not entered, assuming \"Human\"")
				dialog.run()
				dialog.destroy()
				subjectSpeciesText = "Human"

			#commandString = "../datatransfer/trunk/database_insert_data_multiple_subjects_multiple_vists.py -p " + Store.projectName + " -d " + self.directoryEntry.get_text() + " --subject-type=" + subjectTypeText + " --subject-species=" + subjectSpeciesText + " --study-name=" + studyNameText 
			#os.system(commandString)
			#Store.loadProject(Store.projectName)
			self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()
		elif data == "Browse":
			#self.FileWindow = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=None, backend=None)
			# Create a new file selection widget
			self.filew = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_OPEN, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK), backend=None)
		
			#self.filew.connect("destroy", self.destroy)
			# Connect the ok_button to file_ok_sel method
			#self.filew.ok_button.connect("clicked", self.file_ok_sel)
		    
			# Connect the cancel_button to destroy the widget
			#self.filew.cancel_button.connect("clicked", lambda w: self.filew.destroy())
		    
			# Lets set the filename, as if this were a save dialog,
			# and we are giving a default filename
			#self.filew.set_filename("penguin.png")
		   	response = self.filew.run()
			if response == gtk.RESPONSE_OK:
				#oldProjectName = projectName
				if os.path.isfile(self.filew.get_filename()):
					self.fileEntry.set_text(self.filew.get_filename())
					self.filew.destroy()
				else:
					dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The file you selected is not a regular file")
					dialog.run()
					dialog.destroy()
			elif response == gtk.RESPONSE_CANCEL:
				print 'Closed, no files selected'
				self.filew.destroy()

	def comboChanged(self, widget, data):
# clear all the combo boxes lower than the one that has had its selection changed
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		
		curSubjectGroup = self.subjectTypeCombo.get_active_text()
		curSubjectIdx = self.subjectIDCombo.get_active()
		curVisitIdx = self.visitCombo.get_active()
		curStudyIdx = self.studyCombo.get_active()
		curEpisodeIdx = self.episodeCombo.get_active()
		curAcquisitionIdx = self.acquisitionCombo.get_active()
		#print curSubjectGroup + ", " + str(curSubjectIdx) + ", " + str(curVisitIdx) + ", " + str(curStudyIdx) + ", " + str(curEpisodeIdx) + ", " + str(curAcquisitionIdx) + ", data: " + data

		if data == "Subject Type":
			NumNodes = self.subjectTypeCombo.get_model().iter_n_children(None)
			for iterSubjectIdx in range(self.subjectTypeCombo.get_model().iter_n_children(None)):
				self.subjectIDCombo.remove_text(0)
			for iterVisitIdx in range(self.visitCombo.get_model().iter_n_children(None)):
				self.visitCombo.remove_text(0)
			for iterStudyIdx in range(self.studyCombo.get_model().iter_n_children(None)):
				self.studyCombo.remove_text(0)
			for iterEpisodeIdx in range(self.episodeCombo.get_model().iter_n_children(None)):
				self.episodeCombo.remove_text(0)
			for iterAcquisitionIdx in range(self.acquisitionCombo.get_model().iter_n_children(None)):
				self.acquisitionCombo.remove_text(0)
			#self.subjectIDCombo.clear()
			#self.visitCombo.clear()
			#self.studyCombo.clear()
			#self.episodeCombo.clear()
			#self.acquisitionCombo.clear()
			for iterSubjectIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects)):
				self.subjectIDCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[iterSubjectIdx].ID)
		elif data == "Subject ID":
			for iterVisitIdx in range(self.visitCombo.get_model().iter_n_children(None)):
				self.visitCombo.remove_text(0)
			for iterStudyIdx in range(self.studyCombo.get_model().iter_n_children(None)):
				self.studyCombo.remove_text(0)
			for iterEpisodeIdx in range(self.episodeCombo.get_model().iter_n_children(None)):
				self.episodeCombo.remove_text(0)
			for iterAcquisitionIdx in range(self.acquisitionCombo.get_model().iter_n_children(None)):
				self.acquisitionCombo.remove_text(0)
			#self.visitCombo.clear()
			#self.studyCombo.clear()
			#self.episodeCombo.clear()
			#self.acquisitionCombo.clear()
			for iterVisitIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits)):
				self.visitCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[iterVisitIdx].ID)
		elif data == "Visit":
			for iterStudyIdx in range(self.studyCombo.get_model().iter_n_children(None)):
				self.studyCombo.remove_text(0)
			for iterEpisodeIdx in range(self.episodeCombo.get_model().iter_n_children(None)):
				self.episodeCombo.remove_text(0)
			for iterAcquisitionIdx in range(self.acquisitionCombo.get_model().iter_n_children(None)):
				self.acquisitionCombo.remove_text(0)
			#self.studyCombo.clear()
			#self.episodeCombo.clear()
			#self.acquisitionCombo.clear()
			for iterStudyIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies)):
				self.studyCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[iterStudyIdx].ID)
		elif data == "Study":
			for iterEpisodeIdx in range(self.episodeCombo.get_model().iter_n_children(None)):
				self.episodeCombo.remove_text(0)
			for iterAcquisitionIdx in range(self.acquisitionCombo.get_model().iter_n_children(None)):
				self.acquisitionCombo.remove_text(0)
			#self.episodeCombo.clear()
			#self.acquisitionCombo.clear()
			for iterEpisodeIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes)):
				self.episodeCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[iterEpisodeIdx].ID)
		elif data == "Episode":
			for iterAcquisitionIdx in range(self.acquisitionCombo.get_model().iter_n_children(None)):
				self.acquisitionCombo.remove_text(0)
			#self.acquisitionCombo.clear()
			for iterAcquisitionIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions)):
				self.acquisitionCombo.append_text(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[iterAcquisitionIdx].ID)
		else:
			pass
	
class CheckinDataWindow:
	def __init__(self):
		pass

	def showWindow(self):
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.window.set_title("Checkin data")
		self.vbox.show()
		self.table = gtk.Table(5, 3, False)
	   	self.table.show()

		self.subjectTypeLabel = gtk.Label("Subject type:")
		self.table.attach(self.subjectTypeLabel, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.subjectTypeLabel.show()
		
		self.subjectSpeciesLabel = gtk.Label("Subject species:")
		self.table.attach(self.subjectSpeciesLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.subjectSpeciesLabel.show()

		self.studyNameLabel = gtk.Label("Study name:")
		self.table.attach(self.studyNameLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.studyNameLabel.show()

		self.directoryLabel = gtk.Label("Directory:")
		self.table.attach(self.directoryLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.directoryLabel.show()

		self.subjectTypeCombo = gtk.combo_box_new_text()
		self.subjectTypeCombo.append_text("patients")
		self.subjectTypeCombo.append_text("controls")
		self.subjectTypeCombo.show()
		self.table.attach(self.subjectTypeCombo, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.subjectSpeciesEntry = gtk.Entry()
		self.table.attach(self.subjectSpeciesEntry, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 0, 0)
		self.subjectSpeciesEntry.show()

		self.studyNameEntry = gtk.Entry()
		self.table.attach(self.studyNameEntry, 1, 2, 2, 3, gtk.FILL, gtk.FILL, 0, 0)
		self.subjectSpeciesEntry.show()
		
		self.directoryEntry = gtk.Entry()
		self.table.attach(self.directoryEntry, 1, 2, 3, 4, gtk.FILL, gtk.FILL, 0, 0)
		self.directoryEntry.show()
		
		self.ButtonBrowse = gtk.Button(label="Browse...", stock=gtk.STOCK_OPEN)
		self.table.attach(self.ButtonBrowse, 2, 3, 3, 4, gtk.FILL, gtk.FILL, 0, 0)
		self.ButtonBrowse.show()
		self.ButtonBrowse.connect("clicked", self.callback, "Browse")
	
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
	        self.table.attach(self.ButtonOK, 0, 1, 4, 5, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
	        self.table.attach(self.ButtonCancel, 1, 2, 4, 5, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
	
	def callback(self, widget, data=None):
		if data == "OK":
			subjectTypeText = self.subjectTypeCombo.get_active_text()
			subjectSpeciesText = self.subjectSpeciesEntry.get_text()
			studyNameText = self.studyNameEntry.get_text()
			
			if subjectTypeText == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Subject type must be selected")
				dialog.run()
				dialog.destroy()
			elif self.studyNameEntry.get_text() == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="A studyName must be entered")
				dialog.run()
				dialog.destroy()
			elif self.directoryEntry.get_text() == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="A directory must be entered")
				dialog.run()
				dialog.destroy()
			elif subjectSpeciesText == None:
				dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_OK, message_format="Species was not entered, assuming \"Human\"")
				dialog.run()
				dialog.destroy()
				subjectSpeciesText = "Human"

			commandString = "dt_insert_data_dicom.py -p " + Store.projectName + " -d " + self.directoryEntry.get_text() + " --subject-type=" + subjectTypeText + " --subject-species=" + subjectSpeciesText + " --study-name=" + studyNameText 
			os.system(commandString)
			Store.loadProject(Store.projectName)
			self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()
		elif data == "Browse":
			#self.FileWindow = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=None, backend=None)
			# Create a new file selection widget
			self.filew = gtk.FileChooserDialog(title="Choose a project", parent=self.window, action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK), backend=None)
		
			#self.filew.connect("destroy", self.destroy)
			# Connect the ok_button to file_ok_sel method
			#self.filew.ok_button.connect("clicked", self.file_ok_sel)
		    
			# Connect the cancel_button to destroy the widget
			#self.filew.cancel_button.connect("clicked", lambda w: self.filew.destroy())
		    
			# Lets set the filename, as if this were a save dialog,
			# and we are giving a default filename
			#self.filew.set_filename("penguin.png")
		   	response = self.filew.run()
			if response == gtk.RESPONSE_OK:
				#oldProjectName = projectName
				if os.path.isdir(self.filew.get_filename()):
					self.directoryEntry.set_text(self.filew.get_filename())
					self.filew.destroy()
				else:
					dialog = gtk.MessageDialog(parent=self.filew, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The file you selected is not a directory")
					dialog.run()
					dialog.destroy()
			elif response == gtk.RESPONSE_CANCEL:
				print 'Closed, no files selected'
				self.filew.destroy()

class InsertFieldWindow:
	def showWindow(self):
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.window.set_title("Insert field")
		self.vbox.show()
		self.table = gtk.Table(3, 2, False)
	   	self.table.show()
		
		self.fieldNameLabel = gtk.Label("Field Name:")
		self.table.attach(self.fieldNameLabel, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.fieldNameLabel.show()

		self.fieldNameEntry = gtk.Entry()
		self.table.attach(self.fieldNameEntry, 1, 2, 0, 1, gtk.FILL, gtk.FILL, 0, 0)
		self.fieldNameEntry.show()

		self.fieldValueLabel = gtk.Label("Field Value:")
		self.table.attach(self.fieldValueLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.fieldValueLabel.show()

		self.fieldValueEntry = gtk.Entry()
		self.table.attach(self.fieldValueEntry, 1, 2, 1, 2, gtk.FILL, gtk.FILL, 0, 0)
		self.fieldValueEntry.show()

		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 2, 3, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 2, 3, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
			
	def callback(self, widget, data=None):
		if data == "OK":
			if self.fieldValueEntry.get_text() != None and self.fieldNameEntry.get_text() != None:
				# get the current selected value in the table
				Selection = Display.view.get_selection()
				(Model, parentIter) = Selection.get_selected()
				path = Model.get_path(parentIter)
				selectedEntry = Store.getEntryFromPath(path)
				# append the value to the tree store
				# append the value to the subjectGroups
				# find the rootnode to append to in subjectGroups
				
				# append the value to the pathIterators
				curSubjectGroup = Store.pathIterators[selectedEntry].subjectGroup
				if Store.pathIterators[selectedEntry].subjectIdx != -1:
					curSubjectIdx = Store.pathIterators[selectedEntry].subjectIdx
					if Store.pathIterators[selectedEntry].visitIdx != -1:
						curVisitIdx = Store.pathIterators[selectedEntry].visitIdx
						Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID
						if Store.pathIterators[selectedEntry].studyIdx != -1:
							curStudyIdx = Store.pathIterators[selectedEntry].studyIdx
							Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID
							if Store.pathIterators[selectedEntry].episodeIdx != -1:
								curEpisodeIdx = Store.pathIterators[selectedEntry].episodeIdx
								Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID
								if Store.pathIterators[selectedEntry].acquisitionIdx != -1:
									curAcquisitionIdx = Store.pathIterators[selectedEntry].acquisitionIdx
									Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID
									if Store.pathIterators[selectedEntry].dataResourceIdx != -1:
										curDataResourceIdx = Store.pathIterators[selectedEntry].dataResourceIdx
										Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].ID
										Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
										# add this value to the XML
										NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
										NewXMLElement.text = self.fieldValueEntry.get_text()
										Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, dataResourceIdx=curDataResourceIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
										Store.XMLEdited = True
									else: # dataResourceIdx == -1
										Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
										# add this value to the XML
										NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
										NewXMLElement.text(self.fieldValueEntry.get_text())
										Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, acquisitionIdx=curAcquisitionIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
										Store.XMLEdited = True
								else: # acquisitionIdx == -1
									Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
									# add this value to the XML
									NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
									NewXMLElement.text = self.fieldValueEntry.get_text()
									Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, episodeIdx=curEpisodeIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
									Store.XMLEdited = True
							else: # episodeIdx == -1
								Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
								# add this value to the XML
								NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
								NewXMLElement.text = self.fieldValueEntry.get_text()
								Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, studyIdx=curStudyIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
								Store.XMLEdited = True
						else: # studyIdx == -1
							Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
							# add this value to the XML
							NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
							NewXMLElement.text = self.fieldValueEntry.get_text()
							Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, visitIdx=curVisitIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
							Store.XMLEdited = True
					else: # visitIdx == -1
						Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes[self.fieldNameEntry.get_text()] = Store.tree_store.prepend(parentIter, [self.fieldNameEntry.get_text(), self.fieldValueEntry.get_text(), True])
						# add this value to the XML
						NewXMLElement = etree.SubElement(Store.pathIterators[selectedEntry].XMLAppendElement, self.fieldNameEntry.get_text())
						NewXMLElement.text = self.fieldValueEntry.get_text()
						Store.pathIterators[gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].valueNodes[self.fieldNameEntry.get_text()]))] = reversePathLookup(subjectGroup=curSubjectGroup, subjectIdx=curSubjectIdx, XMLElement=NewXMLElement, isEditable=True, isAppendable=False, isDeletable=False)
						Store.XMLEdited = True

#		for selectedEntry in Store.pathIterators.iterkeys():
#			printingString = ""
#			curSubjectGroup = Store.pathIterators[selectedEntry].subjectGroup
#			printingString = printingString + " " + curSubjectGroup

			self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()



class ProjectSelectionWindow:
	def __init__(self):
		pass

	def showWindow(self):
		self.window = gtk.Window()
		self.window.set_title("Select Project")
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(2, 2, False)
	   	self.table.show()
		
		self.directoryCombo = gtk.combo_box_new_text()
		fileList = os.listdir(DataTransferConstants.databaseDirectory)
		for curFile in fileList:
			if os.path.isdir(os.path.join(DataTransferConstants.databaseDirectory, curFile)):
				self.directoryCombo.append_text(curFile)
				
		self.directoryCombo.show()
		self.table.attach(self.directoryCombo, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)

		self.label = gtk.Label("Project Name:")
		self.table.attach(self.label, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
	        self.table.attach(self.ButtonOK, 0, 1, 1, 2, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
	        self.table.attach(self.ButtonCancel, 1, 2, 1, 2, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
	
	def callback(self, widget, data=None):
		if data == "OK":
			if not self.directoryCombo.get_active_text() == None:
				projectName = self.directoryCombo.get_active_text()
				if os.path.isfile(os.path.join(databaseDirectory, projectName, "projects.xml")):
					if Store.XMLEdited == True:
						response = UIExample.ask_to_save_xml()
						if response == gtk.RESPONSE_YES:
							Store.projectXML.write(os.path.join(databaseDirectory, Store.projectName, 'projects.xml'), pretty_print=True)
					Store.loadProject(projectName)
			self.window.destroy()
		elif data == "Cancel":
			self.window.destroy()

		# erase the window

class UIManagerExample:

	WIDTH = 800
	HEIGHT = 600
	def layout_resize(self, widget, event):
		x, y, width, height = widget.get_allocation()
		if width > self.lwidth or height > self.lheight:
			self.lwidth = max(width, self.lwidth)
			self.lheight = max(height, self.lheight)
			widget.set_size(self.lwidth, self.lheight)

	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="OpenProjectXML"/>
			<menuitem action="SaveProjectXML"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
		<menu action="Edit">
			<menuitem action="CheckoutData"/>
			<menuitem action="CheckinData"/>
			<menuitem action="CheckinArbitraryData"/>
			<separator/>
			<menuitem action="AddContributor"/>
			<menuitem action="DeleteContributor"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="OpenProjectXML"/>
		<toolitem action="SaveProjectXML"/>
		<separator/>
		<toolitem action="CheckoutData"/>
		<toolitem action="CheckinData"/>
		<toolitem action="CheckinArbitraryData"/>
		<separator/>
		<toolitem action="AddContributor"/>
		<toolitem action="DeleteContributor"/>
	</toolbar>
	</ui>'''
	def __init__(self):
		# Create the toplevel window
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBI image database")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

		# A gtk.VBox is a container that allows you to insert objects
		# vertically
		vbox = gtk.VBox()
		window.add(vbox)
		
		# Create a UIManager instance
		uimanager = gtk.UIManager()

		# Add the accelerator group to the toplevel window
		accelgroup = uimanager.get_accel_group()
		window.add_accel_group(accelgroup)

		# Create an ActionGroup
		self.actiongroup = gtk.ActionGroup('UIManagerExample')

		# Create actions
		#(action, ?, menu test, ?, tooltiptext, callback)
		#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
		self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
					('OpenProjectXML', gtk.STOCK_OPEN, '_Open Project', None, 'Open a projects XML file', self.open_project_cb),
					('SaveProjectXML', gtk.STOCK_SAVE, '_Save Project', None, 'Save a projects XML file', self.save_project_cb),
					('CheckoutData', gtk.STOCK_NEW, '_Checkout data', None, 'Check out a dataset', self.checkout_data_cb),
					('CheckinData', gtk.STOCK_OPEN, 'C_heckin data', None, 'Check in a dataset', self.checkin_data_cb),
					('CheckinArbitraryData', gtk.STOCK_SAVE, 'Ch_eckin arbitrary data', None, 'Check in an arbitrary data file', self.checkin_arbitrary_data_cb),
					#('InsertField', None, '_Insert field', None, 'Insert a field', self.insert_field_cb),
					#('DeleteField', None, '_Delete field', None, 'Delete a field', self.delete_field_cb),
					('AddContributor', gtk.STOCK_NEW, '_Add contributor', None, 'Add a contributor', self.add_contributor_cb),
					('DeleteContributor', gtk.STOCK_DELETE, '_Delete contributor', None, 'Delete a contributor', self.delete_contributor_cb),
					('File', None, '_File'), ('Edit', None, '_Edit')])
		self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

		self.actiongroup.get_action("CheckinArbitraryData").set_sensitive(False)
		self.actiongroup.get_action("CheckoutData").set_sensitive(False)
		#self.actiongroup.get_action("InsertField").set_sensitive(False)
		#self.actiongroup.get_action("DeleteField").set_sensitive(False)
		self.actiongroup.get_action("DeleteContributor").set_sensitive(False)

		if gtk.pygtk_version >= (2, 16, 0):
			self.actiongroup.get_action('CheckoutData').set_is_important(True)
			self.actiongroup.get_action('CheckinData').set_is_important(True)
			self.actiongroup.get_action('CheckinArbitraryData').set_is_important(True)
			self.actiongroup.get_action('AddContributor').set_is_important(True)
			self.actiongroup.get_action('DeleteContributor').set_is_important(True)
		else:
			self.actiongroup.get_action('CheckoutData').set_property("is-important", True)
			self.actiongroup.get_action('CheckinData').set_property("is-important", True)
			self.actiongroup.get_action('CheckinArbitraryData').set_property("is-important", True)
			self.actiongroup.get_action('AddContributor').set_property("is-important", True)
			self.actiongroup.get_action('DeleteContributor').set_property("is-important", True)


		# Add the actiongroup to the uimanager
		uimanager.insert_action_group(self.actiongroup, 0)

		# Add a UI description
		uimanager.add_ui_from_string(self.ui)

		# Create a MenuBar
		menubar = uimanager.get_widget('/MenuBar')
		vbox.pack_start(menubar, False)

		# Create a Toolbar
		toolbar = uimanager.get_widget('/Toolbar')
		vbox.pack_start(toolbar, False)
        	
		self.mdl = Store.get_model()
		self.view = Display.make_view( self.mdl )
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		tree_selection = self.view.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		tree_selection.set_select_function(self.selectionChanged, full=False, data=None)
		
		scrolled_window.add(self.view)
		#table = gtk.Table(2, 2, False)
	   	#table.show()

		#vScrollbar = gtk.VScrollbar(None)
		#vScrollbar.show()
		#table.attach(vScrollbar, 1, 2, 0, 1, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		#hScrollbar = gtk.HScrollbar(None)
		#hScrollbar.show()
	        #table.attach(hScrollbar, 0, 1, 1, 2, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		#table.attach(self.view, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		#vAdjust = self.view.get_vadjustment()
		#vScrollbar.set_adjustment(vAdjust)
		#hAdjust = self.view.get_hadjustment()
		#hScrollbar.set_adjustment(hAdjust)
		window.connect("delete-event", self.window_deleting_cb)
		# Add our view into the main window
		vbox.pack_start(scrolled_window, True)
	
		window.show_all()
		return

	def ask_to_save_xml(self):
		dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_YES_NO, message_format="The XML file has been modified do you wish to save the changes?")
		response = dialog.run()
		dialog.destroy()
		return response

	def window_deleting_cb(self, widget, event, data=None):
		if Store.XMLEdited == True:
			response = self.ask_to_save_xml()
			if response == gtk.RESPONSE_YES:
				Store.projectXML.write(os.path.join(databaseDirectory, Store.projectName, 'projects.xml'), pretty_print=True)
		return False

	def selectionChanged(self, path, data):
		selectedEntry = Store.getEntryFromPath(path)
		if selectedEntry != None:
			if Store.pathIterators[selectedEntry].isContributorRoot == True:
				self.actiongroup.get_action("DeleteContributor").set_sensitive(True)
			else:
				self.actiongroup.get_action("DeleteContributor").set_sensitive(False)
			# disable check out data if we are not on a data resource tab
			if Store.pathIterators[selectedEntry].dataResourceIdx == -1:
				self.actiongroup.get_action("CheckoutData").set_sensitive(False)
			else:
				self.actiongroup.get_action("CheckoutData").set_sensitive(True)
			#if Store.pathIterators[selectedEntry].isAppendable == True:
			#	self.actiongroup.get_action("InsertField").set_sensitive(True)
			#else:
			#	self.actiongroup.get_action("InsertField").set_sensitive(False)
			#if Store.pathIterators[selectedEntry].isDeletable == True:
			#	self.actiongroup.get_action("DeleteField").set_sensitive(True)
			#else:
			#	self.actiongroup.get_action("DeleteField").set_sensitive(False)
			#if Store.pathIterators[selectedEntry].dataResourceIdx
		else:
			print "Selected Entry not found"
			
		return True

	def add_contributor_cb(self, b):
		#selectedEntry = Store.getEntryFromPath(path)
		addContributorWindow.showWindow()
	
	def delete_contributor_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		
		# delete from XML
		ChildList = list(Store.pathIterators[selectedEntry].XMLElement)
		for curChild in ChildList:
			Store.pathIterators[selectedEntry].XMLElement.remove(curChild)
		CurParent = Store.pathIterators[selectedEntry].XMLElement.getparent()
		CurParent.remove(Store.pathIterators[selectedEntry].XMLElement)
		
		parentIter = Store.mdl.get_iter(path)
		
		# delete from tree
		while True:
			curIter = Store.mdl.iter_children(parentIter)
			if curIter == None:
				break
			else:
				# delete from Store.pathIterators
				curRowRef = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(curIter))
				for i in Store.pathIterators.iterkeys():
					if i.get_path() == curRowRef.get_path():
						del Store.pathIterators[i]
						break
				Store.tree_store.remove(curIter)
		curRowRef = gtk.TreeRowReference(Store.mdl, Store.mdl.get_path(parentIter))
		for i in Store.pathIterators.iterkeys():
			if i.get_path() == curRowRef.get_path():
				del Store.pathIterators[i]
				break
		Store.tree_store.remove(parentIter)

	def insert_field_cb(self, b):
		#selectedEntry = Store.getEntryFromPath(path)
		insertFieldWindow.showWindow()

	
	def delete_field_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		# remove from the store
		Store.tree_store.remove(parentIter)
		# remove from the XML tree
		curParentXML = Store.pathIterators[selectedEntry].XMLElement.getparent()
		curParentXML.remove(Store.pathIterators[selectedEntry].XMLElement)
		# remove from Store.pathIterators
		del Store.pathIterators[selectedEntry]
		self.actiongroup.get_action("DeleteField").set_sensitive(False)
		Store.XMLEdited = True

	def quit_cb(self, b):
		if Store.XMLEdited == True:
			response = self.ask_to_save_xml()
			if response == gtk.RESPONSE_YES:
				Store.projectXML.write(os.path.join(databaseDirectory, Store.projectName, 'projects.xml'), pretty_print=True)
		gtk.main_quit()

	def open_project_cb(self, b):
#		for curSubjectGroup in ["controls", "patients"]:
#			for curSubjectIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects)):
#				print "\tsubject: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects))
#				for curVisitIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits)):
#					print "\t\tvisit: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits))
#					for curStudyIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies)):
#						print "\t\t\tstudy: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies))
#						for curEpisodeIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes)):
#							print "\t\t\t\tepisode: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes))
#							for curAcquisitionIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions)):
#								print "\t\t\t\t\tacquisition: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions))
					#			for curDataResourceIdx in range(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources)):
					#				print "\t\t\t\t\t\tdataResource: " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].ID + str(len(Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources))
		projectSelect.showWindow()

	def save_project_cb(self, b):
		Store.projectXML.write(os.path.join(databaseDirectory, Store.projectName, 'projects.xml'), pretty_print=True)
		Store.XMLEdited = False
	
	def checkout_data_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, Rows) = Selection.get_selected_rows()
		for i in Rows:
			selectedEntry = Store.getEntryFromPath(i)
			
			# so we have to have a dataresource selected to export the data
			if Store.pathIterators[selectedEntry].subjectGroup != None and Store.pathIterators[selectedEntry].subjectIdx != -1 and Store.pathIterators[selectedEntry].visitIdx != -1 and Store.pathIterators[selectedEntry].studyIdx != -1 and Store.pathIterators[selectedEntry].episodeIdx != -1 and Store.pathIterators[selectedEntry].acquisitionIdx != -1 and Store.pathIterators[selectedEntry].dataResourceIdx != -1:
				curSubjectGroup = Store.pathIterators[selectedEntry].subjectGroup
				curSubjectIdx = Store.pathIterators[selectedEntry].subjectIdx
				curSubjectID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].ID
				curVisitIdx = Store.pathIterators[selectedEntry].visitIdx
				curVisitID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID
				curStudyIdx = Store.pathIterators[selectedEntry].studyIdx
				curStudyID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID
				curEpisodeIdx = Store.pathIterators[selectedEntry].episodeIdx
				curEpisodeID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID
				curAcquisitionIdx = Store.pathIterators[selectedEntry].acquisitionIdx
				curAcquisitionID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID
				curDataResourceIdx = Store.pathIterators[selectedEntry].dataResourceIdx
				curDataResourceID = Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].ID
				
				#print curSubjectID + " " + curVisitID + " " + curStudyID + " " + curEpisodeID + " " + curAcquisitionID + " " + curDataResourceID
				# get the format of the data
				dataResourceNode = Store.projectXML.xpath("/XCEDE/dataResource[@ID='" + curDataResourceID + "' and @subjectID='" + curSubjectID + "' and @visitID='" + curVisitID + "' and @episodeID='" + curEpisodeID + "' and @acquisitionID='" + curAcquisitionID + "']")
				if len(dataResourceNode) == 1:
					dataFormat = dataResourceNode[0].get("format")
					if dataFormat == "DICOM":
						# Create a new file selection widget
						self.filew = gtk.FileChooserDialog(title="Select output file", parent=None, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK), backend=None)
						NIFTIfilter = gtk.FileFilter()
						NIFTIfilter.set_name("NIFTI")
						NIFTIfilter.add_pattern("*.nii")
						MINCfilter = gtk.FileFilter()
						MINCfilter.set_name("MINC")
						MINCfilter.add_pattern("*.mnc")
					   	
						self.filew.add_filter(NIFTIfilter)
						self.filew.add_filter(MINCfilter)

						self.filew.run()
						response = self.filew.run()
						if response == gtk.RESPONSE_OK:
							outputFile = self.filew.get_filename()
							outputFormat = self.filew.get_filter().get_name()
							#print outputFile + " " + outputFormat
							commandString = "dt_checkout_volume.py " + " -p " + Store.projectName + " --subject-id=" + curSubjectID + " --visit-id=" + curVisitID + " --study-id=" + curStudyID + " --episode-id=" + curEpisodeID + " --acquisition-id=" + curAcquisitionID + " --data-resource-id=\"" + curDataResourceID + "\" -o " + outputFile + " --output-format=" + outputFormat
							#print commandString
							os.system(commandString)
						elif response == gtk.RESPONSE_CANCEL:
							print 'Closed, no files selected'
						self.filew.destroy()

					elif dataFormat == "GENESIS":
						print "GENESIS FORMAT DATA"
					else:
						self.filew = gtk.FileChooserDialog(title="Select output file", parent=None, action=gtk.FILE_CHOOSER_ACTION_SAVE, buttons=(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OPEN,gtk.RESPONSE_OK), backend=None)
						response = self.filew.run()
						if response == gtk.RESPONSE_OK:
							commandString = "dt_checkout_volume.py -p " + projectName + " --subject-id=" + curSubjectID + " --visit-id=" + curVisitID + " --study-id=" + curStudyID + " --episode-id=" + curEpisodeID + " --acquisition-id=" + curAcquisitionID + " --data-resource-id=" + curDataResourceID + " -o " + self.filew.get_filename()
							print commandString
							os.system(commandString)
						self.filew.destroy()

#			if Store.pathIterators[selectedEntry].subjectIdx != -1:
#				curSubjectIdx = Store.pathIterators[selectedEntry].subjectIdx
#				print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].ID
#				if Store.pathIterators[selectedEntry].visitIdx != -1:
#					curVisitIdx = Store.pathIterators[selectedEntry].visitIdx
#					print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID
#					if Store.pathIterators[selectedEntry].studyIdx != -1:
#						curStudyIdx = Store.pathIterators[selectedEntry].studyIdx
#						print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID
#						if Store.pathIterators[selectedEntry].episodeIdx != -1:
#							curEpisodeIdx = Store.pathIterators[selectedEntry].episodeIdx
#							print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID
#							if Store.pathIterators[selectedEntry].acquisitionIdx != -1:
#								curAcquisitionIdx = Store.pathIterators[selectedEntry].acquisitionIdx
#								print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID
#								if Store.pathIterators[selectedEntry].dataResourceIdx != -1:
#									curDataResourceIdx = Store.pathIterators[selectedEntry].dataResourceIdx
#									print Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].ID
			#print "###"
	
	def file_ok_sel(self, w):
		print "%s" % self.filew.get_filename()


	def checkin_data_cb(self, b):
		CheckinData.showWindow()
#		for selectedEntry in Store.pathIterators.iterkeys():
#			printingString = ""
#			curSubjectGroup = Store.pathIterators[selectedEntry].subjectGroup
#			printingString = printingString + " " + curSubjectGroup
#			if Store.pathIterators[selectedEntry].subjectIdx != -1:
#				curSubjectIdx = Store.pathIterators[selectedEntry].subjectIdx
#				printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].ID
#				if Store.pathIterators[selectedEntry].visitIdx != -1:
#					curVisitIdx = Store.pathIterators[selectedEntry].visitIdx
#					printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].ID
#					if Store.pathIterators[selectedEntry].studyIdx != -1:
#						curStudyIdx = Store.pathIterators[selectedEntry].studyIdx
#						printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].ID
#						if Store.pathIterators[selectedEntry].episodeIdx != -1:
#							curEpisodeIdx = Store.pathIterators[selectedEntry].episodeIdx
#							printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].ID
#							if Store.pathIterators[selectedEntry].acquisitionIdx != -1:
#								curAcquisitionIdx = Store.pathIterators[selectedEntry].acquisitionIdx
#								printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].ID
#								if Store.pathIterators[selectedEntry].dataResourceIdx != -1:
#									curDataResourceIdx = Store.pathIterators[selectedEntry].dataResourceIdx
#									printingString = printingString + " " + Store.subjectGroups[curSubjectGroup].subjects[curSubjectIdx].visits[curVisitIdx].studies[curStudyIdx].episodes[curEpisodeIdx].acquisitions[curAcquisitionIdx].dataResources[curDataResourceIdx].ID
#			print printingString

	def checkin_arbitrary_data_cb(self, b):
		#selectedEntry = Store.getEntryFromPath(path)
		checkinArbitraryDataWindow.showWindow()
	
# This contains the TreeView widget and handles callbacks to do with the tree view
class DisplayModel:
	""" Displays the Info_Model model in a view """
	def make_view( self, model ):
		""" Form a view for the Tree Model """
		self.view = gtk.TreeView( model )
		# setup the text cell renderer and allows these
		# cells to be edited.
		self.renderer = gtk.CellRendererText()
		self.renderer.set_property( 'editable', False )
		self.renderer.connect( 'edited', self.col0_edited_cb, model )
		self.view.connect( 'button-press-event', self.col0_button_press_event_cb, model )

		# The toggle cellrenderer is setup and we allow it to be
		# changed (toggled) by the user.
		self.renderer1 = gtk.CellRendererText()
		self.renderer1.set_property('editable', True)
		self.renderer1.connect( 'edited', self.col1_edited_cb, model )
		self.renderer1.connect( 'editing-started', self.col1_editing_started_cb, model )
		self.renderer1.set_property("cell-background", "cyan")
			
		# Connect column0 of the display with column 0 in our list model
		# The renderer will then display whatever is in column 0 of
		# our model .
		self.column0 = gtk.TreeViewColumn("Name", self.renderer, text=0)
			
		# The columns active state is attached to the second column
		# in the model.  So when the model says True then the button
		# will show as active e.g on.
		self.column1 = gtk.TreeViewColumn("Value", self.renderer1, text=1, cell_background_set=2 )
		self.view.append_column( self.column0 )
		self.view.append_column( self.column1 )
		return self.view

	# the event parameter is a gtk.gdk.Event object
	# it has members that allow you to see what the input was
	# event.type
	#	gtk.gdk.BUTTON_PRESS (single click)
	#	gtk.gdk._2BUTTON_PRESS (double click)
	# for the button press events
	# event.button
	#	1: left mouse button
	#	2: middle mouse button
	#	3: right mouse button
	def col0_button_press_event_cb(self, widget, event, model):
		if widget == self.view:
			if event.type == gtk.gdk.BUTTON_PRESS:
				if event.button == 3:
					Selection = self.view.get_selection()
					(Model, Rows) = Selection.get_selected_rows()
					for i in Rows:
						print ":".join(str(v) for v in i)

			elif event.type == gtk.gdk._2BUTTON_PRESS:
				print "double click"
	def col1_editing_started_cb(self, cellrenderer, editable, path, model):
		"""Called when a text cell is edited.  It puts the new text"""
		"""in the model so that it is displayed properly."""
		#print "Editing started on " + str(cellrenderer) + "\n" + str(editable) + "\n" + str(path) + "\n" + str(model)
		#selectedEntry = Store.getEntryFromPath(path)
		#if selectedEntry != None:
		#	print str(Store.pathIterators[selectedEntry])
		return
	
	def col0_edited_cb( self, cell, path, new_text, model ):
		"""
		Called when a text cell is edited.  It puts the new text
		in the model so that it is displayed properly.
		"""
		print "Change '%s' to '%s'" % (model[path][0], new_text)
		model[path][0] = new_text
		return

	def col1_edited_cb( self, cell, path, new_text, model ):
		"""
		Sets the toggled state on the toggle button to true or false.
		"""
		selectedEntry = Store.getEntryFromPath(path)
		
		if Store.pathIterators[selectedEntry].isEditable == True:
			model[path][1] = new_text
			Store.pathIterators[selectedEntry].XMLElement.text = new_text
			Store.XMLEdited = True
		else:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field that was just modified is read-only, not committing the change")
			dialog.run()
			dialog.destroy()
		
		#print "Toggle '%s' to: %s" % (model[path][0], model[path][1],)
		return

if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "p:", ['database-directory='])
	projectName = None
	databaseDirectory = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
	#for o, a in opts:
	if projectName == None:
		print "Project Name not specified, quitting"
		quit()
	
	projectName = projectName.strip('/')
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if DataTransferConstants.isProjectExist(databaseDirectory, projectName) == False:
		print "project directory was not found"
		quit()

	Store = InfoModel(projectName)	
	Display = DisplayModel()
	projectSelect = ProjectSelectionWindow()
	insertFieldWindow = InsertFieldWindow()
	addContributorWindow = AddContributorWindow()
	CheckinData = CheckinDataWindow()
	checkinArbitraryDataWindow = CheckinArbitraryDataWindow()
	UIExample = UIManagerExample()
	gtk.main()
