#!/usr/bin/env python
import getopt
import sys
import os

import re

from lxml import etree

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import DataTransferConstants

class reversePathLookup:
	# isEditable means that the field's value is editable
	# isProtocolRoot means that the row is the root node at the top of a protocol
	def __init__(self, protocol=None, condition=None, otherData=None, attributeName=""):
		self.protocol = protocol
		self.condition = condition
		self.otherData = otherData
		self.attributeName = attributeName

	def __str__(self):
		return "[protocol: " + self.protocol + ", isEditable: " + str(self.isEditable) + ", isProtocolRoot: " + str(self.isProtocolRoot) + ", isConditionRoot: " + str(self.isConditionRoot) + "]"

# these classes are used to keep track of nodes of attributes in the XML file and the tree
class Protocol:
	def __init__(self, ID="", nodeXML=None, nodeTree=None, conditions=None, otherData=None, protocolTypeNodeTree=None, protocolTypeNodeXML=None, preferredSliceOrientationNodeTree=None, preferredSliceOrientationNodeXML=None, imagingSequenceNodeTree=None, imagingSequenceNodeXML=None, numberOfSlicesNodeTree=None, numberOfSlicesNodeXML=None, sliceThicknessNodeTree=None, sliceThicknessNodeXML=None, TRNodeTree=None, TRNodeXML=None, TENodeTree=None, TENodeXML=None, TINodeTree=None, TINodeXML=None, flipAngleNodeTree=None, flipAngleNodeXML=None, numberOfVolumesNodeTree=None, numberOfVolumesNodeXML=None, acquisitionOrderNodeTree=None, acquisitionOrderNodeXML=None):
		self.ID = ID
		self.nodeTree = nodeTree
		self.nodeXML = nodeXML
		self.protocolTypeNodeTree = protocolTypeNodeTree
		self.protocolTypeNodeXML = protocolTypeNodeXML
		self.preferredSliceOrientationNodeTree = preferredSliceOrientationNodeTree
		self.preferredSliceOrientationNodeXML = preferredSliceOrientationNodeXML
		self.imagingSequenceNodeTree = imagingSequenceNodeTree
		self.imagingSequenceNodeXML = imagingSequenceNodeXML
		self.numberOfSlicesNodeTree = numberOfSlicesNodeTree
		self.numberOfSlicesNodeXML = numberOfSlicesNodeXML
		self.sliceThicknessNodeTree = sliceThicknessNodeTree
		self.sliceThicknessNodeXML = sliceThicknessNodeXML
		self.TRNodeTree = TRNodeTree
		self.TRNodeXML = TRNodeXML
		self.TENodeTree = TENodeTree
		self.TENodeXML = TENodeXML
		self.TINodeTree = TINodeTree
		self.TINodeXML = TINodeXML
		self.flipAngleNodeTree = flipAngleNodeTree
		self.flipAngleNodeXML = flipAngleNodeXML
		self.numberOfVolumesNodeTree = numberOfVolumesNodeTree
		self.numberOfVolumesNodeXML = numberOfVolumesNodeXML
		self.acquisitionOrderNodeTree = acquisitionOrderNodeTree
		self.acquisitionOrderNodeXML = acquisitionOrderNodeXML
		self.conditions = {}
		self.otherData = {}

class Condition:
	def __init__(self, ID="", descriptionNodeXML=None, descriptionNodeTree=None, onsetTimesNodeXML=None, onsetTimesNodeTree=None, durationNodeXML=None, durationNodeTree=None):
		self.ID = ID
		self.descriptionNodeTree = descriptionNodeTree
		self.descriptionNodeXML = descriptionNodeXML
		self.onsetTimesNodeTree = onsetTimesNodeTree
		self.onsetTimesNodeXML = onsetTimesNodeXML
		self.durationNodeTree = durationNodeTree
		self.durationNodeXML = durationNodeXML

class OtherData:
	def __init__(self, ID="", descriptionNodeXML=None, descriptionNodeTree=None, formatNodeXML=None, formatNodeTree=None, acquisitionDeviceNodeXML=None, acquisitionDeviceNodeTree=None):
		self.ID = ID
		self.descriptionNodeTree = descriptionNodeTree
		self.descriptionNodeXML = descriptionNodeXML
		self.formatNodeTree = formatNodeTree
		self.formatNodeXML = formatNodeXML
		self.acquisitionDeviceNodeTree = acquisitionDeviceNodeTree
		self.acquisitionDeviceNodeXML = acquisitionDeviceNodeXML
	
class InfoModel:

	def loadProtocol(self, databaseDirectory):
		
		if os.path.isfile(os.path.join(databaseDirectory, "protocols.xml")) == False:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="There was no XML file found in the project directory")
			dialog.run()
			dialog.destroy()
			quit()
		else:	
			if self.protocols != None:
				#self.deallocateArrays()
				self.tree_store.clear()

			self.protocols = {}
			self.pathIterators = {}
			self.protocolXML = etree.parse(os.path.join(databaseDirectory, "protocols.xml"))
			self.protocolXMLRoot = self.protocolXML.getroot()
		
			self.treeRoot = self.tree_store.append(None, ["protocols", "", False, "", False])
			
			self.curProtocolNodesXML = self.protocolXMLRoot.xpath("//protocol")

			for curProtocolNodeXML in self.curProtocolNodesXML:
				curProtocolID = curProtocolNodeXML.get("ID")
				# so add this protocol root to the tree
				self.protocols[curProtocolID] = Protocol(ID=curProtocolID)
				self.protocols[curProtocolID].nodeTree = self.tree_store.append(self.treeRoot, ["protocol", curProtocolID, True, "", False])
				self.protocols[curProtocolID].nodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].nodeTree))
				self.pathIterators[self.protocols[curProtocolID].nodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="ID")
				self.protocols[curProtocolID].nodeXML = curProtocolNodeXML
				for curProtocolChildNodeXML in curProtocolNodeXML.iterchildren():
					# insert the attribute of the protocol into the tree
					if curProtocolChildNodeXML.tag == "condition":
						curConditionID = curProtocolChildNodeXML.get("ID")
						self.protocols[curProtocolID].conditions[curConditionID] = Condition(ID=curConditionID)
						self.protocols[curProtocolID].conditions[curConditionID].nodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), ["condition", curConditionID, True, "", False])
						self.protocols[curProtocolID].conditions[curConditionID].nodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].conditions[curConditionID].nodeTree))
						self.pathIterators[self.protocols[curProtocolID].conditions[curConditionID].nodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="ID")
						self.protocols[curProtocolID].conditions[curConditionID].nodeXML = curProtocolChildNodeXML
						for curConditionChildNodeXML in curProtocolChildNodeXML.iterchildren():
							if curConditionChildNodeXML.tag == "description":
								self.protocols[curProtocolID].conditions[curConditionID].description = curConditionChildNodeXML.text
								self.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].conditions[curConditionID].nodeTree.get_path()), [curConditionChildNodeXML.tag, curConditionChildNodeXML.text, True, "", False])
								self.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree))
								self.protocols[curProtocolID].conditions[curConditionID].descriptionNodeXML = curConditionChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="description")
							elif curConditionChildNodeXML.tag == "onsetTimes":
								self.protocols[curProtocolID].conditions[curConditionID].onsetTimes = curConditionChildNodeXML.text
								self.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].conditions[curConditionID].nodeTree.get_path()), [curConditionChildNodeXML.tag, curConditionChildNodeXML.text, True, curConditionChildNodeXML.get("units"), False])
								self.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree))
								self.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeXML = curConditionChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="onsetTimes")
							elif curConditionChildNodeXML.tag == "duration":
								self.protocols[curProtocolID].conditions[curConditionID].duration = curConditionChildNodeXML.text
								self.protocols[curProtocolID].conditions[curConditionID].durationNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].conditions[curConditionID].nodeTree.get_path()), [curConditionChildNodeXML.tag, curConditionChildNodeXML.text, True, curConditionChildNodeXML.get("units"), False])
								self.protocols[curProtocolID].conditions[curConditionID].durationNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].conditions[curConditionID].durationNodeTree))
								self.protocols[curProtocolID].conditions[curConditionID].durationNodeXML = curConditionChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].conditions[curConditionID].durationNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="duration")
							else:
								print "unsupported field in condition: " + curConditionChildNodeXML.tag
						#for curConditionChildNodeXML in curProtocolChildNodeXML.iterchildren():
					elif curProtocolChildNodeXML.tag == "otherData":
						curOtherDataID = curProtocolChildNodeXML.get("ID")
						self.protocols[curProtocolID].otherData[curOtherDataID] = OtherData(ID=curOtherDataID)
						self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), ["otherData", curOtherDataID, True, "", False])
						self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree))
						self.pathIterators[self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="ID")
						self.protocols[curProtocolID].otherData[curOtherDataID].nodeXML = curProtocolChildNodeXML
	
						for curOtherDataChildNodeXML in curProtocolChildNodeXML.iterchildren():
							if curOtherDataChildNodeXML.tag == "description":
								self.protocols[curProtocolID].otherData[curOtherDataID].description = curOtherDataChildNodeXML.text
								self.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree.get_path()), [curOtherDataChildNodeXML.tag, curOtherDataChildNodeXML.text, True, "", False])
								self.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree))
								self.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeXML = curOtherDataChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="description")
							elif curOtherDataChildNodeXML.tag == "format":
								self.protocols[curProtocolID].otherData[curOtherDataID].format = curOtherDataChildNodeXML.text
								self.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree.get_path()), [curOtherDataChildNodeXML.tag, curOtherDataChildNodeXML.text, True, "", False])
								self.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree))
								self.protocols[curProtocolID].otherData[curOtherDataID].formatNodeXML = curOtherDataChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="format")
							elif curOtherDataChildNodeXML.tag == "acquisitionDevice":
								self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDevice = curOtherDataChildNodeXML.text
								self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].otherData[curOtherDataID].nodeTree.get_path()), [curOtherDataChildNodeXML.tag, curOtherDataChildNodeXML.text, True, "", False])
								self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree))
								self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeXML = curOtherDataChildNodeXML
								self.pathIterators[self.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="acquisitionDevice")
					else:
						if curProtocolChildNodeXML.tag == "protocolType":
							self.protocols[curProtocolID].protocolType = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].protocolTypeNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].protocolTypeNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].protocolTypeNodeTree))
							self.protocols[curProtocolID].protocolTypeNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].protocolTypeNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="protocolType")
						elif curProtocolChildNodeXML.tag == "preferredSliceOrientation":
							self.protocols[curProtocolID].preferredSliceOrientation = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].preferredSliceOrientationNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].preferredSliceOrientationNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].preferredSliceOrientationNodeTree))
							self.protocols[curProtocolID].preferredSliceOrientationNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].preferredSliceOrientationNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="preferredSliceOrientation")
						elif curProtocolChildNodeXML.tag == "imagingSequence":
							self.protocols[curProtocolID].imagingSequence = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].imagingSequenceNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].imagingSequenceNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].imagingSequenceNodeTree))
							self.protocols[curProtocolID].imagingSequenceNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].imagingSequenceNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="imagingSequence")
						elif curProtocolChildNodeXML.tag == "numberOfSlices":
							self.protocols[curProtocolID].numberOfSlices = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].numberOfSlicesNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].numberOfSlicesNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].numberOfSlicesNodeTree))
							self.protocols[curProtocolID].numberOfSlicesNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].numberOfSlicesNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="numberOfSlices")
						elif curProtocolChildNodeXML.tag == "sliceThickness":
							self.protocols[curProtocolID].sliceThickness = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].sliceThicknessNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, curProtocolChildNodeXML.get("units"), False])
							self.protocols[curProtocolID].sliceThicknessNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].sliceThicknessNodeTree))
							self.protocols[curProtocolID].sliceThicknessNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].sliceThicknessNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="sliceThickness")
						elif curProtocolChildNodeXML.tag == "TR":
							self.protocols[curProtocolID].TR = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].TRNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, curProtocolChildNodeXML.get("units"), False])
							self.protocols[curProtocolID].TRNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].TRNodeTree))
							self.protocols[curProtocolID].TRNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].TRNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TR")
						elif curProtocolChildNodeXML.tag == "TE":
							self.protocols[curProtocolID].TE = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].TENodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, curProtocolChildNodeXML.get("units"), False])
							self.protocols[curProtocolID].TENodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].TENodeTree))
							self.protocols[curProtocolID].TENodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].TENodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TE")
						elif curProtocolChildNodeXML.tag == "TI":
							self.protocols[curProtocolID].TI = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].TINodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, curProtocolChildNodeXML.get("units"), False])
							self.protocols[curProtocolID].TINodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].TINodeTree))
							self.protocols[curProtocolID].TINodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].TINodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TI")
						elif curProtocolChildNodeXML.tag == "flipAngle":
							self.protocols[curProtocolID].flipAngle = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].flipAngleNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].flipAngleNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].flipAngleNodeTree))
							self.protocols[curProtocolID].flipAngleNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].flipAngleNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="flipAngle")
						elif curProtocolChildNodeXML.tag == "numberOfVolumes":
							self.protocols[curProtocolID].numberOfVolumes = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].numberOfVolumesNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].numberOfVolumesNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].numberOfVolumesNodeTree))
							self.protocols[curProtocolID].numberOfVolumesNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].numberOfVolumesNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="numberOfVolumes")
						elif curProtocolChildNodeXML.tag == "acquisitionOrder":
							self.protocols[curProtocolID].acquisitionOrder = curProtocolChildNodeXML.text
							self.protocols[curProtocolID].acquisitionOrderNodeTree = self.tree_store.append(self.tree_store.get_iter(self.protocols[curProtocolID].nodeTree.get_path()), [curProtocolChildNodeXML.tag, curProtocolChildNodeXML.text, True, "", False])
							self.protocols[curProtocolID].acquisitionOrderNodeTree = gtk.TreeRowReference(self.tree_store, self.tree_store.get_path(self.protocols[curProtocolID].acquisitionOrderNodeTree))
							self.protocols[curProtocolID].acquisitionOrderNodeXML = curProtocolChildNodeXML
							self.pathIterators[self.protocols[curProtocolID].acquisitionOrderNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="acquisitionOrder")
				#for curProtocolChildNodeXML in curProtocolNodeXML.iterchildren():
			#for curProtocolNodeXML in self.curProtocolNodesXML:
		#if os.path.isfile(fileName) == False:
	#def loadProtocol(self, databaseDirectory):
		
	""" The model class holds the information we want to display """
	def __init__(self, protocolFile):
		""" Sets up and populates our gtk.TreeStore """
		self.tree_store = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
		self.protocols = None
		self.pathIterators = None
		self.loadProtocol(databaseDirectory)
		
	def get_model(self):
		""" Returns the model """
		if self.tree_store:
			return self.tree_store
		else:
			return None

	def getEntryFromPath(self, path):
		curRowRef = gtk.TreeRowReference(self.tree_store, path)
		#if isinstance(path, str):
		#	searchPath = path
		#elif isinstance(path, tuple):
		#	searchPath = ":".join(str(v) for v in path)
		#print "###"
		#print searchPath
		#print "###"
		selectedEntry = None
		for curEntry in self.pathIterators.iterkeys():
			#curPath = ":".join(str(v) for v in curEntry.get_path())
			#if curPath == searchPath:
			if curRowRef.get_path() == curEntry.get_path():
				selectedEntry = curEntry
				break
		return selectedEntry

	def deleteProtocol(self, protocol):
		curProtocolNodesXML = self.protocolXMLRoot.xpath("//protocol[@ID='" + protocol + "']")
		if len(curProtocolNodesXML) > 0:
			curProtocol = self.protocols[protocol]
			
			conditions = list(curProtocol.conditions)
			for curCondition in conditions:
				self.deleteCondition(curCondition, protocol)

			otherData = list(curProtocol.otherData)
			for curOtherData in otherData:
				self.deleteOtherData(curOtherData, protocol)

			self.tree_store.remove(self.tree_store.get_iter(curProtocol.protocolTypeNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.preferredSliceOrientationNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.imagingSequenceNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.numberOfSlicesNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.sliceThicknessNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.TRNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.TENodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.TINodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.flipAngleNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.numberOfVolumesNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.acquisitionOrderNodeTree.get_path()));
			self.tree_store.remove(self.tree_store.get_iter(curProtocol.nodeTree.get_path()));
			
			curProtocol.nodeXML.remove(curProtocol.protocolTypeNodeXML)
			curProtocol.nodeXML.remove(curProtocol.preferredSliceOrientationNodeXML)
			curProtocol.nodeXML.remove(curProtocol.imagingSequenceNodeXML)
			curProtocol.nodeXML.remove(curProtocol.numberOfSlicesNodeXML)
			curProtocol.nodeXML.remove(curProtocol.sliceThicknessNodeXML)
			curProtocol.nodeXML.remove(curProtocol.TRNodeXML)
			curProtocol.nodeXML.remove(curProtocol.TENodeXML)
			curProtocol.nodeXML.remove(curProtocol.TINodeXML)
			curProtocol.nodeXML.remove(curProtocol.flipAngleNodeXML)
			curProtocol.nodeXML.remove(curProtocol.numberOfVolumesNodeXML)
			curProtocol.nodeXML.remove(curProtocol.acquisitionOrderNodeXML)

			# get the list of conditions
			del self.pathIterators[curProtocol.protocolTypeNodeTree]
			del self.pathIterators[curProtocol.preferredSliceOrientationNodeTree]
			del self.pathIterators[curProtocol.imagingSequenceNodeTree]
			del self.pathIterators[curProtocol.numberOfSlicesNodeTree]
			del self.pathIterators[curProtocol.TRNodeTree]
			del self.pathIterators[curProtocol.TENodeTree]
			del self.pathIterators[curProtocol.TINodeTree]
			del self.pathIterators[curProtocol.flipAngleNodeTree]
			del self.pathIterators[curProtocol.numberOfVolumesNodeTree]
			del self.pathIterators[curProtocol.acquisitionOrderNodeTree]
			del self.pathIterators[curProtocol.nodeTree]

# delete the entries in the current protocol
# cycle through each condition, delete it
# cycle through each 

	def deleteCondition(self, condition, protocol):
		curCondition = self.protocols[protocol].conditions[condition]
		
		# delete from tree
		self.tree_store.remove(self.tree_store.get_iter(curCondition.descriptionNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curCondition.onsetTimesNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curCondition.durationNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curCondition.nodeTree.get_path()));

		# delete from XML
		curCondition.nodeXML.remove(curCondition.descriptionNodeXML)
		curCondition.nodeXML.remove(curCondition.onsetTimesNodeXML)
		curCondition.nodeXML.remove(curCondition.durationNodeXML)
		curParent = curCondition.nodeXML.getparent()
		curParent.remove(curCondition.nodeXML)
		
		# delete from path iterators
		del self.pathIterators[curCondition.descriptionNodeTree]
		del self.pathIterators[curCondition.onsetTimesNodeTree]
		del self.pathIterators[curCondition.durationNodeTree]
		del self.pathIterators[curCondition.nodeTree]

	def deleteOtherData(self, otherData, protocol):
		curOtherData = self.protocols[protocol].otherData[otherData]
		# delete from tree
		self.tree_store.remove(self.tree_store.get_iter(curOtherData.descriptionNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curOtherData.formatNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curOtherData.acquisitionDeviceNodeTree.get_path()));
		self.tree_store.remove(self.tree_store.get_iter(curOtherData.nodeTree.get_path()));

		# delete from XML
		curOtherData.nodeXML.remove(curOtherData.descriptionNodeXML)
		curOtherData.nodeXML.remove(curOtherData.formatNodeXML)
		curOtherData.nodeXML.remove(curOtherData.acquisitionDeviceNodeXML)
		parentXML = curOtherData.nodeXML.getparent()
		parentXML.remove(curOtherData.nodeXML)
		
		# delete from path iterators
		del self.pathIterators[curOtherData.descriptionNodeTree]
		del self.pathIterators[curOtherData.formatNodeTree]
		del self.pathIterators[curOtherData.acquisitionDeviceNodeTree]
		del self.pathIterators[curOtherData.nodeTree]


class InsertOtherDataWindow:
	def __init__(self):
		pass
	
	def errorWindow(self, text):
			dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=text)
			dialog.run()
			dialog.destroy()

	def showWindow(self, protocol=None, otherData=None):
		self.protocol = protocol
		self.otherData = otherData
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(7, 3, False)
	   	self.table.show()
		
		self.otherDataIDLabel = gtk.Label("Other Data ID:")
		self.table.attach(self.otherDataIDLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.otherDataIDLabel.show()
		
		self.descriptionLabel = gtk.Label("Description:")
		self.table.attach(self.descriptionLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.descriptionLabel.show()
		
		self.formatLabel = gtk.Label("Format:")
		self.table.attach(self.formatLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.formatLabel.show()

		self.acquisitionDeviceLabel = gtk.Label("Acquisition Device:")
		self.table.attach(self.acquisitionDeviceLabel, 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.acquisitionDeviceLabel.show()
		
		self.otherDataIDEntry = gtk.Entry()
		self.table.attach(self.otherDataIDEntry, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.otherDataIDEntry.show()
		
		self.descriptionEntry = gtk.Entry()
		self.table.attach(self.descriptionEntry, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.descriptionEntry.show()
		
		self.formatEntry = gtk.Entry()
		self.table.attach(self.formatEntry, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.formatEntry.show()
	
		self.acquisitionDeviceEntry = gtk.Entry()
		self.table.attach(self.acquisitionDeviceEntry, 1, 2, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.acquisitionDeviceEntry.show()
		
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 5, 6, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 5, 6, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		
		if self.otherData != None and self.protocol != None:
			otherDataNodeXML = Store.protocolXML.xpath("//protocol[@ID='" + self.protocol + "']/otherData[@ID='" + self.otherData + "']")

			self.otherDataIDEntry.set_text(otherData)
			self.otherDataIDEntry.set_sensitive(False)

			for curChildNode in otherDataNodeXML[0].iterchildren():
				if curChildNode.tag == 'description':
					self.descriptionEntry.set_text(curChildNode.text)
				elif curChildNode.tag == 'format':
					self.formatEntry.set_text(curChildNode.text)
				elif curChildNode.tag == 'acquisitionDevice':
					self.acquisitionDeviceEntry.set_text(curChildNode.text)

		self.window.show_all()
		#self.window.set_modal(True)
	#def showWindow(self):
	
	def callback(self, widget, data=None):
		if data == "OK":
			if self.otherDataIDEntry.get_text() == "":
				self.errorWindow("OtherDataID must be entered")
			elif self.descriptionEntry.get_text() == "":
				self.errorWindow("Description must be entered")
			elif self.formatEntry.get_text() == "":
				self.errorWindow("Format must be entered")
			elif self.acquisitionDeviceEntry.get_text() == "":
				self.errorWindow("acquisitionDevice must be entered")
			else:
				curProtocolID = self.protocol
				curOtherDataID = self.otherDataIDEntry.get_text()
				otherDataNodes = Store.protocolXML.xpath("//protocol[@ID='" + curProtocolID + "']/otherData[@ID='" + curOtherDataID + "']")
				if len(otherDataNodes) > 0 and self.otherData == None:
					self.errorWindow("otherData ID already present:" + curOtherDataID)
				elif (len(otherDataNodes) > 0 and self.otherData != None) or (self.otherData == None and len(otherDataNodes) == 0):
					protocolNodes = Store.protocolXML.xpath("//protocol[@ID='" + curProtocolID + "']")
					acquisitionOrderNodes = protocolNodes[0].xpath("//acquisitionOrder")
					conditionNodes = protocolNodes[0].xpath("//condition")
					if len(otherDataNodes) == 0:

						Store.protocols[curProtocolID].otherData[curOtherDataID] = OtherData(ID=curOtherDataID)
						Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML = etree.Element('otherData')
						Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML.set("ID", curOtherDataID)
						if(len(conditionNodes) > 0):
							conditionNodes[len(conditionNodes) - 1].addnext(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML)
						else:
							acquisitionOrderNodes[len(acquisitionOrderNodes) - 1].addnext(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML)
						
						Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeXML = etree.SubElement(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML, 'description')
						Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeXML = etree.SubElement(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML, 'format')
						Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeXML = etree.SubElement(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeXML, 'acquisitionDevice')
						
						Store.protocols[curProtocolID].otherData[curOtherDataID].NodeTree = Store.tree_store.append(Store.tree_store.get_iter(Store.protocols[curProtocolID].nodeTree.get_path()), ['otherData', curOtherDataID, True, "", False])
						Store.protocols[curProtocolID].otherData[curOtherDataID].NodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeTree))
						otherDataNodeIter = Store.tree_store.get_iter(Store.protocols[curProtocolID].otherData[curOtherDataID].NodeTree.get_path())
						Store.pathIterators[Store.protocols[curProtocolID].otherData[curOtherDataID].NodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID)
						Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree = Store.tree_store.append(otherDataNodeIter, ['description', self.descriptionEntry.get_text(), True, "", False])
						Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree))
						Store.pathIterators[Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="description")
						Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree = Store.tree_store.append(otherDataNodeIter, ['format', self.formatEntry.get_text(), True, "", False])
						Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree))
						Store.pathIterators[Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="format")
						Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree = Store.tree_store.append(otherDataNodeIter, ['acquisitionDevice', self.acquisitionDeviceEntry.get_text(), True, "", False])
						Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree))
						Store.pathIterators[Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree] = reversePathLookup(protocol=curProtocolID, otherData=curOtherDataID, attributeName="acquisitionDevice")
						
					Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeXML.text = self.descriptionEntry.get_text()
					Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeXML.text = self.formatEntry.get_text()
					Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeXML.text = self.acquisitionDeviceEntry.get_text()
					
					Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].otherData[curOtherDataID].descriptionNodeTree.get_path()), 1, self.descriptionEntry.get_text())
					Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].otherData[curOtherDataID].formatNodeTree.get_path()), 1, self.formatEntry.get_text())
					Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].otherData[curOtherDataID].acquisitionDeviceNodeTree.get_path()), 1, self.acquisitionDeviceEntry.get_text())

					self.window.destroy()
				#if len(otherDataNodes) > 0:
			#if self.otherDataIDEntry.get_text() == "":
		elif data == "Cancel":
			self.window.destroy()
		#if data == "OK":


class InsertConditionWindow:
	def __init__(self):
		pass
	
	def errorWindow(self, text):
			dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=text)
			dialog.run()
			dialog.destroy()

	def showWindow(self, protocol=None, condition=None):
		self.protocol = protocol
		self.condition = condition

		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(7, 3, False)
	   	self.table.show()
		
		self.conditionIDLabel = gtk.Label("Condition ID:")
		self.table.attach(self.conditionIDLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.conditionIDLabel.show()
		
		self.descriptionLabel = gtk.Label("Description:")
		self.table.attach(self.descriptionLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.descriptionLabel.show()
		
		self.onsetTimesLabel = gtk.Label("Onset Times (comma separated list):")
		self.table.attach(self.onsetTimesLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.onsetTimesLabel.show()

		self.durationLabel = gtk.Label("Duration:")
		self.table.attach(self.durationLabel, 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.durationLabel.show()
		
		self.descriptionEntry = gtk.Entry()
		self.table.attach(self.descriptionEntry, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.descriptionEntry.show()
		
		self.conditionIDEntry = gtk.Entry()
		self.table.attach(self.conditionIDEntry, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.conditionIDEntry.show()
		
		self.onsetTimesEntry = gtk.Entry()
		self.table.attach(self.onsetTimesEntry, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.onsetTimesEntry.show()
		
		self.onsetTimesUnitsCombo = gtk.combo_box_new_text()
		self.onsetTimesUnitsCombo.append_text("ms")
		self.onsetTimesUnitsCombo.append_text("s")
		self.table.attach(self.onsetTimesUnitsCombo, 2, 3, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.onsetTimesUnitsCombo.show()
		
		self.durationEntry = gtk.Entry()
		self.table.attach(self.durationEntry, 1, 2, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.durationEntry.show()
		
		self.durationUnitsCombo = gtk.combo_box_new_text()
		self.durationUnitsCombo.append_text("ms")
		self.durationUnitsCombo.append_text("s")
		self.table.attach(self.durationUnitsCombo, 2, 3, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.durationUnitsCombo.show()
		
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 5, 6, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 5, 6, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
		if self.condition != None and self.protocol != None:
			conditionNodeXML = Store.protocolXML.xpath("//protocol[@ID='" + self.protocol + "']/condition[@ID='" + self.condition + "']")

			self.conditionIDEntry.set_text(condition)
			self.conditionIDEntry.set_sensitive(False)

			for curChildNode in conditionNodeXML[0].iterchildren():
				if curChildNode.tag == 'description':
					self.descriptionEntry.set_text(curChildNode.text)
				elif curChildNode.tag == 'onsetTimes':
					self.onsetTimesEntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "ms":
						self.onsetTimesUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "s":
						self.onsetTimesUnitsCombo.set_active(1)
				elif curChildNode.tag == 'duration':
					self.durationEntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "ms":
						self.durationUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "s":
						self.durationUnitsCombo.set_active(1)

	def callback(self, widget, data=None):
		if data == "OK":
			if self.conditionIDEntry.get_text() == "":
				self.errorWindow("ConditionID must be entered")
			elif self.descriptionEntry.get_text() == "":
				self.errorWindow("Description must be entered")
			elif self.onsetTimesEntry.get_text() == "":
				self.errorWindow("Onset times must be entered")
			else:
				# all the fields are entered correctly, now do syntax checking
				onsetTimesEntryMat = re.match("^(\d+[\d,]+\d+|\d+)$", self.onsetTimesEntry.get_text())
				tmp = self.onsetTimesEntry.get_text()
				if onsetTimesEntryMat == None:
					self.errorWindow("Onset times was invalid")
				else:
					onsetTimesUnits = self.onsetTimesUnitsCombo.get_active_text()
					
					curProtocolID = self.protocol
					curConditionID = self.conditionIDEntry.get_text()
					conditionNodes = Store.protocolXML.xpath("//protocol[@ID='" + curProtocolID + "']/condition[@ID='" + curConditionID + "']")
					# if we are editing and 
					if len(conditionNodes) > 0 and self.protocol == None and self.condition == None:
						self.errorWindow("Condition ID already present:" + curConditionID)
					elif (len(conditionNodes) > 0 and self.condition != None) or (self.condition == None and len(conditionNodes) == 0):
						if len(conditionNodes) == 0:
							protocolNodes = Store.protocolXML.xpath("//protocol[@ID='" + curProtocolID + "']")
							acquisitionOrderNodes = protocolNodes[0].getparent().xpath("//acquisitionOrder")
							Store.protocols[curProtocolID].conditions[curConditionID] = Condition(ID=curConditionID)
							Store.protocols[curProtocolID].conditions[curConditionID].NodeXML = etree.Element('condition')
							Store.protocols[curProtocolID].conditions[curConditionID].NodeXML.set("ID", curConditionID)
							acquisitionOrderNodes[len(acquisitionOrderNodes) - 1].addnext(Store.protocols[curProtocolID].conditions[curConditionID].NodeXML)
							
							Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeXML = etree.SubElement(Store.protocols[curProtocolID].conditions[curConditionID].NodeXML, 'description')
							Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeXML = etree.SubElement(Store.protocols[curProtocolID].conditions[curConditionID].NodeXML, 'onsetTimes')
							Store.protocols[curProtocolID].conditions[curConditionID].durationNodeXML = etree.SubElement(Store.protocols[curProtocolID].conditions[curConditionID].NodeXML, 'duration')
							
							#find the position of the acquisitionOrderNode in the tree
							Store.protocols[curProtocolID].conditions[curConditionID].NodeTree = Store.tree_store.insert_after(Store.tree_store.get_iter(Store.protocols[curProtocolID].nodeTree.get_path()), Store.tree_store.get_iter(Store.protocols[curProtocolID].acquisitionOrderNodeTree.get_path()), ['condition', curConditionID, True, "", False])
							Store.protocols[curProtocolID].conditions[curConditionID].NodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].conditions[curConditionID].NodeTree))
							conditionNodeIter = Store.tree_store.get_iter(Store.protocols[curProtocolID].conditions[curConditionID].NodeTree.get_path())
							Store.pathIterators[Store.protocols[curProtocolID].conditions[curConditionID].NodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID)
							Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree = Store.tree_store.append(conditionNodeIter, ['description', self.descriptionEntry.get_text(), True, "", False])
							Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree))
							Store.pathIterators[Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="description")
							Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree = Store.tree_store.append(conditionNodeIter, ['onsetTimes', self.onsetTimesEntry.get_text(), True, self.onsetTimesUnitsCombo.get_active_text(), False])
							Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree))
							Store.pathIterators[Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="onsetTimes")
							Store.protocols[curProtocolID].conditions[curConditionID].durationNodeTree = Store.tree_store.append(conditionNodeIter, ['duration', self.durationEntry.get_text(), True, self.durationUnitsCombo.get_active_text(), False])
							Store.protocols[curProtocolID].conditions[curConditionID].durationNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].conditions[curConditionID].durationNodeTree))
							Store.pathIterators[Store.protocols[curProtocolID].conditions[curConditionID].durationNodeTree] = reversePathLookup(protocol=curProtocolID, condition=curConditionID, attributeName="duration")

						Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeXML.text = self.descriptionEntry.get_text()
						Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeXML.text = self.onsetTimesEntry.get_text()
						Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeXML.set('units', self.onsetTimesUnitsCombo.get_active_text())
						Store.protocols[curProtocolID].conditions[curConditionID].durationNodeXML.text = self.durationEntry.get_text()
						Store.protocols[curProtocolID].conditions[curConditionID].durationNodeXML.set('units', self.durationUnitsCombo.get_active_text())
						
						Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].conditions[curConditionID].descriptionNodeTree.get_path()), 1, self.descriptionEntry.get_text())
						Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].conditions[curConditionID].onsetTimesNodeTree.get_path()), 1, self.onsetTimesEntry.get_text(), 3, self.onsetTimesUnitsCombo.get_active_text())
						Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].conditions[curConditionID].durationNodeTree.get_path()), 1, self.durationEntry.get_text(), 3, self.durationUnitsCombo.get_active_text())
						
						self.window.destroy()
					#if len(conditionNodes) > 0:
				#if onsetTimesEntryMat == None:
			#if self.conditionIDEntry.get_text() == "":
		elif data == "Cancel":
			self.window.destroy()


class InsertProtocolWindow:
	def __init__(self):
		pass
	
	def errorWindow(self, text):
			dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=text)
			dialog.run()
			dialog.destroy()

	def showWindow(self, protocol=None):
		self.protocol = protocol
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(15, 3, False)
	   	self.table.show()
		
		self.unitsLabel = gtk.Label("Units:")
		self.table.attach(self.unitsLabel, 2, 3, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.unitsLabel.show()

		self.protocolIDLabel = gtk.Label("Protocol ID:")
		self.table.attach(self.protocolIDLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.protocolIDLabel.show()

		self.protocolTypeLabel = gtk.Label("Protocol Type:")
		self.table.attach(self.protocolTypeLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.protocolTypeLabel.show()
		
		self.preferredSliceOrientationLabel = gtk.Label("Slice Orientation:")
		self.table.attach(self.preferredSliceOrientationLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.preferredSliceOrientationLabel.show()
	
		self.imagingSequenceLabel = gtk.Label("Imaging Sequence:")
		self.table.attach(self.imagingSequenceLabel, 0, 1, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.imagingSequenceLabel.show()

		self.numberOfSlicesLabel = gtk.Label("Number of Slices:")
		self.table.attach(self.numberOfSlicesLabel, 0, 1, 5, 6, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.numberOfSlicesLabel.show()

		self.sliceThicknessLabel = gtk.Label("Slice Thickness:")
		self.table.attach(self.sliceThicknessLabel, 0, 1, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.sliceThicknessLabel.show()

		self.TRLabel = gtk.Label("TR:")
		self.table.attach(self.TRLabel, 0, 1, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TRLabel.show()
		
		self.TELabel = gtk.Label("TE:")
		self.table.attach(self.TELabel, 0, 1, 8, 9, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TELabel.show()
		
		self.TILabel = gtk.Label("TI:")
		self.table.attach(self.TILabel, 0, 1, 9, 10, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TILabel.show()
	
		self.numberOfVolumesLabel = gtk.Label("Number of Volumes:")
		self.table.attach(self.numberOfVolumesLabel, 0, 1, 10, 11, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.numberOfVolumesLabel.show()
		
		self.acquisitionOrderLabel = gtk.Label("Acquisition Order:")
		self.table.attach(self.acquisitionOrderLabel, 0, 1, 11, 12, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.acquisitionOrderLabel.show()
	
		self.flipAngleLabel = gtk.Label("Flip angle:")
		self.table.attach(self.flipAngleLabel, 0, 1, 12, 13, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.flipAngleLabel.show()
		
		self.protocolIDEntry = gtk.Entry()
		self.table.attach(self.protocolIDEntry, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.protocolIDEntry.show()
		
		self.protocolTypeCombo = gtk.combo_box_new_text()
		self.protocolTypeCombo.append_text("block")
		self.protocolTypeCombo.append_text("event")
		self.protocolTypeCombo.append_text("mixed")
		self.protocolTypeCombo.append_text("mixed sparse")
		self.protocolTypeCombo.show()
		self.table.attach(self.protocolTypeCombo, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.preferredSliceOrientationCombo = gtk.combo_box_new_text()
		self.preferredSliceOrientationCombo.append_text("sagittal")
		self.preferredSliceOrientationCombo.append_text("coronal")
		self.preferredSliceOrientationCombo.append_text("axial")
		self.preferredSliceOrientationCombo.append_text("oblique")
		self.preferredSliceOrientationCombo.show()
		self.table.attach(self.preferredSliceOrientationCombo, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.imagingSequenceCombo = gtk.combo_box_new_text()
		self.imagingSequenceCombo.append_text("EPI-BOLD")
		self.imagingSequenceCombo.append_text("ASL")
		self.imagingSequenceCombo.show()
		self.table.attach(self.imagingSequenceCombo, 1, 2, 4, 5, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
			
		self.numberOfSlicesEntry = gtk.Entry()
		self.table.attach(self.numberOfSlicesEntry, 1, 2, 5, 6, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.numberOfSlicesEntry.show()

		self.sliceThicknessEntry = gtk.Entry()
		self.table.attach(self.sliceThicknessEntry, 1, 2, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.sliceThicknessEntry.show()
			
		self.sliceThicknessUnitsCombo = gtk.combo_box_new_text()
		self.sliceThicknessUnitsCombo.append_text("mm")
		self.sliceThicknessUnitsCombo.append_text("cm")
		self.table.attach(self.sliceThicknessUnitsCombo, 2, 3, 6, 7, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.sliceThicknessUnitsCombo.show()
		
		self.TREntry = gtk.Entry()
		self.table.attach(self.TREntry, 1, 2, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TREntry.show()
			
		self.TRUnitsCombo = gtk.combo_box_new_text()
		self.TRUnitsCombo.append_text("ms")
		self.TRUnitsCombo.append_text("s")
		self.table.attach(self.TRUnitsCombo, 2, 3, 7, 8, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TRUnitsCombo.show()
		
		self.TEEntry = gtk.Entry()
		self.table.attach(self.TEEntry, 1, 2, 8, 9, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TEEntry.show()
			
		self.TEUnitsCombo = gtk.combo_box_new_text()
		self.TEUnitsCombo.append_text("ms")
		self.TEUnitsCombo.append_text("s")
		self.table.attach(self.TEUnitsCombo, 2, 3, 8, 9, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TEUnitsCombo.show()
		
		self.TIEntry = gtk.Entry()
		self.table.attach(self.TIEntry, 1, 2, 9, 10, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TIEntry.show()
			
		self.TIUnitsCombo = gtk.combo_box_new_text()
		self.TIUnitsCombo.append_text("ms")
		self.TIUnitsCombo.append_text("s")
		self.table.attach(self.TIUnitsCombo, 2, 3, 9, 10, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.TIUnitsCombo.show()
		
		self.numberOfVolumesEntry = gtk.Entry()
		self.table.attach(self.numberOfVolumesEntry, 1, 2, 10, 11, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.numberOfVolumesEntry.show()
		
		self.acquisitionOrderCombo = gtk.combo_box_new_text()
		self.acquisitionOrderCombo.append_text("ascending")
		self.acquisitionOrderCombo.append_text("descending")
		self.acquisitionOrderCombo.append_text("ascending interleaved")
		self.acquisitionOrderCombo.append_text("descending interleaved")
		self.acquisitionOrderCombo.append_text("inside out")
		self.acquisitionOrderCombo.show()
		self.table.attach(self.acquisitionOrderCombo, 1, 2, 11, 12, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.flipAngleEntry = gtk.Entry()
		self.table.attach(self.flipAngleEntry, 1, 2, 12, 13, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.flipAngleEntry.show()
		
		if protocol != None:
			protocolNodeXML = Store.protocolXML.xpath("//protocol[@ID='" + protocol + "']")
			self.protocolIDEntry.set_text(protocol)
			self.protocolIDEntry.set_sensitive(False)

			for curChildNode in protocolNodeXML[0].iterchildren():
				if curChildNode.tag == 'protocolType':
					if curChildNode.text == 'block':
						self.protocolTypeCombo.set_active(0)
					elif curChildNode.text == 'event':
						self.protocolTypeCombo.set_active(1)
					elif curChildNode.text == 'mixed':
						self.protocolTypeCombo.set_active(2)
					elif curChildNode.text == 'mixed sparse':
						self.protocolTypeCombo.set_active(3)
				elif curChildNode.tag == 'preferredSliceOrientation':
					if curChildNode.text == 'sagittal':
						self.preferredSliceOrientationCombo.set_active(0)
					elif curChildNode.text == 'coronal':
						self.preferredSliceOrientationCombo.set_active(1)
					elif curChildNode.text == 'axial':
						self.preferredSliceOrientationCombo.set_active(2)
					elif curChildNode.text == 'oblique':
						self.preferredSliceOrientationCombo.set_active(3)
				elif curChildNode.tag == 'imagingSequence':
					if curChildNode.text == 'EPI-BOLD':
						self.imagingSequenceCombo.set_active(0)
					elif curChildNode.text == 'ASL':
						self.imagingSequenceCombo.set_active(1)
				elif curChildNode.tag == 'numberOfSlices':
					self.numberOfSlicesEntry.set_text(curChildNode.text)
				elif curChildNode.tag == 'sliceThickness':
					self.sliceThicknessEntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "mm":
						self.sliceThicknessUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "cm":
						self.sliceThicknessUnitsCombo.set_active(1)
				elif curChildNode.tag == 'TR':
					self.TREntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "ms":
						self.TRUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "s":
						self.TRUnitsCombo.set_active(1)
				elif curChildNode.tag == 'TE':
					self.TEEntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "ms":
						self.TEUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "s":
						self.TEUnitsCombo.set_active(1)
				elif curChildNode.tag == 'TI':
					self.TIEntry.set_text(curChildNode.text)
					if curChildNode.get("units") == "ms":
						self.TIUnitsCombo.set_active(0)
					elif curChildNode.get("units") == "s":
						self.TIUnitsCombo.set_active(1)
				elif curChildNode.tag == 'numberOfVolumes':
					self.numberOfVolumesEntry.set_text(curChildNode.text)
				elif curChildNode.tag == 'acquisitionOrder':
					if curChildNode.text == 'ascending':
						self.acquisitionOrderCombo.set_active(0)
					elif curChildNode.text == 'descending':
						self.acquisitionOrderCombo.set_active(1)
					elif curChildNode.text == 'ascending interleaved':
						self.acquisitionOrderCombo.set_active(2)
					elif curChildNode.text == 'descending interleaved':
						self.acquisitionOrderCombo.set_active(3)
					elif curChildNode.text == 'inside out':
						self.acquisitionOrderCombo.set_active(4)
				elif curChildNode.tag == 'flipAngle':
					self.flipAngleEntry.set_text(curChildNode.text)

		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 13, 14, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 13, 14, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
	
	def callback(self, widget, data=None):
		if data == "OK":
			if self.protocolIDEntry.get_text() == "":
				self.errorWindow("Protocol ID must be entered")
			elif self.protocolTypeCombo.get_active_text() == None:
				self.errorWindow("Protocol type must be entered")
			elif self.preferredSliceOrientationCombo.get_active_text() == None:
				self.errorWindow("Preferred slice orientation must be entered")
			elif self.imagingSequenceCombo.get_active_text() == None:
				self.errorWindow("Imaging sequence must be entered")
			elif self.numberOfSlicesEntry.get_text() == "":
				self.errorWindow("Number of Slices must be entered")
			elif self.sliceThicknessEntry.get_text() == "":
				self.errorWindow("Slice Thickness must be entered")
			elif self.TREntry.get_text() == "":
				self.errorWindow("TR must be entered")
			elif self.TEEntry.get_text() == "":
				self.errorWindow("TE must be entered")
			elif self.TIEntry.get_text() == "":
				self.errorWindow("TI must be entered")
			elif self.numberOfVolumesEntry.get_text() == "":
				self.errorWindow("Number of volumes must be entered")
			elif self.acquisitionOrderCombo.get_active_text() == None:
				self.errorWindow("Acquisition Order must be entered")
			elif self.flipAngleEntry.get_text() == "":
				self.errorWindow("Flip angle must be entered")
			else:
				# all the fields are entered correctly, now do syntax checking
				numberOfSlicesEntryMat = re.match("^\d+$", self.numberOfSlicesEntry.get_text())
				sliceThicknessEntryMat = re.match("^\d+$", self.sliceThicknessEntry.get_text())
				TREntryMat = re.match("^\d+$", self.TREntry.get_text())
				TEEntryMat = re.match("^\d+$", self.TEEntry.get_text())
				TIEntryMat = re.match("^\d+$", self.TIEntry.get_text())
				numberOfVolumesEntryMat = re.match("^\d+$", self.numberOfVolumesEntry.get_text())
				flipAngleEntryMat = re.match("^\d+$", self.flipAngleEntry.get_text())

				if numberOfSlicesEntryMat == None:
					self.errorWindow("Number of slices was invalid")
				elif sliceThicknessEntryMat == None:
					self.errorWindow("Slice gap was invalid")
				elif TREntryMat == None:
					self.errorWindow("TR was invalid")
				elif TEEntryMat == None:
					self.errorWindow("TE was invalid")
				elif TIEntryMat == None:
					self.errorWindow("TI was invalid")
				elif numberOfVolumesEntryMat == None:
					self.errorWindow("Number of volumes was invalid")
				elif flipAngleEntryMat == None:
					self.errorWindow("Flip angle was invalid")
				else:
					if int(self.numberOfSlicesEntry.get_text()) < 1:
						self.errorWindow("Number of slices must be positive")
					elif float(self.TREntry.get_text()) <= 0:
						self.errorWindow("TR must be positive")
					elif float(self.TEEntry.get_text()) <= 0:
						self.errorWindow("TE must be positive")
					elif float(self.TIEntry.get_text()) <= 0:
						self.errorWindow("TI must be positive")
					elif int(self.numberOfVolumesEntry.get_text()) < 1:
						self.errorWindow("Number of volumes must be positive")
					elif float(self.flipAngleEntry.get_text()) <= 0:
						self.errorWindow("Flip angle must be positive")
					else:
						if self.sliceThicknessUnitsCombo.get_active_text() == None:
							sliceThicknessUnits = 'mm'
						else:
							sliceThicknessUnits = self.sliceThicknessUnitsCombo.get_active_text()
						if self.TRUnitsCombo.get_active_text() == None:
							TRUnits = 'ms'
						else:
							TRUnits = self.TRUnitsCombo.get_active_text()
						if self.TEUnitsCombo.get_active_text() == None:
							TEUnits = 'ms'
						else:
							TEUnits = self.TEUnitsCombo.get_active_text()
						if self.TIUnitsCombo.get_active_text() == None:
							TIUnits = 'ms'
						else:
							TIUnits = self.TIUnitsCombo.get_active_text()
						
						if not (sliceThicknessUnits == "mm" or sliceThicknessUnits == "cm"):
							self.errorWindow("Slice gap units must be either \"mm\" or \"cm\"")
						elif not (TRUnits == "ms" or TRUnits == "s"):
							self.errorWindow("TR units must be either \"ms\" or \"s\"")
						elif not (TEUnits == "ms" or TEUnits == "s"):
							self.errorWindow("TE units must be either \"ms\" or \"s\"")
						elif not (TIUnits == "ms" or TIUnits == "s"):
							self.errorWindow("TI units must be either \"ms\" or \"s\"")
						else:
							# we have to make sure that the protocol ID is unique
							curProtocolID = self.protocolIDEntry.get_text()
							protocolNodes = Store.protocolXMLRoot.xpath("//protocol[@ID='" + curProtocolID + "']")
							if len(protocolNodes) > 0 and self.protocol == None:
								self.errorWindow("protocol ID is already in the database: " + curProtocolID)
							elif (len(protocolNodes) > 0 and self.protocol != None) or (self.protocol == None and len(protocolNodes) == 0):
								# all tests passed add to the tree and add to the XML file
								# if the current protocol is not in the database then it is a new one
								if len(protocolNodes) == 0:
									Store.protocols[curProtocolID] = Protocol(ID=curProtocolID)
									# create the XML nodes
									Store.protocols[curProtocolID].nodeXML = etree.SubElement(Store.protocolXMLRoot, "protocol")
									Store.protocols[curProtocolID].protocolTypeNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'protocolType')
									Store.protocols[curProtocolID].preferredSliceOrientationNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'preferredSliceOrientation')
									Store.protocols[curProtocolID].imagingSequenceNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'imagingSequence')
									Store.protocols[curProtocolID].numberOfSlicesNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'numberOfSlices')
									Store.protocols[curProtocolID].sliceThicknessNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'sliceThickness')
									Store.protocols[curProtocolID].TRNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'TR')
									Store.protocols[curProtocolID].TENodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'TE')
									Store.protocols[curProtocolID].TINodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'TI')
									Store.protocols[curProtocolID].numberOfVolumesNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'numberOfVolumes')
									Store.protocols[curProtocolID].acquisitionOrderNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'acquisitionOrder')
									Store.protocols[curProtocolID].flipAngleNodeXML = etree.SubElement(Store.protocols[curProtocolID].nodeXML, 'flipAngle')

									# create the tree nodes
									Store.protocols[curProtocolID].nodeTree = Store.tree_store.append(Store.treeRoot, ['protocol', curProtocolID, True, "", False])
									Store.protocols[curProtocolID].nodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].nodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].nodeTree] = reversePathLookup(protocol=curProtocolID)
									protocolNodeTreeIter = Store.tree_store.get_iter(Store.protocols[curProtocolID].nodeTree.get_path())

									Store.protocols[curProtocolID].protocolTypeNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['protocolType', self.protocolTypeCombo.get_active_text(), True, "", False])
									Store.protocols[curProtocolID].protocolTypeNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].protocolTypeNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].protocolTypeNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="protocolType")

									Store.protocols[curProtocolID].preferredSliceOrientationNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['preferredSliceOrientation', self.preferredSliceOrientationCombo.get_active_text(), True, "", False])
									Store.protocols[curProtocolID].preferredSliceOrientationNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].preferredSliceOrientationNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].preferredSliceOrientationNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="preferredSliceOrientation")

									Store.protocols[curProtocolID].imagingSequenceNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['imagingSequence', self.imagingSequenceCombo.get_active_text(), True, "", False])
									Store.protocols[curProtocolID].imagingSequenceNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].imagingSequenceNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].imagingSequenceNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="imagingSequence")

									Store.protocols[curProtocolID].numberOfSlicesNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['numberOfSlices', self.numberOfSlicesEntry.get_text(), True, "", False])
									Store.protocols[curProtocolID].numberOfSlicesNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].numberOfSlicesNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].numberOfSlicesNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="numberOfSlices")

									Store.protocols[curProtocolID].sliceThicknessNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['sliceThickness', self.sliceThicknessEntry.get_text(), True, sliceThicknessUnits, True])
									Store.protocols[curProtocolID].sliceThicknessNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].sliceThicknessNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].sliceThicknessNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="sliceThickness")

									Store.protocols[curProtocolID].TRNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['TR', self.TREntry.get_text(), True, TRUnits, True])
									Store.protocols[curProtocolID].TRNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].TRNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].TRNodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TR")

									Store.protocols[curProtocolID].TENodeTree = Store.tree_store.append(protocolNodeTreeIter, ['TE', self.TEEntry.get_text(), True, TEUnits, True])
									Store.protocols[curProtocolID].TENodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].TENodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].TENodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TE")

									Store.protocols[curProtocolID].TINodeTree = Store.tree_store.append(protocolNodeTreeIter, ['TI', self.TIEntry.get_text(), True, TIUnits, True])
									Store.protocols[curProtocolID].TINodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].TINodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].TINodeTree] = reversePathLookup(protocol=curProtocolID, attributeName="TI")

									Store.protocols[curProtocolID].numberOfVolumesNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['numberOfVolumes', self.numberOfVolumesEntry.get_text(), True, "", False])
									Store.protocols[curProtocolID].numberOfVolumesNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].numberOfVolumesNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].numberOfVolumesNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="numberOfVolumes")

									Store.protocols[curProtocolID].acquisitionOrderNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['acquisitionOrder', self.acquisitionOrderCombo.get_active_text(), True, "", False])
									Store.protocols[curProtocolID].acquisitionOrderNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].acquisitionOrderNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].acquisitionOrderNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="acquisitionOrder")

									Store.protocols[curProtocolID].flipAngleNodeTree = Store.tree_store.append(protocolNodeTreeIter, ['flipAngle', self.flipAngleEntry.get_text(), True, "", False])
									Store.protocols[curProtocolID].flipAngleNodeTree = gtk.TreeRowReference(Store.tree_store, Store.tree_store.get_path(Store.protocols[curProtocolID].flipAngleNodeTree))
									Store.pathIterators[Store.protocols[curProtocolID].flipAngleNodeTree] = reversePathLookup(protocol=curProtocolID,attributeName="flipAngle")
								#if len(protocolNodes) == 0:
								# set the values of the new entries in the tree uses gtk.TreeStore.set(gtk.TreeIter, ...) name/value pairs
								# for set, column 1 is the "value" column and column 3 is the "units" column
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].protocolTypeNodeTree.get_path()), 1, self.protocolTypeCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].preferredSliceOrientationNodeTree.get_path()), 1, self.preferredSliceOrientationCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].imagingSequenceNodeTree.get_path()), 1, self.imagingSequenceCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].numberOfSlicesNodeTree.get_path()), 1, self.numberOfSlicesEntry.get_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].sliceThicknessNodeTree.get_path()), 1, self.sliceThicknessEntry.get_text(), 3, self.sliceThicknessUnitsCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].TRNodeTree.get_path()), 1, self.TREntry.get_text(), 3, self.TRUnitsCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].TENodeTree.get_path()), 1, self.TEEntry.get_text(), 3, self.TEUnitsCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].TINodeTree.get_path()), 1, self.TIEntry.get_text(), 3, self.TIUnitsCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].numberOfVolumesNodeTree.get_path()), 1, self.numberOfVolumesEntry.get_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].acquisitionOrderNodeTree.get_path()), 1, self.acquisitionOrderCombo.get_active_text())
								Store.tree_store.set(Store.tree_store.get_iter(Store.protocols[curProtocolID].flipAngleNodeTree.get_path()), 1, self.flipAngleEntry.get_text())
	
								Store.protocols[curProtocolID].nodeXML.set("ID", curProtocolID)
								Store.protocols[curProtocolID].protocolTypeNodeXML.text = self.protocolTypeCombo.get_active_text()
								Store.protocols[curProtocolID].preferredSliceOrientationNodeXML.text = self.preferredSliceOrientationCombo.get_active_text()
								Store.protocols[curProtocolID].imagingSequenceNodeXML.text = self.imagingSequenceCombo.get_active_text()
								Store.protocols[curProtocolID].numberOfSlicesNodeXML.text = self.numberOfSlicesEntry.get_text()
								Store.protocols[curProtocolID].sliceThicknessNodeXML.text = self.sliceThicknessEntry.get_text()
								Store.protocols[curProtocolID].sliceThicknessNodeXML.set('units', sliceThicknessUnits)
								Store.protocols[curProtocolID].TRNodeXML.text = self.TREntry.get_text()
								Store.protocols[curProtocolID].TRNodeXML.set('units', TRUnits)
								Store.protocols[curProtocolID].TENodeXML.text = self.TEEntry.get_text()
								Store.protocols[curProtocolID].TENodeXML.set('units', TEUnits)
								Store.protocols[curProtocolID].TINodeXML.text = self.TIEntry.get_text()
								Store.protocols[curProtocolID].TINodeXML.set('units', TIUnits)
								Store.protocols[curProtocolID].numberOfVolumesNodeXML.text = self.numberOfVolumesEntry.get_text()
								Store.protocols[curProtocolID].acquisitionOrderNodeXML.text = self.acquisitionOrderCombo.get_active_text()
								Store.protocols[curProtocolID].flipAngleNodeXML.text = self.flipAngleEntry.get_text()
								
								self.window.destroy()
							#if len(protocolNodes) > 0:
						#if not (sliceThicknessUnits == "mm" or sliceThicknessUnits == "cm"):
					#if int(self.numberOfSlicesEntry.get_text()) < 1:
				#if numberOfSlicesEntryMat == None:
			#if self.protocolIDEntry.get_text() == "":
		elif data == "Cancel":
			self.window.destroy()

class UIManagerExample:

	WIDTH = 800
	HEIGHT = 600
	def layout_resize(self, widget, event):
		x, y, width, height = widget.get_allocation()
		if width > self.lwidth or height > self.lheight:
			self.lwidth = max(width, self.lwidth)
			self.lheight = max(height, self.lheight)
			widget.set_size(self.lwidth, self.lheight)

	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="SaveProtocolXML"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
		<menu action="Edit">
			<menuitem action="InsertProtocol"/>
			<menuitem action="EditProtocol"/>
			<menuitem action="DeleteProtocol"/>
			<separator/>
			<menuitem action="InsertCondition"/>
			<menuitem action="EditCondition"/>
			<menuitem action="DeleteCondition"/>
			<separator/>
			<menuitem action="InsertOtherData"/>
			<menuitem action="EditOtherData"/>
			<menuitem action="DeleteOtherData"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="SaveProtocolXML"/>
		<separator/>
		<toolitem action="InsertProtocol"/>
		<toolitem action="EditProtocol"/>
		<toolitem action="DeleteProtocol"/>
		<separator/>
		<toolitem action="InsertCondition"/>
		<toolitem action="EditCondition"/>
		<toolitem action="DeleteCondition"/>
		<separator/>
		<toolitem action="InsertOtherData"/>
		<toolitem action="EditOtherData"/>
		<toolitem action="DeleteOtherData"/>
	</toolbar>
	</ui>'''
	def __init__(self):
		# Create the toplevel window
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBI protocol database")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

		# A gtk.VBox is a container that allows you to insert objects
		# vertically
		vbox = gtk.VBox()
		window.add(vbox)
		
		# Create a UIManager instance
		uimanager = gtk.UIManager()

		# Add the accelerator group to the toplevel window
		accelgroup = uimanager.get_accel_group()
		window.add_accel_group(accelgroup)

		# Create an ActionGroup
		self.actiongroup = gtk.ActionGroup('UIManagerExample')

		# Create actions
		#(action, ?, menu test, ?, tooltiptext, callback)
		#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
		self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
					('SaveProtocolXML', None, '_Save protocol XML', None, 'Save protocol XML file', self.save_protocol_xml_cb),
					('InsertProtocol', None, '_Insert protocol', None, 'Insert a protocol', self.insert_protocol_cb),
					('EditProtocol', None, '_Edit protocol', None, 'Edit a protocol', self.edit_protocol_cb),
					('DeleteProtocol', None, '_Delete protocol', None, 'Delete a protocol', self.delete_protocol_cb),
					('InsertCondition', None, '_Insert condition', None, 'Insert a condition', self.insert_condition_cb),
					('EditCondition', None, '_Edit condition', None, 'Edit a condition', self.edit_condition_cb),
					('DeleteCondition', None, '_Delete condition', None, 'Delete a condition', self.delete_condition_cb),
					('InsertOtherData', None, '_Insert OtherData', None, 'Insert a OtherData', self.insert_otherdata_cb),
					('EditOtherData', None, '_Edit OtherData', None, 'Edit a OtherData', self.edit_otherdata_cb),
					('DeleteOtherData', None, '_Delete OtherData', None, 'Delete a OtherData', self.delete_otherdata_cb),
					('File', None, '_File'), ('Edit', None, '_Edit')])
		self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

		self.actiongroup.get_action("EditProtocol").set_sensitive(False)
		self.actiongroup.get_action("DeleteProtocol").set_sensitive(False)
		self.actiongroup.get_action("DeleteCondition").set_sensitive(False)
		self.actiongroup.get_action("EditCondition").set_sensitive(False)
		self.actiongroup.get_action("InsertCondition").set_sensitive(False)
		self.actiongroup.get_action("InsertOtherData").set_sensitive(False)
		self.actiongroup.get_action("EditOtherData").set_sensitive(False)
		self.actiongroup.get_action("DeleteOtherData").set_sensitive(False)
		# Add the actiongroup to the uimanager
		uimanager.insert_action_group(self.actiongroup, 0)

		# Add a UI description
		uimanager.add_ui_from_string(self.ui)

		# Create a MenuBar
		menubar = uimanager.get_widget('/MenuBar')
		vbox.pack_start(menubar, False)

		# Create a Toolbar
		toolbar = uimanager.get_widget('/Toolbar')
		vbox.pack_start(toolbar, False)
        	
		self.mdl = Store.get_model()
		self.view = Display.make_view( self.mdl )
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		tree_selection = self.view.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		tree_selection.set_select_function(self.selectionChanged, full=False, data=None)
		
		scrolled_window.add(self.view)
		vbox.pack_start(scrolled_window, True)
	
		window.show_all()
		return

	def selectionChanged(self, path, data):
		selectedEntry = Store.getEntryFromPath(path)
		if selectedEntry != None:
			if selectedEntry in Store.pathIterators:
				if Store.pathIterators[selectedEntry].protocol != "":
					self.actiongroup.get_action("InsertCondition").set_sensitive(True)
					self.actiongroup.get_action("InsertOtherData").set_sensitive(True)
				else:	
					self.actiongroup.get_action("InsertCondition").set_sensitive(False)
					self.actiongroup.get_action("InsertOtherData").set_sensitive(False)
				if Store.pathIterators[selectedEntry].protocol != "" and Store.pathIterators[selectedEntry].attributeName == "ID":
					self.actiongroup.get_action("EditProtocol").set_sensitive(True)
					self.actiongroup.get_action("DeleteProtocol").set_sensitive(True)
				else:
					self.actiongroup.get_action("EditProtocol").set_sensitive(False)
					self.actiongroup.get_action("DeleteProtocol").set_sensitive(False)
				# disable check out data if we are not on a data resource tab
				if Store.pathIterators[selectedEntry].condition != None and Store.pathIterators[selectedEntry].attributeName == "ID":
					self.actiongroup.get_action("EditCondition").set_sensitive(True)
					self.actiongroup.get_action("DeleteCondition").set_sensitive(True)
				else:
					self.actiongroup.get_action("EditCondition").set_sensitive(False)
					self.actiongroup.get_action("DeleteCondition").set_sensitive(False)
				#if Store.pathIterators[selectedEntry].dataResourceIdx

				if Store.pathIterators[selectedEntry].otherData != None and Store.pathIterators[selectedEntry].attributeName == "ID":
					self.actiongroup.get_action("EditOtherData").set_sensitive(True)
					self.actiongroup.get_action("DeleteOtherData").set_sensitive(True)
				else:
					self.actiongroup.get_action("EditOtherData").set_sensitive(False)
					self.actiongroup.get_action("DeleteOtherData").set_sensitive(False)
				#if Store.pathIterators[selectedEntry].dataResourceIdx
			else:
				print "Selected entry not in the pathIterators"
		else:
			print "Selected Entry not found"
			
		return True
	
	def save_protocol_xml_cb(self, b):
		# get the file name that the user wants to save the protocol file to
		Store.protocolXML.write(os.path.join(databaseDirectory, "protocols.xml"), pretty_print=True)
	
	def quit_cb(self, b):
		print 'Quitting program'
		gtk.main_quit()

	def insert_otherdata_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		insertOtherDataWindow.showWindow(protocol=Store.pathIterators[selectedEntry].protocol)
		pass
		
	def edit_otherdata_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		insertOtherDataWindow.showWindow(protocol=Store.pathIterators[selectedEntry].protocol, otherData=Store.pathIterators[selectedEntry].otherData)

	def delete_otherdata_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)

		Store.deleteOtherData(otherData=Store.pathIterators[selectedEntry].otherData, protocol=Store.pathIterators[selectedEntry].protocol)
		Display.actiongroup.get_action("EditOtherData").set_sensitive(False)
		Display.actiongroup.get_action("DeleteOtherData").set_sensitive(False)

	def insert_condition_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		insertConditionWindow.showWindow(protocol=Store.pathIterators[selectedEntry].protocol)
		pass
	
	def edit_condition_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		insertConditionWindow.showWindow(protocol=Store.pathIterators[selectedEntry].protocol, condition=Store.pathIterators[selectedEntry].condition)

	def delete_condition_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)

		curCondition = Store.protocols[Store.pathIterators[selectedEntry].protocol].condition[Store.pathIterators[selectedEntry].condition]
		Store.deleteCondition(Store.pathIterators[selectedEntry].condition, Store.pathIterators[selectedEntry].protocol)
		Display.actiongroup.get_action("EditCondition").set_sensitive(False)
		Display.actiongroup.get_action("DeleteCondition").set_sensitive(False)
	
	def insert_protocol_cb(self, b):
		insertProtocolWindow.showWindow()

	def edit_protocol_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)

		insertProtocolWindow.showWindow(protocol=Store.pathIterators[selectedEntry].protocol)

	def delete_protocol_cb(self, b):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getEntryFromPath(path)
		
		Store.deleteProtocol(Store.pathIterators[selectedEntry].protocol)
		Display.actiongroup.get_action("EditProtocol").set_sensitive(False)
		Display.actiongroup.get_action("DeleteProtocol").set_sensitive(False)

# This contains the TreeView widget and handles callbacks to do with the tree view
class DisplayModel:
	""" Displays the Info_Model model in a view """
	def make_view( self, model ):
		""" Form a view for the Tree Model """
		self.view = gtk.TreeView( model )
		# setup the text cell renderer and allows these
		# cells to be edited.
		self.renderer = gtk.CellRendererText()
		self.renderer.set_property( 'editable', False )
		#self.renderer.connect( 'edited', self.col0_edited_cb, model )
		#self.view.connect( 'button-press-event', self.col0_button_press_event_cb, model )

		# The toggle cellrenderer is setup and we allow it to be
		# changed (toggled) by the user.
		self.renderer1 = gtk.CellRendererText()
		self.renderer1.set_property('editable', False)
		self.renderer1.connect( 'edited', self.col1_edited_cb, model )
		self.renderer1.connect( 'editing-started', self.col1_editing_started_cb, model )
		self.renderer1.set_property("cell-background", "cyan")
		
		self.renderer2 = gtk.CellRendererText()
		self.renderer2.set_property('editable', False)
		self.renderer1.connect( 'edited', self.col2_edited_cb, model )
		self.renderer1.connect( 'editing-started', self.col2_editing_started_cb, model )
		self.renderer1.set_property("cell-background", "cyan")
					
		# Connect column0 of the display with column 0 in our list model
		# The renderer will then display whatever is in column 0 of
		# our model .
		self.column0 = gtk.TreeViewColumn("Name", self.renderer, text=0)
			
		# The columns active state is attached to the second column
		# in the model.  So when the model says True then the button
		# will show as active e.g on.
		self.column1 = gtk.TreeViewColumn("Value", self.renderer1, text=1, cell_background_set=2 )
		self.column2 = gtk.TreeViewColumn("Units (if applicable)", self.renderer2, text=3)
		self.view.append_column( self.column0 )
		self.view.append_column( self.column1 )
		self.view.append_column( self.column2 )
		return self.view

	# the event parameter is a gtk.gdk.Event object
	# it has members that allow you to see what the input was
	# event.type
	#	gtk.gdk.BUTTON_PRESS (single click)
	#	gtk.gdk._2BUTTON_PRESS (double click)
	# for the button press events
	# event.button
	#	1: left mouse button
	#	2: middle mouse button
	#	3: right mouse button
#	def col0_button_press_event_cb(self, widget, event, model):
#		if widget == self.view:
#			if event.type == gtk.gdk.BUTTON_PRESS:
#				if event.button == 3:
#					Selection = self.view.get_selection()
#					(Model, Rows) = Selection.get_selected_rows()
#					for i in Rows:
#						print ":".join(str(v) for v in i)
#
#			elif event.type == gtk.gdk._2BUTTON_PRESS:
#				print "double click"
	def col1_editing_started_cb(self, cellrenderer, editable, path, model):
		"""Called when a text cell is edited.  It puts the new text"""
		"""in the model so that it is displayed properly."""
		#print "Editing started on " + str(cellrenderer) + "\n" + str(editable) + "\n" + str(path) + "\n" + str(model)
		#selectedEntry = Store.getEntryFromPath(path)
		#if selectedEntry != None:
		#	print str(Store.pathIterators[selectedEntry])
		return
	
	def col2_edited_cb( self, cell, path, new_text, model ):
		"""
		Sets the toggled state on the toggle button to true or false.
		"""
		selectedEntry = Store.getEntryFromPath(path)
		if selectedEntry != None:
			if Store.pathIterators[selectedEntry].isUnitsEditable == True:
				model[path][2] = new_text
				Store.pathIterators[selectedEntry].XMLElement.set("units", new_text)
			else:
				dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The units field that was just modified is read-only, not committing the change")
				dialog.run()
				dialog.destroy()
		return
	
	def col2_editing_started_cb(self, cellrenderer, editable, path, model):
		"""Called when a text cell is edited.  It puts the new text"""
		"""in the model so that it is displayed properly."""
		#print "Editing started on " + str(cellrenderer) + "\n" + str(editable) + "\n" + str(path) + "\n" + str(model)
		#selectedEntry = Store.getEntryFromPath(path)
		#if selectedEntry != None:
		#	print str(Store.pathIterators[selectedEntry])
		return

	def col1_edited_cb( self, cell, path, new_text, model ):
		"""
		Sets the toggled state on the toggle button to true or false.
		"""
		selectedEntry = Store.getEntryFromPath(path)
		if Store.pathIterators[selectedEntry].isEditable == True:
			model[path][1] = new_text
			Store.pathIterators[selectedEntry].XMLElement.text = new_text
		else:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field that was just modified is read-only, not committing the change")
			dialog.run()
			dialog.destroy()
		
		#print "Toggle '%s' to: %s" % (model[path][0], model[path][1],)
		return
	
	

if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "", ["database-directory="])
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a

	#for o, a in opts:
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)

	Store = InfoModel(databaseDirectory)	
	Display = DisplayModel()
	insertProtocolWindow = InsertProtocolWindow()
	insertConditionWindow = InsertConditionWindow()
	insertOtherDataWindow = InsertOtherDataWindow()
	UIExample = UIManagerExample()
	gtk.main()
