#!/usr/bin/python

import sys
import getopt
import os

import dicom

import DICOMUtilities


def usage():
	print "Usage: " + sys.argv[0] + " [-v] -d <source directory> --patient-name=<> --patient-birthdate=<> --patient-address=<> [--dry-run]"
	print
	print "Mandatory arguments:"
	print "-d <source directory>: the root directory of the source files to be modified"
	print
	print "AT LEAST ONE OF:"
	print "--patient-name=<> --patient-birthdate=<> --patient-address=<>"
	print "These fields will be overwritten with the values specified in their respective options"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "--dry-run: will not actually modify source files, just print which ones will be modified"

def main():
	DICOMTags = {'subjectName': "0x0010,0x0010"};
	
	opts, args = getopt.getopt(sys.argv[1:], "d:hv", ["patient-name=", "patient-birthdate=", "patient-address=", 'dry-run'])

	baseDirectory = None
	subjectName = None
	subjectAddress = None
	subjectDOB = None
	dryRun = False
	verbose = False

	for o, a in opts:
		if o == '-h':
			usage()
			quit()
		if o == '-d':
			baseDirectory = a
		if o == '-v':
			verbose = True
		if o == '--patient-name':
			subjectName = a
		if o == '--patient-address':
			subjectAddress = a
		if o == '--patient-birthdate':
			subjectDOB = a
		if o == '--dry-run':
			dryRun = True
	#for o, a in opts:

	if baseDirectory == None:
		print "source directory not given"
		usage()
		quit()

	if subjectName == None and subjectAddress == None and subjectDOB == None:
		print "No fields were given to change"
		usage()
		quit()
	
	if not os.path.isdir(baseDirectory):
		print "Directory given is not a directory"
	else:
		DICOMFiles = DICOMUtilities.getDICOMKeysRecursive(baseDirectory, Recurse=True, keyArray = DICOMTags)
		for curFile in DICOMFiles:
			if dryRun == False:
				if verbose == True:
					print curFile
				ds = dicom.filereader.ReadFile(curFile)
				if subjectName != None:
					if not (0x10,0x10) in ds:
						ds.AddNew(0x00100010, "PN", subjectName)
					else:
						ds[0x10,0x10].value = subjectName # subject name

				if subjectAddress != None:
					if not (0x10,0x1040) in ds:
						ds.AddNew(0x00101040, "LO", subjectAddress)
					else:
						ds[0x10,0x1040].value = subjectAddress # subject address
				
				if subjectDOB != None:
					if not (0x10,0x30) in ds:
						ds.AddNew(0x00100030, "DA", subjectDOB)
					else:
						ds[0x10,0x0030].value = subjectDOB # subject DOB
				
				#dicom.filewriter.WriteFile(os.path.join(databaseDirectory, curAcquisitionURI, tail), ds)
				dicom.filewriter.WriteFile(curFile, ds)

				del ds
			else:
				print curFile
		#for curFile in DICOMFiles:
	#if not os.path.isdir(baseDirectory):
#def main():

if __name__ == "__main__":
	main()
