#!/usr/bin/python

# extracts a volume from a raw data resource in the database
# the raw data resource could be DICOM, GENESIS or ANALYZE

import tempfile

from lxml import etree
import sys
import os
import getopt
import DataTransferConstants
import re
import shutil
import errno



def usage():
	print "Usage: " + sys.argv[0] + " [--database-directory=<>]"
	print
	print "Strips the acquisition URI from the URI entries in the dataResource nodes, this is a migration script"
	print
	print "--database-directory: the root directory of the database"
	print "-h: this help message, program will quit"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "h", ["database-directory="])
	
	databaseDirectory = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-h':
			usage()
			quit()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)

	projectList = DataTransferConstants.projectList(databaseDirectory)

	for projectName in projectList:
		projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
		projectXMLRoot = projectXML.getroot()

		dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource")

		for curDataResourceNode in dataResourceNodes:
			acquisitionURI = curDataResourceNode.get("acquisitionURI")
			for curURI in curDataResourceNode.iterchildren(tag = "uri"):
				idx = curURI.text.find(acquisitionURI)
				if idx != -1:
					curURI.text = curURI.text[idx + len(acquisitionURI) + 1:]
		projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print = True)

if __name__ == "__main__":
	main()
