#!/usr/bin/python

from lxml import etree

import re
import os
import errno
import sys
import getopt

import DataTransferConstants

import dicom

from struct import unpack

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --subject-id=<> --new-subject-type=<patients or controls>"
	print
	print "Mandatory arguments:"
	print "-p <project name>: the project name, the project's directory must exist and be created by dt_create_project.py"
	print 
	print "--subject-id=<> the numeric id of a subject"
	print "--new-subject-type=<> the subject type to change to, must be patients or controls"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "--database-directory=<database directory>: the root directory of where the database is situated"

	
# moves a subject's data into another subject type/subject/study/
# rationale:
# Spelling errors in names or errors in birthdates may result in either a new subject being created when the data is already there
# or the user may just wish to change the subject type from patient to control

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "subject-id=", "new-subject-type="])

	projectName = None
	databaseDirectory = None
	baseDirectory = None
	subjectID = None
	newSubjectType = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '-h':
			usage()
			quit()
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-id':
			subjectID = a
		if o == '--new-subject-type':
			newSubjectType = a
	#for o, a in opts:

	#temporaryDirectory = tempfile.mkdtemp()

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or subjectID == None or newSubjectType == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	pat = re.compile('^\d+$')
	
	if re.match(pat, subjectID) == None:
		print "subject id was not numeric"
		quit()


	if not (newSubjectType.lower() == 'patients' or newSubjectType.lower() == 'controls'):
		print "--new-subject-type must be patients or controls"
		quit()

	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	projectPatientsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='patients']")
	if len(projectPatientsListRoot) == 0:
		print "No subjects subject group list, should not happen"
		quit()
	else:
		projectPatientsListRoot = projectPatientsListRoot[0]

	projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/project[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
	if len(projectControlsListRoot) == 0:
		print "No subjects subject group list, should not happen"
		quit()
	else:
		projectControlsListRoot = projectControlsListRoot[0]

	# match all the DataResource nodes with the input pattern

	subjectNode = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + subjectID + "']")
	if len(subjectNode) == 0:
		print "The subject could not be found"
		quit()
	else:
		subjectNode = subjectNode[0]

		curSubjectType = None
		# cycle through
		for curSubjectNode in projectControlsListRoot.iterchildren():
			if curSubjectNode.text == subjectID:
				curSubjectType = 'controls'
				break
		if curSubjectType == None:
			for curSubjectNode in projectPatientsListRoot.iterchildren():
				if curSubjectNode.text == subjectID:
					curSubjectType = 'patients'
					break
		if curSubjectType == None:
			print "Although I found the subject in the database, I couldnt determine its type"
			quit()
		else:
			if curSubjectType != newSubjectType:
				# remove the subject from the current list and put a new entry in the list that we are changing the subject type to
				if curSubjectType == 'controls':
					projectControlsListRoot.remove(curSubjectNode)
					newSubjectTypeNode = etree.SubElement(projectPatientsListRoot, "subjectID")
				else:
					projectPatientsListRoot.remove(curSubjectNode)
					newSubjectTypeNode = etree.SubElement(projectControlsListRoot, "subjectID")
				newSubjectTypeNode.text = subjectID
				visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectGroupID='" + curSubjectType + "' and @subjectID='" + subjectID + "']")
				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectGroupID='" + curSubjectType + "' and @subjectID='" + subjectID + "']")
				episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectGroupID='" + curSubjectType + "' and @subjectID='" + subjectID + "']")
				acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectGroupID='" + curSubjectType + "' and @subjectID='" + subjectID + "']")
				dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectGroupID='" + curSubjectType + "' and @subjectID='" + subjectID + "']")

				for curNodeToChange in visitNodes:
					curNodeToChange.set("subjectGroupID", newSubjectType)
				for curNodeToChange in studyNodes:
					curNodeToChange.set("subjectGroupID", newSubjectType)
				for curNodeToChange in episodeNodes:
					curNodeToChange.set("subjectGroupID", newSubjectType)
				for curNodeToChange in acquisitionNodes:
					curNodeToChange.set("subjectGroupID", newSubjectType)
				for curNodeToChange in dataResourceNodes:
					curNodeToChange.set("subjectGroupID", newSubjectType)
				projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
		#if curSubjectType == None:
	#if len(subjectNode) == 0:
#def main():

if __name__ == "__main__":
	main()
