#!/usr/bin/python

from lxml import etree

import re
import os
import errno
import sys
import getopt
import shutil

import DataTransferConstants

from struct import unpack

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --subject-id=<> --visit-id=<> --new-visit-id=<>"
	print
	print "Mandatory arguments:"
	print "-p <project name>: the project name, the project's directory must exist and be created by dt_create_project.py"
	print 
	print "--subject-id=<> the numeric id of a subject"
	print "--visit-id=<> the visit id that you wish to change"
	print "--new-visit-id=<>, matched visits will have their dates changed to whatever is specified here"
	print
	print "Optional arguments:"
	print "-h: help (this message)"
	print "--database-directory=<database directory>: the root directory of where the database is situated"


# moves a subject's data into another subject type/subject/study/
# rationale:
# Spelling errors in names or errors in birthdates may result in either a new subject being created when the data is already there
# or the user may just wish to change the subject type from patient to control

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "subject-id=", "visit-id=", "new-visit-id="])

	projectName = None
	databaseDirectory = None
	baseDirectory = None
	subjectID = None
	visitID = None
	newVisitID = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '-h':
			usage()
			quit()
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-id':
			subjectID = a
		if o == '--visit-id':
			visitID = a
		if o == '--new-visit-id':
			newVisitID = a
	#for o, a in opts:

	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or subjectID == None or visitID == None or newVisitID == None:
		print "Mandatory argument not given"
		usage()
		quit()

	pat = re.compile('^\d+$')
	
	if re.match(pat, subjectID) == None:
		print "subject id was not numeric"
		quit()
	
	if DataTransferConstants.isValidDate(visitID) == False:
		print "visit id was not numeric"
		quit()

	if DataTransferConstants.isValidDate(newVisitID) == False:
		print "new visit id was not numeric"
		quit()
	
	projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
	projectXMLRoot = projectXML.getroot()
	
	# match all the DataResource nodes with the input pattern

	subjectNode = projectXMLRoot.xpath("/XCEDE/subject[@ID='" + subjectID + "']")
	if len(subjectNode) == 0:
		print "The subject could not be found"
		quit()
	else:
		subjectNode = subjectNode[0]
		
		visitNode = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + subjectID + "' and @ID='" + visitID + "']")
		if len(visitNode) == 0:
			print "The subject was found but they had no visits matching your pattern"
		else:
			visitNode = visitNode[0]
			# check to see if the new visit is already there
			existingVisitNode = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + subjectID + "' and @ID='" + newVisitID + "']")

			if len(existingVisitNode) > 0:
				# move this visit into the existing one
				# go through each episode and see if it exists
				# if the current episode exists and there is data then move each acquisition into it as a new acquisition
				existingStudyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + subjectID + "' and @visitID='" + newVisitID + "']")
				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
				for curStudyNode in studyNodes:
					existingStudyNode = None
					for curExistingStudyNode in existingStudyNodes:
						if curStudyNode.get("ID") == curExistingStudyNode.get("ID"):
							existingStudyNode = curExistingStudyNode
					#for curExistingStudyNode in existingStudyNodes:
					if existingStudyNode != None:
						episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
						existingEpisodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + subjectID + "' and @visitID='" + newVisitID + "']")
						for curEpisodeNode in episodeNodes:
							existingEpisodeNode = None
							for curExistingEpisodeNode in existingEpisodeNodes:
								if curEpisodeNode.get("ID") == curExistingEpisodeNode.get("ID"):
									existingEpisodeNode = curExistingEpisodeNode
							if existingEpisodeNode != None:
								existingAcquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + subjectID + "' and @visitID='" + newVisitID + "' and @studyID='" + curEpisodeNode.get("studyID") + "' and @episodeID='" + curEpisodeNode.get("ID") + "']")
								acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + subjectID + "' and @visitID='" + visitID + "' and @studyID='" + curEpisodeNode.get("studyID") + "' and @episodeID='" + curEpisodeNode.get("ID") + "']")
								nextAcquisitionID = -1
								for curExistingAcquisitionNode in existingAcquisitionNodes:
									if int(curExistingAcquisitionNode.get("ID")) >= nextAcquisitionID:
										nextAcquisitionID = int(curExistingAcquisitionNode.get("ID")) + 1
								for curAcquisitionNode in acquisitionNodes:
									dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + subjectID + "' and @visitID='" + visitID + "' and @studyID='" + curAcquisitionNode.get("studyID") + "' and @episodeID='" + curEpisodeNode.get("ID") + "' and @acquisitionID='" + curAcquisitionNode.get("ID") + "']")
									for curDataResourceNode in dataResourceNodes:
										curDataResourceNode.set("acquisitionID", str(nextAcquisitionID))
										curDataResourceNode.set("visitID", newVisitID)
									# move the episodes into the new visit now
									print "mv " + os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID, curEpisodeNode.get("studyID"), curEpisodeNode.get("ID"), curAcquisitionNode.get("ID").zfill(DataTransferConstants.padZerosLength)) + " > " + \
									os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID, curEpisodeNode.get("studyID"), curEpisodeNode.get("ID"), str(nextAcquisitionID).zfill(DataTransferConstants.padZerosLength))

									shutil.move(os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID, curEpisodeNode.get("studyID"), curEpisodeNode.get("ID"), curAcquisitionNode.get("ID").zfill(DataTransferConstants.padZerosLength)), \
									os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID, curEpisodeNode.get("studyID"), curEpisodeNode.get("ID"), str(nextAcquisitionID).zfill(DataTransferConstants.padZerosLength)))

									curAcquisitionNode.set("ID", str(nextAcquisitionID))
									nextAcquisitionID = nextAcquisitionID + 1
								projectXMLRoot.remove(curEpisodeNode)
							else:
								# the study exists but the episode doesnt, move the episode directory in and change all the IDs
								acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
								dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")

								for curNodeToChange in acquisitionNodes:
									curNodeToChange.set("visitID", newVisitID)
									curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
									curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
									curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
								
								for curNodeToChange in dataResourceNodes:
									curNodeToChange.set("visitID", newVisitID)
									curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
									curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
									curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
									curNodeToChange.set("acquisitionURI", os.path.join(curNodeToChange.get("episodeURI"), curNodeToChange.get("acquisitionID")))
								# move the study from the old visit to the new visit
								print "mv " + os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID, curStudyNode.get("ID"), curEpisodeNode.get("ID")) + " > " + \
								os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID, curStudyNode.get("ID"), curEpisodeNode.get("ID"))
								shutil.move(\
								os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID, curStudyNode.get("ID"), curEpisodeNode.get("ID")), \
								os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID, curStudyNode.get("ID"), curEpisodeNode.get("ID")))

								curEpisodeNode.set("visitID", newVisitID)
								curEpisodeNode.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID"), newVisitID))
							#if existingEpisodeNode != None:
						projectXMLRoot.remove(curStudyNode)
					else:
						# no study with this name in the new visit, move the files and change all the IDs
						episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + subjectID + "' and @visitID='" + visitID + "]'")
						acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + subjectID + "' and @visitID='" + visitID + "]'")
						dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
						
						curStudyNode.set("visitID", newVisitID)
						curStudyNode.set("visitURI", os.path.join(curNodeToChange.get("projectID"), curNodeToChange.get("subjectID").zfill(DataTransferConstants.padZerosLength), newVisitID))

						for curNodeToChange in episodeNodes:
							curNodeToChange.set("visitID", newVisitID)
							curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), curNodeToChange.get("subjectID").zfill(DataTransferConstants.padZerosLength), newVisitID))
							curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
						
						for curNodeToChange in acquisitionNodes:
							curNodeToChange.set("visitID", newVisitID)
							curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
							curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
							curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
						
						for curNodeToChange in dataResourceNodes:
							curNodeToChange.set("visitID", newVisitID)
							curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
							curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
							curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
							curNodeToChange.set("acquisitionURI", os.path.join(curNodeToChange.get("episodeURI"), curNodeToChange.get("acquisitionID")))
						# move the study from the old visit to the new visit
						shutil.move(os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID, curStudyNode.get("ID")), os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID, curStudyNode.get("ID")))
					#if existingStudyNode != None:
				#for curStudyNode in studyNodes:
				shutil.rmtree(os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID))
				projectXMLRoot.remove(visitNode)
				projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
			else:
				# simply rename all the visit ids
				visitNode.set("ID", newVisitID)
				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
				episodeNodes = projectXMLRoot.xpath("/XCEDE/episode[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
				acquisitionNodes = projectXMLRoot.xpath("/XCEDE/acquisition[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
				dataResourceNodes = projectXMLRoot.xpath("/XCEDE/dataResource[@subjectID='" + subjectID + "' and @visitID='" + visitID + "']")
				
				for curNodeToChange in studyNodes:
					curNodeToChange.set("visitID", newVisitID)
					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
				
				for curNodeToChange in episodeNodes:
					curNodeToChange.set("visitID", newVisitID)
					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
				
				for curNodeToChange in acquisitionNodes:
					curNodeToChange.set("visitID", newVisitID)
					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
					curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
				
				for curNodeToChange in dataResourceNodes:
					curNodeToChange.set("visitID", newVisitID)
					curNodeToChange.set("visitURI", os.path.join(curNodeToChange.get("subjectURI"), newVisitID))
					curNodeToChange.set("studyURI", os.path.join(curNodeToChange.get("visitURI"), curNodeToChange.get("studyID")))
					curNodeToChange.set("episodeURI", os.path.join(curNodeToChange.get("studyURI"), curNodeToChange.get("episodeID")))
					curNodeToChange.set("acquisitionURI", os.path.join(curNodeToChange.get("episodeURI"), curNodeToChange.get("acquisitionID")))

				# move the visit directory
				print "mv " + os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID) + " > " + os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID)
				shutil.move(os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), visitID), os.path.join(databaseDirectory, projectName, subjectID.zfill(DataTransferConstants.padZerosLength), newVisitID))
				projectXML.write(os.path.join(databaseDirectory, projectName, "projects.xml"), pretty_print=True)
			#if len(existingVisitNode) > 0:
	#if len(subjectNode) == 0:
#def main():

#
#
#			# find the existing
#			newSubjectNodes = []
#			if subjectID != None:
#				for curSubjectNode in subjectNodes:
#					if DataTransferPatterns.matchNumericPattern(curSubjectNode.get("ID"), subjectID) == True:
#						newSubjectNodes.append(curSubjectNode)
#				subjectNodes = newSubjectNodes
#				del newSubjectNodes
#			newVisitNodes = []
#			for curSubjectNode in subjectNodes:
#				visitNodes = projectXMLRoot.xpath("/XCEDE/visit[@subjectID='" + curSubjectNode.get("ID") + "']")\
#				if visitID != None:
#					for curVisitNode in visitNodes:
#						if DataTransferPatterns.matchNumericPattern(curVisitNode.get("ID"), visitID) == True:
#							newVisitNodes.append(curVisitNode)
#				else:
#					for curVisitNode in visitNodes:
#						newVisitNodes.append(curVisitNode)
#			visitNodes = newVisitNodes
#			del newVisitNodes
#			for curVisitNode in visitNodes:
#				studyNodes = projectXMLRoot.xpath("/XCEDE/study[@subjectID='" + curVisitNode.get("subjectID") + "' and @visitID='" + curVisitNode.get("ID") + "']")
#				if studyID != None:
#					for curStudyNode in StudyNodes:
#						if DataTransferPatterns.matchNumericPattern(curStudyNode.get("ID"), studyID) == True:
#							newStudyNodes.append(curStudyNode)
#				else:
#					for curStudyNode in StudyNodes:
#						newStudyNodes.append(curStudyNode)
#			StudyNodes = newStudyNodes
#			del newStudyNodes
#
#		projectControlsListRoot = projectXMLRoot.xpath("/XCEDE/dataResource[@ID='" + projectName + "']/projectInfo/subjectGroupList/subjectGroup[@ID='controls']")
#		
if __name__ == "__main__":
	main()
