#!/usr/bin/python

import DataTransferPatterns
import DataTransferConstants

def usage():
	print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>] --output-directory=<> --output-format=<> --output-pattern=<> --dti-output-format=<fsl or mrtrix> --sort-output=sort1,sort2... [--subject-type-pattern=<> --subject-pattern=<> --visit-pattern=<> --study-pattern=<> --episode-pattern=<> --acquisition-pattern=<> --data-resource-pattern=<>] [--match-output-file=] [--dry-run]"

	print ""
	print "-p: the project name, this directory must exist: " + os.path.join("<project directory>", "<project name>")
	print "--database-directory: the root directory of the database"
	print "-h: prints this help message, program exits"
	print "--output-directory: the directory to place the output files"
	print "--output-format: must be NIFTI or MINC"
	print "--output-pattern: each file will be named according to this pattern. Special sequences of characters can be used to substitute IDs into the filenames as follows:"
	print ""
	print "%p: project name"
	print "%t: subject type"
	print "%s: subject ID"
	print "%v: visit ID (date)"
	print "%e: episode ID"
	print "%a: acquisition ID"
	print "%d: today's date (YYYYMMDD)"
	print "%n: the index of the file in the current set of checked out files, so the first file will be 1, second this will be 2, and so on"
	print "%%: the character %"
	print ""
	print "Input field patterns:"
	print ""
	print "String fields (study-pattern, episode-pattern) permits the use of wildcard patterns in the same way as used in the shell for file matching"
	print ""
	print "Numeric patterns (subject-pattern, visit-pattern, acquisition-pattern) uses MATLAB-like syntax"
	print "\t--subject-pattern=lowerLimit:increment:upperLimit, lowerLimit:*, *:upperlimit"
	print "\t\t so --subject-pattern=1:5:21 will select subjects with IDs 1, 6, 11, 16, 21"
	print "\t\t so --subject-pattern=*:21 will select subjects with IDs below and including 21"
	print "\t\t so --subject-pattern=21:* will select subjects with IDs with 21 or above"
	print ""
	print "IMPORTANT:"
	print "\tThe wildcard character, *, is interpreted by the shell prior to program execution and thus may return a list of files in the current directory"
	print "\tTo avoid this, escape any wildcards with two backslashes, \\\\*, so that bash does not perform file matching"
	print ""
	print "--database-directory NOTES"
	print "\tThe project directory is the root directory of the database and it is an optional command line argument."
	print "\tIf the project directory given on the command line then that value will be used."
	print "\tIf it is not specified on the command line it will search the environment variable DT_PROJECT_DIR"
	print "\tIf the environment variable is empty, the value in DataTransferConstants.databaseDirectory will be used: " + DataTransferConstants.databaseDirectory
	print
	print "--dti-output-pattern=fsl (default) will produce a bval and bvecs file with each DTI volume found"
	print "--dti-output-pattern=mrtrix will produce a grad file, compatible with mrtrix, with each DTI volume found"
	print
	print "--sort-output=key1,key2,key3... is used to place output files in subdirectories of the output directory (--output-directory)"
	print " refer to the substitution rules for --output-pattern for how to insert the meta-data into the directory names"
	print " the extra key %, is used here to insert a comma into the directory name"
	print " This feature is intended to be used if you want to sort data into a particular directory structure after checkout"
	print "\tEXAMPLE"
	print "--sort-output=%e,preop,%a will place each output file into <output-directory>/<episodeid of each data file>/preop/<acquisitionid of each file>"
	print
	print "--match-output-file=<filename> prints out the checked out volumes to a file so you can check out the same list of files again or use this to"
	print " make lists of files across multiple projects and check them out all at once"
	print " if you use --match-output-file=- then the matches will be printed to standard output"
	print "each match is printed to a line in the output file with the following format:"
	print "project id\tsubject type id\tsubject id\tvisit id\tstudy id\tepisode id\tacquisition id\tdata resource id"
	print "--dry-run will not actually check the files out, useful if you just want to generate lists of matches"

def main():
	opts, args = getopt.getopt(sys.argv[1:], "p:h", ["database-directory=", "subject-type-pattern=", "subject-pattern=", "visit-pattern=", "study-pattern=", "episode-pattern=", "acquisition-pattern=", "data-resource-pattern=", "output-directory=", "output-pattern=", "output-format=", "dti-output-format=", "sort-output=", "match-output-file=", "dry-run"])

	projectName = None
	databaseDirectory = None
	subjectTypePattern = None
	subjectPattern = None
	visitPattern = None
	studyPattern = None
	episodePattern = None
	acquisitionPattern = None
	dataResourcePattern = None
	outputDirectory = None
	outputFormat = None
	outputPattern = None
	DTIOutputFormat = None
	sortOutputKeys = None
	matchOutputFile = None
	dryRun = False

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
		if o == '--subject-type-pattern':
			subjectTypePattern = a
		if o == '--subject-pattern':
			subjectPattern = a
		if o == '--visit-pattern':
			visitPattern = a
		if o == '--study-pattern':
			studyPattern = a
		if o == '--episode-pattern':
			episodePattern = a
		if o == '--acquisition-pattern':
			acquisitionPattern = a
		if o == '--data-resource-pattern':
			dataResourcePattern = a
		if o == '--output-directory':
			outputDirectory = a
		if o == '--output-format':
			outputFormat = a
		if o == '--output-pattern':
			outputPattern = a
		if o == '--dti-output-format':
			DTIOutputFormat = a
		if o == '--sort-output':
			sortOutputKeys = a
		if o == '--match-output-file':
			matchOutputFile = a
		if o == '--dry-run':
			dryRun = True
		if o == '-h':
			usage()
			quit()
	#for o, a in opts:
	
	if DTIOutputFormat == None:
		DTIOutputFormat = "fsl"
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	
	if projectName == None or outputDirectory == None or outputFormat == None or outputPattern == None:
		print "Mandatory argument not given"
		usage()
		quit()
	
	if outputFormat.lower() != "nifti" and outputFormat.lower() != "minc":
		print "output format not supported: " + outputFormat
		quit()

	if not os.path.isdir(outputDirectory):
		print "Output directory specified does not exist or is not a directory: " + outputDirectory
		quit()

if __name__ == "__main__":
	main()
