#!/usr/bin/python

# PURPOSE
#	print a list of subjects contained within a project
#	each subject list will be printed

from lxml import etree

import sys
import os
import errno
import getopt
import hashlib

import DataTransferConstants
import re

def main(): 
	opts, args = getopt.getopt(sys.argv[1:], "p:", ["database-directory="])
	projectName = None
	databaseDirectory = None

	for o, a in opts:
		if o == '-p':
			projectName = a
		if o == '--database-directory':
			databaseDirectory = a
	if projectName == None:
		print sys.argv[0] + ": prints out a list of subjects from a project"
		print ""
		print "Usage: " + sys.argv[0] + " -p <project name> [--database-directory=<>]"

		quit()
	
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)
	if databaseDirectory == None:
		databaseDirectory = "./"

	if not os.path.isfile(os.path.join(databaseDirectory, projectName, 'projects.xml')):
		print "Cannot find the project xml file: " + os.path.join(databaseDirectory, projectName, 'projects.xml')
	else:
		projectXML = etree.parse(os.path.join(databaseDirectory, projectName, 'projects.xml'))
		projectXMLRoot = projectXML.getroot()
		
		# PSEUDOCODE
		#
		# find all subject groups in the project
		# for each subject group
		#	print out the subject group name
		#	find all subjects in subject group
		#	for each subject in subject group
		#		print ID, age, sex

		subjectGroups = projectXML.xpath('//subjectGroup')
		if len(subjectGroups) > 0:
			for curSubjectGroup in subjectGroups:
				print curSubjectGroup.get("ID")
				for curSubject in curSubjectGroup.iterchildren():
					# get the subject's details in their subjectInfo 
					curSubjectDetails = projectXML.xpath("/XCEDE/subject[@ID='" + curSubject.text + "']/subjectInfo")
					if len(curSubjectDetails) == 0:
						print "Subject ID " + curSubject.text + " has no information"
					elif len(curSubjectDetails) > 1:
						print "Subject ID " + curSubject.text + " has multiple information records"
					else:
						subjectSex = None
						subjectSpecies = None
						subjectDOB = None
						for curSubjectDetailsChild in curSubjectDetails[0].iterchildren():
							if curSubjectDetailsChild.tag == 'sex':
								subjectSex = curSubjectDetailsChild.text
							elif curSubjectDetailsChild.tag == 'species':
								subjectSpecies = curSubjectDetailsChild.text
							elif curSubjectDetailsChild.tag == 'birthdate':
								subjectDOB = curSubjectDetailsChild.text
						if subjectSex == None:
							subjectSex = ""
						if subjectSpecies == None:
							subjectSpecies = ""
						if subjectDOB == None:
							subjectDOB = ""
						print "\tID: " + curSubject.text + ", Sex: " + subjectSex + ", Birth date: " + subjectDOB + ", Species: " + subjectSpecies
					#if len(curSubjectDetails) == 0:
				#for curSubject in curSubjectGroup.iterchildren():
			#for curSubjectGroup in subjectGroups:
		else:
			print "No subject groups in this project, this should not happen"
		

if __name__ == "__main__":
    main()

