#!/usr/bin/env python
import getopt
import sys
import os

import re

from lxml import etree

import pygtk
pygtk.require('2.0')
import gtk
import gobject

import DataTransferConstants
import DataTransferUserFields

class reversePathLookup:
	def __init__(self, level=None, name=None):
		self.level = level
		self.name = name

	def __str__(self):
		return "[level: " + str(self.level) + ", name: " + str(self.name)

class InfoModel:
	def migrateProjectFieldToDatabaseWide(self, level = None, name = None):
		pass

	def loadAndDisplayFields(self, databaseDirectory=None, projectName=None):
		self.listStore.clear()
		self.userFields = DataTransferUserFields.UserFields(databaseDirectory=databaseDirectory, projectName=projectName)
		
		self.projectName = projectName
		self.databaseDirectory = databaseDirectory

		# get the fields from the other projects
		projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
		self.otherProjectFields = {}
		
		self.databaseRowReferences = []
		self.projectRowReferences = []
		self.otherProjectRowReferences = {}
		self.otherProjectXML = {}
		self.otherProjectXMLRoot = {}

		if self.projectName != None:
			self.projectXML = etree.parse(os.path.join(databaseDirectory, projectName, "projects.xml")) 
			self.projectXMLRoot = self.projectXML.getroot()

		self.needToCommit = False

		# this is the array that contains the queued fields to delete in the XML files
		self.fieldsToDeleteInProjects = {}

		# this is set to true if we have added or deleted a database-wide field and it has to be added or deleted from all projects
		self.commitAllProjects = False
		
		if self.projectName != None:
			self.fieldsToDeleteInProjects[self.projectName] = []

		for curProject in projectList:
			self.fieldsToDeleteInProjects[curProject] = []
			if curProject != projectName:
				self.otherProjectFields[curProject] = DataTransferUserFields.UserFields(databaseDirectory=databaseDirectory, projectName=curProject)
				self.otherProjectXML[curProject] = etree.parse(os.path.join(databaseDirectory, curProject, "projects.xml"))
				self.otherProjectXMLRoot[curProject] = self.otherProjectXML[curProject].getroot()

		for curDatabaseField in self.userFields.databaseFields:
			curTreeNode = self.listStore.append([curDatabaseField['level'], curDatabaseField['name'], curDatabaseField['format'], curDatabaseField['defaultValue'], "database-wide", self.projectName != None])
			self.databaseRowReferences.append(gtk.TreeRowReference(self.listStore, self.listStore.get_path(curTreeNode)))
		for curDatabaseField in self.userFields.projectFields:
			curTreeNode = self.listStore.append([curDatabaseField['level'], curDatabaseField['name'], curDatabaseField['format'], curDatabaseField['defaultValue'], projectName, self.projectName == None])
			self.projectRowReferences.append(gtk.TreeRowReference(self.listStore, self.listStore.get_path(curTreeNode)))
		for curProject in projectList:
			if curProject != projectName:
				self.otherProjectRowReferences[curProject] = []
				for curDatabaseField in self.otherProjectFields[curProject].projectFields:
					curTreeNode = self.listStore.append([curDatabaseField['level'], curDatabaseField['name'], curDatabaseField['format'], curDatabaseField['defaultValue'], curProject, self.projectName == None])
					self.otherProjectRowReferences[curProject].append(gtk.TreeRowReference(self.listStore, self.listStore.get_path(curTreeNode)))

	""" The model class holds the information we want to display """
	def __init__(self, databaseDirectory=None, projectName=None):
		""" Sets up and populates our gtk.ListStore """
		""" The format of the widget is dictated by the type of store we have, since this will be a liststore it will give us a tabular output """ 
		self.listStore = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
		self.protocols = None
		self.pathIterators = None
		self.loadAndDisplayFields(databaseDirectory=databaseDirectory, projectName=projectName)
		# fix this to look at file permissions
		self.readOnly = False
	
	def __str__(self):
		return str(self.userFields)

	def get_model(self):
		""" Returns the model """
		if self.listStore:
			return self.listStore
		else:
			return None

	def getFieldFromPath(self, path):
		curRowRef = gtk.TreeRowReference(self.listStore, path)
		#print curRowRef
		for curIdx in range(len(self.databaseRowReferences)):
			if curRowRef.get_path() == self.databaseRowReferences[curIdx].get_path():
				return self.userFields.databaseFields[curIdx]
		for curIdx in range(len(self.projectRowReferences)):
			if curRowRef.get_path() == self.projectRowReferences[curIdx].get_path():
				return self.userFields.projectFields[curIdx]
		projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
		for curProject in projectList:
			if curProject != self.projectName:
				for curIdx in range(len(self.otherProjectRowReferences[curProject])):
					if curRowRef.get_path() == self.otherProjectRowReferences[curProject][curIdx].get_path():
						return self.otherProjectFields[curProject].projectFields[curIdx]
		return None
	
	def addFieldDatabase(self, level=None, name=None):
		projectList = DataTransferConstants.projectList(databaseDirectory=self.databaseDirectory)
		
		if self.projectName != None:
			curFieldToDelete = {}
			curFieldToDelete['project'] = self.projectName
			curFieldToDelete['level'] = level
			curFieldToDelete['name'] = name
			self.fieldsToDeleteInProjects.append(curFieldToDelete)

		for curProject in projectList:
			self.fieldsToDeleteInProjects[curProject] = []
			if curProject != self.projectName:
				curFieldToDelete = {}
				curFieldToDelete['project'] = curProject
				curFieldToDelete['level'] = level
				curFieldToDelete['name'] = name
				self.fieldsToDeleteInProjects.append(curFieldToDelete)

		self.needToCommit = True
		self.commitAllProjects = True

	def deleteFieldDatabase(self, level=None, name=None, path=None):
		curIter = self.listStore.get_iter(path)
		self.listStore.remove(curIter)
		projectList = DataTransferConstants.projectList(databaseDirectory=self.databaseDirectory)
		
		curIdx = self.userFields.databaseFieldIdx(level=level, name=name)
		del self.userFields.databaseFields[curIdx]

		if self.projectName != None:
			curFieldToDelete = {}
			curFieldToDelete['level'] = level
			curFieldToDelete['name'] = name
			self.fieldsToDeleteInProjects[self.projectName].append(curFieldToDelete)

		for curProject in projectList:
			if curProject != self.projectName:
				curFieldToDelete = {}
				curFieldToDelete['level'] = level
				curFieldToDelete['name'] = name
				self.fieldsToDeleteInProjects[curProject].append(curFieldToDelete)

		self.needToCommit = True
		self.commitAllProjects = True
	
	def deleteFieldProject(self, level=None, name=None, path=None):
		curIter = self.listStore.get_iter(path)
		self.listStore.remove(curIter)
		projectList = DataTransferConstants.projectList(databaseDirectory=self.databaseDirectory)
		
		curIdx = self.userFields.projectFieldIdx(level=level, name=name)
		del self.userFields.projectFields[curIdx]

		if self.projectName != None:
			curFieldToDelete = {}
			curFieldToDelete['level'] = level
			curFieldToDelete['name'] = name
			self.fieldsToDeleteInProjects[self.projectName].append(curFieldToDelete)

		self.needToCommit = True
		self.commitAllProjects = True


	# returns the actual record for a given field, whether it is a project-based or database-wide field
	def getField(self, level=None, name=None):
		databaseIdx = self.userFields.databaseFieldIdx(level=level, name=name)
		if databaseIdx != -1:
			return self.userFields.databaseFields[databaseIdx]
		projectIdx = self.userFields.projectFieldIdx(level=level, name=name)
		if projectIdx != -1:
			return self.userFields.projectFields[projectIdx]
		return None

	def fieldScope(self, level=None, name=None):
		if self.userFields.databaseFieldIdx(level=level, name=name) != -1:
			return 'database'
		else:
			if self.userFields.projectFieldIdx(level=level, name=name) != -1:
				return 'project'
			# look for it in the other projects
			projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
			for curProject in projectList:
				if curProject != Store.projectName:
					if self.otherProjectFields[curProject].projectFieldIdx(level=level, name=name) != -1:
						return 'project'
			#self.userFields.projectFieldIdx(level=level, name=name) != -1:
		if DataTransferUserFields.isFieldReserved(level = level, name = name):
			return 'reserved'
		else:
			return 'not exist'
		#elif isinstance(Present, list):
		#	return 'project'

	# returns a list of projects that a field is contained in 

	def getFieldProjectList(level = None, name = None):
		projectsPresent = []
		if self.userFields.projectFieldIdx(level=level, name=name) != -1:
			projectsPresent.append(self.projectName)
		projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
		for curProject in projectList:
			if curProject != Store.projectName:
				if self.otherProjectFields[curProject].projectFieldIdx(level=level, name=name) != -1:
					projectsPresent.append(curProject)
		return projectsPresent

	# searches through the database-wide and project-based fields to see if a field is present
	# if it is in a database-wide field, the return value will be True
	# if it is not present at all, the return value will be False
	# if it not in the database-wide set but in a number of projects, then a list of strings of project names will be returned
	#def isFieldPresent(self, level=None, name=None):
	#	projectsPresent = []
	#	if self.userFields.databaseFieldIdx(level=level, name=name) != -1:
	#		return True
	#	if self.userFields.projectFieldIdx(level=level, name=name) != -1:
	#		projectsPresent.append(self.projectName)
	#	projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
	#	for curProject in projectList:
	#		if curProject != Store.projectName:
	#			if self.otherProjectFields[curProject].projectFieldIdx(level=level, name=name) != -1:
	#				projectsPresent.append(curProject)
	#	if len(projectsPresent) == 0:
	#		return False
	#	else:
	#		return projectsPresent
	
	def replaceField(self, level=None, name=None, format=None, defaultValue=None):
		self.userFields.replaceField(level=level, name = name, format = format, defaultValue = defaultValue)

	def appendDatabaseField(self, level=None, name=None, format=None, defaultValue=None):
		# find the treeIter of the last database entry
		if len(Store.userFields.databaseFields) > 0:
			treeRowReference = Store.databaseRowReferences[len(Store.userFields.databaseFields) - 1]
			curTreeNode = Store.listStore.insert_after(Store.listStore.get_iter(treeRowReference.get_path()), row = [level, name, format, defaultValue, 'database-wide', self.projectName != None])
		else:
			curTreeNode = Store.listStore.prepend([level, name, format, defaultValue, 'database-wide', self.projectName != None])

		self.databaseRowReferences.append(gtk.TreeRowReference(self.listStore, self.listStore.get_path(curTreeNode)))
		self.userFields.appendDatabaseField(level=level, name=name, format=format, defaultValue=defaultValue)
		self.needToCommit = True
		self.commitAllProjects = True
	
	def appendProjectField(self, level=None, name=None, format=None, defaultValue=None):
		# find the treeIter of the last database entry
		if len(Store.userFields.projectFields) > 0:
			treeRowReference = Store.projectRowReferences[len(Store.userFields.projectFields) - 1]
			curTreeNode = Store.listStore.insert_after(Store.listStore.get_iter(treeRowReference.get_path()), row = [level, name, format, defaultValue, self.projectName, self.projectName == None])
		else:
			if len(Store.userFields.databaseFields) > 0:
				treeRowReference = Store.databaseRowReferences[len(Store.userFields.projectFields) - 1]
				curTreeNode = Store.listStore.insert_after(Store.listStore.get_iter(treeRowReference.get_path()), row = [level, name, format, defaultValue, self.projectName, self.projectName == None])
			else:
				curTreeNode = Store.listStore.append([level, name, format, defaultValue, self.projectName, self.projectName == None])

		self.projectRowReferences.append(gtk.TreeRowReference(self.listStore, self.listStore.get_path(curTreeNode)))
		self.userFields.appendProjectField(level=level, name=name, format=format, defaultValue=defaultValue)
		self.needToCommit = True

	def commit(self):
		if self.needToCommit == True:
			if self.projectName == None:
				self.userFields.saveDatabaseFieldFile()
				if self.commitAllProjects == True:
					# we are committing all projects, so run through all the field deletions and delete from the XML files
					projectList = DataTransferConstants.projectList(databaseDirectory=databaseDirectory)
					for curProject in projectList:
						if curProject != self.projectName:
							# need to add the DATABASE fields from the current database
							self.otherProjectFields[curProject].addFieldsFromOneSet(curFields=self.userFields.databaseFields, projectXMLRoot=self.otherProjectXMLRoot[curProject])
							self.otherProjectFields[curProject].deleteFieldInXML(curFields=self.fieldsToDeleteInProjects[curProject], projectXMLRoot=self.otherProjectXMLRoot[curProject])
							self.fieldsToDeleteInProjects[curProject] = {}
							self.otherProjectXML[curProject].write(os.path.join(self.databaseDirectory, curProject, 'projects.xml'))
			else:
				self.userFields.addFieldsFromOneSet(curFields=self.userFields.projectFields, projectXMLRoot=self.projectXMLRoot)
				self.userFields.deleteFieldInXML(curFields=self.fieldsToDeleteInProjects[self.projectName], projectXMLRoot=self.projectXMLRoot)
				self.fieldsToDeleteInProjects[self.projectName] = {}
				self.projectXML.write(os.path.join(self.databaseDirectory, self.projectName, 'projects.xml'))
				self.userFields.saveProjectFieldFile()

		self.needToCommit = False
		self.commitAllProjects = False


class AddFieldWindow:
	def __init__(self):
		pass
	
	def showWindow(self, level = None, name = None):
		self.window = gtk.Window()
		self.vbox = gtk.VBox()
		self.window.add(self.vbox)
		self.vbox.show()
		self.table = gtk.Table(5, 2, False)
	   	self.table.show()
		
		self.levelLabel = gtk.Label("Level:")
		self.table.attach(self.levelLabel, 0, 1, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.levelLabel.show()
		
		self.nameLabel = gtk.Label("Name:")
		self.table.attach(self.nameLabel, 0, 1, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.nameLabel.show()
		
		self.formatLabel = gtk.Label("Format:")
		self.table.attach(self.formatLabel, 0, 1, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.formatLabel.show()
		
		self.defaultValueLabel = gtk.Label("Default Value:")
		self.table.attach(self.defaultValueLabel, 0, 1, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.defaultValueLabel.show()
		
		self.levelCombo = gtk.combo_box_new_text()
		for curLevel in DataTransferUserFields.userFieldLevels:
			self.levelCombo.append_text(curLevel)
		self.levelCombo.show()
		self.table.attach(self.levelCombo, 1, 2, 0, 1, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.nameEntry = gtk.Entry()
		self.table.attach(self.nameEntry, 1, 2, 1, 2, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.nameEntry.show()
		
		self.formatCombo = gtk.combo_box_new_text()
		for curFormat in DataTransferUserFields.userFieldFormats:
			self.formatCombo.append_text(curFormat)
		self.formatCombo.show()
		self.table.attach(self.formatCombo, 1, 2, 2, 3, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		
		self.defaultValueEntry = gtk.Entry()
		self.table.attach(self.defaultValueEntry, 1, 2, 3, 4, gtk.FILL|gtk.EXPAND, gtk.FILL|gtk.EXPAND, 0, 0)
		self.defaultValueEntry.show()
	
		self.ButtonOK = gtk.Button(label="OK", stock=gtk.STOCK_OK)
		self.table.attach(self.ButtonOK, 0, 1, 4, 5, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonOK.show()
		self.ButtonOK.connect("clicked", self.callback, "OK")
		self.ButtonCancel = gtk.Button(label="Cancel", stock=gtk.STOCK_CANCEL)
		self.table.attach(self.ButtonCancel, 1, 2, 4, 5, gtk.FILL|gtk.SHRINK, gtk.FILL|gtk.SHRINK, 0, 0)
		self.ButtonCancel.show()
		self.ButtonCancel.connect("clicked", self.callback, "Cancel")
		self.vbox.pack_start(self.table)
		self.window.show_all()
		#self.window.set_modal(True)
		
		self.level = level
		self.name = name

		if self.level != None:
			self.levelCombo.set_sensitive(False)
			self.nameEntry.set_sensitive(False)
			curField = Store.getField(level=level, name=name)
			for curLevelIdx in range(len(DataTransferUserFields.userFieldLevels)):
				if curField['level'] == DataTransferUserFields.userFieldLevels[curLevelIdx]:
					self.levelCombo.set_active(curLevelIdx)
					break
			self.nameEntry.set_text(curField['name'])
			for curFormatIdx in range(len(DataTransferUserFields.userFieldFormats)):
				if curField['format'] == DataTransferUserFields.userFieldFormats[curFormatIdx]:
					self.formatCombo.set_active(curFormatIdx)
					break
			self.defaultValueEntry.set_text(curField['defaultValue'])

	def callback(self, widget, data=None):
		curLevel = self.levelCombo.get_active_text()
		curName = self.nameEntry.get_text()
		curFormat = self.formatCombo.get_active_text()
		curDefaultValue = self.defaultValueEntry.get_text()
		if curDefaultValue == None:
			curDefaultValue = ''

		if data == "OK":
			# so we have to have a level, name and format
			if self.levelCombo.get_active_text() == None:
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Level must be entered")
				dialog.run()
				dialog.destroy()
			elif self.nameEntry.get_text() == None:
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Name must be entered")
				dialog.run()
				dialog.destroy()
			elif self.formatCombo.get_active_text() == None:
				dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="Format must be entered")
				dialog.run()
				dialog.destroy()
			else:
				if self.name != None:
					# we are replacing, so just edit the field
					Store.replaceField(level = self.level, name = self.name, format = curFormat, defaultValue = curDefaultValue)
					# replace the row in the GUI
					Selection = Display.view.get_selection()
					(Model, parentIter) = Selection.get_selected()
					path = Model.get_path(parentIter)
					if Store.projectName == None:
						databaseField = True
						scopeString = 'database-wide'
					else:
						scopeString = Store.projectName
						databaseField = False

					Store.listStore.set(parentIter, 0, self.level, 1, self.name, 2, curFormat, 3, curDefaultValue, 4, scopeString, 5, databaseField)
					self.window.destroy()
				else:
					# if we are adding to the database-wide fields
					# then we have to check see if the field is in any of the projects
					fieldScope = Store.fieldScope(level=self.levelCombo.get_active_text(), name=self.nameEntry.get_text())
					if Store.projectName == None:
						if fieldScope == 'database':
							# its already in the database-wide fields, so put up a box and say that
							dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field is already in the database-wide set of fields")
							dialog.run()
							dialog.destroy()
						elif fieldScope == 'project':
							dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=("The field is in the following projects: " + " ".join(Store.getFieldProjectList(level = self.levelCombo.get_active_text(), name = self.nameEntry.get_text()))))
							dialog.run()
							dialog.destroy()
						else:
							# the field is not in the database, so add it
							# check see if the field is in the prohibited list
							if fieldScope == 'reserved':
								dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field is in the reserved list")
								dialog.run()
								dialog.destroy()
							else:
								if self.defaultValueEntry.get_text() == None:
									valText = ""
								else:
									valText = self.defaultValueEntry.get_text()
								Store.appendDatabaseField(level=self.levelCombo.get_active_text(), name=self.nameEntry.get_text(), format=self.formatCombo.get_active_text(), defaultValue=valText)
								self.window.destroy()
							#if fieldScope == 'reserved':
						#if fieldScope == 'database':
					else:
						# we are in "project" mode so we are modifying the project-specific fields
						isInCurrentProject = False
						if fieldScope == 'database':
							# its already in the database-wide fields, so put up a box and say that
							dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field is already in the database-wide set of fields")
							dialog.run()
							dialog.destroy()
						elif fieldScope == 'project':
							for curProject in Store.getFieldProjectList(level = self.levelCombo.get_active_text(), name = self.nameEntry.get_text()):
								if curProject == Store.projectName:
									isInCurrentProject = True
									dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format=("The field is in the following projects: " + " ".join(Store.getFieldProjectList(level = self.levelCombo.get_active_text(), name = self.nameEntry.get_text()))))
									dialog.run()
									dialog.destroy()
						if isInCurrentProject == False and fieldScope == 'not exist':
							# the field is not in the database, so add it
							# check see if the field is in the prohibited list
							if fieldScope == 'reserved':
								dialog = gtk.MessageDialog(parent=self.window, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_OK, message_format="The field is in the reserved list")
								dialog.run()
								dialog.destroy()
							else:
								if self.defaultValueEntry.get_text() == None:
									valText = ""
								else:
									valText = self.defaultValueEntry.get_text()
								Store.appendProjectField(level=self.levelCombo.get_active_text(), name=self.nameEntry.get_text(), format=self.formatCombo.get_active_text(), defaultValue=valText)
								self.window.destroy()
							#if fieldScope == 'reserved':
						#if fieldScope == 'database':
					#if Store.projectName == None:
				#if self.name != None:
		#if data == "OK":
		elif data == "Cancel":
			self.window.destroy()

class UIManagerExample:

	WIDTH = 800
	HEIGHT = 600
	def layout_resize(self, widget, event):
		x, y, width, height = widget.get_allocation()
		if width > self.lwidth or height > self.lheight:
			self.lwidth = max(width, self.lwidth)
			self.lheight = max(height, self.lheight)
			widget.set_size(self.lwidth, self.lheight)

	ui = '''<ui>
	<menubar name="MenuBar">
		<menu action="File">
			<menuitem action="OpenFieldFile"/>
			<menuitem action="SaveFieldFile"/>
			<separator/>
			<menuitem action="Quit"/>
		</menu>
		<menu action="Edit">
			<menuitem action="InsertField"/>
			<menuitem action="EditField"/>
			<menuitem action="DeleteField"/>
			<menuitem action="MigrateField"/>
		</menu>
	</menubar>
	<toolbar name="Toolbar">
		<toolitem action="OpenFieldFile"/>
		<toolitem action="SaveFieldFile"/>
		<separator/>
		<toolitem action="InsertField"/>
		<toolitem action="EditField"/>
		<toolitem action="DeleteField"/>
		<toolitem action="MigrateField"/>
	</toolbar>
	</ui>'''
	def __init__(self):
		# Create the toplevel window
		window = gtk.Window()
		window.connect('destroy', lambda w: gtk.main_quit())
		window.set_size_request(self.WIDTH, self.HEIGHT)
		window.set_title("DFBI user field interface")
		self.lwidth = self.WIDTH
		self.lheight = self.HEIGHT

		# A gtk.VBox is a container that allows you to insert objects
		# vertically
		vbox = gtk.VBox()
		window.add(vbox)
		
		# Create a UIManager instance
		uimanager = gtk.UIManager()

		# Add the accelerator group to the toplevel window
		accelgroup = uimanager.get_accel_group()
		window.add_accel_group(accelgroup)

		# Create an ActionGroup
		self.actiongroup = gtk.ActionGroup('UIManagerExample')

		# Create actions
		#(action, ?, menu test, ?, tooltiptext, callback)
		#('SaveProjectXML', stock id for the action, '_Save Project', None, 'Save a projects XML file', self.save_project_cb)
		self.actiongroup.add_actions([('Quit', gtk.STOCK_QUIT, '_Quit me!', None, 'Quit the Program', self.quit_cb),
					('OpenFieldFile', None, '_Open field files', None, 'Open field files', self.open_field_files_cb),
					('SaveFieldFile', None, '_Save field files', None, 'Save field files', self.save_field_files_cb),
					('InsertField', None, '_Insert Field', None, 'Insert a Field', self.insert_field_cb),
					('EditField', None, '_Edit Selected Field', None, 'Edit Selected Field', self.edit_field_cb),
					('DeleteField', None, '_Delete Selected Field', None, 'Delete Selected Field', self.delete_field_cb),
					('MigrateField', None, '_Migrate Selected Field to database-wide', None, 'Migrate Selected Field to database-wide', self.migrate_field_cb),
					('File', None, '_File'), ('Edit', None, '_Edit')])
		self.actiongroup.get_action('Quit').set_property('short-label', '_Quit')

		self.actiongroup.get_action("EditField").set_sensitive(False)
		self.actiongroup.get_action("DeleteField").set_sensitive(False)
		# Add the actiongroup to the uimanager
		uimanager.insert_action_group(self.actiongroup, 0)

		# Add a UI description
		uimanager.add_ui_from_string(self.ui)

		# Create a MenuBar
		menubar = uimanager.get_widget('/MenuBar')
		vbox.pack_start(menubar, False)

		# Create a Toolbar
		toolbar = uimanager.get_widget('/Toolbar')
		vbox.pack_start(toolbar, False)
        	
		self.mdl = Store.get_model()
		self.view = Display.make_view( self.mdl )
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		tree_selection = self.view.get_selection()
		tree_selection.set_mode(gtk.SELECTION_SINGLE)
		#tree_selection.select_path(0)	
		#tree_selection.set_select_function(self.selectionChanged, full=False, data=None)
		tree_selection.connect("changed", self.selectionChanged)
		scrolled_window.add(self.view)
		vbox.pack_start(scrolled_window, True)
		window.connect("delete-event", self.window_deleting_cb)
	
		window.show_all()
		return
	
	def open_field_files_cb(self, w):
		pass
	
	def save_field_files_cb(self, w):
		Store.commit()
		pass

	def insert_field_cb(self, w):
		addFieldWindow.showWindow()
	
	def edit_field_cb(self, path):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getFieldFromPath(path)

		addFieldWindow.showWindow(level = selectedEntry['level'], name = selectedEntry['name'])
		return True

	def delete_field_cb(self, w):
		Selection = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		path = Model.get_path(parentIter)
		selectedEntry = Store.getFieldFromPath(path)

		# if it is a database-wide field, then we have to queue up deletion for all projects
		if Store.projectName == None:
			Store.deleteFieldDatabase(level=selectedEntry['level'], name=selectedEntry['name'], path=path)
		else:
			Store.deleteFieldProject(level=selectedEntry['level'], name=selectedEntry['name'], path=path)
		print Store

	def migrate_field_cb(self):
		pass
	
	def selectionChanged(self, Selection):
		# see if this field is in the current project's field list, if it is, then enable the migrate option
		#election = Display.view.get_selection()
		(Model, parentIter) = Selection.get_selected()
		if parentIter != None:
			path = Model.get_path(parentIter)
			selectedEntry = Store.getFieldFromPath(path)
			curScope = Store.fieldScope(level = selectedEntry['level'], name = selectedEntry['name'])
			if Store.readOnly == False:
				if Store.projectName == None and curScope == 'project':
					self.actiongroup.get_action("EditField").set_sensitive(False)
					self.actiongroup.get_action("DeleteField").set_sensitive(False)
				elif Store.projectName != None and curScope == 'project':
					self.actiongroup.get_action("EditField").set_sensitive(True)
					self.actiongroup.get_action("DeleteField").set_sensitive(True)
				elif Store.projectName == None and curScope == 'database':
					self.actiongroup.get_action("EditField").set_sensitive(True)
					self.actiongroup.get_action("DeleteField").set_sensitive(True)
				elif Store.projectName != None and curScope == 'database':
					self.actiongroup.get_action("EditField").set_sensitive(False)
					self.actiongroup.get_action("DeleteField").set_sensitive(False)

		return True
	
	def ask_and_save(self):
		if Store.needToCommit == True:
			dialog = gtk.MessageDialog(parent=None, flags=0, type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_YES_NO, message_format="The fields have been modified. Do you wish to save the changes?")
			response = dialog.run()
			dialog.destroy()
			if response == gtk.RESPONSE_YES:
				Store.commit()
				return False
			else:
				return True

	def quit_cb(self, b):
		self.ask_and_save()
		gtk.main_quit()
	
	def window_deleting_cb(self, widget, event, data=None):
		response = self.ask_and_save()
		return response


# This contains the TreeView widget and handles callbacks to do with the tree view
class DisplayModel:
	""" Displays the Info_Model model in a view """
	def make_view( self, model ):
		""" Form a view for the Tree Model """
		self.view = gtk.TreeView( model )

		self.renderers = []
		self.treeViewColumns = []
		self.columnTitles = ["Level", "Name", "Format", "Default Value", "Scope"]
		for i in range(len(self.columnTitles)):
			self.renderers.append(gtk.CellRendererText())
			self.treeViewColumns.append(gtk.TreeViewColumn(self.columnTitles[i], self.renderers[i], text=i, foreground_set=5))

		for i in range(len(self.renderers)):
			 self.renderers[i].set_property('editable', False)
			 self.renderers[i].set_property('foreground', 'black')
			 self.view.append_column(self.treeViewColumns[i])
		self.view.set_rules_hint(True)

		return self.view

if __name__ == '__main__':
	opts, args = getopt.getopt(sys.argv[1:], "p:", ["database-directory="])
	databaseDirectory = None
	projectName = None

	for o, a in opts:
		if o == '--database-directory':
			databaseDirectory = a
		if o == '-p':
			projectName = a

	#for o, a in opts:
	databaseDirectory = DataTransferConstants.getDatabaseDirectory(databaseDirectory)

	Store = InfoModel(databaseDirectory, projectName)	
	Display = DisplayModel()
	addFieldWindow = AddFieldWindow()
	#insertProtocolWindow = InsertProtocolWindow()
	#insertConditionWindow = InsertConditionWindow()
	#insertOtherDataWindow = InsertOtherDataWindow()
	
	# check the permissions for the database user field files
	# if we are editing the database-wide fields, the user has to have write access to all the field files for all the projects
	# otherwise the user only needs write access to the current project
	# if we have permission problems then enable read-only access

	UIExample = UIManagerExample()
	gtk.main()
